/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.asset;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.engine.AbstractService;
import org.apache.tapestry.engine.IEngineServiceView;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.request.ResponseOutputStream;
import org.apache.tapestry.services.RequestExceptionReporter;

/**
 * A service for building URLs to and accessing {@link org.apache.tapestry.IAsset}s. Most of the
 * work is deferred to the {@link org.apache.tapestry.IAsset}instance.
 * <p>
 * The retrieval part is directly linked to {@link PrivateAsset}. The service responds to a URL
 * that encodes the path of a resource within the classpath. The
 * {@link #service(IEngineServiceView, IRequestCycle, ResponseOutputStream)}method reads the
 * resource and streams it out.
 * <p>
 * TBD: Security issues. Should only be able to retrieve a resource that was previously registerred
 * in some way ... otherwise, hackers will be able to suck out the .class files of the application!
 * 
 * @author Howard Lewis Ship
 */

public class AssetService extends AbstractService
{public static com.cortexeb.tools.clover.d __CLOVER_37_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1097439627296L);
    /**
     * Defaults MIME types, by extension, used when the servlet container doesn't provide MIME
     * types. ServletExec Debugger, for example, fails to do provide these.
     */

    private final static Map _mimeTypes;

    static
    {
        __CLOVER_37_0.S[12075]++;_mimeTypes = new HashMap(17);
        __CLOVER_37_0.S[12076]++;_mimeTypes.put("css", "text/css");
        __CLOVER_37_0.S[12077]++;_mimeTypes.put("gif", "image/gif");
        __CLOVER_37_0.S[12078]++;_mimeTypes.put("jpg", "image/jpeg");
        __CLOVER_37_0.S[12079]++;_mimeTypes.put("jpeg", "image/jpeg");
        __CLOVER_37_0.S[12080]++;_mimeTypes.put("htm", "text/html");
        __CLOVER_37_0.S[12081]++;_mimeTypes.put("html", "text/html");
    }

    private static final int BUFFER_SIZE = 10240;

    /** @since 3.1 */

    private RequestExceptionReporter _exceptionReporter;

    /**
     * Builds a {@link ILink}for a {@link PrivateAsset}.
     * <p>
     * A single parameter is expected, the resource path of the asset (which is expected to start
     * with a leading slash).
     */

    public ILink getLink(IRequestCycle cycle, IComponent component, Object[] parameters)
    {try { __CLOVER_37_0.M[2941]++;
        __CLOVER_37_0.S[12082]++;if ((((Tapestry.size(parameters) != 1) && (++__CLOVER_37_0.CT[2080] != 0)) || (++__CLOVER_37_0.CF[2080] == 0))){
            __CLOVER_37_0.S[12083]++;throw new ApplicationRuntimeException(Tapestry.format(
                    "service-single-parameter",
                    Tapestry.ASSET_SERVICE));}

        // Service is stateless

        __CLOVER_37_0.S[12084]++;return constructLink(cycle, Tapestry.ASSET_SERVICE, null, parameters, false);
    } finally { }}

    public String getName()
    {try { __CLOVER_37_0.M[2942]++;
        __CLOVER_37_0.S[12085]++;return Tapestry.ASSET_SERVICE;
    } finally { }}

    private static String getMimeType(String path)
    {try { __CLOVER_37_0.M[2943]++;
        __CLOVER_37_0.S[12086]++;String key;
        __CLOVER_37_0.S[12087]++;String result;
        __CLOVER_37_0.S[12088]++;int dotx;

        __CLOVER_37_0.S[12089]++;dotx = path.lastIndexOf('.');
        __CLOVER_37_0.S[12090]++;key = path.substring(dotx + 1).toLowerCase();

        __CLOVER_37_0.S[12091]++;result = (String) _mimeTypes.get(key);

        __CLOVER_37_0.S[12092]++;if ((((result == null) && (++__CLOVER_37_0.CT[2081] != 0)) || (++__CLOVER_37_0.CF[2081] == 0))){
            __CLOVER_37_0.S[12093]++;result = "text/plain";}

        __CLOVER_37_0.S[12094]++;return result;
    } finally { }}

    /**
     * Retrieves a resource from the classpath and returns it to the client in a binary output
     * stream.
     * <p>
     * TBD: Security issues. Hackers can download .class files.
     */

    public void service(IEngineServiceView engine, IRequestCycle cycle, ResponseOutputStream output)
            throws ServletException, IOException
    {try { __CLOVER_37_0.M[2944]++;
        __CLOVER_37_0.S[12095]++;Object[] parameters = getParameters(cycle);

        __CLOVER_37_0.S[12096]++;if ((((Tapestry.size(parameters) != 1) && (++__CLOVER_37_0.CT[2082] != 0)) || (++__CLOVER_37_0.CF[2082] == 0))){
            __CLOVER_37_0.S[12097]++;throw new ApplicationRuntimeException(Tapestry.format(
                    "service-single-parameter",
                    Tapestry.ASSET_SERVICE));}

        __CLOVER_37_0.S[12098]++;String resourcePath = (String) parameters[0];

        __CLOVER_37_0.S[12099]++;URL resourceURL = cycle.getEngine().getClassResolver().getResource(resourcePath);

        __CLOVER_37_0.S[12100]++;if ((((resourceURL == null) && (++__CLOVER_37_0.CT[2083] != 0)) || (++__CLOVER_37_0.CF[2083] == 0))){
            __CLOVER_37_0.S[12101]++;throw new ApplicationRuntimeException(Tapestry.format("missing-resource", resourcePath));}

        __CLOVER_37_0.S[12102]++;URLConnection resourceConnection = resourceURL.openConnection();

        __CLOVER_37_0.S[12103]++;ServletContext servletContext = cycle.getRequestContext().getServlet().getServletContext();

        __CLOVER_37_0.S[12104]++;writeAssetContent(engine, cycle, output, resourcePath, resourceConnection, servletContext);
    } finally { }}

    /** @since 2.2 * */

    private void writeAssetContent(IEngineServiceView engine, IRequestCycle cycle,
            ResponseOutputStream output, String resourcePath, URLConnection resourceConnection,
            ServletContext servletContext)
    {try { __CLOVER_37_0.M[2945]++;
        // Getting the content type and length is very dependant
        // on support from the application server (represented
        // here by the servletContext).

        __CLOVER_37_0.S[12105]++;String contentType = servletContext.getMimeType(resourcePath);
        __CLOVER_37_0.S[12106]++;int contentLength = resourceConnection.getContentLength();

        __CLOVER_37_0.S[12107]++;try
        {
            __CLOVER_37_0.S[12108]++;if ((((contentLength > 0) && (++__CLOVER_37_0.CT[2084] != 0)) || (++__CLOVER_37_0.CF[2084] == 0))){
                __CLOVER_37_0.S[12109]++;cycle.getRequestContext().getResponse().setContentLength(contentLength);}

            // Set the content type. If the servlet container doesn't
            // provide it, try and guess it by the extension.

            __CLOVER_37_0.S[12110]++;if ((((contentType == null || contentType.length() == 0) && (++__CLOVER_37_0.CT[2085] != 0)) || (++__CLOVER_37_0.CF[2085] == 0))){
                __CLOVER_37_0.S[12111]++;contentType = getMimeType(resourcePath);}

            __CLOVER_37_0.S[12112]++;output.setContentType(contentType);

            // Disable any further buffering inside the ResponseOutputStream

            __CLOVER_37_0.S[12113]++;output.forceFlush();

            __CLOVER_37_0.S[12114]++;InputStream input = resourceConnection.getInputStream();

            __CLOVER_37_0.S[12115]++;byte[] buffer = new byte[BUFFER_SIZE];

            __CLOVER_37_0.S[12116]++;while (true){
            {
                __CLOVER_37_0.S[12117]++;int bytesRead = input.read(buffer);

                __CLOVER_37_0.S[12118]++;if ((((bytesRead < 0) && (++__CLOVER_37_0.CT[2086] != 0)) || (++__CLOVER_37_0.CF[2086] == 0))){
                    __CLOVER_37_0.S[12119]++;break;}

                __CLOVER_37_0.S[12120]++;output.write(buffer, 0, bytesRead);
            }}

            __CLOVER_37_0.S[12121]++;input.close();
        }
        catch (Throwable ex)
        {
            __CLOVER_37_0.S[12122]++;String title = Tapestry.format("AssetService.exception-report-title", resourcePath);

            __CLOVER_37_0.S[12123]++;_exceptionReporter.reportRequestException(title, ex);
        }
    } finally { }}

    /** @since 3.1 */

    public void setExceptionReporter(RequestExceptionReporter exceptionReporter)
    {try { __CLOVER_37_0.M[2946]++;
        __CLOVER_37_0.S[12124]++;_exceptionReporter = exceptionReporter;
    } finally { }}
}