/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.pageload;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.Location;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.event.PageDetachListener;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.services.ExpressionEvaluator;

/**
 * Given a component, a property and a value, this object will reset the property to the value
 * whenever the page (containing the component) is detached. This is related to support for
 * {@link org.apache.tapestry.spec.IPropertySpecification}s.
 * 
 * @author Howard Lewis Ship
 * @since 3.0
 */

public class PropertyInitializer implements PageDetachListener
{public static com.cortexeb.tools.clover.d __CLOVER_197_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1097439627296L);
    private IComponent _component;

    private String _propertyName;

    private String _expression;

    private boolean _invariant;

    private Object _value;

    private Location _location;

    private ExpressionEvaluator _evaluator;

    public PropertyInitializer(IComponent component, String propertyName, String expression,
            Location location, ExpressionEvaluator evaluator)
    {try { __CLOVER_197_0.M[3098]++;
        __CLOVER_197_0.S[12858]++;_component = component;
        __CLOVER_197_0.S[12859]++;_propertyName = propertyName;
        __CLOVER_197_0.S[12860]++;_expression = expression;
        __CLOVER_197_0.S[12861]++;_location = location;
        __CLOVER_197_0.S[12862]++;_evaluator = evaluator;

        __CLOVER_197_0.S[12863]++;prepareInvariant();
    } finally { }}

    private void prepareInvariant()
    {try { __CLOVER_197_0.M[3099]++;
        __CLOVER_197_0.S[12864]++;_invariant = false;

        __CLOVER_197_0.S[12865]++;try
        {
            // If no initial value expression is provided, then read the current
            // property of the expression. This may be null, or may be
            // a value set in finishLoad() (via an abstract accessor).

            __CLOVER_197_0.S[12866]++;if ((((Tapestry.isBlank(_expression)) && (++__CLOVER_197_0.CT[2188] != 0)) || (++__CLOVER_197_0.CF[2188] == 0))){
            {
                __CLOVER_197_0.S[12867]++;_invariant = true;
                __CLOVER_197_0.S[12868]++;_value = _evaluator.read(_component, _propertyName);
            }}
            else{ __CLOVER_197_0.S[12869]++;if ((((_evaluator.isConstant(_expression)) && (++__CLOVER_197_0.CT[2189] != 0)) || (++__CLOVER_197_0.CF[2189] == 0))){
            {
                // If the expression is a constant, evaluate it and remember the value
                __CLOVER_197_0.S[12870]++;_invariant = true;
                __CLOVER_197_0.S[12871]++;_value = _evaluator.read(_component, _expression);
            }}}
        }
        catch (Exception ex)
        {
            __CLOVER_197_0.S[12872]++;throw new ApplicationRuntimeException(PageloadMessages.unableToInitializeProperty(
                    _propertyName,
                    _component,
                    ex), _location, ex);
        }
    } finally { }}

    public void pageDetached(PageEvent event)
    {try { __CLOVER_197_0.M[3100]++;
        __CLOVER_197_0.S[12873]++;try
        {
            __CLOVER_197_0.S[12874]++;if ((((_invariant) && (++__CLOVER_197_0.CT[2190] != 0)) || (++__CLOVER_197_0.CF[2190] == 0))){
                __CLOVER_197_0.S[12875]++;_evaluator.write(_component, _propertyName, _value);}
            else{
            {
                __CLOVER_197_0.S[12876]++;Object value = _evaluator.read(_component, _expression);
                __CLOVER_197_0.S[12877]++;_evaluator.write(_component, _propertyName, value);
            }}
        }
        catch (Exception ex)
        {
            __CLOVER_197_0.S[12878]++;throw new ApplicationRuntimeException(PageloadMessages.unableToInitializeProperty(
                    _propertyName,
                    _component,
                    ex), _location, ex);
        }
    } finally { }}

}