/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import ognl.OgnlRuntime;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.ClassResolver;
import org.apache.hivemind.Messages;
import org.apache.hivemind.impl.BaseLocatable;
import org.apache.hivemind.util.PropertyAdaptor;
import org.apache.hivemind.util.PropertyUtils;
import org.apache.tapestry.bean.BeanProvider;
import org.apache.tapestry.bean.BeanProviderPropertyAccessor;
import org.apache.tapestry.engine.IPageLoader;
import org.apache.tapestry.event.ChangeObserver;
import org.apache.tapestry.event.PageDetachListener;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.event.PageValidateListener;
import org.apache.tapestry.listener.ListenerMap;
import org.apache.tapestry.param.ParameterManager;
import org.apache.tapestry.spec.IComponentSpecification;
import org.apache.tapestry.util.prop.OgnlUtils;

/**
 * Abstract base class implementing the {@link IComponent}interface.
 * 
 * @author Howard Lewis Ship
 */

public abstract class AbstractComponent extends BaseLocatable implements IComponent
{public static com.cortexeb.tools.clover.d __CLOVER_0_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /**
     * Used to check that subclasses invoke this implementation of prepareForRender().
     * 
     * @see Tapestry#checkMethodInvocation(Object, String, Object)
     * @since 3.0
     */
    private final static String PREPAREFORRENDER_METHOD_ID = "AbstractComponent.prepareForRender()";

    /**
     * Used to check that subclasses invoke this implementation of cleanupAfterRender().
     * 
     * @see Tapestry#checkMethodInvocation(Object, String, Object)
     * @since 3.0
     */

    private final static String CLEANUPAFTERRENDER_METHOD_ID = "AbstractComponent.cleanupAfterRender()";

    static
    {
        // Register the BeanProviderHelper to provide access to the
        // beans of a bean provider as named properties.

        __CLOVER_0_0.S[0]++;OgnlRuntime.setPropertyAccessor(IBeanProvider.class, new BeanProviderPropertyAccessor());
    }

    /**
     * The specification used to originally build the component.
     */

    private IComponentSpecification _specification;

    /**
     * The page that contains the component, possibly itself (if the component is in fact, a page).
     */

    private IPage _page;

    /**
     * The component which contains the component. This will only be null if the component is
     * actually a page.
     */

    private IComponent _container;

    /**
     * The simple id of this component.
     */

    private String _id;

    /**
     * The fully qualified id of this component. This is calculated the first time it is needed,
     * then cached for later.
     */

    private String _idPath;

    private static final int MAP_SIZE = 5;

    /**
     * A {@link Map}of all bindings (for which there isn't a corresponding JavaBeans property); the
     * keys are the names of formal and informal parameters.
     */

    private Map _bindings;

    private Map _components;

    private static final int BODY_INIT_SIZE = 5;

    private INamespace _namespace;

    /**
     * Used in place of JDK 1.3's Collections.EMPTY_MAP (which is not available in JDK 1.2).
     */

    private static final Map EMPTY_MAP = Collections.unmodifiableMap(new HashMap(1));

    /**
     * The number of {@link IRender}objects in the body of this component.
     */

    private int _bodyCount = 0;

    /**
     * An aray of elements in the body of this component.
     */

    private IRender[] _body;

    /**
     * The components' asset map.
     */

    private Map _assets;

    /**
     * A mapping that allows public instance methods to be dressed up as {@link IActionListener}
     * listener objects.
     * 
     * @since 1.0.2
     */

    private ListenerMap _listeners;

    /**
     * A bean provider; these are lazily created as needed.
     * 
     * @since 1.0.4
     */

    private IBeanProvider _beans;

    /**
     * Manages setting and clearing parameter properties for the component.
     * 
     * @since 2.0.3
     */

    private ParameterManager _parameterManager;

    /**
     * Provides access to localized Strings for this component.
     * 
     * @since 2.0.4
     */

    private Messages _strings;

    public void addAsset(String name, IAsset asset)
    {try { __CLOVER_0_0.M[0]++;
        __CLOVER_0_0.S[1]++;if ((((_assets == null) && (++__CLOVER_0_0.CT[0] != 0)) || (++__CLOVER_0_0.CF[0] == 0))){
            __CLOVER_0_0.S[2]++;_assets = new HashMap(MAP_SIZE);}

        __CLOVER_0_0.S[3]++;_assets.put(name, asset);
    } finally { }}

    public void addComponent(IComponent component)
    {try { __CLOVER_0_0.M[1]++;
        __CLOVER_0_0.S[4]++;if ((((_components == null) && (++__CLOVER_0_0.CT[1] != 0)) || (++__CLOVER_0_0.CF[1] == 0))){
            __CLOVER_0_0.S[5]++;_components = new HashMap(MAP_SIZE);}

        __CLOVER_0_0.S[6]++;_components.put(component.getId(), component);
    } finally { }}

    /**
     * Adds an element (which may be static text or a component) as a body element of this
     * component. Such elements are rendered by {@link #renderBody(IMarkupWriter, IRequestCycle)}.
     * 
     * @since 2.2
     */

    public void addBody(IRender element)
    {try { __CLOVER_0_0.M[2]++;
        // Should check the specification to see if this component
        // allows body. Curently, this is checked by the component
        // in render(), which is silly.

        __CLOVER_0_0.S[7]++;if ((((_body == null) && (++__CLOVER_0_0.CT[2] != 0)) || (++__CLOVER_0_0.CF[2] == 0))){
        {
            __CLOVER_0_0.S[8]++;_body = new IRender[BODY_INIT_SIZE];
            __CLOVER_0_0.S[9]++;_body[0] = element;

            __CLOVER_0_0.S[10]++;_bodyCount = 1;
            __CLOVER_0_0.S[11]++;return;
        }}

        // No more room? Make the array bigger.

        __CLOVER_0_0.S[12]++;if ((((_bodyCount == _body.length) && (++__CLOVER_0_0.CT[3] != 0)) || (++__CLOVER_0_0.CF[3] == 0))){
        {
            __CLOVER_0_0.S[13]++;IRender[] newWrapped;

            __CLOVER_0_0.S[14]++;newWrapped = new IRender[_body.length * 2];

            __CLOVER_0_0.S[15]++;System.arraycopy(_body, 0, newWrapped, 0, _bodyCount);

            __CLOVER_0_0.S[16]++;_body = newWrapped;
        }}

        __CLOVER_0_0.S[17]++;_body[_bodyCount++] = element;
    } finally { }}

    /**
     * Registers this component as a listener of the page if it implements
     * {@link org.apache.tapestry.event.PageDetachListener}or
     * {@link org.apache.tapestry.event.PageRenderListener}.
     * <p>
     * Invokes {@link #finishLoad()}. Subclasses may overide as needed, but must invoke this
     * implementation. {@link BaseComponent}loads its HTML template.
     */

    public void finishLoad(IRequestCycle cycle, IPageLoader loader,
            IComponentSpecification specification)
    {try { __CLOVER_0_0.M[3]++;
        __CLOVER_0_0.S[18]++;if ((((this instanceof PageDetachListener) && (++__CLOVER_0_0.CT[4] != 0)) || (++__CLOVER_0_0.CF[4] == 0))){
            __CLOVER_0_0.S[19]++;_page.addPageDetachListener((PageDetachListener) this);}

        __CLOVER_0_0.S[20]++;if ((((this instanceof PageRenderListener) && (++__CLOVER_0_0.CT[5] != 0)) || (++__CLOVER_0_0.CF[5] == 0))){
            __CLOVER_0_0.S[21]++;_page.addPageRenderListener((PageRenderListener) this);}

        __CLOVER_0_0.S[22]++;if ((((this instanceof PageValidateListener) && (++__CLOVER_0_0.CT[6] != 0)) || (++__CLOVER_0_0.CF[6] == 0))){
            __CLOVER_0_0.S[23]++;_page.addPageValidateListener((PageValidateListener) this);}

        __CLOVER_0_0.S[24]++;finishLoad();
    } finally { }}

    /**
     * @deprecated To be removed in 3.1. Use
     *             {@link Tapestry#fireObservedChange(IComponent, String, int)}instead.
     */
    protected void fireObservedChange(String propertyName, int newValue)
    {try { __CLOVER_0_0.M[4]++;
        __CLOVER_0_0.S[25]++;Tapestry.fireObservedChange(this, propertyName, newValue);
    } finally { }}

    /**
     * @deprecated To be removed in 3.1. Use
     *             {@link Tapestry#fireObservedChange(IComponent, String, Object)}instead.
     */
    protected void fireObservedChange(String propertyName, Object newValue)
    {try { __CLOVER_0_0.M[5]++;
        __CLOVER_0_0.S[26]++;Tapestry.fireObservedChange(this, propertyName, newValue);
    } finally { }}

    /**
     * @deprecated To be removed in 3.1. Use
     *             {@link Tapestry#fireObservedChange(IComponent, String, boolean)}instead.
     */
    protected void fireObservedChange(String propertyName, boolean newValue)
    {try { __CLOVER_0_0.M[6]++;
        __CLOVER_0_0.S[27]++;Tapestry.fireObservedChange(this, propertyName, newValue);
    } finally { }}

    /**
     * @deprecated To be removed in 3.1. Use
     *             {@link Tapestry#fireObservedChange(IComponent, String, double)}instead.
     */
    protected void fireObservedChange(String propertyName, double newValue)
    {try { __CLOVER_0_0.M[7]++;
        __CLOVER_0_0.S[28]++;Tapestry.fireObservedChange(this, propertyName, newValue);
    } finally { }}

    /**
     * @deprecated To be removed in 3.1. Use
     *             {@link Tapestry#fireObservedChange(IComponent, String, float)}instead.
     */
    protected void fireObservedChange(String propertyName, float newValue)
    {try { __CLOVER_0_0.M[8]++;
        __CLOVER_0_0.S[29]++;Tapestry.fireObservedChange(this, propertyName, newValue);
    } finally { }}

    /**
     * @deprecated To be removed in 3.1. Use
     *             {@link Tapestry#fireObservedChange(IComponent, String, long)}instead.
     */
    protected void fireObservedChange(String propertyName, long newValue)
    {try { __CLOVER_0_0.M[9]++;
        __CLOVER_0_0.S[30]++;Tapestry.fireObservedChange(this, propertyName, newValue);
    } finally { }}

    /**
     * @deprecated To be removed in 3.1. Use
     *             {@link Tapestry#fireObservedChange(IComponent, String, char)}instead.
     */
    protected void fireObservedChange(String propertyName, char newValue)
    {try { __CLOVER_0_0.M[10]++;
        __CLOVER_0_0.S[31]++;Tapestry.fireObservedChange(this, propertyName, newValue);
    } finally { }}

    /**
     * @deprecated To be removed in 3.1. Use
     *             {@link Tapestry#fireObservedChange(IComponent, String, byte)}instead.
     */
    protected void fireObservedChange(String propertyName, byte newValue)
    {try { __CLOVER_0_0.M[11]++;
        __CLOVER_0_0.S[32]++;Tapestry.fireObservedChange(this, propertyName, newValue);
    } finally { }}

    /**
     * @deprecated To be removed in 3.1. Use
     *             {@link Tapestry#fireObservedChange(IComponent, String, short)}instead.
     */
    protected void fireObservedChange(String propertyName, short newValue)
    {try { __CLOVER_0_0.M[12]++;
        __CLOVER_0_0.S[33]++;Tapestry.fireObservedChange(this, propertyName, newValue);
    } finally { }}

    /**
     * @deprecated To be removed in 3.1. Use
     *             {@link #renderInformalParameters(IMarkupWriter, IRequestCycle)}instead.
     */

    protected void generateAttributes(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_0_0.M[13]++;
        __CLOVER_0_0.S[34]++;renderInformalParameters(writer, cycle);
    } finally { }}

    /**
     * Converts informal parameters into additional attributes on the curently open tag.
     * <p>
     * Invoked from subclasses to allow additional attributes to be specified within a tag (this
     * works best when there is a one-to-one corespondence between an {@link IComponent}and a HTML
     * element.
     * <p>
     * Iterates through the bindings for this component. Filters out bindings when the name matches
     * a formal parameter (as of 1.0.5, informal bindings are weeded out at page load / template
     * load time, if they match a formal parameter, or a specificied reserved name). For the most
     * part, all the bindings here are either informal parameter, or formal parameter without a
     * corresponding JavaBeans property.
     * <p>
     * For each acceptible key, the value is extracted using {@link IBinding#getObject()}. If the
     * value is null, no attribute is written.
     * <p>
     * If the value is an instance of {@link IAsset}, then {@link IAsset#buildURL(IRequestCycle)}
     * is invoked to convert the asset to a URL.
     * <p>
     * Finally, {@link IMarkupWriter#attribute(String,String)}is invoked with the value (or the
     * URL).
     * <p>
     * The most common use for informal parameters is to support the HTML class attribute (for use
     * with cascading style sheets) and to specify JavaScript event handlers.
     * <p>
     * Components are only required to generate attributes on the result phase; this can be skipped
     * during the rewind phase.
     */

    protected void renderInformalParameters(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_0_0.M[14]++;
        __CLOVER_0_0.S[35]++;String attribute;

        __CLOVER_0_0.S[36]++;if ((((_bindings == null) && (++__CLOVER_0_0.CT[7] != 0)) || (++__CLOVER_0_0.CF[7] == 0))){
            __CLOVER_0_0.S[37]++;return;}

        __CLOVER_0_0.S[38]++;Iterator i = _bindings.entrySet().iterator();

        __CLOVER_0_0.S[39]++;while ((((i.hasNext()) && (++__CLOVER_0_0.CT[8] != 0)) || (++__CLOVER_0_0.CF[8] == 0))){
        {
            __CLOVER_0_0.S[40]++;Map.Entry entry = (Map.Entry) i.next();
            __CLOVER_0_0.S[41]++;String name = (String) entry.getKey();

            __CLOVER_0_0.S[42]++;IBinding binding = (IBinding) entry.getValue();

            __CLOVER_0_0.S[43]++;Object value = binding.getObject();
            __CLOVER_0_0.S[44]++;if ((((value == null) && (++__CLOVER_0_0.CT[9] != 0)) || (++__CLOVER_0_0.CF[9] == 0))){
                __CLOVER_0_0.S[45]++;continue;}

            __CLOVER_0_0.S[46]++;if ((((value instanceof IAsset) && (++__CLOVER_0_0.CT[10] != 0)) || (++__CLOVER_0_0.CF[10] == 0))){
            {
                __CLOVER_0_0.S[47]++;IAsset asset = (IAsset) value;

                // Get the URL of the asset and insert that.

                __CLOVER_0_0.S[48]++;attribute = asset.buildURL(cycle);
            }}
            else{
                __CLOVER_0_0.S[49]++;attribute = value.toString();}

            __CLOVER_0_0.S[50]++;writer.attribute(name, attribute);
        }}

    } finally { }}

    /**
     * Returns an object used to resolve classes.
     * 
     * @since 3.0
     */
    private ClassResolver getClassResolver()
    {try { __CLOVER_0_0.M[15]++;
        __CLOVER_0_0.S[51]++;return getPage().getEngine().getClassResolver();
    } finally { }}

    /**
     * Returns the named binding, or null if it doesn't exist.
     * <p>
     * This method looks for a JavaBeans property with an appropriate name, of type {@link IBinding}.
     * The property should be named <code><i>name</i>Binding</code>. If it exists and is both
     * readable and writable, then it is accessor method is invoked. Components which implement such
     * methods can access their own binding through their instance variables instead of invoking
     * this method, a performance optimization.
     * 
     * @see #setBinding(String,IBinding)
     */

    public IBinding getBinding(String name)
    {try { __CLOVER_0_0.M[16]++;
        __CLOVER_0_0.S[52]++;String bindingPropertyName = name + Tapestry.PARAMETER_PROPERTY_NAME_SUFFIX;

        __CLOVER_0_0.S[53]++;if ((((PropertyUtils.isReadable(this, bindingPropertyName)
                && PropertyUtils.getPropertyType(this, bindingPropertyName).equals(IBinding.class)) && (++__CLOVER_0_0.CT[11] != 0)) || (++__CLOVER_0_0.CF[11] == 0))){
        {
            __CLOVER_0_0.S[54]++;return (IBinding) PropertyUtils.read(this, bindingPropertyName);
        }}

        __CLOVER_0_0.S[55]++;if ((((_bindings == null) && (++__CLOVER_0_0.CT[12] != 0)) || (++__CLOVER_0_0.CF[12] == 0))){
            __CLOVER_0_0.S[56]++;return null;}

        __CLOVER_0_0.S[57]++;return (IBinding) _bindings.get(name);
    } finally { }}

    /**
     * Return's the page's change observer. In practical terms, this will be an
     * {@link org.apache.tapestry.engine.IPageRecorder}.
     * 
     * @see IPage#getChangeObserver()
     * @deprecated To be removed in 3.1; use {@link IPage#getChangeObserver()}.
     */

    public ChangeObserver getChangeObserver()
    {try { __CLOVER_0_0.M[17]++;
        __CLOVER_0_0.S[58]++;return _page.getChangeObserver();
    } finally { }}

    public IComponent getComponent(String id)
    {try { __CLOVER_0_0.M[18]++;
        __CLOVER_0_0.S[59]++;IComponent result = null;

        __CLOVER_0_0.S[60]++;if ((((_components != null) && (++__CLOVER_0_0.CT[13] != 0)) || (++__CLOVER_0_0.CF[13] == 0))){
            __CLOVER_0_0.S[61]++;result = (IComponent) _components.get(id);}

        __CLOVER_0_0.S[62]++;if ((((result == null) && (++__CLOVER_0_0.CT[14] != 0)) || (++__CLOVER_0_0.CF[14] == 0))){
            __CLOVER_0_0.S[63]++;throw new ApplicationRuntimeException(Tapestry.format("no-such-component", this, id),
                    this, null, null);}

        __CLOVER_0_0.S[64]++;return result;
    } finally { }}

    public IComponent getContainer()
    {try { __CLOVER_0_0.M[19]++;
        __CLOVER_0_0.S[65]++;return _container;
    } finally { }}

    public void setContainer(IComponent value)
    {try { __CLOVER_0_0.M[20]++;
        __CLOVER_0_0.S[66]++;if ((((_container != null) && (++__CLOVER_0_0.CT[15] != 0)) || (++__CLOVER_0_0.CF[15] == 0))){
            __CLOVER_0_0.S[67]++;throw new ApplicationRuntimeException(Tapestry
                    .getMessage("AbstractComponent.attempt-to-change-container"));}

        __CLOVER_0_0.S[68]++;_container = value;
    } finally { }}

    /**
     * Returns the name of the page, a slash, and this component's id path. Pages are different,
     * they simply return their name.
     * 
     * @see #getIdPath()
     */

    public String getExtendedId()
    {try { __CLOVER_0_0.M[21]++;
        __CLOVER_0_0.S[69]++;if ((((_page == null) && (++__CLOVER_0_0.CT[16] != 0)) || (++__CLOVER_0_0.CF[16] == 0))){
            __CLOVER_0_0.S[70]++;return null;}

        __CLOVER_0_0.S[71]++;return _page.getPageName() + "/" + getIdPath();
    } finally { }}

    public String getId()
    {try { __CLOVER_0_0.M[22]++;
        __CLOVER_0_0.S[72]++;return _id;
    } finally { }}

    public void setId(String value)
    {try { __CLOVER_0_0.M[23]++;
        __CLOVER_0_0.S[73]++;if ((((_id != null) && (++__CLOVER_0_0.CT[17] != 0)) || (++__CLOVER_0_0.CF[17] == 0))){
            __CLOVER_0_0.S[74]++;throw new ApplicationRuntimeException(Tapestry
                    .getMessage("AbstractComponent.attempt-to-change-component-id"));}

        __CLOVER_0_0.S[75]++;_id = value;
    } finally { }}

    public String getIdPath()
    {try { __CLOVER_0_0.M[24]++;
        __CLOVER_0_0.S[76]++;String containerIdPath;

        __CLOVER_0_0.S[77]++;if ((((_container == null) && (++__CLOVER_0_0.CT[18] != 0)) || (++__CLOVER_0_0.CF[18] == 0))){
            __CLOVER_0_0.S[78]++;throw new NullPointerException(Tapestry
                    .format("AbstractComponent.null-container", this));}

        __CLOVER_0_0.S[79]++;containerIdPath = _container.getIdPath();

        __CLOVER_0_0.S[80]++;if ((((containerIdPath == null) && (++__CLOVER_0_0.CT[19] != 0)) || (++__CLOVER_0_0.CF[19] == 0))){
            __CLOVER_0_0.S[81]++;_idPath = _id;}
        else{
            __CLOVER_0_0.S[82]++;_idPath = containerIdPath + "." + _id;}

        __CLOVER_0_0.S[83]++;return _idPath;
    } finally { }}

    public IPage getPage()
    {try { __CLOVER_0_0.M[25]++;
        __CLOVER_0_0.S[84]++;return _page;
    } finally { }}

    public void setPage(IPage value)
    {try { __CLOVER_0_0.M[26]++;
        __CLOVER_0_0.S[85]++;if ((((_page != null) && (++__CLOVER_0_0.CT[20] != 0)) || (++__CLOVER_0_0.CF[20] == 0))){
            __CLOVER_0_0.S[86]++;throw new ApplicationRuntimeException(Tapestry
                    .getMessage("AbstractComponent.attempt-to-change-page"));}

        __CLOVER_0_0.S[87]++;_page = value;
    } finally { }}

    public IComponentSpecification getSpecification()
    {try { __CLOVER_0_0.M[27]++;
        __CLOVER_0_0.S[88]++;return _specification;
    } finally { }}

    public void setSpecification(IComponentSpecification value)
    {try { __CLOVER_0_0.M[28]++;
        __CLOVER_0_0.S[89]++;if ((((_specification != null) && (++__CLOVER_0_0.CT[21] != 0)) || (++__CLOVER_0_0.CF[21] == 0))){
            __CLOVER_0_0.S[90]++;throw new ApplicationRuntimeException(Tapestry
                    .getMessage("AbstractComponent.attempt-to-change-spec"));}

        __CLOVER_0_0.S[91]++;_specification = value;
    } finally { }}

    /**
     * Renders all elements wrapped by the receiver.
     */

    public void renderBody(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_0_0.M[29]++;
        __CLOVER_0_0.S[92]++;for (int i = 0; (((i < _bodyCount) && (++__CLOVER_0_0.CT[22] != 0)) || (++__CLOVER_0_0.CF[22] == 0)); i++){
            __CLOVER_0_0.S[93]++;_body[i].render(writer, cycle);}
    } finally { }}

    /**
     * Adds the binding with the given name, replacing any existing binding with that name.
     * <p>
     * This method checks to see if a matching JavaBeans property (with a name of
     * <code><i>name</i>Binding</code> and a type of {@link IBinding}) exists. If so, that
     * property is updated. An optimized component can simply implement accessor and mutator methods
     * and then access its bindings via its own instance variables, rather than going through {@link
     * #getBinding(String)}.
     * <p>
     * Informal parameters should <em>not</em> be stored in instance variables if {@link
     * #renderInformalParameters(IMarkupWriter, IRequestCycle)} is to be used. It relies on using
     * the collection of bindings (to store informal parameters).
     */

    public void setBinding(String name, IBinding binding)
    {try { __CLOVER_0_0.M[30]++;
        __CLOVER_0_0.S[94]++;String bindingPropertyName = name + Tapestry.PARAMETER_PROPERTY_NAME_SUFFIX;

        __CLOVER_0_0.S[95]++;if ((((PropertyUtils.isWritable(this, bindingPropertyName)
                && PropertyUtils.getPropertyType(this, bindingPropertyName).equals(IBinding.class)) && (++__CLOVER_0_0.CT[23] != 0)) || (++__CLOVER_0_0.CF[23] == 0))){
        {
            __CLOVER_0_0.S[96]++;PropertyUtils.write(this, bindingPropertyName, binding);
            __CLOVER_0_0.S[97]++;return;
        }}

        __CLOVER_0_0.S[98]++;if ((((_bindings == null) && (++__CLOVER_0_0.CT[24] != 0)) || (++__CLOVER_0_0.CF[24] == 0))){
            __CLOVER_0_0.S[99]++;_bindings = new HashMap(MAP_SIZE);}

        __CLOVER_0_0.S[100]++;_bindings.put(name, binding);
    } finally { }}

    public String toString()
    {try { __CLOVER_0_0.M[31]++;
        __CLOVER_0_0.S[101]++;StringBuffer buffer;

        __CLOVER_0_0.S[102]++;buffer = new StringBuffer(super.toString());

        __CLOVER_0_0.S[103]++;buffer.append('[');

        __CLOVER_0_0.S[104]++;buffer.append(getExtendedId());

        __CLOVER_0_0.S[105]++;buffer.append(']');

        __CLOVER_0_0.S[106]++;return buffer.toString();
    } finally { }}

    /**
     * Returns an unmodifiable {@link Map}of components, keyed on component id. Never returns null,
     * but may return an empty map. The returned map is immutable.
     */

    public Map getComponents()
    {try { __CLOVER_0_0.M[32]++;
        __CLOVER_0_0.S[107]++;if ((((_components == null) && (++__CLOVER_0_0.CT[25] != 0)) || (++__CLOVER_0_0.CF[25] == 0))){
            __CLOVER_0_0.S[108]++;return EMPTY_MAP;}

        __CLOVER_0_0.S[109]++;return Collections.unmodifiableMap(_components);

    } finally { }}

    public Map getAssets()
    {try { __CLOVER_0_0.M[33]++;
        __CLOVER_0_0.S[110]++;if ((((_assets == null) && (++__CLOVER_0_0.CT[26] != 0)) || (++__CLOVER_0_0.CF[26] == 0))){
            __CLOVER_0_0.S[111]++;return EMPTY_MAP;}

        __CLOVER_0_0.S[112]++;return Collections.unmodifiableMap(_assets);
    } finally { }}

    public IAsset getAsset(String name)
    {try { __CLOVER_0_0.M[34]++;
        __CLOVER_0_0.S[113]++;if ((((_assets == null) && (++__CLOVER_0_0.CT[27] != 0)) || (++__CLOVER_0_0.CF[27] == 0))){
            __CLOVER_0_0.S[114]++;return null;}

        __CLOVER_0_0.S[115]++;return (IAsset) _assets.get(name);
    } finally { }}

    public Collection getBindingNames()
    {try { __CLOVER_0_0.M[35]++;
        // If no conainer, i.e. a page, then no bindings.

        __CLOVER_0_0.S[116]++;if ((((_container == null) && (++__CLOVER_0_0.CT[28] != 0)) || (++__CLOVER_0_0.CF[28] == 0))){
            __CLOVER_0_0.S[117]++;return null;}

        __CLOVER_0_0.S[118]++;HashSet result = new HashSet();

        // All the informal bindings go into the bindings Map.

        __CLOVER_0_0.S[119]++;if ((((_bindings != null) && (++__CLOVER_0_0.CT[29] != 0)) || (++__CLOVER_0_0.CF[29] == 0))){
            __CLOVER_0_0.S[120]++;result.addAll(_bindings.keySet());}

        // Now, iterate over the formal parameters and add the formal parameters
        // that have a binding.

        __CLOVER_0_0.S[121]++;List names = _specification.getParameterNames();

        __CLOVER_0_0.S[122]++;int count = names.size();

        __CLOVER_0_0.S[123]++;for (int i = 0; (((i < count) && (++__CLOVER_0_0.CT[30] != 0)) || (++__CLOVER_0_0.CF[30] == 0)); i++){
        {
            __CLOVER_0_0.S[124]++;String name = (String) names.get(i);

            __CLOVER_0_0.S[125]++;if ((((result.contains(name)) && (++__CLOVER_0_0.CT[31] != 0)) || (++__CLOVER_0_0.CF[31] == 0))){
                __CLOVER_0_0.S[126]++;continue;}

            __CLOVER_0_0.S[127]++;if ((((getBinding(name) != null) && (++__CLOVER_0_0.CT[32] != 0)) || (++__CLOVER_0_0.CF[32] == 0))){
                __CLOVER_0_0.S[128]++;result.add(name);}
        }}

        __CLOVER_0_0.S[129]++;return result;
    } finally { }}

    /**
     * Returns a {@link Map}of all bindings for this component. This implementation is expensive,
     * since it has to merge the disassociated bindings (informal parameters, and parameters without
     * a JavaBeans property) with the associated bindings (formal parameters with a JavaBeans
     * property).
     * 
     * @since 1.0.5
     */

    public Map getBindings()
    {try { __CLOVER_0_0.M[36]++;
        __CLOVER_0_0.S[130]++;Map result = new HashMap();

        // Add any informal parameters.

        __CLOVER_0_0.S[131]++;if ((((_bindings != null) && (++__CLOVER_0_0.CT[33] != 0)) || (++__CLOVER_0_0.CF[33] == 0))){
            __CLOVER_0_0.S[132]++;result.putAll(_bindings);}

        // Now work on the formal parameters

        __CLOVER_0_0.S[133]++;Iterator i = _specification.getParameterNames().iterator();
        __CLOVER_0_0.S[134]++;while ((((i.hasNext()) && (++__CLOVER_0_0.CT[34] != 0)) || (++__CLOVER_0_0.CF[34] == 0))){
        {
            __CLOVER_0_0.S[135]++;String name = (String) i.next();

            __CLOVER_0_0.S[136]++;if ((((result.containsKey(name)) && (++__CLOVER_0_0.CT[35] != 0)) || (++__CLOVER_0_0.CF[35] == 0))){
                __CLOVER_0_0.S[137]++;continue;}

            __CLOVER_0_0.S[138]++;IBinding binding = getBinding(name);

            __CLOVER_0_0.S[139]++;if ((((binding != null) && (++__CLOVER_0_0.CT[36] != 0)) || (++__CLOVER_0_0.CF[36] == 0))){
                __CLOVER_0_0.S[140]++;result.put(name, binding);}
        }}

        __CLOVER_0_0.S[141]++;return result;
    } finally { }}

    /**
     * Returns a {@link ListenerMap}for the component. A {@link ListenerMap}contains a number of
     * synthetic read-only properties that implement the {@link IActionListener}interface, but in
     * fact, cause public instance methods to be invoked.
     * 
     * @since 1.0.2
     */

    public ListenerMap getListeners()
    {try { __CLOVER_0_0.M[37]++;
        __CLOVER_0_0.S[142]++;if ((((_listeners == null) && (++__CLOVER_0_0.CT[37] != 0)) || (++__CLOVER_0_0.CF[37] == 0))){
            __CLOVER_0_0.S[143]++;_listeners = new ListenerMap(this);}

        __CLOVER_0_0.S[144]++;return _listeners;
    } finally { }}

    /**
     * Returns the {@link IBeanProvider}for this component. This is lazily created the first time
     * it is needed.
     * 
     * @since 1.0.4
     */

    public IBeanProvider getBeans()
    {try { __CLOVER_0_0.M[38]++;
        __CLOVER_0_0.S[145]++;if ((((_beans == null) && (++__CLOVER_0_0.CT[38] != 0)) || (++__CLOVER_0_0.CF[38] == 0))){
            __CLOVER_0_0.S[146]++;_beans = new BeanProvider(this);}

        __CLOVER_0_0.S[147]++;return _beans;
    } finally { }}

    /**
     * Invoked, as a convienience, from
     * {@link #finishLoad(IRequestCycle, IPageLoader, IComponentSpecification)}. This implemenation
     * does nothing. Subclasses may override without invoking this implementation.
     * 
     * @since 1.0.5
     */

    protected void finishLoad()
    {try { __CLOVER_0_0.M[39]++;
    } finally { }}

    /**
     * The main method used to render the component. Invokes
     * {@link #prepareForRender(IRequestCycle)}, then
     * {@link #renderComponent(IMarkupWriter, IRequestCycle)}.
     * {@link #cleanupAfterRender(IRequestCycle)}is invoked in a <code>finally</code> block.
     * <p>
     * Subclasses should not override this method; instead they will implement
     * {@link #renderComponent(IMarkupWriter, IRequestCycle)}.
     * 
     * @since 2.0.3
     */

    public final void render(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_0_0.M[40]++;
        __CLOVER_0_0.S[148]++;try
        {
            __CLOVER_0_0.S[149]++;Tapestry.clearMethodInvocations();

            __CLOVER_0_0.S[150]++;prepareForRender(cycle);

            __CLOVER_0_0.S[151]++;Tapestry.checkMethodInvocation(PREPAREFORRENDER_METHOD_ID, "prepareForRender()", this);

            __CLOVER_0_0.S[152]++;renderComponent(writer, cycle);
        }
        finally
        {
            __CLOVER_0_0.S[153]++;Tapestry.clearMethodInvocations();

            __CLOVER_0_0.S[154]++;cleanupAfterRender(cycle);

            __CLOVER_0_0.S[155]++;Tapestry.checkMethodInvocation(
                    CLEANUPAFTERRENDER_METHOD_ID,
                    "cleanupAfterRender()",
                    this);
        }
    } finally { }}

    /**
     * Invoked by {@link #render(IMarkupWriter, IRequestCycle)}to prepare the component to render.
     * This implementation sets JavaBeans properties from matching bound parameters. Subclasses that
     * override this method must invoke this implementation as well.
     * 
     * @since 2.0.3
     */

    protected void prepareForRender(IRequestCycle cycle)
    {try { __CLOVER_0_0.M[41]++;
        __CLOVER_0_0.S[156]++;Tapestry.addMethodInvocation(PREPAREFORRENDER_METHOD_ID);

        __CLOVER_0_0.S[157]++;if ((((_parameterManager == null) && (++__CLOVER_0_0.CT[39] != 0)) || (++__CLOVER_0_0.CF[39] == 0))){
        {
            // Pages inherit from this class too, but pages (by definition)
            // never have parameters.

            __CLOVER_0_0.S[158]++;if ((((getSpecification().isPageSpecification()) && (++__CLOVER_0_0.CT[40] != 0)) || (++__CLOVER_0_0.CF[40] == 0))){
                __CLOVER_0_0.S[159]++;return;}

            __CLOVER_0_0.S[160]++;_parameterManager = new ParameterManager(this);
        }}

        __CLOVER_0_0.S[161]++;_parameterManager.setParameters(cycle);
    } finally { }}

    /**
     * Invoked by {@link #render(IMarkupWriter, IRequestCycle)}to actually render the component
     * (with any parameter values already set). This is the method that subclasses must implement.
     * 
     * @since 2.0.3
     */

    protected abstract void renderComponent(IMarkupWriter writer, IRequestCycle cycle);

    /**
     * Invoked by {@link #render(IMarkupWriter, IRequestCycle)}after the component renders, to
     * clear any parameters back to null (or 0, or false, or whatever the correct default is).
     * Primarily, this is used to ensure that the component doesn't hold onto any objects that could
     * otherwise be garbage collected.
     * <p>
     * Subclasses may override this implementation, but must also invoke it.
     * 
     * @since 2.0.3
     */

    protected void cleanupAfterRender(IRequestCycle cycle)
    {try { __CLOVER_0_0.M[42]++;
        __CLOVER_0_0.S[162]++;Tapestry.addMethodInvocation(CLEANUPAFTERRENDER_METHOD_ID);

        __CLOVER_0_0.S[163]++;if ((((_parameterManager != null) && (++__CLOVER_0_0.CT[41] != 0)) || (++__CLOVER_0_0.CF[41] == 0))){
            __CLOVER_0_0.S[164]++;_parameterManager.resetParameters(cycle);}
    } finally { }}

    /** @since 3.0 * */

    public Messages getMessages()
    {try { __CLOVER_0_0.M[43]++;
        __CLOVER_0_0.S[165]++;if ((((_strings == null) && (++__CLOVER_0_0.CT[42] != 0)) || (++__CLOVER_0_0.CF[42] == 0))){
            __CLOVER_0_0.S[166]++;_strings = getPage().getEngine().getComponentMessagesSource().getMessages(this);}

        __CLOVER_0_0.S[167]++;return _strings;
    } finally { }}

    /**
     * Obtains the {@link IMessages}for this component (if necessary), and gets the string from it.
     */

    public String getString(String key)
    {try { __CLOVER_0_0.M[44]++;
        __CLOVER_0_0.S[168]++;return getMessages().getMessage(key);
    } finally { }}

    public String getMessage(String key)
    {try { __CLOVER_0_0.M[45]++;
        // Invoke the deprecated implementation (for code coverage reasons).
        // In 3.1, remove getString() and move its implementation
        // here.

        __CLOVER_0_0.S[169]++;return getString(key);
    } finally { }}

    /**
     * Formats a message string, using
     * {@link IMessages#format(java.lang.String, java.lang.Object[])}.
     * 
     * @param key
     *            the key used to obtain a localized pattern using {@link #getString(String)}
     * @param arguments
     *            passed to the formatter
     * @since 2.2
     * @deprecated To be removed in 3.1. Use {@link #format(String, Object[])}instead.
     */

    public String formatString(String key, Object[] arguments)
    {try { __CLOVER_0_0.M[46]++;
        __CLOVER_0_0.S[170]++;return getMessages().format(key, arguments);
    } finally { }}

    /**
     * Formats a localized message string, using
     * {@link IMessages#format(java.lang.String, java.lang.Object[])}.
     * 
     * @param key
     *            the key used to obtain a localized pattern using {@link #getString(String)}
     * @param arguments
     *            passed to the formatter
     * @since 3.0
     */

    public String format(String key, Object[] arguments)
    {try { __CLOVER_0_0.M[47]++;
        // SOP: New name invokes deprecated method (consistency and
        // code coverage); in 3.1 we move the implementation here.

        __CLOVER_0_0.S[171]++;return formatString(key, arguments);
    } finally { }}

    /**
     * Convienience method for invoking {@link IMessages#format(String, Object[])}
     * 
     * @since 2.2
     * @deprecated To be removed in 3.1. Use {@link #format(String, Object)}instead.
     */

    public String formatString(String key, Object argument)
    {try { __CLOVER_0_0.M[48]++;
        __CLOVER_0_0.S[172]++;return getMessages().format(key, argument);
    } finally { }}

    /**
     * Convienience method for invoking {@link IMessages#format(String, Object)}
     * 
     * @since 3.0
     */

    public String format(String key, Object argument)
    {try { __CLOVER_0_0.M[49]++;
        __CLOVER_0_0.S[173]++;return formatString(key, argument);
    } finally { }}

    /**
     * Convienience method for invoking {@link IMessages#format(String, Object, Object)}.
     * 
     * @since 2.2
     * @deprecated To be removed in 3.1. Use {@link #format(String, Object, Object)}instead.
     */

    public String formatString(String key, Object argument1, Object argument2)
    {try { __CLOVER_0_0.M[50]++;
        __CLOVER_0_0.S[174]++;return getMessages().format(key, argument1, argument2);
    } finally { }}

    /**
     * Convienience method for invoking {@link IMessages#format(String, Object, Object)}.
     * 
     * @since 3.0
     */

    public String format(String key, Object argument1, Object argument2)
    {try { __CLOVER_0_0.M[51]++;
        __CLOVER_0_0.S[175]++;return formatString(key, argument1, argument2);
    } finally { }}

    /**
     * Convienience method for {@link IMessages#format(String, Object, Object, Object)}.
     * 
     * @since 2.2
     * @deprecated To be removed in 3.1. Use {@link #format(String, Object, Object, Object)}
     *             instead.
     */

    public String formatString(String key, Object argument1, Object argument2, Object argument3)
    {try { __CLOVER_0_0.M[52]++;
        __CLOVER_0_0.S[176]++;return getMessages().format(key, argument1, argument2, argument3);
    } finally { }}

    /**
     * Convienience method for {@link IMessages#format(String, Object, Object, Object)}.
     * 
     * @since 3.0
     */

    public String format(String key, Object argument1, Object argument2, Object argument3)
    {try { __CLOVER_0_0.M[53]++;
        __CLOVER_0_0.S[177]++;return formatString(key, argument1, argument2, argument3);
    } finally { }}

    public INamespace getNamespace()
    {try { __CLOVER_0_0.M[54]++;
        __CLOVER_0_0.S[178]++;return _namespace;
    } finally { }}

    public void setNamespace(INamespace namespace)
    {try { __CLOVER_0_0.M[55]++;
        __CLOVER_0_0.S[179]++;_namespace = namespace;
    } finally { }}

    /**
     * Returns the body of the component, the element (which may be static HTML or components) that
     * the component immediately wraps. May return null. Do not modify the returned array. The array
     * may be padded with nulls.
     * 
     * @since 2.3
     * @see #getBodyCount()
     */

    public IRender[] getBody()
    {try { __CLOVER_0_0.M[56]++;
        __CLOVER_0_0.S[180]++;return _body;
    } finally { }}

    /**
     * Returns the active number of elements in the the body, which may be zero.
     * 
     * @since 2.3
     * @see #getBody()
     */

    public int getBodyCount()
    {try { __CLOVER_0_0.M[57]++;
        __CLOVER_0_0.S[181]++;return _bodyCount;
    } finally { }}

    /**
     * Empty implementation of
     * {@link org.apache.tapestry.event.PageRenderListener#pageEndRender(PageEvent)}. This allows
     * classes to implement {@link org.apache.tapestry.event.PageRenderListener}and only implement
     * the {@link org.apache.tapestry.event.PageRenderListener#pageBeginRender(PageEvent)}method.
     * 
     * @since 3.0
     */

    public void pageEndRender(PageEvent event)
    {try { __CLOVER_0_0.M[58]++;
    } finally { }}

    /**
     * Sets a property of a component.
     * 
     * @see IComponent
     * @since 3.0
     */
    public void setProperty(String propertyName, Object value)
    {try { __CLOVER_0_0.M[59]++;
        __CLOVER_0_0.S[182]++;OgnlUtils.set(propertyName, this, value);
    } finally { }}

    /**
     * Gets a property of a component.
     * 
     * @see IComponent
     * @since 3.0
     */
    public Object getProperty(String propertyName)
    {try { __CLOVER_0_0.M[60]++;
        __CLOVER_0_0.S[183]++;return OgnlUtils.get(propertyName, this);
    } finally { }}
}