/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.asset;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.ServletException;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.engine.AbstractService;
import org.apache.tapestry.engine.IEngineServiceView;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.request.ResponseOutputStream;

/**
 *  A service for building URLs to and accessing {@link org.apache.tapestry.IAsset}s.
 *  Most of the work is deferred to the {@link org.apache.tapestry.IAsset} instance.
 *
 *  <p>The retrieval part is directly linked to {@link PrivateAsset}.
 *  The service responds to a URL that encodes the path of a resource
 *  within the classpath.  The 
 *  {@link #service(IEngineServiceView, IRequestCycle, ResponseOutputStream)} 
 *  method reads the resource and streams it out.
 *
 *  <p>TBD: Security issues.  Should only be able to retrieve a
 *  resource that was previously registerred in some way
 *  ... otherwise, hackers will be able to suck out the .class files
 *  of the application!
 *
 *  @author Howard Lewis Ship
 **/

public class AssetService extends AbstractService
{public static com.cortexeb.tools.clover.d __CLOVER_37_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /**
     *  Defaults MIME types, by extension, used when the servlet container
     *  doesn't provide MIME types.  ServletExec Debugger, for example,
     *  fails to do provide these.
     *
     **/

    private final static Map _mimeTypes;

    static {
        __CLOVER_37_0.S[922]++;_mimeTypes = new HashMap(17);
        __CLOVER_37_0.S[923]++;_mimeTypes.put("css", "text/css");
        __CLOVER_37_0.S[924]++;_mimeTypes.put("gif", "image/gif");
        __CLOVER_37_0.S[925]++;_mimeTypes.put("jpg", "image/jpeg");
        __CLOVER_37_0.S[926]++;_mimeTypes.put("jpeg", "image/jpeg");
        __CLOVER_37_0.S[927]++;_mimeTypes.put("htm", "text/html");
        __CLOVER_37_0.S[928]++;_mimeTypes.put("html", "text/html");
    }

    private static final int BUFFER_SIZE = 10240;

    /**
     *  Builds a {@link ILink} for a {@link PrivateAsset}.
     *
     *  <p>A single parameter is expected, the resource path of the asset
     *  (which is expected to start with a leading slash).
     *
     **/

    public ILink getLink(IRequestCycle cycle, IComponent component, Object[] parameters)
    {try { __CLOVER_37_0.M[240]++;
        __CLOVER_37_0.S[929]++;if ((((Tapestry.size(parameters) != 1) && (++__CLOVER_37_0.CT[184] != 0)) || (++__CLOVER_37_0.CF[184] == 0))){
            __CLOVER_37_0.S[930]++;throw new ApplicationRuntimeException(
                Tapestry.format("service-single-parameter", Tapestry.ASSET_SERVICE));}

        // Service is stateless

        __CLOVER_37_0.S[931]++;return constructLink(cycle, Tapestry.ASSET_SERVICE, null, parameters, false);
    } finally { }}

    public String getName()
    {try { __CLOVER_37_0.M[241]++;
        __CLOVER_37_0.S[932]++;return Tapestry.ASSET_SERVICE;
    } finally { }}

    private static String getMimeType(String path)
    {try { __CLOVER_37_0.M[242]++;
        __CLOVER_37_0.S[933]++;String key;
        __CLOVER_37_0.S[934]++;String result;
        __CLOVER_37_0.S[935]++;int dotx;

        __CLOVER_37_0.S[936]++;dotx = path.lastIndexOf('.');
        __CLOVER_37_0.S[937]++;key = path.substring(dotx + 1).toLowerCase();

        __CLOVER_37_0.S[938]++;result = (String) _mimeTypes.get(key);

        __CLOVER_37_0.S[939]++;if ((((result == null) && (++__CLOVER_37_0.CT[185] != 0)) || (++__CLOVER_37_0.CF[185] == 0))){
            __CLOVER_37_0.S[940]++;result = "text/plain";}

        __CLOVER_37_0.S[941]++;return result;
    } finally { }}

    /**
     *  Retrieves a resource from the classpath and returns it to the
     *  client in a binary output stream.
     *
     *  <p>TBD: Security issues.  Hackers can download .class files.
     *
     *
     **/

    public void service(
        IEngineServiceView engine,
        IRequestCycle cycle,
        ResponseOutputStream output)
        throws ServletException, IOException
    {try { __CLOVER_37_0.M[243]++;
        __CLOVER_37_0.S[942]++;Object[] parameters = getParameters(cycle);

        __CLOVER_37_0.S[943]++;if ((((Tapestry.size(parameters) != 1) && (++__CLOVER_37_0.CT[186] != 0)) || (++__CLOVER_37_0.CF[186] == 0))){
            __CLOVER_37_0.S[944]++;throw new ApplicationRuntimeException(
                Tapestry.format("service-single-parameter", Tapestry.ASSET_SERVICE));}

        __CLOVER_37_0.S[945]++;String resourcePath = (String) parameters[0];

        __CLOVER_37_0.S[946]++;URL resourceURL = cycle.getEngine().getClassResolver().getResource(resourcePath);

        __CLOVER_37_0.S[947]++;if ((((resourceURL == null) && (++__CLOVER_37_0.CT[187] != 0)) || (++__CLOVER_37_0.CF[187] == 0))){
            __CLOVER_37_0.S[948]++;throw new ApplicationRuntimeException(
                Tapestry.format("missing-resource", resourcePath));}

        __CLOVER_37_0.S[949]++;URLConnection resourceConnection = resourceURL.openConnection();

        __CLOVER_37_0.S[950]++;ServletContext servletContext = cycle.getRequestContext().getServlet().getServletContext();

        __CLOVER_37_0.S[951]++;writeAssetContent(engine, cycle, output, resourcePath, resourceConnection, servletContext);
    } finally { }}

    /**  @since 2.2 **/

    private void writeAssetContent(
        IEngineServiceView engine,
        IRequestCycle cycle,
        ResponseOutputStream output,
        String resourcePath,
        URLConnection resourceConnection,
        ServletContext servletContext)
    {try { __CLOVER_37_0.M[244]++;
        // Getting the content type and length is very dependant
        // on support from the application server (represented
        // here by the servletContext).

        __CLOVER_37_0.S[952]++;String contentType = servletContext.getMimeType(resourcePath);
        __CLOVER_37_0.S[953]++;int contentLength = resourceConnection.getContentLength();

        __CLOVER_37_0.S[954]++;try
        {
            __CLOVER_37_0.S[955]++;if ((((contentLength > 0) && (++__CLOVER_37_0.CT[188] != 0)) || (++__CLOVER_37_0.CF[188] == 0))){
                __CLOVER_37_0.S[956]++;cycle.getRequestContext().getResponse().setContentLength(contentLength);}

            // Set the content type.  If the servlet container doesn't
            // provide it, try and guess it by the extension.

            __CLOVER_37_0.S[957]++;if ((((contentType == null || contentType.length() == 0) && (++__CLOVER_37_0.CT[189] != 0)) || (++__CLOVER_37_0.CF[189] == 0))){
                __CLOVER_37_0.S[958]++;contentType = getMimeType(resourcePath);}

            __CLOVER_37_0.S[959]++;output.setContentType(contentType);

            // Disable any further buffering inside the ResponseOutputStream

            __CLOVER_37_0.S[960]++;output.forceFlush();

            __CLOVER_37_0.S[961]++;InputStream input = resourceConnection.getInputStream();

            __CLOVER_37_0.S[962]++;byte[] buffer = new byte[BUFFER_SIZE];

            __CLOVER_37_0.S[963]++;while (true){
            {
                __CLOVER_37_0.S[964]++;int bytesRead = input.read(buffer);

                __CLOVER_37_0.S[965]++;if ((((bytesRead < 0) && (++__CLOVER_37_0.CT[190] != 0)) || (++__CLOVER_37_0.CF[190] == 0))){
                    __CLOVER_37_0.S[966]++;break;}

                __CLOVER_37_0.S[967]++;output.write(buffer, 0, bytesRead);
            }}

            __CLOVER_37_0.S[968]++;input.close();
        }
        catch (Throwable ex)
        {
            __CLOVER_37_0.S[969]++;String title = Tapestry.format("AssetService.exception-report-title", resourcePath);

            __CLOVER_37_0.S[970]++;engine.reportException(title, ex);
        }
    } finally { }}
}