/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.callback;

import org.apache.tapestry.IPage;
import org.apache.tapestry.IRequestCycle;

/**
 *  Simple callback for returning to a page.
 *  <p>
 *  Example usage of <tt>PageCallback</tt>:
 *  <p>
 *  The Home page ensure a user is 
 *  authenticated in the {@link org.apache.tapestry.IPage#validate(IRequestCycle)} 
 *  method.  If the user is not authenticated, they are redirected to the Login 
 *  page, after setting a callback in the Login page.
 *  <p>
 *  The Login page <tt>formSubmit()</tt> {@link org.apache.tapestry.IActionListener} 
 *  authenticates the user and then invokes {@link ICallback#performCallback(IRequestCycle)} 
 *  to the Home page.
 *  <pre>
 *  public class Home extends BasePage {
 * 
 *      public void validate(IRequestCycle cycle) {            
 *          Visit visit = (Visit) getVisit();
 *      
 *          if (!visit.isAuthenticated()) {
 *              Login login = (Login) cycle.getPage("Login");
 *
 *              login.setCallback(new PageCallback(this));
 *              
 *              throw new PageRedirectException(login);
 *          }            
 *      }
 *  }
 *
 *  public Login extends BasePage {
 * 
 *      private ICallback _callback;
 *
 *      public void setCallback(ICallback _callback) {
 *          _callback = callback;
 *      }
 *
 *      public void formSubmit(IRequestCycle cycle) {
 *          // Authentication code
 *          ..
 *   
 *          Visit visit = (Visit) getVisit();
 *
 *          visit.setAuthenticated(true);
 *  
 *          if (_callback != null) {
 *              _callback.performCallback(cycle);
 *          }
 *      }
 *  }    
 *  </pre>
 *
 *  @author Howard Lewis Ship
 *  @since 0.2.9
 *
 **/

public class PageCallback implements ICallback
{public static com.cortexeb.tools.clover.d __CLOVER_59_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /**
     *  @since 2.0.4
     * 
     **/

    private static final long serialVersionUID = -3286806776105690068L;

    private String _pageName;

    public PageCallback(String pageName)
    {try { __CLOVER_59_0.M[351]++;
        __CLOVER_59_0.S[1440]++;_pageName = pageName;
    } finally { }}

    public PageCallback(IPage page)
    {
        this(page.getPageName());__CLOVER_59_0.S[1441]++;try { __CLOVER_59_0.M[352]++;
    } finally { }}

    public String toString()
    {try { __CLOVER_59_0.M[353]++;
        __CLOVER_59_0.S[1442]++;return "PageCallback[" + _pageName + "]";
    } finally { }}

    /**
     *  Invokes {@link IRequestCycle#activate(String)} to select the previously
     *  identified page as the response page.
     *
     **/

    public void performCallback(IRequestCycle cycle)
    {try { __CLOVER_59_0.M[354]++;
        __CLOVER_59_0.S[1443]++;cycle.activate(_pageName);
    } finally { }}
}