/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.enhance;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.ClassResolver;
import org.apache.hivemind.service.ClassFactory;
import org.apache.hivemind.service.MethodSignature;
import org.apache.tapestry.engine.IComponentClassEnhancer;
import org.apache.tapestry.event.ResetEventListener;
import org.apache.tapestry.spec.IComponentSpecification;

/**
 *  Default implementation of {@link IComponentClassEnhancer}.
 *
 *  @author Howard Lewis Ship
 *  @since 3.0
 * 
 */

public class DefaultComponentClassEnhancer implements IComponentClassEnhancer, ResetEventListener
{public static com.cortexeb.tools.clover.d __CLOVER_110_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private Log _log;

    /**
     *  Map of Class, keyed on IComponentSpecification.
     * 
     */

    private Map _cachedClasses = Collections.synchronizedMap(new HashMap());

    private ClassResolver _classResolver;
    private boolean _disableValidation;
    private ClassFactory _classFactory;

    public void resetEventDidOccur()
    {try { __CLOVER_110_0.M[623]++;
        __CLOVER_110_0.S[2728]++;_cachedClasses.clear();
    } finally { }}

    public Class getEnhancedClass(IComponentSpecification specification, String className)
    {try { __CLOVER_110_0.M[624]++;
        __CLOVER_110_0.S[2729]++;Class result = getCachedClass(specification);

        __CLOVER_110_0.S[2730]++;if ((((result == null) && (++__CLOVER_110_0.CT[499] != 0)) || (++__CLOVER_110_0.CF[499] == 0))){
        {
            __CLOVER_110_0.S[2731]++;synchronized (this)
            {
                __CLOVER_110_0.S[2732]++;result = getCachedClass(specification);
                __CLOVER_110_0.S[2733]++;if ((((result == null) && (++__CLOVER_110_0.CT[500] != 0)) || (++__CLOVER_110_0.CF[500] == 0))){
                {
                    __CLOVER_110_0.S[2734]++;result = constructComponentClass(specification, className);
                    __CLOVER_110_0.S[2735]++;storeCachedClass(specification, result);
                }}
            }
        }}

        __CLOVER_110_0.S[2736]++;return result;
    } finally { }}

    private void storeCachedClass(IComponentSpecification specification, Class cachedClass)
    {try { __CLOVER_110_0.M[625]++;
        __CLOVER_110_0.S[2737]++;_cachedClasses.put(specification, cachedClass);
    } finally { }}

    private Class getCachedClass(IComponentSpecification specification)
    {try { __CLOVER_110_0.M[626]++;
        __CLOVER_110_0.S[2738]++;return (Class) _cachedClasses.get(specification);
    } finally { }}

    /**
     *  Returns the class to be used for the component, which is either
     *  the class with the given name, or an enhanced subclass.
     * 
     */

    private Class constructComponentClass(IComponentSpecification specification, String className)
    {try { __CLOVER_110_0.M[627]++;
        __CLOVER_110_0.S[2739]++;Class result = null;

        __CLOVER_110_0.S[2740]++;try
        {
            __CLOVER_110_0.S[2741]++;result = _classResolver.findClass(className);
        }
        catch (Exception ex)
        {
            __CLOVER_110_0.S[2742]++;throw new ApplicationRuntimeException(ex.getMessage(), specification.getLocation(), ex);
        }

        __CLOVER_110_0.S[2743]++;try
        {
            __CLOVER_110_0.S[2744]++;ComponentClassFactory factory = createComponentClassFactory(specification, result);

            __CLOVER_110_0.S[2745]++;if ((((factory.needsEnhancement()) && (++__CLOVER_110_0.CT[501] != 0)) || (++__CLOVER_110_0.CF[501] == 0))){
            {
                __CLOVER_110_0.S[2746]++;result = factory.createEnhancedSubclass();

                __CLOVER_110_0.S[2747]++;if ((((!_disableValidation) && (++__CLOVER_110_0.CT[502] != 0)) || (++__CLOVER_110_0.CF[502] == 0))){
                    __CLOVER_110_0.S[2748]++;validateEnhancedClass(result, className, specification);}
            }}
        }
        catch (Throwable ex)
        {
            __CLOVER_110_0.S[2749]++;throw new ApplicationRuntimeException(
                EnhanceMessages.codeGenerationError(className, ex),
                ex);
        }

        __CLOVER_110_0.S[2750]++;return result;
    } finally { }}

    /**
     *  Constructs a new factory for enhancing the specified class. Advanced users
     *  may want to provide thier own enhancements to classes and this method
     *  is the hook that allows them to provide a subclass of
     *  {@link org.apache.tapestry.enhance.ComponentClassFactory} adding those
     *  enhancements.
     * 
     */

    private ComponentClassFactory createComponentClassFactory(
        IComponentSpecification specification,
        Class componentClass)
    {try { __CLOVER_110_0.M[628]++;
        __CLOVER_110_0.S[2751]++;return new ComponentClassFactory(
            _classResolver,
            specification,
            componentClass,
            _classFactory);
    } finally { }}

    /**
     *  Invoked to validate that an enhanced class is acceptible.  Primarily, this is to ensure
     *  that the class contains no unimplemented abstract methods or fields.  Normally,
     *  this kind of checking is done at compile time, but for generated
     *  classes, there is no compile time check (!) and you can get runtime
     *  errors when accessing unimplemented abstract methods.
     * 
     *
     */

    private void validateEnhancedClass(
        Class subject,
        String className,
        IComponentSpecification specification)
    {try { __CLOVER_110_0.M[629]++;
        __CLOVER_110_0.S[2752]++;boolean debug = _log.isDebugEnabled();

        __CLOVER_110_0.S[2753]++;if ((((debug) && (++__CLOVER_110_0.CT[503] != 0)) || (++__CLOVER_110_0.CF[503] == 0))){
            __CLOVER_110_0.S[2754]++;_log.debug("Validating " + subject);}

        __CLOVER_110_0.S[2755]++;Set implementedMethods = new HashSet();
        __CLOVER_110_0.S[2756]++;Class current = subject;

        __CLOVER_110_0.S[2757]++;while (true){
        {
            __CLOVER_110_0.S[2758]++;Method m = checkForAbstractMethods(current, implementedMethods);

            __CLOVER_110_0.S[2759]++;if ((((m != null) && (++__CLOVER_110_0.CT[504] != 0)) || (++__CLOVER_110_0.CF[504] == 0))){
                __CLOVER_110_0.S[2760]++;throw new ApplicationRuntimeException(
                    EnhanceMessages.noImplForAbstractMethod(m, current, className, subject),
                    specification.getLocation(),
                    null);}

            // An earlier version of this code walked the interfaces directly,
            // but it appears that implementing an interface actually
            // puts abstract method declarations into the class
            // (at least, in terms of what getDeclaredMethods() returns).

            // March up to the super class.

            __CLOVER_110_0.S[2761]++;current = current.getSuperclass();

            // Once advanced up to a concrete class, we trust that
            // the compiler did its checking.

            __CLOVER_110_0.S[2762]++;if ((((!Modifier.isAbstract(current.getModifiers())) && (++__CLOVER_110_0.CT[505] != 0)) || (++__CLOVER_110_0.CF[505] == 0))){
                __CLOVER_110_0.S[2763]++;break;}
        }}

    } finally { }}

    /**
     *  Searches the class for abstract methods, returning the first found.
     *  Records non-abstract methods in the implementedMethods set.
     * 
     */

    private Method checkForAbstractMethods(Class current, Set implementedMethods)
    {try { __CLOVER_110_0.M[630]++;
        __CLOVER_110_0.S[2764]++;boolean debug = _log.isDebugEnabled();

        __CLOVER_110_0.S[2765]++;if ((((debug) && (++__CLOVER_110_0.CT[506] != 0)) || (++__CLOVER_110_0.CF[506] == 0))){
            __CLOVER_110_0.S[2766]++;_log.debug("Searching for abstract methods in " + current);}

        __CLOVER_110_0.S[2767]++;Method[] methods = current.getDeclaredMethods();

        __CLOVER_110_0.S[2768]++;for (int i = 0; (((i < methods.length) && (++__CLOVER_110_0.CT[507] != 0)) || (++__CLOVER_110_0.CF[507] == 0)); i++){
        {
            __CLOVER_110_0.S[2769]++;Method m = methods[i];

            __CLOVER_110_0.S[2770]++;if ((((debug) && (++__CLOVER_110_0.CT[508] != 0)) || (++__CLOVER_110_0.CF[508] == 0))){
                __CLOVER_110_0.S[2771]++;_log.debug("Checking " + m);}

            __CLOVER_110_0.S[2772]++;boolean isAbstract = Modifier.isAbstract(m.getModifiers());

            __CLOVER_110_0.S[2773]++;MethodSignature s = new MethodSignature(m);

            __CLOVER_110_0.S[2774]++;if ((((isAbstract) && (++__CLOVER_110_0.CT[509] != 0)) || (++__CLOVER_110_0.CF[509] == 0))){
            {
                __CLOVER_110_0.S[2775]++;if ((((implementedMethods.contains(s)) && (++__CLOVER_110_0.CT[510] != 0)) || (++__CLOVER_110_0.CF[510] == 0))){
                    __CLOVER_110_0.S[2776]++;continue;}

                __CLOVER_110_0.S[2777]++;return m;
            }}

            __CLOVER_110_0.S[2778]++;implementedMethods.add(s);
        }}

        __CLOVER_110_0.S[2779]++;return null;
    } finally { }}

    public void setLog(Log log)
    {try { __CLOVER_110_0.M[631]++;
        __CLOVER_110_0.S[2780]++;_log = log;
    } finally { }}

	/**
	 * Because this is set from the application's property source, the value is obtained
	 * as a string (HiveMind doesn't have an opportunity to coerce it to boolean for us),
	 * so we do the conversion here. 
	 */
    public void setDisableValidation(String value)
    {try { __CLOVER_110_0.M[632]++;
        __CLOVER_110_0.S[2781]++;_disableValidation = "true".equals(value);
    } finally { }}

    public void setClassResolver(ClassResolver resolver)
    {try { __CLOVER_110_0.M[633]++;
        __CLOVER_110_0.S[2782]++;_classResolver = resolver;
    } finally { }}

    public void setClassFactory(ClassFactory factory)
    {try { __CLOVER_110_0.M[634]++;
        __CLOVER_110_0.S[2783]++;_classFactory = factory;
    } finally { }}

}