/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.form;

import java.util.ResourceBundle;

import org.apache.commons.lang.enum.Enum;

/**
 *  Implementation of {@link IPropertySelectionModel} that wraps around
 *  a set of {@link Enum}s.
 * 
 *  <p>Uses a simple index number as the value (used to represent the option).
 *
 *  <p>The resource bundle from which labels are extracted is usually
 *  a resource within the Tapestry application.  Since 
 *  {@link ResourceBundle#getBundle(String, java.util.Locale)} uses its caller's class loader,
 *  and that classloader will be the Tapestry framework's classloader, the application's
 *  resources won't be visible.  This requires that the application resolve
 *  the resource to a {@link ResourceBundle} before creating this model.
 *  
 *  @author Howard Lewis Ship
 * 
 **/

public class EnumPropertySelectionModel implements IPropertySelectionModel
{public static com.cortexeb.tools.clover.d __CLOVER_129_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private Enum[] _options;
    private String[] _labels;

    private String _resourcePrefix;
    private ResourceBundle _bundle;

    /**
     *  Standard constructor.
     *
     *  <p>Labels for the options are extracted from a resource bundle.  resourceBaseName
     *  identifies the bundle.  Typically, the bundle will be a <code>.properties</code>
     *  file within the classpath.  Specify the fully qualified class name equivalent, i.e.,
     *  for file <code>/com/example/foo/LabelStrings.properties</code> use
     *  <code>com.example.foo.LabelStrings</code> as the resource base name.
     *
     *  <p>Normally (when resourcePrefix is null), the keys used to extract labels
     *  matches the {@link Enum#getName() enumeration id} of the option.  By
     *  convention, the enumeration id matches the name of the static variable.
     *
     *  <p>To avoid naming conflicts when using a single resource bundle for multiple
     *  models, use a resource prefix.  This is a string which is prepended to
     *  the enumeration id (they prefix and enumeration id are seperated with a period).
     *
     *  @param   options The list of possible values for this model, in the order they
     *  should appear. This exact array is retained (not copied).
     *
     *  @param   bundle The {@link ResourceBundle} from which labels may be extracted.
     *
     *  @param   resourcePrefix An optional prefix used when accessing keys within the bundle. 
     *  Used to allow a single ResouceBundle to contain labels for multiple Enums.
     **/

    public EnumPropertySelectionModel(Enum[] options, ResourceBundle bundle, String resourcePrefix)
    {try { __CLOVER_129_0.M[679]++;
        __CLOVER_129_0.S[3016]++;_options = options;
        __CLOVER_129_0.S[3017]++;_bundle = bundle;
        __CLOVER_129_0.S[3018]++;_resourcePrefix = resourcePrefix;
    } finally { }}

    /**
     *  Simplified constructor using no prefix.
     *
     **/

    public EnumPropertySelectionModel(Enum[] options, ResourceBundle bundle)
    {
        this(options, bundle, null);__CLOVER_129_0.S[3019]++;try { __CLOVER_129_0.M[680]++;
    } finally { }}

    public int getOptionCount()
    {try { __CLOVER_129_0.M[681]++;
        __CLOVER_129_0.S[3020]++;return _options.length;
    } finally { }}

    public Object getOption(int index)
    {try { __CLOVER_129_0.M[682]++;
        __CLOVER_129_0.S[3021]++;return _options[index];
    } finally { }}

    public String getLabel(int index)
    {try { __CLOVER_129_0.M[683]++;
        __CLOVER_129_0.S[3022]++;if ((((_labels == null) && (++__CLOVER_129_0.CT[549] != 0)) || (++__CLOVER_129_0.CF[549] == 0))){
            __CLOVER_129_0.S[3023]++;readLabels();}

        __CLOVER_129_0.S[3024]++;return _labels[index];
    } finally { }}

    public String getValue(int index)
    {try { __CLOVER_129_0.M[684]++;
        __CLOVER_129_0.S[3025]++;return Integer.toString(index);
    } finally { }}

    public Object translateValue(String value)
    {try { __CLOVER_129_0.M[685]++;
        __CLOVER_129_0.S[3026]++;int index;

        __CLOVER_129_0.S[3027]++;index = Integer.parseInt(value);

        __CLOVER_129_0.S[3028]++;return _options[index];
    } finally { }}

    private void readLabels()
    {try { __CLOVER_129_0.M[686]++;
        __CLOVER_129_0.S[3029]++;_labels = new String[_options.length];

        __CLOVER_129_0.S[3030]++;for (int i = 0; (((i < _options.length) && (++__CLOVER_129_0.CT[550] != 0)) || (++__CLOVER_129_0.CF[550] == 0)); i++){
        {
            __CLOVER_129_0.S[3031]++;String enumerationId = _options[i].getName();

            __CLOVER_129_0.S[3032]++;String key;

            __CLOVER_129_0.S[3033]++;if ((((_resourcePrefix == null) && (++__CLOVER_129_0.CT[551] != 0)) || (++__CLOVER_129_0.CF[551] == 0))){
                __CLOVER_129_0.S[3034]++;key = enumerationId;}
            else{
                __CLOVER_129_0.S[3035]++;key = _resourcePrefix + "." + enumerationId;}

            __CLOVER_129_0.S[3036]++;_labels[i] = _bundle.getString(key);
        }}

    } finally { }}
}