/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.form;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.tapestry.AbstractComponent;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IActionListener;
import org.apache.tapestry.IBinding;
import org.apache.tapestry.IDirect;
import org.apache.tapestry.IEngine;
import org.apache.tapestry.IForm;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.RenderRewoundException;
import org.apache.tapestry.StaleLinkException;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.html.Body;
import org.apache.tapestry.util.IdAllocator;
import org.apache.tapestry.util.StringSplitter;
import org.apache.tapestry.valid.IValidationDelegate;

/**
 *  Component which contains form element components.  Forms use the
 *  action or direct services to handle the form submission.  A Form will wrap
 *  other components and static HTML, including
 *  form components such as {@link TextArea}, {@link TextField}, {@link Checkbox}, etc.
 * 
 *  [<a href="../../../../../ComponentReference/Form.html">Component Reference</a>]
 * 
 *  <p>When a form is submitted, it continues through the rewind cycle until
 *  <em>after</em> all of its wrapped elements have renderred.  As the form
 *  component render (in the rewind cycle), they will be updating
 *  properties of the containing page and notifying thier listeners.  Again:
 *  each form component is responsible not only for rendering HTML (to present the
 *  form), but for handling it's share of the form submission.
 *
 *  <p>Only after all that is done will the Form notify its listener.
 *
 *  <p>Starting in release 1.0.2, a Form can use either the direct service or
 *  the action service.  The default is the direct service, even though
 *  in earlier releases, only the action service was available.
 *
 *  @author Howard Lewis Ship, David Solis
 **/

public abstract class Form extends AbstractComponent implements IForm, IDirect
{public static com.cortexeb.tools.clover.d __CLOVER_130_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static class HiddenValue
    {
        String _name;
        String _value;
        String _id;

        private HiddenValue(String name, String value)
        {
			this(name, null, value);__CLOVER_130_0.S[3037]++;try { __CLOVER_130_0.M[687]++;
        } finally { }}

		private HiddenValue(String name, String id, String value)
		{try { __CLOVER_130_0.M[688]++;
			__CLOVER_130_0.S[3038]++;_name	= name;
			__CLOVER_130_0.S[3039]++;_id		= id;
			__CLOVER_130_0.S[3040]++;_value	= value;
		} finally { }}
    }

    private boolean _rewinding;
    private boolean _rendering;
    private String _name;

    /**
     *  Used when rewinding the form to figure to match allocated ids (allocated during
     *  the rewind) against expected ids (allocated in the previous request cycle, when
     *  the form was rendered).
     * 
     *  @since 3.0
     * 
     **/

    private int _allocatedIdIndex;

    /**
     *  The list of allocated ids for form elements within this form.  This list
     *  is constructed when a form renders, and is validated against when the
     *  form is rewound.
     * 
     *  @since 3.0
     * 
     **/

    private List _allocatedIds = new ArrayList();

    /**
     *  {@link Map}, keyed on {@link FormEventType}.  Values are either a String (the name
     *  of a single event), or a {@link List} of Strings.
     *
     *  @since 1.0.2
     **/

    private Map _events;

    private static final int EVENT_MAP_SIZE = 3;

    private IdAllocator _elementIdAllocator = new IdAllocator();

    private String _encodingType;

    private List _hiddenValues;

    /**
     *  Returns the currently active {@link IForm}, or null if no form is
     *  active.  This is a convienience method, the result will be
     *  null, or an instance of {@link IForm}, but not necessarily a
     *  <code>Form</code>.
     *
     **/

    public static IForm get(IRequestCycle cycle)
    {try { __CLOVER_130_0.M[689]++;
        __CLOVER_130_0.S[3041]++;return (IForm) cycle.getAttribute(ATTRIBUTE_NAME);
    } finally { }}

    /**
     *  Indicates to any wrapped form components that they should respond to the form
     *  submission.
     *
     *  @throws ApplicationRuntimeException if not rendering.
     **/

    public boolean isRewinding()
    {try { __CLOVER_130_0.M[690]++;
        __CLOVER_130_0.S[3042]++;if ((((!_rendering) && (++__CLOVER_130_0.CT[552] != 0)) || (++__CLOVER_130_0.CF[552] == 0))){
            __CLOVER_130_0.S[3043]++;throw Tapestry.createRenderOnlyPropertyException(this, "rewinding");}

        __CLOVER_130_0.S[3044]++;return _rewinding;
    } finally { }}

    /**
     *  Returns true if this Form is configured to use the direct
     *  service.
     *
     *  <p>This is derived from the direct parameter, and defaults
     *  to true if not bound.
     *
     *  @since 1.0.2
     **/

    public abstract boolean isDirect();

    /**
     *  Returns true if the stateful parameter is bound to
     *  a true value.  If stateful is not bound, also returns
     *  the default, true.
     *
     *  @since 1.0.1
     **/

    public boolean getRequiresSession()
    {try { __CLOVER_130_0.M[691]++;
        __CLOVER_130_0.S[3045]++;return isStateful();
    } finally { }}

    /**
     *  Constructs a unique identifier (within the Form).  The identifier
     *  consists of the component's id, with an index number added to
     *  ensure uniqueness.
     *
     *  <p>Simply invokes {@link #getElementId(org.apache.tapestry.form.IFormComponent, java.lang.String)}
     *  with the component's id.
     *
     *
     *  @since 1.0.2
     **/

    public String getElementId(IFormComponent component)
    {try { __CLOVER_130_0.M[692]++;
        __CLOVER_130_0.S[3046]++;return getElementId(component, component.getId());
    } finally { }}

    /**
     *  Constructs a unique identifier from the base id.  If possible, the
     *  id is used as-is.  Otherwise, a unique identifier is appended
     *  to the id.
     *
     *  <p>This method is provided simply so that some components
     * ({@link ImageSubmit}) have more specific control over
     *  their names.
     *
     *  @since 1.0.3
     *
     **/

    public String getElementId(IFormComponent component, String baseId)
    {try { __CLOVER_130_0.M[693]++;
        __CLOVER_130_0.S[3047]++;String result = _elementIdAllocator.allocateId(baseId);

        __CLOVER_130_0.S[3048]++;if ((((_rewinding) && (++__CLOVER_130_0.CT[553] != 0)) || (++__CLOVER_130_0.CF[553] == 0))){
        {
            __CLOVER_130_0.S[3049]++;if ((((_allocatedIdIndex >= _allocatedIds.size()) && (++__CLOVER_130_0.CT[554] != 0)) || (++__CLOVER_130_0.CF[554] == 0))){
            {
                __CLOVER_130_0.S[3050]++;throw new StaleLinkException(
                    Tapestry.format(
                        "Form.too-many-ids",
                        getExtendedId(),
                        Integer.toString(_allocatedIds.size()),
                        component.getExtendedId()),
                    this);
            }}

            __CLOVER_130_0.S[3051]++;String expected = (String) _allocatedIds.get(_allocatedIdIndex);

            __CLOVER_130_0.S[3052]++;if ((((!result.equals(expected)) && (++__CLOVER_130_0.CT[555] != 0)) || (++__CLOVER_130_0.CF[555] == 0))){
                __CLOVER_130_0.S[3053]++;throw new StaleLinkException(
                    Tapestry.format(
                        "Form.id-mismatch",
                        new Object[] {
                            getExtendedId(),
                            Integer.toString(_allocatedIdIndex + 1),
                            expected,
                            result,
                            component.getExtendedId()}),
                    this);}
        }}
        else{
        {
            __CLOVER_130_0.S[3054]++;_allocatedIds.add(result);
        }}

        __CLOVER_130_0.S[3055]++;_allocatedIdIndex++;

        __CLOVER_130_0.S[3056]++;component.setName(result);

        __CLOVER_130_0.S[3057]++;return result;
    } finally { }}

    /**
     *  Returns the name generated for the form.  This is used to faciliate
     *  components that write JavaScript and need to access the form or
     *  its contents.
     *
     *  <p>This value is generated when the form renders, and is not cleared.
     *  If the Form is inside a {@link org.apache.tapestry.components.Foreach}, 
     *  this will be the most recently
     *  generated name for the Form.
     *
     *  <p>This property is exposed so that sophisticated applications can write
     *  JavaScript handlers for the form and components within the form.
     *
     *  @see AbstractFormComponent#getName()
     *
     **/

    public String getName()
    {try { __CLOVER_130_0.M[694]++;
        __CLOVER_130_0.S[3058]++;return _name;
    } finally { }}

    /** @since 3.0 **/

    protected void prepareForRender(IRequestCycle cycle)
    {try { __CLOVER_130_0.M[695]++;
        __CLOVER_130_0.S[3059]++;super.prepareForRender(cycle);

        __CLOVER_130_0.S[3060]++;if ((((cycle.getAttribute(ATTRIBUTE_NAME) != null) && (++__CLOVER_130_0.CT[556] != 0)) || (++__CLOVER_130_0.CF[556] == 0))){
            __CLOVER_130_0.S[3061]++;throw new ApplicationRuntimeException(
                Tapestry.getMessage("Form.forms-may-not-nest"),
                this,
                null,
                null);}

        __CLOVER_130_0.S[3062]++;cycle.setAttribute(ATTRIBUTE_NAME, this);
    } finally { }}

    protected void cleanupAfterRender(IRequestCycle cycle)
    {try { __CLOVER_130_0.M[696]++;
        __CLOVER_130_0.S[3063]++;_rendering = false;

        __CLOVER_130_0.S[3064]++;_allocatedIdIndex = 0;
        __CLOVER_130_0.S[3065]++;_allocatedIds.clear();

        __CLOVER_130_0.S[3066]++;_events = null;

        __CLOVER_130_0.S[3067]++;_elementIdAllocator.clear();

        __CLOVER_130_0.S[3068]++;if ((((_hiddenValues != null) && (++__CLOVER_130_0.CT[557] != 0)) || (++__CLOVER_130_0.CF[557] == 0))){
            __CLOVER_130_0.S[3069]++;_hiddenValues.clear();}

        __CLOVER_130_0.S[3070]++;cycle.removeAttribute(ATTRIBUTE_NAME);

        __CLOVER_130_0.S[3071]++;_encodingType = null;

        __CLOVER_130_0.S[3072]++;IValidationDelegate delegate = getDelegate();

        __CLOVER_130_0.S[3073]++;if ((((delegate != null) && (++__CLOVER_130_0.CT[558] != 0)) || (++__CLOVER_130_0.CF[558] == 0))){
            __CLOVER_130_0.S[3074]++;delegate.setFormComponent(null);}

        __CLOVER_130_0.S[3075]++;super.cleanupAfterRender(cycle);
    } finally { }}

    protected void writeAttributes(IMarkupWriter writer, ILink link)
    {try { __CLOVER_130_0.M[697]++;
        __CLOVER_130_0.S[3076]++;String method = getMethod();

        __CLOVER_130_0.S[3077]++;writer.begin(getTag());
        __CLOVER_130_0.S[3078]++;writer.attribute("method", ((((method == null) ) && (++__CLOVER_130_0.CT[559] != 0)) || (++__CLOVER_130_0.CF[559] == 0))? "post" : method);
        __CLOVER_130_0.S[3079]++;writer.attribute("name", _name);
        __CLOVER_130_0.S[3080]++;writer.attribute("action", link.getURL(null, false));

        __CLOVER_130_0.S[3081]++;if ((((_encodingType != null) && (++__CLOVER_130_0.CT[560] != 0)) || (++__CLOVER_130_0.CF[560] == 0))){
            __CLOVER_130_0.S[3082]++;writer.attribute("enctype", _encodingType);}
    } finally { }}

    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_130_0.M[698]++;
        __CLOVER_130_0.S[3083]++;String actionId = cycle.getNextActionId();
        __CLOVER_130_0.S[3084]++;_name = getDisplayName() + actionId;

        __CLOVER_130_0.S[3085]++;boolean renderForm = !cycle.isRewinding();
        __CLOVER_130_0.S[3086]++;boolean rewound = cycle.isRewound(this);

        __CLOVER_130_0.S[3087]++;_rewinding = rewound;

        __CLOVER_130_0.S[3088]++;_allocatedIdIndex = 0;

        __CLOVER_130_0.S[3089]++;_rendering = true;

        __CLOVER_130_0.S[3090]++;if ((((rewound) && (++__CLOVER_130_0.CT[561] != 0)) || (++__CLOVER_130_0.CF[561] == 0))){
        {
            __CLOVER_130_0.S[3091]++;String storedIdList = cycle.getRequestContext().getParameter(_name);

            __CLOVER_130_0.S[3092]++;reconstructAllocatedIds(storedIdList);
        }}

        __CLOVER_130_0.S[3093]++;ILink link = getLink(cycle, actionId);

        // When rendering, use a nested writer so that an embedded Upload
        // component can force the encoding type.

        __CLOVER_130_0.S[3094]++;IMarkupWriter nested = writer.getNestedWriter();

        __CLOVER_130_0.S[3095]++;renderBody(nested, cycle);

        __CLOVER_130_0.S[3096]++;if ((((renderForm) && (++__CLOVER_130_0.CT[562] != 0)) || (++__CLOVER_130_0.CF[562] == 0))){
        {
            __CLOVER_130_0.S[3097]++;writeAttributes(writer, link);

            __CLOVER_130_0.S[3098]++;renderInformalParameters(writer, cycle);
            __CLOVER_130_0.S[3099]++;writer.println();
        }}

        // Write the hidden's, or at least, reserve the query parameters
        // required by the Gesture.

        __CLOVER_130_0.S[3100]++;writeLinkParameters(writer, link, !renderForm);

        __CLOVER_130_0.S[3101]++;if ((((renderForm) && (++__CLOVER_130_0.CT[563] != 0)) || (++__CLOVER_130_0.CF[563] == 0))){
        {
            // What's this for?  It's part of checking for stale links.  
            // We record the list of allocated ids.
            // On rewind, we check that the stored list against which
            // ids were allocated.  If the persistent state of the page or
            // application changed between render (previous request cycle)
            // and rewind (current request cycle), then the list
            // of ids will change as well.

            __CLOVER_130_0.S[3102]++;writeHiddenField(writer, _name, buildAllocatedIdList());
            __CLOVER_130_0.S[3103]++;writeHiddenValues(writer);

            __CLOVER_130_0.S[3104]++;nested.close();

            __CLOVER_130_0.S[3105]++;writer.end(getTag());

            // Write out event handlers collected during the rendering.

            __CLOVER_130_0.S[3106]++;emitEventHandlers(writer, cycle);
        }}

        __CLOVER_130_0.S[3107]++;if ((((rewound) && (++__CLOVER_130_0.CT[564] != 0)) || (++__CLOVER_130_0.CF[564] == 0))){
        {
            __CLOVER_130_0.S[3108]++;int expected = _allocatedIds.size();

            // The other case, _allocatedIdIndex > expected, is
            // checked for inside getElementId().  Remember that
            // _allocatedIdIndex is incremented after allocating.

            __CLOVER_130_0.S[3109]++;if ((((_allocatedIdIndex < expected) && (++__CLOVER_130_0.CT[565] != 0)) || (++__CLOVER_130_0.CF[565] == 0))){
            {
                __CLOVER_130_0.S[3110]++;String nextExpectedId = (String) _allocatedIds.get(_allocatedIdIndex);

                __CLOVER_130_0.S[3111]++;throw new StaleLinkException(
                    Tapestry.format(
                        "Form.too-few-ids",
                        getExtendedId(),
                        Integer.toString(expected - _allocatedIdIndex),
                        nextExpectedId),
                    this);
            }}

            __CLOVER_130_0.S[3112]++;IActionListener listener = getListener();

            __CLOVER_130_0.S[3113]++;if ((((listener != null) && (++__CLOVER_130_0.CT[566] != 0)) || (++__CLOVER_130_0.CF[566] == 0))){
                __CLOVER_130_0.S[3114]++;listener.actionTriggered(this, cycle);}

            // Abort the rewind render.

            __CLOVER_130_0.S[3115]++;throw new RenderRewoundException(this);
        }}
    } finally { }}

    /**
     *  Adds an additional event handler.
     *
     *  @since 1.0.2
     * 
     **/

    public void addEventHandler(FormEventType type, String functionName)
    {try { __CLOVER_130_0.M[699]++;
        __CLOVER_130_0.S[3116]++;if ((((_events == null) && (++__CLOVER_130_0.CT[567] != 0)) || (++__CLOVER_130_0.CF[567] == 0))){
            __CLOVER_130_0.S[3117]++;_events = new HashMap(EVENT_MAP_SIZE);}

        __CLOVER_130_0.S[3118]++;Object value = _events.get(type);

        // The value can either be a String, or a List of String.  Since
        // it is rare for there to be more than one event handling function,
        // we start with just a String.

        __CLOVER_130_0.S[3119]++;if ((((value == null) && (++__CLOVER_130_0.CT[568] != 0)) || (++__CLOVER_130_0.CF[568] == 0))){
        {
            __CLOVER_130_0.S[3120]++;_events.put(type, functionName);
            __CLOVER_130_0.S[3121]++;return;
        }}

        // The second function added converts it to a List.

        __CLOVER_130_0.S[3122]++;if ((((value instanceof String) && (++__CLOVER_130_0.CT[569] != 0)) || (++__CLOVER_130_0.CF[569] == 0))){
        {
            __CLOVER_130_0.S[3123]++;List list = new ArrayList();
            __CLOVER_130_0.S[3124]++;list.add(value);
            __CLOVER_130_0.S[3125]++;list.add(functionName);

            __CLOVER_130_0.S[3126]++;_events.put(type, list);
            __CLOVER_130_0.S[3127]++;return;
        }}

        // The third and subsequent function just
        // adds to the List.

        __CLOVER_130_0.S[3128]++;List list = (List) value;
        __CLOVER_130_0.S[3129]++;list.add(functionName);
    } finally { }}

    protected void emitEventHandlers(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_130_0.M[700]++;

        __CLOVER_130_0.S[3130]++;if ((((_events == null || _events.isEmpty()) && (++__CLOVER_130_0.CT[570] != 0)) || (++__CLOVER_130_0.CF[570] == 0))){
            __CLOVER_130_0.S[3131]++;return;}

        __CLOVER_130_0.S[3132]++;Body body = Body.get(cycle);

        __CLOVER_130_0.S[3133]++;if ((((body == null) && (++__CLOVER_130_0.CT[571] != 0)) || (++__CLOVER_130_0.CF[571] == 0))){
            __CLOVER_130_0.S[3134]++;throw new ApplicationRuntimeException(
                Tapestry.getMessage("Form.needs-body-for-event-handlers"),
                this,
                null,
                null);}

        __CLOVER_130_0.S[3135]++;StringBuffer buffer = new StringBuffer();

        __CLOVER_130_0.S[3136]++;Iterator i = _events.entrySet().iterator();
        __CLOVER_130_0.S[3137]++;while ((((i.hasNext()) && (++__CLOVER_130_0.CT[572] != 0)) || (++__CLOVER_130_0.CF[572] == 0))){
        {

            __CLOVER_130_0.S[3138]++;Map.Entry entry = (Map.Entry) i.next();
            __CLOVER_130_0.S[3139]++;FormEventType type = (FormEventType) entry.getKey();
            __CLOVER_130_0.S[3140]++;Object value = entry.getValue();

            __CLOVER_130_0.S[3141]++;buffer.append("document.");
            __CLOVER_130_0.S[3142]++;buffer.append(_name);
            __CLOVER_130_0.S[3143]++;buffer.append(".");
            __CLOVER_130_0.S[3144]++;buffer.append(type.getPropertyName());
            __CLOVER_130_0.S[3145]++;buffer.append(" = ");

            // The typical case; one event one event handler.  Easy enough.

            __CLOVER_130_0.S[3146]++;if ((((value instanceof String) && (++__CLOVER_130_0.CT[573] != 0)) || (++__CLOVER_130_0.CF[573] == 0))){
            {
                __CLOVER_130_0.S[3147]++;buffer.append(value.toString());
                __CLOVER_130_0.S[3148]++;buffer.append(";");
            }}
            else{
            {
                // Build a composite function in-place

                __CLOVER_130_0.S[3149]++;buffer.append("function ()\n{\n");

                __CLOVER_130_0.S[3150]++;boolean combineWithAnd = type.getCombineUsingAnd();

                __CLOVER_130_0.S[3151]++;List l = (List) value;
                __CLOVER_130_0.S[3152]++;int count = l.size();

                __CLOVER_130_0.S[3153]++;for (int j = 0; (((j < count) && (++__CLOVER_130_0.CT[574] != 0)) || (++__CLOVER_130_0.CF[574] == 0)); j++){
                {
                    __CLOVER_130_0.S[3154]++;String functionName = (String) l.get(j);

                    __CLOVER_130_0.S[3155]++;if ((((j > 0) && (++__CLOVER_130_0.CT[575] != 0)) || (++__CLOVER_130_0.CF[575] == 0))){
                    {

                        __CLOVER_130_0.S[3156]++;if ((((combineWithAnd) && (++__CLOVER_130_0.CT[576] != 0)) || (++__CLOVER_130_0.CF[576] == 0))){
                            __CLOVER_130_0.S[3157]++;buffer.append(" &&");}
                        else{
                            __CLOVER_130_0.S[3158]++;buffer.append(";");}
                    }}

                    __CLOVER_130_0.S[3159]++;buffer.append("\n  ");

                    __CLOVER_130_0.S[3160]++;if ((((combineWithAnd) && (++__CLOVER_130_0.CT[577] != 0)) || (++__CLOVER_130_0.CF[577] == 0))){
                    {
                        __CLOVER_130_0.S[3161]++;if ((((j == 0) && (++__CLOVER_130_0.CT[578] != 0)) || (++__CLOVER_130_0.CF[578] == 0))){
                            __CLOVER_130_0.S[3162]++;buffer.append("return ");}
                        else{
                            __CLOVER_130_0.S[3163]++;buffer.append("  ");}
                    }}

                    __CLOVER_130_0.S[3164]++;buffer.append(functionName);
                    __CLOVER_130_0.S[3165]++;buffer.append("()");
                }}

                __CLOVER_130_0.S[3166]++;buffer.append(";\n}");
            }}

            __CLOVER_130_0.S[3167]++;buffer.append("\n\n");
        }}

        __CLOVER_130_0.S[3168]++;body.addInitializationScript(buffer.toString());
    } finally { }}

    /**
     *  Simply invokes {@link #render(IMarkupWriter, IRequestCycle)}.
     *
     *  @since 1.0.2
     * 
     **/

    public void rewind(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_130_0.M[701]++;
        __CLOVER_130_0.S[3169]++;render(writer, cycle);
    } finally { }}

    /**
     *  Method invoked by the direct service.
     *
     *  @since 1.0.2
     *
     **/

    public void trigger(IRequestCycle cycle)
    {try { __CLOVER_130_0.M[702]++;
        __CLOVER_130_0.S[3170]++;Object[] parameters = cycle.getServiceParameters();

        __CLOVER_130_0.S[3171]++;cycle.rewindForm(this, (String) parameters[0]);
    } finally { }}

    /**
     *  Builds the EngineServiceLink for the form, using either the direct or
     *  action service. 
     *
     *  @since 1.0.3
     *
     **/

    private ILink getLink(IRequestCycle cycle, String actionId)
    {try { __CLOVER_130_0.M[703]++;
        __CLOVER_130_0.S[3172]++;String serviceName = null;

        __CLOVER_130_0.S[3173]++;if ((((isDirect()) && (++__CLOVER_130_0.CT[579] != 0)) || (++__CLOVER_130_0.CF[579] == 0))){
            __CLOVER_130_0.S[3174]++;serviceName = Tapestry.DIRECT_SERVICE;}
        else{
            __CLOVER_130_0.S[3175]++;serviceName = Tapestry.ACTION_SERVICE;}

        __CLOVER_130_0.S[3176]++;IEngine engine = cycle.getEngine();
        __CLOVER_130_0.S[3177]++;IEngineService service = engine.getService(serviceName);

        // A single service parameter is used to store the actionId.

        __CLOVER_130_0.S[3178]++;return service.getLink(cycle, this, new String[] { actionId });
    } finally { }}

    private void writeLinkParameters(IMarkupWriter writer, ILink link, boolean reserveOnly)
    {try { __CLOVER_130_0.M[704]++;
        __CLOVER_130_0.S[3179]++;String[] names = link.getParameterNames();
        __CLOVER_130_0.S[3180]++;int count = Tapestry.size(names);

        __CLOVER_130_0.S[3181]++;for (int i = 0; (((i < count) && (++__CLOVER_130_0.CT[580] != 0)) || (++__CLOVER_130_0.CF[580] == 0)); i++){
        {
            __CLOVER_130_0.S[3182]++;String name = names[i];

            // Reserve the name.

            __CLOVER_130_0.S[3183]++;_elementIdAllocator.allocateId(name);

            __CLOVER_130_0.S[3184]++;if ((((!reserveOnly) && (++__CLOVER_130_0.CT[581] != 0)) || (++__CLOVER_130_0.CF[581] == 0))){
                __CLOVER_130_0.S[3185]++;writeHiddenFieldsForParameter(writer, link, name);}
        }}
    } finally { }}

    /**
     *  @since 3.0
     *
     **/

    protected void writeHiddenField(IMarkupWriter writer, String name, String value)
    {try { __CLOVER_130_0.M[705]++;
		__CLOVER_130_0.S[3186]++;writeHiddenField(writer, name, null, value);
    } finally { }}

	protected void writeHiddenField(IMarkupWriter writer, String name, String id, String value)
	{try { __CLOVER_130_0.M[706]++;
		__CLOVER_130_0.S[3187]++;writer.beginEmpty("input");
		__CLOVER_130_0.S[3188]++;writer.attribute("type", "hidden");
		__CLOVER_130_0.S[3189]++;writer.attribute("name", name);

		__CLOVER_130_0.S[3190]++;if((((id != null && id.length() != 0) && (++__CLOVER_130_0.CT[582] != 0)) || (++__CLOVER_130_0.CF[582] == 0))){
			__CLOVER_130_0.S[3191]++;writer.attribute("id", id);}
		
		__CLOVER_130_0.S[3192]++;writer.attribute("value", value);
		__CLOVER_130_0.S[3193]++;writer.println();
	} finally { }}

    /**
     *  @since 2.2
     * 
     **/

    private void writeHiddenFieldsForParameter(
        IMarkupWriter writer,
        ILink link,
        String parameterName)
    {try { __CLOVER_130_0.M[707]++;
        __CLOVER_130_0.S[3194]++;String[] values = link.getParameterValues(parameterName);

        __CLOVER_130_0.S[3195]++;for (int i = 0; (((i < values.length) && (++__CLOVER_130_0.CT[583] != 0)) || (++__CLOVER_130_0.CF[583] == 0)); i++){
        {
            __CLOVER_130_0.S[3196]++;writeHiddenField(writer, parameterName, values[i]);
        }}
    } finally { }}

    /**
     *  Converts the allocateIds property into a string, a comma-separated list of ids.
     *  This is included as a hidden field in the form and is used to identify
     *  discrepencies when the form is submitted.
     * 
     *  @since 3.0
     * 
     **/

    protected String buildAllocatedIdList()
    {try { __CLOVER_130_0.M[708]++;
        __CLOVER_130_0.S[3197]++;StringBuffer buffer = new StringBuffer();
        __CLOVER_130_0.S[3198]++;int count = _allocatedIds.size();

        __CLOVER_130_0.S[3199]++;for (int i = 0; (((i < count) && (++__CLOVER_130_0.CT[584] != 0)) || (++__CLOVER_130_0.CF[584] == 0)); i++){
        {
            __CLOVER_130_0.S[3200]++;if ((((i > 0) && (++__CLOVER_130_0.CT[585] != 0)) || (++__CLOVER_130_0.CF[585] == 0))){
                __CLOVER_130_0.S[3201]++;buffer.append(',');}

            __CLOVER_130_0.S[3202]++;buffer.append(_allocatedIds.get(i));
        }}

        __CLOVER_130_0.S[3203]++;return buffer.toString();
    } finally { }}

    /**
     *  Converts a string passed as a parameter (and containing a comma
     *  separated list of ids) back into the allocateIds property.
     * 
     *  @see #buildAllocatedIdList()
     * 
     *  @since 3.0
     * 
     **/

    protected void reconstructAllocatedIds(String storedIdList)
    {try { __CLOVER_130_0.M[709]++;
        __CLOVER_130_0.S[3204]++;if ((((Tapestry.isBlank(storedIdList)) && (++__CLOVER_130_0.CT[586] != 0)) || (++__CLOVER_130_0.CF[586] == 0))){
            __CLOVER_130_0.S[3205]++;return;}

        __CLOVER_130_0.S[3206]++;StringSplitter splitter = new StringSplitter(',');

        __CLOVER_130_0.S[3207]++;String[] ids = splitter.splitToArray(storedIdList);

        __CLOVER_130_0.S[3208]++;for (int i = 0; (((i < ids.length) && (++__CLOVER_130_0.CT[587] != 0)) || (++__CLOVER_130_0.CF[587] == 0)); i++){
            __CLOVER_130_0.S[3209]++;_allocatedIds.add(ids[i]);}
    } finally { }}

    public abstract IValidationDelegate getDelegate();

    public abstract void setDelegate(IValidationDelegate delegate);

    public abstract void setDirect(boolean direct);

    public abstract IActionListener getListener();

    public abstract String getMethod();

    /**
     *  Invoked when not rendering, so it uses the stateful binding.
     *  If not bound, returns true.
     * 
     **/

    public boolean isStateful()
    {try { __CLOVER_130_0.M[710]++;
        __CLOVER_130_0.S[3210]++;IBinding statefulBinding = getStatefulBinding();

        __CLOVER_130_0.S[3211]++;if ((((statefulBinding == null) && (++__CLOVER_130_0.CT[588] != 0)) || (++__CLOVER_130_0.CF[588] == 0))){
            __CLOVER_130_0.S[3212]++;return true;}

        __CLOVER_130_0.S[3213]++;return statefulBinding.getBoolean();
    } finally { }}

    public abstract IBinding getStatefulBinding();

    protected void finishLoad()
    {try { __CLOVER_130_0.M[711]++;
        __CLOVER_130_0.S[3214]++;setDirect(true);
    } finally { }}

    public void setEncodingType(String encodingType)
    {try { __CLOVER_130_0.M[712]++;
        __CLOVER_130_0.S[3215]++;if ((((_encodingType != null && !_encodingType.equals(encodingType)) && (++__CLOVER_130_0.CT[589] != 0)) || (++__CLOVER_130_0.CF[589] == 0))){
            __CLOVER_130_0.S[3216]++;throw new ApplicationRuntimeException(
                Tapestry.format(
                    "Form.encoding-type-contention",
                    getExtendedId(),
                    _encodingType,
                    encodingType),
                this,
                null,
                null);}

        __CLOVER_130_0.S[3217]++;_encodingType = encodingType;
    } finally { }}

    /**
     *  Returns the tag of the form.
     *
     *  @since 3.0
     *
     **/

    protected String getTag()
    {try { __CLOVER_130_0.M[713]++;
        __CLOVER_130_0.S[3218]++;return "form";
    } finally { }}

    /**
     * Returns the name of the element.
     *
     *
     *  @since 3.0
     **/

    protected String getDisplayName()
    {try { __CLOVER_130_0.M[714]++;
        __CLOVER_130_0.S[3219]++;return "Form";
    } finally { }}

    /** @since 3.0 */

    public void addHiddenValue(String name, String value)
    {try { __CLOVER_130_0.M[715]++;
		__CLOVER_130_0.S[3220]++;if ((((_hiddenValues == null) && (++__CLOVER_130_0.CT[590] != 0)) || (++__CLOVER_130_0.CF[590] == 0))){
			__CLOVER_130_0.S[3221]++;_hiddenValues = new ArrayList();}

		__CLOVER_130_0.S[3222]++;_hiddenValues.add(new HiddenValue(name, value));
    } finally { }}

	/** @since 3.0 */

	public void addHiddenValue(String name, String id, String value)
	{try { __CLOVER_130_0.M[716]++;
		__CLOVER_130_0.S[3223]++;if ((((_hiddenValues == null) && (++__CLOVER_130_0.CT[591] != 0)) || (++__CLOVER_130_0.CF[591] == 0))){
			__CLOVER_130_0.S[3224]++;_hiddenValues = new ArrayList();}

		__CLOVER_130_0.S[3225]++;_hiddenValues.add(new HiddenValue(name, id, value));
	} finally { }}

    /** 
     * Writes hidden values accumulated during the render
     * (by components invoking {@link #addHiddenValue(String, String)}.
     * 
     * @since 3.0
     */

    protected void writeHiddenValues(IMarkupWriter writer)
    {try { __CLOVER_130_0.M[717]++;
        __CLOVER_130_0.S[3226]++;int count = Tapestry.size(_hiddenValues);

        __CLOVER_130_0.S[3227]++;for (int i = 0; (((i < count) && (++__CLOVER_130_0.CT[592] != 0)) || (++__CLOVER_130_0.CF[592] == 0)); i++){
        {
            __CLOVER_130_0.S[3228]++;HiddenValue hv = (HiddenValue) _hiddenValues.get(i);

            __CLOVER_130_0.S[3229]++;writeHiddenField(writer, hv._name, hv._id, hv._value);
        }}
    } finally { }}
}