/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.form;

import java.awt.Point;

import org.apache.tapestry.IActionListener;
import org.apache.tapestry.IAsset;
import org.apache.tapestry.IBinding;
import org.apache.tapestry.IForm;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.request.RequestContext;

/**
 *  Used to create an image button inside a {@link Form}.  Although it
 *  is occasionally useful to know the {@link Point} on the image that was clicked
 *  (i.e., use the image as a kind of image map, which was the original intent
 *  of the HTML element), it is more commonly used to provide a graphic
 *  image for the user to click, rather than the rather plain &lt;input type=submit&gt;.
 *
 *  [<a href="../../../../../ComponentReference/ImageSubmit.html">Component Reference</a>]
 *
 *
 *  @author Howard Lewis Ship
 **/

public abstract class ImageSubmit extends AbstractFormComponent
{public static com.cortexeb.tools.clover.d __CLOVER_136_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);

    public abstract IBinding getPointBinding();

    public abstract IBinding getSelectedBinding();

    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_136_0.M[726]++;
        __CLOVER_136_0.S[3269]++;IForm form = getForm(cycle);

        __CLOVER_136_0.S[3270]++;boolean rewinding = form.isRewinding();

        __CLOVER_136_0.S[3271]++;String nameOverride = getNameOverride();

        __CLOVER_136_0.S[3272]++;String name =
            (((nameOverride == null ) && (++__CLOVER_136_0.CT[599] != 0)) || (++__CLOVER_136_0.CF[599] == 0))? form.getElementId(this) : form.getElementId(this, nameOverride);

        __CLOVER_136_0.S[3273]++;if ((((rewinding) && (++__CLOVER_136_0.CT[600] != 0)) || (++__CLOVER_136_0.CF[600] == 0))){
        {
            // If disabled, do nothing.

            __CLOVER_136_0.S[3274]++;if ((((isDisabled()) && (++__CLOVER_136_0.CT[601] != 0)) || (++__CLOVER_136_0.CF[601] == 0))){
                __CLOVER_136_0.S[3275]++;return;}

            __CLOVER_136_0.S[3276]++;RequestContext context = cycle.getRequestContext();

            // Image clicks get submitted as two request parameters: 
            // foo.x and foo.y

            __CLOVER_136_0.S[3277]++;String parameterName = name + ".x";

            __CLOVER_136_0.S[3278]++;String value = context.getParameter(parameterName);

            __CLOVER_136_0.S[3279]++;if ((((value == null) && (++__CLOVER_136_0.CT[602] != 0)) || (++__CLOVER_136_0.CF[602] == 0))){
                __CLOVER_136_0.S[3280]++;return;}

            // The point parameter is not really used, unless the
            // ImageButton is used for its original purpose (as a kind
            // of image map).  In modern usage, we only care about
            // whether the user clicked on the image (and thus submitted
            // the form), not where in the image the user actually clicked.

            __CLOVER_136_0.S[3281]++;IBinding pointBinding = getPointBinding();

            __CLOVER_136_0.S[3282]++;if ((((pointBinding != null) && (++__CLOVER_136_0.CT[603] != 0)) || (++__CLOVER_136_0.CF[603] == 0))){
            {
                __CLOVER_136_0.S[3283]++;int x = Integer.parseInt(value);

                __CLOVER_136_0.S[3284]++;parameterName = name + ".y";
                __CLOVER_136_0.S[3285]++;value = context.getParameter(parameterName);

                __CLOVER_136_0.S[3286]++;int y = Integer.parseInt(value);

                __CLOVER_136_0.S[3287]++;pointBinding.setObject(new Point(x, y));
            }}

            // Notify the application, by setting the select parameter
            // to the tag parameter.

            __CLOVER_136_0.S[3288]++;IBinding selectedBinding = getSelectedBinding();

            __CLOVER_136_0.S[3289]++;if ((((selectedBinding != null) && (++__CLOVER_136_0.CT[604] != 0)) || (++__CLOVER_136_0.CF[604] == 0))){
                __CLOVER_136_0.S[3290]++;selectedBinding.setObject(getTag());}

            __CLOVER_136_0.S[3291]++;IActionListener listener = getListener();

            __CLOVER_136_0.S[3292]++;if ((((listener != null) && (++__CLOVER_136_0.CT[605] != 0)) || (++__CLOVER_136_0.CF[605] == 0))){
                __CLOVER_136_0.S[3293]++;listener.actionTriggered(this, cycle);}

            __CLOVER_136_0.S[3294]++;return;
        }}

        // Not rewinding, do the real render

        __CLOVER_136_0.S[3295]++;boolean disabled = isDisabled();
        __CLOVER_136_0.S[3296]++;IAsset disabledImage = getDisabledImage();

        __CLOVER_136_0.S[3297]++;IAsset finalImage = ((((disabled && disabledImage != null) ) && (++__CLOVER_136_0.CT[606] != 0)) || (++__CLOVER_136_0.CF[606] == 0))? disabledImage : getImage();

        __CLOVER_136_0.S[3298]++;String imageURL = finalImage.buildURL(cycle);

        __CLOVER_136_0.S[3299]++;writer.beginEmpty("input");
        __CLOVER_136_0.S[3300]++;writer.attribute("type", "image");
        __CLOVER_136_0.S[3301]++;writer.attribute("name", name);

        __CLOVER_136_0.S[3302]++;if ((((disabled) && (++__CLOVER_136_0.CT[607] != 0)) || (++__CLOVER_136_0.CF[607] == 0))){
            __CLOVER_136_0.S[3303]++;writer.attribute("disabled", "disabled");}

        // NN4 places a border unless you tell it otherwise.
        // IE ignores the border attribute and never shows a border.

        __CLOVER_136_0.S[3304]++;writer.attribute("border", 0);

        __CLOVER_136_0.S[3305]++;writer.attribute("src", imageURL);

        __CLOVER_136_0.S[3306]++;renderInformalParameters(writer, cycle);

        __CLOVER_136_0.S[3307]++;writer.closeTag();
    } finally { }}

    public abstract boolean isDisabled();

    public abstract IAsset getDisabledImage();

    public abstract IAsset getImage();

    public abstract IActionListener getListener();

    public abstract Object getTag();

    public abstract String getNameOverride();

    protected void prepareForRender(IRequestCycle cycle)
    {try { __CLOVER_136_0.M[727]++;
        __CLOVER_136_0.S[3308]++;super.prepareForRender(cycle);

        __CLOVER_136_0.S[3309]++;if ((((getImage() == null) && (++__CLOVER_136_0.CT[608] != 0)) || (++__CLOVER_136_0.CF[608] == 0))){
            __CLOVER_136_0.S[3310]++;throw Tapestry.createRequiredParameterException(this, "image");}
    } finally { }}

}