/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.form;

import org.apache.tapestry.IForm;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;

/**
 *  A component used to render a drop-down list of options that
 *  the user may select.
 * 
 *  [<a href="../../../../../ComponentReference/PropertySelection.html">Component Reference</a>]
 *
 *  <p>Earlier versions of PropertySelection (through release 2.2)
 *  were more flexible, they included a <b>renderer</b> property
 *  that controlled how the selection was rendered.  Ultimately,
 *  this proved of little value and this portion of
 *  functionality was deprecated in 2.3 and will be removed in 2.3.
 * 
 *  <p>Typically, the values available to be selected
 *  are defined using an {@link org.apache.commons.lang.enum.Enum}.
 *  A PropertySelection is dependent on
 *  an {@link IPropertySelectionModel} to provide the list of possible values.
 *
 *  <p>Often, this is used to select a particular 
 *  {@link org.apache.commons.lang.enum.Enum} to assign to a property; the
 *  {@link EnumPropertySelectionModel} class simplifies this.
 *
 *
 *  @author Howard Lewis Ship
 *
 **/

public abstract class PropertySelection extends AbstractFormComponent
{public static com.cortexeb.tools.clover.d __CLOVER_141_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /**
     *  A shared instance of {@link SelectPropertySelectionRenderer}.
     *
     * 
     **/

    public static final IPropertySelectionRenderer DEFAULT_SELECT_RENDERER =
        new SelectPropertySelectionRenderer();

    /**
     *  A shared instance of {@link RadioPropertySelectionRenderer}.
     *
     * 
     **/

    public static final IPropertySelectionRenderer DEFAULT_RADIO_RENDERER =
        new RadioPropertySelectionRenderer();

    /**
     *  Renders the component, much of which is the responsiblity
     *  of the {@link IPropertySelectionRenderer renderer}.  The possible options,
     *  thier labels, and the values to be encoded in the form are provided
     *  by the {@link IPropertySelectionModel model}.
     *
     **/

    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_141_0.M[748]++;
        __CLOVER_141_0.S[3458]++;IForm form = getForm(cycle);

        __CLOVER_141_0.S[3459]++;boolean rewinding = form.isRewinding();

        __CLOVER_141_0.S[3460]++;String name = form.getElementId(this);

        __CLOVER_141_0.S[3461]++;if ((((rewinding) && (++__CLOVER_141_0.CT[642] != 0)) || (++__CLOVER_141_0.CF[642] == 0))){
        {
            // If disabled, ignore anything that comes up from the client.

            __CLOVER_141_0.S[3462]++;if ((((isDisabled()) && (++__CLOVER_141_0.CT[643] != 0)) || (++__CLOVER_141_0.CF[643] == 0))){
                __CLOVER_141_0.S[3463]++;return;}

            __CLOVER_141_0.S[3464]++;String optionValue = cycle.getRequestContext().getParameter(name);

            __CLOVER_141_0.S[3465]++;Object value = ((((optionValue == null) ) && (++__CLOVER_141_0.CT[644] != 0)) || (++__CLOVER_141_0.CF[644] == 0))? null : getModel().translateValue(optionValue);

            __CLOVER_141_0.S[3466]++;setValue(value);

            __CLOVER_141_0.S[3467]++;return;
        }}

        __CLOVER_141_0.S[3468]++;IPropertySelectionRenderer renderer = getRenderer();

        __CLOVER_141_0.S[3469]++;if ((((renderer != null) && (++__CLOVER_141_0.CT[645] != 0)) || (++__CLOVER_141_0.CF[645] == 0))){
        {
            __CLOVER_141_0.S[3470]++;renderWithRenderer(writer, cycle, renderer);
            __CLOVER_141_0.S[3471]++;return;
        }}

        __CLOVER_141_0.S[3472]++;writer.begin("select");
        __CLOVER_141_0.S[3473]++;writer.attribute("name", name);

        __CLOVER_141_0.S[3474]++;if ((((isDisabled()) && (++__CLOVER_141_0.CT[646] != 0)) || (++__CLOVER_141_0.CF[646] == 0))){
            __CLOVER_141_0.S[3475]++;writer.attribute("disabled", "disabled");}

        __CLOVER_141_0.S[3476]++;if ((((getSubmitOnChange()) && (++__CLOVER_141_0.CT[647] != 0)) || (++__CLOVER_141_0.CF[647] == 0))){
            __CLOVER_141_0.S[3477]++;writer.attribute("onchange", "javascript:this.form.submit();");}

        // Apply informal attributes.

        __CLOVER_141_0.S[3478]++;renderInformalParameters(writer, cycle);

        __CLOVER_141_0.S[3479]++;writer.println();

        __CLOVER_141_0.S[3480]++;IPropertySelectionModel model = getModel();

        __CLOVER_141_0.S[3481]++;if ((((model == null) && (++__CLOVER_141_0.CT[648] != 0)) || (++__CLOVER_141_0.CF[648] == 0))){
            __CLOVER_141_0.S[3482]++;throw Tapestry.createRequiredParameterException(this, "model");}

        __CLOVER_141_0.S[3483]++;int count = model.getOptionCount();
        __CLOVER_141_0.S[3484]++;boolean foundSelected = false;
        __CLOVER_141_0.S[3485]++;boolean selected = false;
        __CLOVER_141_0.S[3486]++;Object value = getValue();

        __CLOVER_141_0.S[3487]++;for (int i = 0; (((i < count) && (++__CLOVER_141_0.CT[649] != 0)) || (++__CLOVER_141_0.CF[649] == 0)); i++){
        {
            __CLOVER_141_0.S[3488]++;Object option = model.getOption(i);

            __CLOVER_141_0.S[3489]++;if ((((!foundSelected) && (++__CLOVER_141_0.CT[650] != 0)) || (++__CLOVER_141_0.CF[650] == 0))){
            {
                __CLOVER_141_0.S[3490]++;selected = isEqual(option, value);
                __CLOVER_141_0.S[3491]++;if ((((selected) && (++__CLOVER_141_0.CT[651] != 0)) || (++__CLOVER_141_0.CF[651] == 0))){
                    __CLOVER_141_0.S[3492]++;foundSelected = true;}
            }}

            __CLOVER_141_0.S[3493]++;writer.begin("option");
            __CLOVER_141_0.S[3494]++;writer.attribute("value", model.getValue(i));

            __CLOVER_141_0.S[3495]++;if ((((selected) && (++__CLOVER_141_0.CT[652] != 0)) || (++__CLOVER_141_0.CF[652] == 0))){
                __CLOVER_141_0.S[3496]++;writer.attribute("selected", "selected");}

            __CLOVER_141_0.S[3497]++;writer.print(model.getLabel(i));

            __CLOVER_141_0.S[3498]++;writer.end();

            __CLOVER_141_0.S[3499]++;writer.println();

            __CLOVER_141_0.S[3500]++;selected = false;
        }}

        __CLOVER_141_0.S[3501]++;writer.end(); // <select>

    } finally { }}

    /**
     *  Renders the property selection using a {@link IPropertySelectionRenderer}.
     *  Support for this will be removed in 2.3.
     * 
     **/

    private void renderWithRenderer(
        IMarkupWriter writer,
        IRequestCycle cycle,
        IPropertySelectionRenderer renderer)
    {try { __CLOVER_141_0.M[749]++;
        __CLOVER_141_0.S[3502]++;renderer.beginRender(this, writer, cycle);

        __CLOVER_141_0.S[3503]++;IPropertySelectionModel model = getModel();

        __CLOVER_141_0.S[3504]++;int count = model.getOptionCount();

        __CLOVER_141_0.S[3505]++;boolean foundSelected = false;
        __CLOVER_141_0.S[3506]++;boolean selected = false;

        __CLOVER_141_0.S[3507]++;Object value = getValue();

        __CLOVER_141_0.S[3508]++;for (int i = 0; (((i < count) && (++__CLOVER_141_0.CT[653] != 0)) || (++__CLOVER_141_0.CF[653] == 0)); i++){
        {
            __CLOVER_141_0.S[3509]++;Object option = model.getOption(i);

            __CLOVER_141_0.S[3510]++;if ((((!foundSelected) && (++__CLOVER_141_0.CT[654] != 0)) || (++__CLOVER_141_0.CF[654] == 0))){
            {
                __CLOVER_141_0.S[3511]++;selected = isEqual(option, value);
                __CLOVER_141_0.S[3512]++;if ((((selected) && (++__CLOVER_141_0.CT[655] != 0)) || (++__CLOVER_141_0.CF[655] == 0))){
                    __CLOVER_141_0.S[3513]++;foundSelected = true;}
            }}

            __CLOVER_141_0.S[3514]++;renderer.renderOption(this, writer, cycle, model, option, i, selected);

            __CLOVER_141_0.S[3515]++;selected = false;
        }}

        // A PropertySelection doesn't allow a body, so no need to worry about
        // wrapped components.

        __CLOVER_141_0.S[3516]++;renderer.endRender(this, writer, cycle);
    } finally { }}

    private boolean isEqual(Object left, Object right)
    {try { __CLOVER_141_0.M[750]++;
        // Both null, or same object, then are equal

        __CLOVER_141_0.S[3517]++;if ((((left == right) && (++__CLOVER_141_0.CT[656] != 0)) || (++__CLOVER_141_0.CF[656] == 0))){
            __CLOVER_141_0.S[3518]++;return true;}

        // If one is null, the other isn't, then not equal.

        __CLOVER_141_0.S[3519]++;if ((((left == null || right == null) && (++__CLOVER_141_0.CT[657] != 0)) || (++__CLOVER_141_0.CF[657] == 0))){
            __CLOVER_141_0.S[3520]++;return false;}

        // Both non-null; use standard comparison.

        __CLOVER_141_0.S[3521]++;return left.equals(right);
    } finally { }}

    public abstract IPropertySelectionModel getModel();

    public abstract IPropertySelectionRenderer getRenderer();

    /** @since 2.2 **/

    public abstract boolean getSubmitOnChange();

    /** @since 2.2 **/

    public abstract Object getValue();

    /** @since 2.2 **/

    public abstract void setValue(Object value);

    /**
     *  Returns true if this PropertySelection's disabled parameter yields true.
     *  The corresponding HTML control(s) should be disabled.
     **/

    public abstract boolean isDisabled();

}