/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.form;

import java.util.HashSet;
import java.util.Set;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.IForm;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.request.RequestContext;

/**
 *  Implements a component that manages an HTML &lt;select&gt; form element.
 *  The most common situation, using a &lt;select&gt; to set a specific
 *  property of some object, is best handled using a {@link PropertySelection} component.
 *
 *  [<a href="../../../../../ComponentReference/Select.html">Component Reference</a>]
 * 
 *  <p>Otherwise, this component is very similar to {@link RadioGroup}.
 *
 *
 *  @author Howard Lewis Ship
 * 
 **/

public abstract class Select extends AbstractFormComponent
{public static com.cortexeb.tools.clover.d __CLOVER_145_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private boolean _rewinding;
    private boolean _rendering;

    private Set _selections;
    private int _nextOptionId;

    /**
     *  Used by the <code>Select</code> to record itself as a
     *  {@link IRequestCycle} attribute, so that the
     *  {@link Option} components it wraps can have access to it.
     *
     **/

    private final static String ATTRIBUTE_NAME = "org.apache.tapestry.active.Select";

    public static Select get(IRequestCycle cycle)
    {try { __CLOVER_145_0.M[762]++;
        __CLOVER_145_0.S[3598]++;return (Select) cycle.getAttribute(ATTRIBUTE_NAME);
    } finally { }}

    public abstract boolean isDisabled();

    public abstract boolean isMultiple();

    public boolean isRewinding()
    {try { __CLOVER_145_0.M[763]++;
        __CLOVER_145_0.S[3599]++;if ((((!_rendering) && (++__CLOVER_145_0.CT[674] != 0)) || (++__CLOVER_145_0.CF[674] == 0))){
            __CLOVER_145_0.S[3600]++;throw Tapestry.createRenderOnlyPropertyException(this, "rewinding");}

        __CLOVER_145_0.S[3601]++;return _rewinding;
    } finally { }}

    public String getNextOptionId()
    {try { __CLOVER_145_0.M[764]++;
        __CLOVER_145_0.S[3602]++;if ((((!_rendering) && (++__CLOVER_145_0.CT[675] != 0)) || (++__CLOVER_145_0.CF[675] == 0))){
            __CLOVER_145_0.S[3603]++;throw Tapestry.createRenderOnlyPropertyException(this, "nextOptionId");}

        // Return it as a hex value.

        __CLOVER_145_0.S[3604]++;return Integer.toString(_nextOptionId++);
    } finally { }}

    public boolean isSelected(String value)
    {try { __CLOVER_145_0.M[765]++;
        __CLOVER_145_0.S[3605]++;if ((((_selections == null) && (++__CLOVER_145_0.CT[676] != 0)) || (++__CLOVER_145_0.CF[676] == 0))){
            __CLOVER_145_0.S[3606]++;return false;}

        __CLOVER_145_0.S[3607]++;return _selections.contains(value);
    } finally { }}

    /**
     *  Renders the &lt;option&gt; element, or responds when the form containing the element
     *  is submitted (by checking {@link IForm#isRewinding()}.
     **/

    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_145_0.M[766]++;
        __CLOVER_145_0.S[3608]++;IForm form = getForm(cycle);

        __CLOVER_145_0.S[3609]++;if ((((cycle.getAttribute(ATTRIBUTE_NAME) != null) && (++__CLOVER_145_0.CT[677] != 0)) || (++__CLOVER_145_0.CF[677] == 0))){
            __CLOVER_145_0.S[3610]++;throw new ApplicationRuntimeException(
                Tapestry.getMessage("Select.may-not-nest"),
                this,
                null,
                null);}

        // It isn't enough to know whether the cycle in general is rewinding, need to know
        // specifically if the form which contains this component is rewinding.

        __CLOVER_145_0.S[3611]++;_rewinding = form.isRewinding();

        // Used whether rewinding or not.

        __CLOVER_145_0.S[3612]++;String name = form.getElementId(this);

        __CLOVER_145_0.S[3613]++;cycle.setAttribute(ATTRIBUTE_NAME, this);

        __CLOVER_145_0.S[3614]++;if ((((_rewinding) && (++__CLOVER_145_0.CT[678] != 0)) || (++__CLOVER_145_0.CF[678] == 0))){
        {
            __CLOVER_145_0.S[3615]++;_selections = buildSelections(cycle, name);
        }}
        else{
        {
            __CLOVER_145_0.S[3616]++;writer.begin("select");

            __CLOVER_145_0.S[3617]++;writer.attribute("name", name);

            __CLOVER_145_0.S[3618]++;if ((((isMultiple()) && (++__CLOVER_145_0.CT[679] != 0)) || (++__CLOVER_145_0.CF[679] == 0))){
                __CLOVER_145_0.S[3619]++;writer.attribute("multiple", "multiple");}

            __CLOVER_145_0.S[3620]++;if ((((isDisabled()) && (++__CLOVER_145_0.CT[680] != 0)) || (++__CLOVER_145_0.CF[680] == 0))){
                __CLOVER_145_0.S[3621]++;writer.attribute("disabled", "disabled");}

            __CLOVER_145_0.S[3622]++;renderInformalParameters(writer, cycle);
        }}

        __CLOVER_145_0.S[3623]++;_rendering = true;
        __CLOVER_145_0.S[3624]++;_nextOptionId = 0;

        __CLOVER_145_0.S[3625]++;renderBody(writer, cycle);

        __CLOVER_145_0.S[3626]++;if ((((!_rewinding) && (++__CLOVER_145_0.CT[681] != 0)) || (++__CLOVER_145_0.CF[681] == 0))){
        {
            __CLOVER_145_0.S[3627]++;writer.end();
        }}

        __CLOVER_145_0.S[3628]++;cycle.removeAttribute(ATTRIBUTE_NAME);

    } finally { }}

    protected void cleanupAfterRender(IRequestCycle cycle)
    {try { __CLOVER_145_0.M[767]++;
        __CLOVER_145_0.S[3629]++;_rendering = false;
        __CLOVER_145_0.S[3630]++;_selections = null;

        __CLOVER_145_0.S[3631]++;super.cleanupAfterRender(cycle);
    } finally { }}

    /**
     *  Cut-and-paste with {@link RadioGroup}!
     *
     **/

    private Set buildSelections(IRequestCycle cycle, String parameterName)
    {try { __CLOVER_145_0.M[768]++;
        __CLOVER_145_0.S[3632]++;RequestContext context = cycle.getRequestContext();

        __CLOVER_145_0.S[3633]++;String[] parameters = context.getParameters(parameterName);

        __CLOVER_145_0.S[3634]++;if ((((parameters == null) && (++__CLOVER_145_0.CT[682] != 0)) || (++__CLOVER_145_0.CF[682] == 0))){
            __CLOVER_145_0.S[3635]++;return null;}

        __CLOVER_145_0.S[3636]++;int length = parameters.length;

        __CLOVER_145_0.S[3637]++;int size = ((((parameters.length > 30) ) && (++__CLOVER_145_0.CT[683] != 0)) || (++__CLOVER_145_0.CF[683] == 0))? 101 : 7;

        __CLOVER_145_0.S[3638]++;Set result = new HashSet(size);

        __CLOVER_145_0.S[3639]++;for (int i = 0; (((i < length) && (++__CLOVER_145_0.CT[684] != 0)) || (++__CLOVER_145_0.CF[684] == 0)); i++){
            __CLOVER_145_0.S[3640]++;result.add(parameters[i]);}

        __CLOVER_145_0.S[3641]++;return result;
    } finally { }}
}