/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.html;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.hivemind.util.ClasspathResource;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.Resource;
import org.apache.tapestry.AbstractComponent;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.IScriptProcessor;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.asset.PrivateAsset;
import org.apache.tapestry.util.IdAllocator;

/**
 *  The body of a Tapestry page.  This is used since it allows components on the
 *  page access to an initialization script (that is written the start, just inside
 *  the &lt;body&gt; tag).  This is currently used by {@link Rollover} and {@link Script}
 *  components.
 * 
 *  [<a href="../../../../../ComponentReference/Body.html">Component Reference</a>]
 * 
 *  @author Howard Lewis Ship
 * 
 **/

public abstract class Body extends AbstractComponent implements IScriptProcessor
{public static com.cortexeb.tools.clover.d __CLOVER_153_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    // Lines that belong inside the onLoad event handler for the <body> tag.
    private StringBuffer _initializationScript;

    // The writer initially passed to render() ... wrapped elements render
    // into a nested response writer.

    private IMarkupWriter _outerWriter;

    // Any other scripting desired

    private StringBuffer _bodyScript;

    // Contains text lines related to image initializations

    private StringBuffer _imageInitializations;

    /**
     *  Map of URLs to Strings (preloaded image references).
     *
     **/

    private Map _imageMap;

    /**
     *  List of included scripts.  Values are Strings.
     *
     *  @since 1.0.5
     *
     **/

    private List _externalScripts;

    private IdAllocator _idAllocator;

    private static final String ATTRIBUTE_NAME = "org.apache.tapestry.active.Body";

    /**
     *  Tracks a particular preloaded image.
     *
     **/

    /**
     *  Adds to the script an initialization for the named variable as
     *  an Image(), to the given URL.
     *
     *  <p>Returns a reference, a string that can be used to represent
     *  the preloaded image in a JavaScript function.
     *
     *  @since 1.0.2
     **/

    public String getPreloadedImageReference(String URL)
    {try { __CLOVER_153_0.M[784]++;
        __CLOVER_153_0.S[3723]++;if ((((_imageMap == null) && (++__CLOVER_153_0.CT[702] != 0)) || (++__CLOVER_153_0.CF[702] == 0))){
            __CLOVER_153_0.S[3724]++;_imageMap = new HashMap();}

        __CLOVER_153_0.S[3725]++;String reference = (String) _imageMap.get(URL);

        __CLOVER_153_0.S[3726]++;if ((((reference == null) && (++__CLOVER_153_0.CT[703] != 0)) || (++__CLOVER_153_0.CF[703] == 0))){
        {
            __CLOVER_153_0.S[3727]++;int count = _imageMap.size();
            __CLOVER_153_0.S[3728]++;String varName = "tapestry_preload[" + count + "]";
            __CLOVER_153_0.S[3729]++;reference = varName + ".src";

            __CLOVER_153_0.S[3730]++;if ((((_imageInitializations == null) && (++__CLOVER_153_0.CT[704] != 0)) || (++__CLOVER_153_0.CF[704] == 0))){
                __CLOVER_153_0.S[3731]++;_imageInitializations = new StringBuffer();}

            __CLOVER_153_0.S[3732]++;_imageInitializations.append("  ");
            __CLOVER_153_0.S[3733]++;_imageInitializations.append(varName);
            __CLOVER_153_0.S[3734]++;_imageInitializations.append(" = new Image();\n");
            __CLOVER_153_0.S[3735]++;_imageInitializations.append("  ");
            __CLOVER_153_0.S[3736]++;_imageInitializations.append(reference);
            __CLOVER_153_0.S[3737]++;_imageInitializations.append(" = \"");
            __CLOVER_153_0.S[3738]++;_imageInitializations.append(URL);
            __CLOVER_153_0.S[3739]++;_imageInitializations.append("\";\n");

            __CLOVER_153_0.S[3740]++;_imageMap.put(URL, reference);
        }}

        __CLOVER_153_0.S[3741]++;return reference;
    } finally { }}

    /**
     *  Adds other initialization, in the form of additional JavaScript
     *  code to execute from the &lt;body&gt;'s <code>onLoad</code> event
     *  handler.  The caller is responsible for adding a semicolon (statement
     *  terminator).  This method will add a newline after the script.
     *
     **/

    public void addInitializationScript(String script)
    {try { __CLOVER_153_0.M[785]++;
        __CLOVER_153_0.S[3742]++;if ((((_initializationScript == null) && (++__CLOVER_153_0.CT[705] != 0)) || (++__CLOVER_153_0.CF[705] == 0))){
            __CLOVER_153_0.S[3743]++;_initializationScript = new StringBuffer(script.length() + 1);}

        __CLOVER_153_0.S[3744]++;_initializationScript.append(script);
        __CLOVER_153_0.S[3745]++;_initializationScript.append('\n');

    } finally { }}

    /**
     *  Adds additional scripting code to the page.  This code
     *  will be added to a large block of scripting code at the
     *  top of the page (i.e., the before the &lt;body&gt; tag).
     *
     *  <p>This is typically used to add some form of JavaScript
     *  event handler to a page.  For example, the
     *  {@link Rollover} component makes use of this.
     *
     *  <p>Another way this is invoked is by using the
     *  {@link Script} component.
     *
     *  <p>The string will be added, as-is, within
     *  the &lt;script&gt; block generated by this <code>Body</code> component.
     *  The script should <em>not</em> contain HTML comments, those will
     *  be supplied by this Body component.
     *
     *  <p>A frequent use is to add an initialization function using
     *  this method, then cause it to be executed using
     *  {@link #addInitializationScript(String)}.
     *
     **/

    public void addBodyScript(String script)
    {try { __CLOVER_153_0.M[786]++;
        __CLOVER_153_0.S[3746]++;if ((((_bodyScript == null) && (++__CLOVER_153_0.CT[706] != 0)) || (++__CLOVER_153_0.CF[706] == 0))){
            __CLOVER_153_0.S[3747]++;_bodyScript = new StringBuffer(script.length());}

        __CLOVER_153_0.S[3748]++;_bodyScript.append(script);
    } finally { }}

    /**
     *  Used to include a script from an outside URL (the scriptLocation
     *  is a URL, probably obtained from an asset.  This adds
     *  an &lt;script src="..."&gt; tag before the main
     *  &lt;script&gt; tag.  The Body component ensures
     *  that each URL is included only once.
     *
     *  @since 1.0.5
     *
     **/

    public void addExternalScript(Resource scriptLocation)
    {try { __CLOVER_153_0.M[787]++;
        __CLOVER_153_0.S[3749]++;if ((((_externalScripts == null) && (++__CLOVER_153_0.CT[707] != 0)) || (++__CLOVER_153_0.CF[707] == 0))){
            __CLOVER_153_0.S[3750]++;_externalScripts = new ArrayList();}

        __CLOVER_153_0.S[3751]++;if ((((_externalScripts.contains(scriptLocation)) && (++__CLOVER_153_0.CT[708] != 0)) || (++__CLOVER_153_0.CF[708] == 0))){
            __CLOVER_153_0.S[3752]++;return;}

        // Alas, this won't give a good ILocation for the actual problem.

        __CLOVER_153_0.S[3753]++;if ((((!(scriptLocation instanceof ClasspathResource)) && (++__CLOVER_153_0.CT[709] != 0)) || (++__CLOVER_153_0.CF[709] == 0))){
            __CLOVER_153_0.S[3754]++;throw new ApplicationRuntimeException(
                Tapestry.format("Body.include-classpath-script-only", scriptLocation),
                this,
                null,
                null);}

        // Record the URL so we don't include it twice.

        __CLOVER_153_0.S[3755]++;_externalScripts.add(scriptLocation);
    } finally { }}

    /**
     * Writes &lt;script&gt; elements for all the external scripts.
     */
    private void writeExternalScripts(IMarkupWriter writer)
    {try { __CLOVER_153_0.M[788]++;
        __CLOVER_153_0.S[3756]++;int count = Tapestry.size(_externalScripts);
        __CLOVER_153_0.S[3757]++;for (int i = 0; (((i < count) && (++__CLOVER_153_0.CT[710] != 0)) || (++__CLOVER_153_0.CF[710] == 0)); i++){
        {
            __CLOVER_153_0.S[3758]++;ClasspathResource scriptLocation =
                (ClasspathResource) _externalScripts.get(i);

            // This is still very awkward!  Should move the code inside PrivateAsset somewhere
            // else, so that an asset does not have to be created to to build the URL.
            __CLOVER_153_0.S[3759]++;PrivateAsset asset = new PrivateAsset(scriptLocation, null);
            __CLOVER_153_0.S[3760]++;String url = asset.buildURL(getPage().getRequestCycle());

            // Note: important to use begin(), not beginEmpty(), because browser don't
            // interpret <script .../> properly.

            __CLOVER_153_0.S[3761]++;writer.begin("script");
            __CLOVER_153_0.S[3762]++;writer.attribute("language", "JavaScript");
            __CLOVER_153_0.S[3763]++;writer.attribute("type", "text/javascript");
            __CLOVER_153_0.S[3764]++;writer.attribute("src", url);
            __CLOVER_153_0.S[3765]++;writer.end();
            __CLOVER_153_0.S[3766]++;writer.println();
        }}

    } finally { }}

    /**
     *  Retrieves the <code>Body</code> that was stored into the
     *  request cycle.  This allows components wrapped by the
     *  <code>Body</code> to locate it and access the services it
     *  provides.
     *
     **/

    public static Body get(IRequestCycle cycle)
    {try { __CLOVER_153_0.M[789]++;
        __CLOVER_153_0.S[3767]++;return (Body) cycle.getAttribute(ATTRIBUTE_NAME);
    } finally { }}

    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_153_0.M[790]++;
        __CLOVER_153_0.S[3768]++;if ((((cycle.getAttribute(ATTRIBUTE_NAME) != null) && (++__CLOVER_153_0.CT[711] != 0)) || (++__CLOVER_153_0.CF[711] == 0))){
            __CLOVER_153_0.S[3769]++;throw new ApplicationRuntimeException(
                Tapestry.getMessage("Body.may-not-nest"),
                this,
                null,
                null);}

        __CLOVER_153_0.S[3770]++;cycle.setAttribute(ATTRIBUTE_NAME, this);

        __CLOVER_153_0.S[3771]++;_outerWriter = writer;

        __CLOVER_153_0.S[3772]++;IMarkupWriter nested = writer.getNestedWriter();

        __CLOVER_153_0.S[3773]++;renderBody(nested, cycle);

        // Start the body tag.
        __CLOVER_153_0.S[3774]++;writer.println();
        __CLOVER_153_0.S[3775]++;writer.begin(getElement());
        __CLOVER_153_0.S[3776]++;renderInformalParameters(writer, cycle);

        __CLOVER_153_0.S[3777]++;writer.println();

        // Write the page's scripting.  This is included scripts
        // and dynamic JavaScript, including initialization.

        __CLOVER_153_0.S[3778]++;writeScript();

        // Close the nested writer, which dumps its buffered content
        // into its parent.

        __CLOVER_153_0.S[3779]++;nested.close();

        __CLOVER_153_0.S[3780]++;writer.end(); // <body>

    } finally { }}

    protected void cleanupAfterRender(IRequestCycle cycle)
    {try { __CLOVER_153_0.M[791]++;
        __CLOVER_153_0.S[3781]++;super.cleanupAfterRender(cycle);

        __CLOVER_153_0.S[3782]++;if ((((_idAllocator != null) && (++__CLOVER_153_0.CT[712] != 0)) || (++__CLOVER_153_0.CF[712] == 0))){
            __CLOVER_153_0.S[3783]++;_idAllocator.clear();}

        __CLOVER_153_0.S[3784]++;if ((((_imageMap != null) && (++__CLOVER_153_0.CT[713] != 0)) || (++__CLOVER_153_0.CF[713] == 0))){
            __CLOVER_153_0.S[3785]++;_imageMap.clear();}

        __CLOVER_153_0.S[3786]++;if ((((_externalScripts != null) && (++__CLOVER_153_0.CT[714] != 0)) || (++__CLOVER_153_0.CF[714] == 0))){
            __CLOVER_153_0.S[3787]++;_externalScripts.clear();}

        __CLOVER_153_0.S[3788]++;if ((((_initializationScript != null) && (++__CLOVER_153_0.CT[715] != 0)) || (++__CLOVER_153_0.CF[715] == 0))){
            __CLOVER_153_0.S[3789]++;_initializationScript.setLength(0);}

        __CLOVER_153_0.S[3790]++;if ((((_imageInitializations != null) && (++__CLOVER_153_0.CT[716] != 0)) || (++__CLOVER_153_0.CF[716] == 0))){
            __CLOVER_153_0.S[3791]++;_imageInitializations.setLength(0);}

        __CLOVER_153_0.S[3792]++;if ((((_bodyScript != null) && (++__CLOVER_153_0.CT[717] != 0)) || (++__CLOVER_153_0.CF[717] == 0))){
            __CLOVER_153_0.S[3793]++;_bodyScript.setLength(0);}

        __CLOVER_153_0.S[3794]++;_outerWriter = null;
        __CLOVER_153_0.S[3795]++;_outerWriter = null;
    } finally { }}

    /**
     *  Writes a single large JavaScript block containing:
     *  <ul>
     *  <li>Any image initializations
     *  <li>Any scripting
     *  <li>Any initializations
     *  </ul>
     *
     *  <p>The script is written into a nested markup writer.
     *
     *  <p>If there are any other initializations 
     *  (see {@link #addOtherInitialization(String)}),
     *  then a function to execute them is created.
     **/

    private void writeScript()
    {try { __CLOVER_153_0.M[792]++;
        __CLOVER_153_0.S[3796]++;if ((((!Tapestry.isEmpty(_externalScripts)) && (++__CLOVER_153_0.CT[718] != 0)) || (++__CLOVER_153_0.CF[718] == 0))){
            __CLOVER_153_0.S[3797]++;writeExternalScripts(_outerWriter);}

        __CLOVER_153_0.S[3798]++;if ((((!(any(_initializationScript) || any(_bodyScript) || any(_imageInitializations))) && (++__CLOVER_153_0.CT[719] != 0)) || (++__CLOVER_153_0.CF[719] == 0))){
            __CLOVER_153_0.S[3799]++;return;}

        __CLOVER_153_0.S[3800]++;_outerWriter.begin("script");
        __CLOVER_153_0.S[3801]++;_outerWriter.attribute("language", "JavaScript");
        __CLOVER_153_0.S[3802]++;_outerWriter.printRaw("<!--");

        __CLOVER_153_0.S[3803]++;if ((((any(_imageInitializations)) && (++__CLOVER_153_0.CT[720] != 0)) || (++__CLOVER_153_0.CF[720] == 0))){
        {
            __CLOVER_153_0.S[3804]++;_outerWriter.printRaw("\n\nvar tapestry_preload = new Array();\n");
            __CLOVER_153_0.S[3805]++;_outerWriter.printRaw("if (document.images)\n");
            __CLOVER_153_0.S[3806]++;_outerWriter.printRaw("{\n");
            __CLOVER_153_0.S[3807]++;_outerWriter.printRaw(_imageInitializations.toString());
            __CLOVER_153_0.S[3808]++;_outerWriter.printRaw("}\n");
        }}

        __CLOVER_153_0.S[3809]++;if ((((any(_bodyScript)) && (++__CLOVER_153_0.CT[721] != 0)) || (++__CLOVER_153_0.CF[721] == 0))){
        {
            __CLOVER_153_0.S[3810]++;_outerWriter.printRaw("\n\n");
            __CLOVER_153_0.S[3811]++;_outerWriter.printRaw(_bodyScript.toString());
        }}

        __CLOVER_153_0.S[3812]++;if ((((any(_initializationScript)) && (++__CLOVER_153_0.CT[722] != 0)) || (++__CLOVER_153_0.CF[722] == 0))){
        {

            __CLOVER_153_0.S[3813]++;_outerWriter.printRaw("\n\n" + "window.onload = function ()\n" + "{\n");

            __CLOVER_153_0.S[3814]++;_outerWriter.printRaw(_initializationScript.toString());

            __CLOVER_153_0.S[3815]++;_outerWriter.printRaw("}");
        }}

        __CLOVER_153_0.S[3816]++;_outerWriter.printRaw("\n\n// -->");
        __CLOVER_153_0.S[3817]++;_outerWriter.end();
    } finally { }}

    private boolean any(StringBuffer buffer)
    {try { __CLOVER_153_0.M[793]++;
        __CLOVER_153_0.S[3818]++;if ((((buffer == null) && (++__CLOVER_153_0.CT[723] != 0)) || (++__CLOVER_153_0.CF[723] == 0))){
            __CLOVER_153_0.S[3819]++;return false;}

        __CLOVER_153_0.S[3820]++;return buffer.length() > 0;
    } finally { }}

    public abstract String getElement();

    public abstract void setElement(String element);

    /**
     * Sets the element parameter property to its default, "body".
     * 
     * @since 3.0
     */
    protected void finishLoad()
    {try { __CLOVER_153_0.M[794]++;
        __CLOVER_153_0.S[3821]++;setElement("body");
    } finally { }}

    /** @since 3.0 */

    public String getUniqueString(String baseValue)
    {try { __CLOVER_153_0.M[795]++;
        __CLOVER_153_0.S[3822]++;if ((((_idAllocator == null) && (++__CLOVER_153_0.CT[724] != 0)) || (++__CLOVER_153_0.CF[724] == 0))){
            __CLOVER_153_0.S[3823]++;_idAllocator = new IdAllocator();}

        __CLOVER_153_0.S[3824]++;return _idAllocator.allocateId(baseValue);
    } finally { }}

}