/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.link;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.tapestry.AbstractComponent;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.components.ILinkComponent;
import org.apache.tapestry.components.LinkEventType;
import org.apache.tapestry.engine.IEngineService;
import org.apache.tapestry.engine.ILink;
import org.apache.tapestry.html.Body;

/**
 *  Base class for
 *  implementations of {@link ILinkComponent}.  Includes a disabled attribute
 *  (that should be bound to a disabled parameter), 
 *  an anchor attribute, and a
 *  renderer attribute (that should be bound to a renderer parameter).  A default,
 *  shared instance of {@link org.apache.tapestry.link.DefaultLinkRenderer} is
 *  used when no specific renderer is provided.
 *
 *  @author Howard Lewis Ship
 *
 **/

public abstract class AbstractLinkComponent extends AbstractComponent implements ILinkComponent
{public static com.cortexeb.tools.clover.d __CLOVER_173_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private Map _eventHandlers;

    public abstract boolean isDisabled();

    /**
     *  Adds an event handler (typically, from a wrapped component such
     *  as a {@link org.apache.tapestry.html.Rollover}).
     *
     **/

    public void addEventHandler(LinkEventType eventType, String functionName)
    {try { __CLOVER_173_0.M[873]++;
        __CLOVER_173_0.S[4160]++;Object currentValue;

        __CLOVER_173_0.S[4161]++;if ((((_eventHandlers == null) && (++__CLOVER_173_0.CT[768] != 0)) || (++__CLOVER_173_0.CF[768] == 0))){
            __CLOVER_173_0.S[4162]++;_eventHandlers = new HashMap();}

        __CLOVER_173_0.S[4163]++;currentValue = _eventHandlers.get(eventType);

        // The first value is added as a String

        __CLOVER_173_0.S[4164]++;if ((((currentValue == null) && (++__CLOVER_173_0.CT[769] != 0)) || (++__CLOVER_173_0.CF[769] == 0))){
        {
            __CLOVER_173_0.S[4165]++;_eventHandlers.put(eventType, functionName);
            __CLOVER_173_0.S[4166]++;return;
        }}

        // When adding the second value, convert to a List

        __CLOVER_173_0.S[4167]++;if ((((currentValue instanceof String) && (++__CLOVER_173_0.CT[770] != 0)) || (++__CLOVER_173_0.CF[770] == 0))){
        {
            __CLOVER_173_0.S[4168]++;List list = new ArrayList();
            __CLOVER_173_0.S[4169]++;list.add(currentValue);
            __CLOVER_173_0.S[4170]++;list.add(functionName);

            __CLOVER_173_0.S[4171]++;_eventHandlers.put(eventType, list);
            __CLOVER_173_0.S[4172]++;return;
        }}

        // For the third and up, add the new function to the List

        __CLOVER_173_0.S[4173]++;List list = (List) currentValue;
        __CLOVER_173_0.S[4174]++;list.add(functionName);
    } finally { }}

    /**
     *  Renders the link by delegating to an instance
     *  of {@link ILinkRenderer}.
     *
     **/

    protected void renderComponent(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_173_0.M[874]++;
        __CLOVER_173_0.S[4175]++;getRenderer().renderLink(writer, cycle, this);
    } finally { }}

    protected void cleanupAfterRender(IRequestCycle cycle)
    {try { __CLOVER_173_0.M[875]++;
        __CLOVER_173_0.S[4176]++;_eventHandlers = null;

        __CLOVER_173_0.S[4177]++;super.cleanupAfterRender(cycle);
    } finally { }}

    protected void writeEventHandlers(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_173_0.M[876]++;
        __CLOVER_173_0.S[4178]++;String name = null;

        __CLOVER_173_0.S[4179]++;if ((((_eventHandlers == null) && (++__CLOVER_173_0.CT[771] != 0)) || (++__CLOVER_173_0.CF[771] == 0))){
            __CLOVER_173_0.S[4180]++;return;}

        __CLOVER_173_0.S[4181]++;Body body = Body.get(cycle);

        __CLOVER_173_0.S[4182]++;if ((((body == null) && (++__CLOVER_173_0.CT[772] != 0)) || (++__CLOVER_173_0.CF[772] == 0))){
            __CLOVER_173_0.S[4183]++;throw new ApplicationRuntimeException(
                Tapestry.getMessage("AbstractLinkComponent.events-need-body"),
                this,
                null,
                null);}

        __CLOVER_173_0.S[4184]++;Iterator i = _eventHandlers.entrySet().iterator();

        __CLOVER_173_0.S[4185]++;while ((((i.hasNext()) && (++__CLOVER_173_0.CT[773] != 0)) || (++__CLOVER_173_0.CF[773] == 0))){
        {
            __CLOVER_173_0.S[4186]++;Map.Entry entry = (Map.Entry) i.next();
            __CLOVER_173_0.S[4187]++;LinkEventType type = (LinkEventType) entry.getKey();

            __CLOVER_173_0.S[4188]++;name = writeEventHandler(writer, body, name, type.getAttributeName(), entry.getValue());
        }}

    } finally { }}

    protected String writeEventHandler(
        IMarkupWriter writer,
        Body body,
        String name,
        String attributeName,
        Object value)
    {try { __CLOVER_173_0.M[877]++;
        __CLOVER_173_0.S[4189]++;String wrapperFunctionName;

        __CLOVER_173_0.S[4190]++;if ((((value instanceof String) && (++__CLOVER_173_0.CT[774] != 0)) || (++__CLOVER_173_0.CF[774] == 0))){
        {
            __CLOVER_173_0.S[4191]++;wrapperFunctionName = (String) value;
        }}
        else{
        {
            __CLOVER_173_0.S[4192]++;String finalName = (((name == null ) && (++__CLOVER_173_0.CT[775] != 0)) || (++__CLOVER_173_0.CF[775] == 0))? body.getUniqueString("Link") : name;

            __CLOVER_173_0.S[4193]++;wrapperFunctionName = attributeName + "_" + finalName;

            __CLOVER_173_0.S[4194]++;StringBuffer buffer = new StringBuffer();

            __CLOVER_173_0.S[4195]++;buffer.append("function ");
            __CLOVER_173_0.S[4196]++;buffer.append(wrapperFunctionName);
            __CLOVER_173_0.S[4197]++;buffer.append(" ()\n{\n");

            __CLOVER_173_0.S[4198]++;Iterator i = ((List) value).iterator();
            __CLOVER_173_0.S[4199]++;while ((((i.hasNext()) && (++__CLOVER_173_0.CT[776] != 0)) || (++__CLOVER_173_0.CF[776] == 0))){
            {
                __CLOVER_173_0.S[4200]++;String functionName = (String) i.next();
                __CLOVER_173_0.S[4201]++;buffer.append("  ");
                __CLOVER_173_0.S[4202]++;buffer.append(functionName);
                __CLOVER_173_0.S[4203]++;buffer.append("();\n");
            }}

            __CLOVER_173_0.S[4204]++;buffer.append("}\n\n");

            __CLOVER_173_0.S[4205]++;body.addBodyScript(buffer.toString());
        }}

        __CLOVER_173_0.S[4206]++;writer.attribute(attributeName, "javascript:" + wrapperFunctionName + "();");

        __CLOVER_173_0.S[4207]++;return name;
    } finally { }}

    /** @since 3.0 **/

    public abstract ILinkRenderer getRenderer();

    public abstract void setRenderer(ILinkRenderer renderer);

    public void renderAdditionalAttributes(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_173_0.M[878]++;
        __CLOVER_173_0.S[4208]++;writeEventHandlers(writer, cycle);

        // Generate additional attributes from informal parameters.

        __CLOVER_173_0.S[4209]++;renderInformalParameters(writer, cycle);
    } finally { }}

    /**
     *  Utility method for subclasses; Gets the named service from the engine
     *  and invokes {@link IEngineService#getLink(IRequestCycle, org.apache.tapestry.IComponent, Object[])}
     *  on it.
     * 
     *  @since 3.0
     * 
     **/

    protected ILink getLink(IRequestCycle cycle, String serviceName, Object[] serviceParameters)
    {try { __CLOVER_173_0.M[879]++;
        __CLOVER_173_0.S[4210]++;IEngineService service = cycle.getEngine().getService(serviceName);

        __CLOVER_173_0.S[4211]++;return service.getLink(cycle, this, serviceParameters);
    } finally { }}

    public abstract String getAnchor();

    public ILink getLink(IRequestCycle cycle)
    {try { __CLOVER_173_0.M[880]++;
        __CLOVER_173_0.S[4212]++;return null;
    } finally { }}

    /**
     * Sets the renderer parameter property to its default value
     * {@link DefaultLinkRenderer#SHARED_INSTANCE}.
     * 
     * @since 3.0
     */
    protected void finishLoad()
    {try { __CLOVER_173_0.M[881]++;
        __CLOVER_173_0.S[4213]++;setRenderer(DefaultLinkRenderer.SHARED_INSTANCE);
    } finally { }}

}