/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.param;

import org.apache.hivemind.ClassResolver;
import org.apache.hivemind.util.PropertyUtils;
import org.apache.tapestry.IBinding;
import org.apache.tapestry.IComponent;
import org.apache.tapestry.IForm;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.Form;
import org.apache.tapestry.form.IFormComponent;
import org.apache.tapestry.spec.Direction;
import org.apache.tapestry.spec.IParameterSpecification;
import org.apache.tapestry.util.prop.OgnlUtils;

/**
 * Standard implementation of {@link IParameterConnector}. Subclasses add in the ability to clear
 * parameters.
 * 
 * @author Howard Lewis Ship
 * @since 2.0.3
 */

public abstract class AbstractParameterConnector implements IParameterConnector
{public static com.cortexeb.tools.clover.d __CLOVER_203_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private String _parameterName;

    private String _propertyName;

    private IBinding _binding;

    private IComponent _component;

    private boolean _required;

    private Object _clearValue;

    private Direction _direction;

    private ClassResolver _resolver;

    /**
     * Creates a connector. In addition, obtains the current value of the component property; this
     * value will be used to restore the component property.
     */

    protected AbstractParameterConnector(IComponent component, String parameterName,
            IBinding binding)
    {try { __CLOVER_203_0.M[1015]++;
        __CLOVER_203_0.S[4779]++;_component = component;
        __CLOVER_203_0.S[4780]++;_parameterName = parameterName;
        __CLOVER_203_0.S[4781]++;_binding = binding;

        __CLOVER_203_0.S[4782]++;IParameterSpecification pspec = _component.getSpecification().getParameter(_parameterName);
        __CLOVER_203_0.S[4783]++;_required = pspec.isRequired();
        __CLOVER_203_0.S[4784]++;_propertyName = pspec.getPropertyName();
        __CLOVER_203_0.S[4785]++;_direction = pspec.getDirection();

        __CLOVER_203_0.S[4786]++;_clearValue = readCurrentPropertyValue();
    } finally { }}

    /** @since 2.2 */

    private Object readCurrentPropertyValue()
    {try { __CLOVER_203_0.M[1016]++;
        __CLOVER_203_0.S[4787]++;return OgnlUtils.get(_propertyName, _component);
    } finally { }}

    /**
     * Sets the property of the component to the specified value.
     */

    protected void setPropertyValue(Object value)
    {try { __CLOVER_203_0.M[1017]++;
        __CLOVER_203_0.S[4788]++;PropertyUtils.write(_component, _propertyName, value);
    } finally { }}

    /**
     * Gets the value of the binding.
     * 
     * @param requiredType
     *            if not null, the expected type of the value object.
     * @see IBinding#getObject()
     * @see IBinding#getObject(String, Class)
     */

    protected Object getBindingValue(Class requiredType)
    {try { __CLOVER_203_0.M[1018]++;
        __CLOVER_203_0.S[4789]++;Object result;

        __CLOVER_203_0.S[4790]++;if ((((requiredType == null) && (++__CLOVER_203_0.CT[866] != 0)) || (++__CLOVER_203_0.CF[866] == 0))){
            __CLOVER_203_0.S[4791]++;result = _binding.getObject();}
        else{
            __CLOVER_203_0.S[4792]++;result = _binding.getObject(_parameterName, requiredType);}

        __CLOVER_203_0.S[4793]++;return result;
    } finally { }}

    protected IBinding getBinding()
    {try { __CLOVER_203_0.M[1019]++;
        __CLOVER_203_0.S[4794]++;return _binding;
    } finally { }}

    public String toString()
    {try { __CLOVER_203_0.M[1020]++;
        __CLOVER_203_0.S[4795]++;StringBuffer buffer = new StringBuffer(super.toString());
        __CLOVER_203_0.S[4796]++;buffer.append('[');
        __CLOVER_203_0.S[4797]++;buffer.append(_component.getExtendedId());
        __CLOVER_203_0.S[4798]++;buffer.append(' ');
        __CLOVER_203_0.S[4799]++;buffer.append(_parameterName);
        __CLOVER_203_0.S[4800]++;buffer.append(' ');
        __CLOVER_203_0.S[4801]++;buffer.append(_binding);

        __CLOVER_203_0.S[4802]++;buffer.append(' ');
        __CLOVER_203_0.S[4803]++;buffer.append(_direction.getName());

        __CLOVER_203_0.S[4804]++;if ((((_required) && (++__CLOVER_203_0.CT[867] != 0)) || (++__CLOVER_203_0.CF[867] == 0))){
            __CLOVER_203_0.S[4805]++;buffer.append(" required");}

        __CLOVER_203_0.S[4806]++;buffer.append(']');

        __CLOVER_203_0.S[4807]++;return buffer.toString();
    } finally { }}

    /**
     * Restores the property to its default value. For {@link Direction#FORM}parameters, extracts
     * the property value and sets the binding form it (when appropriate).
     */

    public void resetParameter(IRequestCycle cycle)
    {try { __CLOVER_203_0.M[1021]++;
        __CLOVER_203_0.S[4808]++;if ((((_direction == Direction.FORM && cycle.isRewinding()) && (++__CLOVER_203_0.CT[868] != 0)) || (++__CLOVER_203_0.CF[868] == 0))){
        {
            __CLOVER_203_0.S[4809]++;IFormComponent component = (IFormComponent) _component;

            __CLOVER_203_0.S[4810]++;if ((((!component.isDisabled()) && (++__CLOVER_203_0.CT[869] != 0)) || (++__CLOVER_203_0.CF[869] == 0))){
            {
                __CLOVER_203_0.S[4811]++;IForm form = Form.get(cycle);

                __CLOVER_203_0.S[4812]++;if ((((form != null && form.isRewinding()) && (++__CLOVER_203_0.CT[870] != 0)) || (++__CLOVER_203_0.CF[870] == 0))){
                {
                    __CLOVER_203_0.S[4813]++;Object value = readCurrentPropertyValue();

                    __CLOVER_203_0.S[4814]++;_binding.setObject(value);
                }}
            }}
        }}

        // Either way, clear the value.

        __CLOVER_203_0.S[4815]++;setPropertyValue(_clearValue);
    } finally { }}

    /**
     * Returns true if the connector should update the property value from the binding. For
     * {@link org.apache.tapestry.spec.Direction#IN}, this always returns true. For
     * {@link org.apache.tapestry.spec.Direction#FORM}, this returns true only if the request cycle
     * and the active form are rewinding.
     * 
     * @since 2.2
     */

    protected boolean shouldSetPropertyValue(IRequestCycle cycle)
    {try { __CLOVER_203_0.M[1022]++;
        __CLOVER_203_0.S[4816]++;if ((((_direction == Direction.IN) && (++__CLOVER_203_0.CT[871] != 0)) || (++__CLOVER_203_0.CF[871] == 0))){
            __CLOVER_203_0.S[4817]++;return true;}

        // Must be FORM

        __CLOVER_203_0.S[4818]++;return !cycle.isRewinding();
    } finally { }}

}