/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.parse;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.hivemind.Location;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRender;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;

/**
 *  Represents static text in the template that may be passed through
 *  to the client unchanged (except, perhaps, for the removal of
 *  some whitespace).
 *
 *  @see TokenType#TEXT
 *  @author Howard Lewis Ship
 *  @since 3.0
 *
 **/

public class TextToken extends TemplateToken implements IRender
{public static com.cortexeb.tools.clover.d __CLOVER_230_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private char[] _templateData;

    private int _startIndex = -1;
    private int _endIndex = -1;

    private int _offset;
    private int _length;
    private boolean _needsTrim = true;

    public TextToken(char[] templateData, int startIndex, int endIndex, Location location)
    {
        super(TokenType.TEXT, location);__CLOVER_230_0.S[5950]++;try { __CLOVER_230_0.M[1221]++;

        __CLOVER_230_0.S[5951]++;if ((((startIndex < 0
            || endIndex < 0
            || startIndex > templateData.length
            || endIndex > templateData.length) && (++__CLOVER_230_0.CT[1050] != 0)) || (++__CLOVER_230_0.CF[1050] == 0))){
            __CLOVER_230_0.S[5952]++;throw new IllegalArgumentException(
                Tapestry.format(
                    "TextToken.range-error",
                    this,
                    Integer.toString(templateData.length)));}

        __CLOVER_230_0.S[5953]++;_templateData = templateData;
        __CLOVER_230_0.S[5954]++;_startIndex = startIndex;
        __CLOVER_230_0.S[5955]++;_endIndex = endIndex;

        // Values actually used to render, may be adjusted to remove excess
        // leading and trailing whitespace.

        __CLOVER_230_0.S[5956]++;_offset = startIndex;
        __CLOVER_230_0.S[5957]++;_length = endIndex - startIndex + 1;
    } finally { }}

    public synchronized void render(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_230_0.M[1222]++;
        __CLOVER_230_0.S[5958]++;if ((((_needsTrim) && (++__CLOVER_230_0.CT[1051] != 0)) || (++__CLOVER_230_0.CF[1051] == 0))){
        {
            __CLOVER_230_0.S[5959]++;trim();
            __CLOVER_230_0.S[5960]++;_needsTrim = false;
        }}

        __CLOVER_230_0.S[5961]++;if ((((_length == 0) && (++__CLOVER_230_0.CT[1052] != 0)) || (++__CLOVER_230_0.CF[1052] == 0))){
            __CLOVER_230_0.S[5962]++;return;}

        // At one time, we would check to see if the cycle was rewinding and
        // only invoke printRaw() if it was.  However, that slows down
        // normal rendering (microscopically) and, with the new
        // NullResponseWriter class, the "cost" of invoking cycle.isRewinding()
        // is approximately the same as the "cost" of invoking writer.printRaw().

        __CLOVER_230_0.S[5963]++;writer.printRaw(_templateData, _offset, _length);
    } finally { }}

    /**
      *  Strip off all leading and trailing whitespace by adjusting offset and length.
      *
      **/

    private void trim()
    {try { __CLOVER_230_0.M[1223]++;
        __CLOVER_230_0.S[5964]++;if ((((_length == 0) && (++__CLOVER_230_0.CT[1053] != 0)) || (++__CLOVER_230_0.CF[1053] == 0))){
            __CLOVER_230_0.S[5965]++;return;}

        __CLOVER_230_0.S[5966]++;try
        {
            __CLOVER_230_0.S[5967]++;boolean didTrim = false;

            // Shave characters off the end until we hit a non-whitespace
            // character.

            __CLOVER_230_0.S[5968]++;while ((((_length > 0) && (++__CLOVER_230_0.CT[1054] != 0)) || (++__CLOVER_230_0.CF[1054] == 0))){
            {
                __CLOVER_230_0.S[5969]++;char ch = _templateData[_offset + _length - 1];

                __CLOVER_230_0.S[5970]++;if ((((!Character.isWhitespace(ch)) && (++__CLOVER_230_0.CT[1055] != 0)) || (++__CLOVER_230_0.CF[1055] == 0))){
                    __CLOVER_230_0.S[5971]++;break;}

                __CLOVER_230_0.S[5972]++;_length--;
                __CLOVER_230_0.S[5973]++;didTrim = true;
            }}

            // Restore one character of whitespace to the end

            __CLOVER_230_0.S[5974]++;if ((((didTrim) && (++__CLOVER_230_0.CT[1056] != 0)) || (++__CLOVER_230_0.CF[1056] == 0))){
                __CLOVER_230_0.S[5975]++;_length++;}

            __CLOVER_230_0.S[5976]++;didTrim = false;

            // Strip characters off the front until we hit a non-whitespace
            // character.

            __CLOVER_230_0.S[5977]++;while ((((_length > 0) && (++__CLOVER_230_0.CT[1057] != 0)) || (++__CLOVER_230_0.CF[1057] == 0))){
            {
                __CLOVER_230_0.S[5978]++;char ch = _templateData[_offset];

                __CLOVER_230_0.S[5979]++;if ((((!Character.isWhitespace(ch)) && (++__CLOVER_230_0.CT[1058] != 0)) || (++__CLOVER_230_0.CF[1058] == 0))){
                    __CLOVER_230_0.S[5980]++;break;}

                __CLOVER_230_0.S[5981]++;_offset++;
                __CLOVER_230_0.S[5982]++;_length--;
                __CLOVER_230_0.S[5983]++;didTrim = true;
            }}

            // Again, restore one character of whitespace.

            __CLOVER_230_0.S[5984]++;if ((((didTrim) && (++__CLOVER_230_0.CT[1059] != 0)) || (++__CLOVER_230_0.CF[1059] == 0))){
            {
                __CLOVER_230_0.S[5985]++;_offset--;
                __CLOVER_230_0.S[5986]++;_length++;
            }}

        }
        catch (IndexOutOfBoundsException ex)
        {
            __CLOVER_230_0.S[5987]++;throw new RuntimeException(Tapestry.format("TextToken.error-trimming", this));
        }

        // Ok, this isn't perfect.  I don't want to write into templateData[] even
        // though I'd prefer that my single character of whitespace was always a space.
        // It would also be kind of neat to shave whitespace within the static HTML, rather
        // than just on the edges.
    } finally { }}

    protected void extendDescription(ToStringBuilder builder)
    {try { __CLOVER_230_0.M[1224]++;
        __CLOVER_230_0.S[5988]++;builder.append("startIndex", _startIndex);
        __CLOVER_230_0.S[5989]++;builder.append("endIndex", _endIndex);
    } finally { }}

    public int getEndIndex()
    {try { __CLOVER_230_0.M[1225]++;
        __CLOVER_230_0.S[5990]++;return _endIndex;
    } finally { }}

    public int getStartIndex()
    {try { __CLOVER_230_0.M[1226]++;
        __CLOVER_230_0.S[5991]++;return _startIndex;
    } finally { }}

    public char[] getTemplateData()
    {try { __CLOVER_230_0.M[1227]++;
        __CLOVER_230_0.S[5992]++;return _templateData;
    } finally { }}

}
