/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.request;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRender;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.multipart.DefaultMultipartDecoder;
import org.apache.tapestry.multipart.IMultipartDecoder;
import org.apache.tapestry.spec.IApplicationSpecification;

/**
 *  This class encapsulates all the relevant data for one request cycle of an
 *  {@link ApplicationServlet}.  This includes:
 *  <ul>
 *  	<li>{@link HttpServletRequest}
 *		<li>{@link HttpServletResponse}
 *		<li>{@link HttpSession}
 * 		<li>{@link javax.servlet.http.HttpServlet}
 *  </ul>
 *  <p>It also provides methods for:
 *  <ul>
 *  <li>Retrieving the request parameters (even if a file upload is involved)
 *  <li>Getting, setting and removing request attributes
 *  <li>Forwarding requests
 *  <li>Redirecting requests
 *  <li>Getting and setting Cookies
 *  <li>Intepreting the request path info
 *  <li>Writing an HTML description of the <code>RequestContext</code> (for debugging).
 *  </ul>
 *
 * 
 *  <p>
 *  If some cases, it is necesary to provide an implementation of
 *  {@link IRequestDecoder} (often, due to a firewall).
 *  If the application specifification
 *  provides an extension named
 *  <code>org.apache.tapestry.request-decoder</code>
 *  then it will be used, instead of a default decoder.
 * 
 *  <p>This class is not a component, but does implement {@link IRender}.  When asked to render
 *  (perhaps as the delegate of a {@link org.apache.tapestry.components.Delegator} component}
 *  it simply invokes {@link #write(IMarkupWriter)} to display all debugging output.
 *
 *  <p>This class is derived from the original class 
 *  <code>com.primix.servlet.RequestContext</code>,
 *  part of the <b>ServletUtils</b> framework available from
 *  <a href="http://www.gjt.org/servlets/JCVSlet/list/gjt/com/primix/servlet">The Giant 
 *  Java Tree</a>.
 *
 *
 *  @author Howard Lewis Ship
 * 
 **/

public class RequestContext implements IRender
{public static com.cortexeb.tools.clover.d __CLOVER_240_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /** @since 2.2 **/

    private static class DefaultRequestDecoder implements IRequestDecoder
    {
        public DecodedRequest decodeRequest(HttpServletRequest request)
        {try { __CLOVER_240_0.M[1264]++;
            __CLOVER_240_0.S[6134]++;DecodedRequest result = new DecodedRequest();

            __CLOVER_240_0.S[6135]++;result.setRequestURI(request.getRequestURI());
            __CLOVER_240_0.S[6136]++;result.setScheme(request.getScheme());
            __CLOVER_240_0.S[6137]++;result.setServerName(request.getServerName());
            __CLOVER_240_0.S[6138]++;result.setServerPort(request.getServerPort());

            __CLOVER_240_0.S[6139]++;return result;
        } finally { }}
    }

    private static final Log LOG = LogFactory.getLog(RequestContext.class);

    private HttpSession _session;
    private HttpServletRequest _request;
    private HttpServletResponse _response;
    private HttpServlet _servlet;
    private DecodedRequest _decodedRequest;
    private IMultipartDecoder _decoder;
    private boolean _decoded;
    private IApplicationSpecification _specification;

    /**
     * A mapping of the cookies available in the request.
     *
     **/

    private Map _cookieMap;

    /**
     *  Used during {@link #write(IMarkupWriter)}.
     * 
     **/

    private boolean _evenRow;

    /**
     * Creates a <code>RequestContext</code> from its components.
     *
     **/

    public RequestContext(
        HttpServlet servlet,
        HttpServletRequest request,
        HttpServletResponse response,
        IApplicationSpecification specification)
        throws IOException
    {try { __CLOVER_240_0.M[1265]++;
        __CLOVER_240_0.S[6140]++;_servlet = servlet;
        __CLOVER_240_0.S[6141]++;_request = request;
        __CLOVER_240_0.S[6142]++;_response = response;
        __CLOVER_240_0.S[6143]++;_specification = specification;

        // All three parameters may be null if created from
        // AbstractEngine.cleanupEngine().

        __CLOVER_240_0.S[6144]++;if ((((_request != null && DefaultMultipartDecoder.isMultipartRequest(request)) && (++__CLOVER_240_0.CT[1085] != 0)) || (++__CLOVER_240_0.CF[1085] == 0))){
        {
            __CLOVER_240_0.S[6145]++;IMultipartDecoder decoder = obtainMultipartDecoder(servlet, request);
            __CLOVER_240_0.S[6146]++;setDecoder(decoder);
        }}
    } finally { }}

    /**
     *  Invoked from the constructor to create a {@link DefaultMultipartDecoder} instance.
     *  Applications with specific upload needs may need to override this to
     *  provide a subclass instance instead.  The caller will invoke
     *  {@link IMultipartDecoder#decode(HttpServletRequest)} on the
     *  returned object.
     * 
     *  <p>
     *  This implementation checks for application extension
     *  {@link Tapestry#MULTIPART_DECODER_EXTENSION_NAME}.  If that is not
     *  defined, a shared instance of {@link DefaultMultipartDecoder}
     *  is returned.  
     *
     * 
     *  @see ApplicationServlet#createRequestContext(HttpServletRequest, HttpServletResponse)
     *  @since 3.0
     * 
     **/

    protected IMultipartDecoder obtainMultipartDecoder(
        HttpServlet servlet,
        HttpServletRequest request)
        throws IOException
    {try { __CLOVER_240_0.M[1266]++;
        __CLOVER_240_0.S[6147]++;if ((((_specification.checkExtension(Tapestry.MULTIPART_DECODER_EXTENSION_NAME)) && (++__CLOVER_240_0.CT[1086] != 0)) || (++__CLOVER_240_0.CF[1086] == 0))){
            __CLOVER_240_0.S[6148]++;return (IMultipartDecoder) _specification.getExtension(
                Tapestry.MULTIPART_DECODER_EXTENSION_NAME,
                IMultipartDecoder.class);}

        __CLOVER_240_0.S[6149]++;return DefaultMultipartDecoder.getSharedInstance();
    } finally { }}

    /**
     * Adds a simple {@link Cookie}. To set a Cookie with attributes,
     * use {@link #addCookie(Cookie)}.
     *
     **/

    public void addCookie(String name, String value)
    {try { __CLOVER_240_0.M[1267]++;
        __CLOVER_240_0.S[6150]++;addCookie(new Cookie(name, value));
    } finally { }}

    /**
     * Adds a {@link Cookie} to the response. Once added, the
     * Cookie will also be available to {@link #getCookie(String)} method.
     *
     * <p>Cookies should only be added <em>before</em> invoking
     * {@link HttpServletResponse#getWriter()}..
     *
     **/

    public void addCookie(Cookie cookie)
    {try { __CLOVER_240_0.M[1268]++;
        __CLOVER_240_0.S[6151]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_240_0.CT[1087] != 0)) || (++__CLOVER_240_0.CF[1087] == 0))){
            __CLOVER_240_0.S[6152]++;LOG.debug("Adding cookie " + cookie);}

        __CLOVER_240_0.S[6153]++;_response.addCookie(cookie);

        __CLOVER_240_0.S[6154]++;if ((((_cookieMap == null) && (++__CLOVER_240_0.CT[1088] != 0)) || (++__CLOVER_240_0.CF[1088] == 0))){
            __CLOVER_240_0.S[6155]++;readCookieMap();}

        __CLOVER_240_0.S[6156]++;_cookieMap.put(cookie.getName(), cookie);
    } finally { }}

    private void datePair(IMarkupWriter writer, String name, long value)
    {try { __CLOVER_240_0.M[1269]++;
        __CLOVER_240_0.S[6157]++;pair(writer, name, new Date(value));
    } finally { }}

    /** @since 2.2 **/

    private DecodedRequest getDecodedRequest()
    {try { __CLOVER_240_0.M[1270]++;
        __CLOVER_240_0.S[6158]++;if ((((_decodedRequest != null) && (++__CLOVER_240_0.CT[1089] != 0)) || (++__CLOVER_240_0.CF[1089] == 0))){
            __CLOVER_240_0.S[6159]++;return _decodedRequest;}

        __CLOVER_240_0.S[6160]++;IRequestDecoder decoder = null;

        __CLOVER_240_0.S[6161]++;if ((((!_specification.checkExtension(Tapestry.REQUEST_DECODER_EXTENSION_NAME)) && (++__CLOVER_240_0.CT[1090] != 0)) || (++__CLOVER_240_0.CF[1090] == 0))){
            __CLOVER_240_0.S[6162]++;decoder = new DefaultRequestDecoder();}
        else{
            __CLOVER_240_0.S[6163]++;decoder =
                (IRequestDecoder) _specification.getExtension(
                    Tapestry.REQUEST_DECODER_EXTENSION_NAME,
                    IRequestDecoder.class);}

        __CLOVER_240_0.S[6164]++;_decodedRequest = decoder.decodeRequest(_request);

        __CLOVER_240_0.S[6165]++;return _decodedRequest;
    } finally { }}

    /** 
     * 
     *  Returns the actual scheme, possibly decoded from the request.
     * 
     *  @see IRequestDecoder
     *  @see javax.servlet.ServletRequest#getScheme()
     *  @since 2.2  
     * 
     **/

    public String getScheme()
    {try { __CLOVER_240_0.M[1271]++;
        __CLOVER_240_0.S[6166]++;return getDecodedRequest().getScheme();
    } finally { }}

    /** 
     * 
     *  Returns the actual server name, possibly decoded from the request.
     * 
     *  @see IRequestDecoder
     *  @see javax.servlet.ServletRequest#getServerName()
     *  @since 2.2  
     * 
     **/

    public String getServerName()
    {try { __CLOVER_240_0.M[1272]++;
        __CLOVER_240_0.S[6167]++;return getDecodedRequest().getServerName();
    } finally { }}

    /** 
     * 
     *  Returns the actual server port, possibly decoded from the request.
     * 
     *  @see IRequestDecoder
     *  @see javax.servlet.ServletRequest#getServerPort()
     *  @since 2.2  
     * 
     **/

    public int getServerPort()
    {try { __CLOVER_240_0.M[1273]++;
        __CLOVER_240_0.S[6168]++;return getDecodedRequest().getServerPort();
    } finally { }}

    /** 
     * 
     *  Returns the actual request URI, possibly decoded from the request.
     * 
     *  @see IRequestDecoder
     *  @see HttpServletRequest#getRequestURI()
     *  @since 2.2  
     * 
     **/

    public String getRequestURI()
    {try { __CLOVER_240_0.M[1274]++;
        __CLOVER_240_0.S[6169]++;return getDecodedRequest().getRequestURI();
    } finally { }}

    /**
     * Builds an absolute URL from the given URI, using the {@link HttpServletRequest}
     * as the source for scheme, server name and port.
     *
     * @see #getAbsoluteURL(String, String, String, int)
     * 
     **/

    public String getAbsoluteURL(String URI)
    {try { __CLOVER_240_0.M[1275]++;
        __CLOVER_240_0.S[6170]++;String scheme = getScheme();
        __CLOVER_240_0.S[6171]++;String server = getServerName();
        __CLOVER_240_0.S[6172]++;int port = getServerPort();

        // Keep things simple ... port 80 is accepted as the
        // standard port for http so it can be ommitted.
        // Some of the Tomcat code indicates that port 443 is the default
        // for https, and that needs to be researched.

        __CLOVER_240_0.S[6173]++;if ((((scheme.equals("http") && port == 80) && (++__CLOVER_240_0.CT[1091] != 0)) || (++__CLOVER_240_0.CF[1091] == 0))){
            __CLOVER_240_0.S[6174]++;port = 0;}

        __CLOVER_240_0.S[6175]++;return getAbsoluteURL(URI, scheme, server, port);
    } finally { }}

    /**
     * Does some easy checks to turn a path (or URI) into an absolute URL. We assume
     * <ul>
     * <li>The presense of a colon means the path is complete already (any other colons
     * in the URI portion should have been converted to %3A).
     *
     * <li>A leading pair of forward slashes means the path is simply missing
     * the scheme.
     * <li>Otherwise, we assemble the scheme, server, port (if non-zero) and the URI
     * as given.
     * </ul>
     *
     **/

    public String getAbsoluteURL(String URI, String scheme, String server, int port)
    {try { __CLOVER_240_0.M[1276]++;
        __CLOVER_240_0.S[6176]++;StringBuffer buffer = new StringBuffer();

        // Though, really, what does a leading colon with no scheme before it
        // mean?

        __CLOVER_240_0.S[6177]++;if ((((URI.indexOf(':') >= 0) && (++__CLOVER_240_0.CT[1092] != 0)) || (++__CLOVER_240_0.CF[1092] == 0))){
            __CLOVER_240_0.S[6178]++;return URI;}

        // Should check the length here, first.

        __CLOVER_240_0.S[6179]++;if ((((URI.substring(0, 1).equals("//")) && (++__CLOVER_240_0.CT[1093] != 0)) || (++__CLOVER_240_0.CF[1093] == 0))){
        {
            __CLOVER_240_0.S[6180]++;buffer.append(scheme);
            __CLOVER_240_0.S[6181]++;buffer.append(':');
            __CLOVER_240_0.S[6182]++;buffer.append(URI);
            __CLOVER_240_0.S[6183]++;return buffer.toString();
        }}

        __CLOVER_240_0.S[6184]++;buffer.append(scheme);
        __CLOVER_240_0.S[6185]++;buffer.append("://");
        __CLOVER_240_0.S[6186]++;buffer.append(server);

        __CLOVER_240_0.S[6187]++;if ((((port > 0) && (++__CLOVER_240_0.CT[1094] != 0)) || (++__CLOVER_240_0.CF[1094] == 0))){
        {
            __CLOVER_240_0.S[6188]++;buffer.append(':');
            __CLOVER_240_0.S[6189]++;buffer.append(port);
        }}

        __CLOVER_240_0.S[6190]++;if ((((URI.charAt(0) != '/') && (++__CLOVER_240_0.CT[1095] != 0)) || (++__CLOVER_240_0.CF[1095] == 0))){
            __CLOVER_240_0.S[6191]++;buffer.append('/');}

        __CLOVER_240_0.S[6192]++;buffer.append(URI);

        __CLOVER_240_0.S[6193]++;return buffer.toString();
    } finally { }}

    /**
     * Gets a named {@link Cookie}.
     *
     * @param name The name of the Cookie.
     * @return The Cookie, or null if no Cookie with that
     * name exists.
     *
     **/

    public Cookie getCookie(String name)
    {try { __CLOVER_240_0.M[1277]++;
        __CLOVER_240_0.S[6194]++;if ((((_cookieMap == null) && (++__CLOVER_240_0.CT[1096] != 0)) || (++__CLOVER_240_0.CF[1096] == 0))){
            __CLOVER_240_0.S[6195]++;readCookieMap();}

        __CLOVER_240_0.S[6196]++;return (Cookie) _cookieMap.get(name);
    } finally { }}

    /**
     * Reads the named {@link Cookie} and returns its value (if it exists), or
     * null if it does not exist.
     **/

    public String getCookieValue(String name)
    {try { __CLOVER_240_0.M[1278]++;
        __CLOVER_240_0.S[6197]++;Cookie cookie;

        __CLOVER_240_0.S[6198]++;cookie = getCookie(name);

        __CLOVER_240_0.S[6199]++;if ((((cookie == null) && (++__CLOVER_240_0.CT[1097] != 0)) || (++__CLOVER_240_0.CF[1097] == 0))){
            __CLOVER_240_0.S[6200]++;return null;}

        __CLOVER_240_0.S[6201]++;return cookie.getValue();
    } finally { }}

    /**
     *  Returns the named parameter from the {@link HttpServletRequest}.
     *
     *  <p>Use {@link #getParameters(String)} for parameters that may
     *  include multiple values.
     * 
     *  <p>This is the preferred way to obtain parameter values (rather than
     *  obtaining the {@link HttpServletRequest} itself).  For form/multipart-data
     *  encoded requests, this method will still work.
     *
     **/

    public String getParameter(String name)
    {try { __CLOVER_240_0.M[1279]++;
        __CLOVER_240_0.S[6202]++;IMultipartDecoder decoder = getDecoder();
        __CLOVER_240_0.S[6203]++;if ((((decoder != null) && (++__CLOVER_240_0.CT[1098] != 0)) || (++__CLOVER_240_0.CF[1098] == 0))){
            __CLOVER_240_0.S[6204]++;return decoder.getString(_request, name);}

        __CLOVER_240_0.S[6205]++;return _request.getParameter(name);
    } finally { }}

    /**
     *  Convienience method for getting a {@link HttpServletRequest} attribute.
     * 
     *  @since 2.3
     * 
     **/

    public Object getAttribute(String name)
    {try { __CLOVER_240_0.M[1280]++;
        __CLOVER_240_0.S[6206]++;return _request.getAttribute(name);
    } finally { }}

    /**
     * For parameters that are, or are possibly, multi-valued, this
     * method returns all the values as an array of Strings.
     * 
     *  @see #getParameter(String)
     *
     **/

    public String[] getParameters(String name)
    {try { __CLOVER_240_0.M[1281]++;
        // Note: this may not be quite how we want it to work; we'll have to see.

        __CLOVER_240_0.S[6207]++;IMultipartDecoder decoder = getDecoder();
        __CLOVER_240_0.S[6208]++;if ((((decoder != null) && (++__CLOVER_240_0.CT[1099] != 0)) || (++__CLOVER_240_0.CF[1099] == 0))){
            __CLOVER_240_0.S[6209]++;return decoder.getStrings(_request, name);}

        __CLOVER_240_0.S[6210]++;return _request.getParameterValues(name);
    } finally { }}

    /**
     * Returns the named {@link IUploadFile}, if it exists, or null if it doesn't.
     * Uploads require an encoding of <code>multipart/form-data</code>
     * (this is specified in the
     * form's enctype attribute).  If the encoding type
     * is not so, or if no upload matches the name, then this method returns null.
     * 
     **/

    public IUploadFile getUploadFile(String name)
    {try { __CLOVER_240_0.M[1282]++;
        __CLOVER_240_0.S[6211]++;IMultipartDecoder decoder = getDecoder();
        __CLOVER_240_0.S[6212]++;if ((((decoder == null) && (++__CLOVER_240_0.CT[1100] != 0)) || (++__CLOVER_240_0.CF[1100] == 0))){
            __CLOVER_240_0.S[6213]++;return null;}

        __CLOVER_240_0.S[6214]++;return decoder.getUploadFile(_request, name);
    } finally { }}

    /**
     *  Invoked at the end of the request cycle to cleanup and temporary resources.
     *  This is chained to the {@link DefaultMultipartDecoder}, if there is one.
     * 
     *  @since 2.0.1
     **/

    public void cleanup()
    {try { __CLOVER_240_0.M[1283]++;
        __CLOVER_240_0.S[6215]++;if ((((_decoder != null) && (++__CLOVER_240_0.CT[1101] != 0)) || (++__CLOVER_240_0.CF[1101] == 0))){
            __CLOVER_240_0.S[6216]++;_decoder.cleanup(_request);}
    } finally { }}

    /**
     *  Returns the request which initiated the current request cycle.  Note that
     *  the methods {@link #getParameter(String)} and {@link #getParameters(String)}
     *  should be used, rather than obtaining parameters directly from the request
     *  (since the RequestContext handles the differences between normal and multipart/form
     *  requests).
     * 
     **/

    public HttpServletRequest getRequest()
    {try { __CLOVER_240_0.M[1284]++;
        __CLOVER_240_0.S[6217]++;return _request;
    } finally { }}

    public HttpServletResponse getResponse()
    {try { __CLOVER_240_0.M[1285]++;
        __CLOVER_240_0.S[6218]++;return _response;
    } finally { }}

    private String getRowClass()
    {try { __CLOVER_240_0.M[1286]++;
        __CLOVER_240_0.S[6219]++;String result;

        __CLOVER_240_0.S[6220]++;result = (((_evenRow ) && (++__CLOVER_240_0.CT[1102] != 0)) || (++__CLOVER_240_0.CF[1102] == 0))? "even" : "odd";

        __CLOVER_240_0.S[6221]++;_evenRow = !_evenRow;

        __CLOVER_240_0.S[6222]++;return result;
    } finally { }}

    public HttpServlet getServlet()
    {try { __CLOVER_240_0.M[1287]++;
        __CLOVER_240_0.S[6223]++;return _servlet;
    } finally { }}

    /**
     *  Returns the {@link HttpSession}, if necessary, invoking
     * {@link HttpServletRequest#getSession(boolean)}.  However,
     * this method will <em>not</em> create a session.
     *
     **/

    public HttpSession getSession()
    {try { __CLOVER_240_0.M[1288]++;
        __CLOVER_240_0.S[6224]++;if ((((_session == null) && (++__CLOVER_240_0.CT[1103] != 0)) || (++__CLOVER_240_0.CF[1103] == 0))){
            __CLOVER_240_0.S[6225]++;_session = _request.getSession(false);}

        __CLOVER_240_0.S[6226]++;return _session;
    } finally { }}

    /**
     *  Like {@link #getSession()}, but forces the creation of
     *  the {@link HttpSession}, if necessary.
     *
     **/

    public HttpSession createSession()
    {try { __CLOVER_240_0.M[1289]++;
        __CLOVER_240_0.S[6227]++;if ((((_session == null) && (++__CLOVER_240_0.CT[1104] != 0)) || (++__CLOVER_240_0.CF[1104] == 0))){
        {
            __CLOVER_240_0.S[6228]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_240_0.CT[1105] != 0)) || (++__CLOVER_240_0.CF[1105] == 0))){
                __CLOVER_240_0.S[6229]++;LOG.debug("Creating HttpSession");}

            __CLOVER_240_0.S[6230]++;_session = _request.getSession(true);
        }}

        __CLOVER_240_0.S[6231]++;return _session;
    } finally { }}

    private void header(IMarkupWriter writer, String valueName, String dataName)
    {try { __CLOVER_240_0.M[1290]++;
        __CLOVER_240_0.S[6232]++;writer.begin("tr");
        __CLOVER_240_0.S[6233]++;writer.attribute("class", "request-context-header");

        __CLOVER_240_0.S[6234]++;writer.begin("th");
        __CLOVER_240_0.S[6235]++;writer.print(valueName);
        __CLOVER_240_0.S[6236]++;writer.end();

        __CLOVER_240_0.S[6237]++;writer.begin("th");
        __CLOVER_240_0.S[6238]++;writer.print(dataName);
        __CLOVER_240_0.S[6239]++;writer.end("tr");

        __CLOVER_240_0.S[6240]++;_evenRow = true;
    } finally { }}

    private void object(IMarkupWriter writer, String objectName)
    {try { __CLOVER_240_0.M[1291]++;
        __CLOVER_240_0.S[6241]++;writer.begin("span");
        __CLOVER_240_0.S[6242]++;writer.attribute("class", "request-context-object");
        __CLOVER_240_0.S[6243]++;writer.print(objectName);
        __CLOVER_240_0.S[6244]++;writer.end();
    } finally { }}

    private void pair(IMarkupWriter writer, String name, int value)
    {try { __CLOVER_240_0.M[1292]++;
        __CLOVER_240_0.S[6245]++;pair(writer, name, Integer.toString(value));
    } finally { }}

    private void pair(IMarkupWriter writer, String name, Object value)
    {try { __CLOVER_240_0.M[1293]++;
        __CLOVER_240_0.S[6246]++;if ((((value == null) && (++__CLOVER_240_0.CT[1106] != 0)) || (++__CLOVER_240_0.CF[1106] == 0))){
            __CLOVER_240_0.S[6247]++;return;}

        __CLOVER_240_0.S[6248]++;pair(writer, name, value.toString());
    } finally { }}

    private void pair(IMarkupWriter writer, String name, String value)
    {try { __CLOVER_240_0.M[1294]++;
        __CLOVER_240_0.S[6249]++;if ((((value == null) && (++__CLOVER_240_0.CT[1107] != 0)) || (++__CLOVER_240_0.CF[1107] == 0))){
            __CLOVER_240_0.S[6250]++;return;}

        __CLOVER_240_0.S[6251]++;if ((((value.length() == 0) && (++__CLOVER_240_0.CT[1108] != 0)) || (++__CLOVER_240_0.CF[1108] == 0))){
            __CLOVER_240_0.S[6252]++;return;}

        __CLOVER_240_0.S[6253]++;writer.begin("tr");
        __CLOVER_240_0.S[6254]++;writer.attribute("class", getRowClass());

        __CLOVER_240_0.S[6255]++;writer.begin("th");
        __CLOVER_240_0.S[6256]++;writer.print(name);
        __CLOVER_240_0.S[6257]++;writer.end();

        __CLOVER_240_0.S[6258]++;writer.begin("td");
        __CLOVER_240_0.S[6259]++;writer.print(value);
        __CLOVER_240_0.S[6260]++;writer.end("tr");
        __CLOVER_240_0.S[6261]++;writer.println();
    } finally { }}

    private void pair(IMarkupWriter writer, String name, boolean value)
    {try { __CLOVER_240_0.M[1295]++;
        __CLOVER_240_0.S[6262]++;pair(writer, name, (((value ) && (++__CLOVER_240_0.CT[1109] != 0)) || (++__CLOVER_240_0.CF[1109] == 0))? "yes" : "no");
    } finally { }}

    private void readCookieMap()
    {try { __CLOVER_240_0.M[1296]++;
        __CLOVER_240_0.S[6263]++;_cookieMap = new HashMap();

        __CLOVER_240_0.S[6264]++;Cookie[] cookies = _request.getCookies();

        __CLOVER_240_0.S[6265]++;if ((((cookies != null) && (++__CLOVER_240_0.CT[1110] != 0)) || (++__CLOVER_240_0.CF[1110] == 0))){
            __CLOVER_240_0.S[6266]++;for (int i = 0; (((i < cookies.length) && (++__CLOVER_240_0.CT[1111] != 0)) || (++__CLOVER_240_0.CF[1111] == 0)); i++){
                __CLOVER_240_0.S[6267]++;_cookieMap.put(cookies[i].getName(), cookies[i]);}}
    } finally { }}

    /**
     *  Invokes {@link HttpServletResponse#sendRedirect(String)}</code>, 
     *  but massages <code>path</code>, supplying missing elements to
     *  make it an absolute URL (i.e., specifying scheme, server, port, etc.).
     *
     *  <p>The 2.2 Servlet API will do this automatically, and a little more,
     *  according to the early documentation.
     *
     **/

    public void redirect(String path) throws IOException
    {try { __CLOVER_240_0.M[1297]++;
        // Now a little magic to convert path into a complete URL. The Servlet
        // 2.2 API does this automatically.

        __CLOVER_240_0.S[6268]++;String absolutePath = getAbsoluteURL(path);

        __CLOVER_240_0.S[6269]++;String encodedURL = _response.encodeRedirectURL(absolutePath);

        __CLOVER_240_0.S[6270]++;_response.sendRedirect(encodedURL);
    } finally { }}

    private void section(IMarkupWriter writer, String sectionName)
    {try { __CLOVER_240_0.M[1298]++;
        __CLOVER_240_0.S[6271]++;writer.begin("tr");
        __CLOVER_240_0.S[6272]++;writer.attribute("class", "request-context-section");
        __CLOVER_240_0.S[6273]++;writer.begin("th");
        __CLOVER_240_0.S[6274]++;writer.attribute("colspan", 2);

        __CLOVER_240_0.S[6275]++;writer.print(sectionName);
        __CLOVER_240_0.S[6276]++;writer.end("tr");
    } finally { }}

    private List getSorted(Enumeration e)
    {try { __CLOVER_240_0.M[1299]++;
        __CLOVER_240_0.S[6277]++;List result = new ArrayList();

        // JDK 1.4 includes a helper method in Collections for
        // this; but we want 1.2 compatibility for the
        // forseable future.

        __CLOVER_240_0.S[6278]++;while ((((e.hasMoreElements()) && (++__CLOVER_240_0.CT[1112] != 0)) || (++__CLOVER_240_0.CF[1112] == 0))){
            __CLOVER_240_0.S[6279]++;result.add(e.nextElement());}

        __CLOVER_240_0.S[6280]++;Collections.sort(result);

        __CLOVER_240_0.S[6281]++;return result;
    } finally { }}

    /**
     * Writes the state of the context to the writer, typically for inclusion
     * in a HTML page returned to the user. This is useful
     * when debugging.  The Inspector uses this as well.
     *
     **/

    public void write(IMarkupWriter writer)
    {try { __CLOVER_240_0.M[1300]++;
        // Create a box around all of this stuff ...

        __CLOVER_240_0.S[6282]++;writer.begin("table");
        __CLOVER_240_0.S[6283]++;writer.attribute("class", "request-context-border");
        __CLOVER_240_0.S[6284]++;writer.begin("tr");
        __CLOVER_240_0.S[6285]++;writer.begin("td");

        // Get the session, if it exists, and display it.

        __CLOVER_240_0.S[6286]++;HttpSession session = getSession();

        __CLOVER_240_0.S[6287]++;if ((((session != null) && (++__CLOVER_240_0.CT[1113] != 0)) || (++__CLOVER_240_0.CF[1113] == 0))){
        {
            __CLOVER_240_0.S[6288]++;object(writer, "Session");
            __CLOVER_240_0.S[6289]++;writer.begin("table");
            __CLOVER_240_0.S[6290]++;writer.attribute("class", "request-context-object");

            __CLOVER_240_0.S[6291]++;section(writer, "Properties");
            __CLOVER_240_0.S[6292]++;header(writer, "Name", "Value");

            __CLOVER_240_0.S[6293]++;pair(writer, "id", session.getId());
            __CLOVER_240_0.S[6294]++;datePair(writer, "creationTime", session.getCreationTime());
            __CLOVER_240_0.S[6295]++;datePair(writer, "lastAccessedTime", session.getLastAccessedTime());
            __CLOVER_240_0.S[6296]++;pair(writer, "maxInactiveInterval", session.getMaxInactiveInterval());
            __CLOVER_240_0.S[6297]++;pair(writer, "new", session.isNew());

            __CLOVER_240_0.S[6298]++;List names = getSorted(session.getAttributeNames());
            __CLOVER_240_0.S[6299]++;int count = names.size();

            __CLOVER_240_0.S[6300]++;for (int i = 0; (((i < count) && (++__CLOVER_240_0.CT[1114] != 0)) || (++__CLOVER_240_0.CF[1114] == 0)); i++){
            {
                __CLOVER_240_0.S[6301]++;if ((((i == 0) && (++__CLOVER_240_0.CT[1115] != 0)) || (++__CLOVER_240_0.CF[1115] == 0))){
                {
                    __CLOVER_240_0.S[6302]++;section(writer, "Attributes");
                    __CLOVER_240_0.S[6303]++;header(writer, "Name", "Value");
                }}

                __CLOVER_240_0.S[6304]++;String name = (String) names.get(i);
                __CLOVER_240_0.S[6305]++;pair(writer, name, session.getAttribute(name));
            }}

            __CLOVER_240_0.S[6306]++;writer.end(); // Session

        }}

        __CLOVER_240_0.S[6307]++;object(writer, "Request");
        __CLOVER_240_0.S[6308]++;writer.begin("table");
        __CLOVER_240_0.S[6309]++;writer.attribute("class", "request-context-object");

        // Parameters ...

        __CLOVER_240_0.S[6310]++;List parameters = getSorted(_request.getParameterNames());
        __CLOVER_240_0.S[6311]++;int count = parameters.size();

        __CLOVER_240_0.S[6312]++;for (int i = 0; (((i < count) && (++__CLOVER_240_0.CT[1116] != 0)) || (++__CLOVER_240_0.CF[1116] == 0)); i++){
        {

            __CLOVER_240_0.S[6313]++;if ((((i == 0) && (++__CLOVER_240_0.CT[1117] != 0)) || (++__CLOVER_240_0.CF[1117] == 0))){
            {
                __CLOVER_240_0.S[6314]++;section(writer, "Parameters");
                __CLOVER_240_0.S[6315]++;header(writer, "Name", "Value(s)");
            }}

            __CLOVER_240_0.S[6316]++;String name = (String) parameters.get(i);
            __CLOVER_240_0.S[6317]++;String[] values = _request.getParameterValues(name);

            __CLOVER_240_0.S[6318]++;writer.begin("tr");
            __CLOVER_240_0.S[6319]++;writer.attribute("class", getRowClass());
            __CLOVER_240_0.S[6320]++;writer.begin("th");
            __CLOVER_240_0.S[6321]++;writer.print(name);
            __CLOVER_240_0.S[6322]++;writer.end();
            __CLOVER_240_0.S[6323]++;writer.begin("td");

            __CLOVER_240_0.S[6324]++;if ((((values.length > 1) && (++__CLOVER_240_0.CT[1118] != 0)) || (++__CLOVER_240_0.CF[1118] == 0))){
                __CLOVER_240_0.S[6325]++;writer.begin("ul");}

            __CLOVER_240_0.S[6326]++;for (int j = 0; (((j < values.length) && (++__CLOVER_240_0.CT[1119] != 0)) || (++__CLOVER_240_0.CF[1119] == 0)); j++){
            {
                __CLOVER_240_0.S[6327]++;if ((((values.length > 1) && (++__CLOVER_240_0.CT[1120] != 0)) || (++__CLOVER_240_0.CF[1120] == 0))){
                    __CLOVER_240_0.S[6328]++;writer.beginEmpty("li");}

                __CLOVER_240_0.S[6329]++;writer.print(values[j]);

            }}

            __CLOVER_240_0.S[6330]++;writer.end("tr");
        }}

        __CLOVER_240_0.S[6331]++;section(writer, "Properties");
        __CLOVER_240_0.S[6332]++;header(writer, "Name", "Value");

        __CLOVER_240_0.S[6333]++;pair(writer, "authType", _request.getAuthType());
        __CLOVER_240_0.S[6334]++;pair(writer, "characterEncoding", _request.getCharacterEncoding());
        __CLOVER_240_0.S[6335]++;pair(writer, "contentLength", _request.getContentLength());
        __CLOVER_240_0.S[6336]++;pair(writer, "contentType", _request.getContentType());
        __CLOVER_240_0.S[6337]++;pair(writer, "method", _request.getMethod());
        __CLOVER_240_0.S[6338]++;pair(writer, "pathInfo", _request.getPathInfo());
        __CLOVER_240_0.S[6339]++;pair(writer, "pathTranslated", _request.getPathTranslated());
        __CLOVER_240_0.S[6340]++;pair(writer, "protocol", _request.getProtocol());
        __CLOVER_240_0.S[6341]++;pair(writer, "queryString", _request.getQueryString());
        __CLOVER_240_0.S[6342]++;pair(writer, "remoteAddr", _request.getRemoteAddr());
        __CLOVER_240_0.S[6343]++;pair(writer, "remoteHost", _request.getRemoteHost());
        __CLOVER_240_0.S[6344]++;pair(writer, "remoteUser", _request.getRemoteUser());
        __CLOVER_240_0.S[6345]++;pair(writer, "requestedSessionId", _request.getRequestedSessionId());
        __CLOVER_240_0.S[6346]++;pair(writer, "requestedSessionIdFromCookie", _request.isRequestedSessionIdFromCookie());
        __CLOVER_240_0.S[6347]++;pair(writer, "requestedSessionIdFromURL", _request.isRequestedSessionIdFromURL());
        __CLOVER_240_0.S[6348]++;pair(writer, "requestedSessionIdValid", _request.isRequestedSessionIdValid());
        __CLOVER_240_0.S[6349]++;pair(writer, "requestURI", _request.getRequestURI());
        __CLOVER_240_0.S[6350]++;pair(writer, "scheme", _request.getScheme());
        __CLOVER_240_0.S[6351]++;pair(writer, "serverName", _request.getServerName());
        __CLOVER_240_0.S[6352]++;pair(writer, "serverPort", _request.getServerPort());
        __CLOVER_240_0.S[6353]++;pair(writer, "contextPath", _request.getContextPath());
        __CLOVER_240_0.S[6354]++;pair(writer, "servletPath", _request.getServletPath());

        // Now deal with any headers

        __CLOVER_240_0.S[6355]++;List headers = getSorted(_request.getHeaderNames());
        __CLOVER_240_0.S[6356]++;count = headers.size();

        __CLOVER_240_0.S[6357]++;for (int i = 0; (((i < count) && (++__CLOVER_240_0.CT[1121] != 0)) || (++__CLOVER_240_0.CF[1121] == 0)); i++){
        {
            __CLOVER_240_0.S[6358]++;if ((((i == 0) && (++__CLOVER_240_0.CT[1122] != 0)) || (++__CLOVER_240_0.CF[1122] == 0))){
            {
                __CLOVER_240_0.S[6359]++;section(writer, "Headers");
                __CLOVER_240_0.S[6360]++;header(writer, "Name", "Value");
            }}

            __CLOVER_240_0.S[6361]++;String name = (String) headers.get(i);
            __CLOVER_240_0.S[6362]++;String value = _request.getHeader(name);

            __CLOVER_240_0.S[6363]++;pair(writer, name, value);
        }}

        // Attributes

        __CLOVER_240_0.S[6364]++;List attributes = getSorted(_request.getAttributeNames());
        __CLOVER_240_0.S[6365]++;count = attributes.size();

        __CLOVER_240_0.S[6366]++;for (int i = 0; (((i < count) && (++__CLOVER_240_0.CT[1123] != 0)) || (++__CLOVER_240_0.CF[1123] == 0)); i++){
        {
            __CLOVER_240_0.S[6367]++;if ((((i == 0) && (++__CLOVER_240_0.CT[1124] != 0)) || (++__CLOVER_240_0.CF[1124] == 0))){
            {
                __CLOVER_240_0.S[6368]++;section(writer, "Attributes");
                __CLOVER_240_0.S[6369]++;header(writer, "Name", "Value");
            }}

            __CLOVER_240_0.S[6370]++;String name = (String) attributes.get(i);

            __CLOVER_240_0.S[6371]++;pair(writer, name, _request.getAttribute(name));
        }}

        // Cookies ...

        __CLOVER_240_0.S[6372]++;Cookie[] cookies = _request.getCookies();

        __CLOVER_240_0.S[6373]++;if ((((cookies != null) && (++__CLOVER_240_0.CT[1125] != 0)) || (++__CLOVER_240_0.CF[1125] == 0))){
        {
            __CLOVER_240_0.S[6374]++;for (int i = 0; (((i < cookies.length) && (++__CLOVER_240_0.CT[1126] != 0)) || (++__CLOVER_240_0.CF[1126] == 0)); i++){
            {

                __CLOVER_240_0.S[6375]++;if ((((i == 0) && (++__CLOVER_240_0.CT[1127] != 0)) || (++__CLOVER_240_0.CF[1127] == 0))){
                {
                    __CLOVER_240_0.S[6376]++;section(writer, "Cookies");
                    __CLOVER_240_0.S[6377]++;header(writer, "Name", "Value");
                }}

                __CLOVER_240_0.S[6378]++;Cookie cookie = cookies[i];

                __CLOVER_240_0.S[6379]++;pair(writer, cookie.getName(), cookie.getValue());

            }} // Cookies loop
        }}

        __CLOVER_240_0.S[6380]++;writer.end(); // Request

        __CLOVER_240_0.S[6381]++;object(writer, "Servlet");
        __CLOVER_240_0.S[6382]++;writer.begin("table");
        __CLOVER_240_0.S[6383]++;writer.attribute("class", "request-context-object");

        __CLOVER_240_0.S[6384]++;section(writer, "Properties");
        __CLOVER_240_0.S[6385]++;header(writer, "Name", "Value");

        __CLOVER_240_0.S[6386]++;pair(writer, "servlet", _servlet);
        __CLOVER_240_0.S[6387]++;pair(writer, "name", _servlet.getServletName());
        __CLOVER_240_0.S[6388]++;pair(writer, "servletInfo", _servlet.getServletInfo());

        __CLOVER_240_0.S[6389]++;ServletConfig config = _servlet.getServletConfig();

        __CLOVER_240_0.S[6390]++;List names = getSorted(config.getInitParameterNames());
        __CLOVER_240_0.S[6391]++;count = names.size();

        __CLOVER_240_0.S[6392]++;for (int i = 0; (((i < count) && (++__CLOVER_240_0.CT[1128] != 0)) || (++__CLOVER_240_0.CF[1128] == 0)); i++){
        {

            __CLOVER_240_0.S[6393]++;if ((((i == 0) && (++__CLOVER_240_0.CT[1129] != 0)) || (++__CLOVER_240_0.CF[1129] == 0))){
            {
                __CLOVER_240_0.S[6394]++;section(writer, "Init Parameters");
                __CLOVER_240_0.S[6395]++;header(writer, "Name", "Value");
            }}

            __CLOVER_240_0.S[6396]++;String name = (String) names.get(i);
            ;
            __CLOVER_240_0.S[6397]++;pair(writer, name, config.getInitParameter(name));

        }}

        __CLOVER_240_0.S[6398]++;writer.end(); // Servlet

        __CLOVER_240_0.S[6399]++;ServletContext context = config.getServletContext();

        __CLOVER_240_0.S[6400]++;object(writer, "Servlet Context");
        __CLOVER_240_0.S[6401]++;writer.begin("table");
        __CLOVER_240_0.S[6402]++;writer.attribute("class", "request-context-object");

        __CLOVER_240_0.S[6403]++;section(writer, "Properties");
        __CLOVER_240_0.S[6404]++;header(writer, "Name", "Value");

        __CLOVER_240_0.S[6405]++;pair(writer, "majorVersion", context.getMajorVersion());
        __CLOVER_240_0.S[6406]++;pair(writer, "minorVersion", context.getMinorVersion());
        __CLOVER_240_0.S[6407]++;pair(writer, "serverInfo", context.getServerInfo());

        __CLOVER_240_0.S[6408]++;names = getSorted(context.getInitParameterNames());
        __CLOVER_240_0.S[6409]++;count = names.size();
        __CLOVER_240_0.S[6410]++;for (int i = 0; (((i < count) && (++__CLOVER_240_0.CT[1130] != 0)) || (++__CLOVER_240_0.CF[1130] == 0)); i++){
        {
            __CLOVER_240_0.S[6411]++;if ((((i == 0) && (++__CLOVER_240_0.CT[1131] != 0)) || (++__CLOVER_240_0.CF[1131] == 0))){
            {
                __CLOVER_240_0.S[6412]++;section(writer, "Initial Parameters");
                __CLOVER_240_0.S[6413]++;header(writer, "Name", "Value");
            }}

            __CLOVER_240_0.S[6414]++;String name = (String) names.get(i);
            __CLOVER_240_0.S[6415]++;pair(writer, name, context.getInitParameter(name));
        }}

        __CLOVER_240_0.S[6416]++;names = getSorted(context.getAttributeNames());
        __CLOVER_240_0.S[6417]++;count = names.size();
        __CLOVER_240_0.S[6418]++;for (int i = 0; (((i < count) && (++__CLOVER_240_0.CT[1132] != 0)) || (++__CLOVER_240_0.CF[1132] == 0)); i++){
        {
            __CLOVER_240_0.S[6419]++;if ((((i == 0) && (++__CLOVER_240_0.CT[1133] != 0)) || (++__CLOVER_240_0.CF[1133] == 0))){
            {
                __CLOVER_240_0.S[6420]++;section(writer, "Attributes");
                __CLOVER_240_0.S[6421]++;header(writer, "Name", "Value");
            }}

            __CLOVER_240_0.S[6422]++;String name = (String) names.get(i);
            __CLOVER_240_0.S[6423]++;pair(writer, name, context.getAttribute(name));
        }}

        __CLOVER_240_0.S[6424]++;writer.end(); // Servlet Context

        __CLOVER_240_0.S[6425]++;writeSystemProperties(writer);

        __CLOVER_240_0.S[6426]++;writer.end("table"); // The enclosing border
    } finally { }}

    private void writeSystemProperties(IMarkupWriter writer)
    {try { __CLOVER_240_0.M[1301]++;
        __CLOVER_240_0.S[6427]++;Properties properties = null;

        __CLOVER_240_0.S[6428]++;object(writer, "JVM System Properties");

        __CLOVER_240_0.S[6429]++;try
        {
            __CLOVER_240_0.S[6430]++;properties = System.getProperties();
        }
        catch (SecurityException se)
        {
            __CLOVER_240_0.S[6431]++;writer.print("<p>");
            __CLOVER_240_0.S[6432]++;writer.print(se.toString());
            __CLOVER_240_0.S[6433]++;return;
        }

        __CLOVER_240_0.S[6434]++;String pathSeparator = System.getProperty("path.separator", ";");

        __CLOVER_240_0.S[6435]++;writer.begin("table");
        __CLOVER_240_0.S[6436]++;writer.attribute("class", "request-context-object");

        __CLOVER_240_0.S[6437]++;List names = new ArrayList(properties.keySet());
        __CLOVER_240_0.S[6438]++;Collections.sort(names);
        __CLOVER_240_0.S[6439]++;int count = names.size();

        __CLOVER_240_0.S[6440]++;for (int i = 0; (((i < count) && (++__CLOVER_240_0.CT[1134] != 0)) || (++__CLOVER_240_0.CF[1134] == 0)); i++){
        {

            __CLOVER_240_0.S[6441]++;if ((((i == 0) && (++__CLOVER_240_0.CT[1135] != 0)) || (++__CLOVER_240_0.CF[1135] == 0))){
                __CLOVER_240_0.S[6442]++;header(writer, "Name", "Value");}

            __CLOVER_240_0.S[6443]++;String name = (String) names.get(i);

            __CLOVER_240_0.S[6444]++;String property = properties.getProperty(name);

            __CLOVER_240_0.S[6445]++;if ((((property != null && property.indexOf(pathSeparator) > 0 && name.endsWith(".path")) && (++__CLOVER_240_0.CT[1136] != 0)) || (++__CLOVER_240_0.CF[1136] == 0))){
            {
                __CLOVER_240_0.S[6446]++;writer.begin("tr");
                __CLOVER_240_0.S[6447]++;writer.attribute("class", getRowClass());

                __CLOVER_240_0.S[6448]++;writer.begin("th");
                __CLOVER_240_0.S[6449]++;writer.print(name);
                __CLOVER_240_0.S[6450]++;writer.end();

                __CLOVER_240_0.S[6451]++;writer.begin("td");
                __CLOVER_240_0.S[6452]++;writer.begin("ul");

                __CLOVER_240_0.S[6453]++;StringTokenizer tokenizer = new StringTokenizer(property, pathSeparator);

                __CLOVER_240_0.S[6454]++;while ((((tokenizer.hasMoreTokens()) && (++__CLOVER_240_0.CT[1137] != 0)) || (++__CLOVER_240_0.CF[1137] == 0))){
                {
                    __CLOVER_240_0.S[6455]++;writer.beginEmpty("li");
                    __CLOVER_240_0.S[6456]++;writer.print(tokenizer.nextToken());
                }}

                __CLOVER_240_0.S[6457]++;writer.end("tr");
            }}
            else{
            {
                __CLOVER_240_0.S[6458]++;pair(writer, name, property);
            }}
        }}

        __CLOVER_240_0.S[6459]++;writer.end(); // System Properties
    } finally { }}

    /**
     *  Invokes {@link #write(IMarkupWriter)}, which is used for debugging.
     *  Does nothing if the cycle is rewinding.
     *
     **/

    public void render(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_240_0.M[1302]++;
        __CLOVER_240_0.S[6460]++;if ((((!cycle.isRewinding()) && (++__CLOVER_240_0.CT[1138] != 0)) || (++__CLOVER_240_0.CF[1138] == 0))){
            __CLOVER_240_0.S[6461]++;write(writer);}
    } finally { }}

    /**
     *  Returns the multipart decoder and lazily decodes the request parameters.
     *  This allows both for this operation to be performed only when really needed
     *  and for opening the request for reading much later, so that the Engine can
     *  have a chance to set the encoding that the request needs to use.
     * 
     *  @return the multipart decoder or null if not needed for this request
     *  @since 3.0
     **/
    private IMultipartDecoder getDecoder()
    {try { __CLOVER_240_0.M[1303]++;
        __CLOVER_240_0.S[6462]++;if ((((_decoder != null && !_decoded) && (++__CLOVER_240_0.CT[1139] != 0)) || (++__CLOVER_240_0.CF[1139] == 0))){
        {
            __CLOVER_240_0.S[6463]++;_decoder.decode(_request);
            __CLOVER_240_0.S[6464]++;_decoded = true;
        }}

        __CLOVER_240_0.S[6465]++;return _decoder;
    } finally { }}

    /**
     *  Sets the multipart decoder to be used for the request.
     * 
     *  @param decoder the multipart decoder
     *  @since 3.0
     **/
    public void setDecoder(IMultipartDecoder decoder)
    {try { __CLOVER_240_0.M[1304]++;
        __CLOVER_240_0.S[6466]++;_decoder = decoder;
        __CLOVER_240_0.S[6467]++;_decoded = false;
    } finally { }}

    /**
     * @since 3.1 --- but probably pretty temporary!
     */
    public IApplicationSpecification getApplicationSpecification()
    {try { __CLOVER_240_0.M[1305]++;
        __CLOVER_240_0.S[6468]++;return _specification;
    } finally { }}

}