/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.resolver;

import javax.servlet.ServletContext;

import org.apache.hivemind.Resource;
import org.apache.tapestry.INamespace;
import org.apache.tapestry.engine.ISpecificationSource;
import org.apache.tapestry.resource.ContextResource;
import org.apache.tapestry.spec.IComponentSpecification;

/**
 *  Base class for resolving a {@link org.apache.tapestry.spec.IComponentSpecification}
 *  for a particular page or component, within a specified 
 *  {@link org.apache.tapestry.INamespace}.  In some cases, a search is necessary.
 *
 *  @author Howard Lewis Ship
 *  @since 3.0
 *
 */

public class AbstractSpecificationResolver
{public static com.cortexeb.tools.clover.d __CLOVER_242_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /** Set by resolve() */
    private INamespace _namespace;

    /** Set by resolve() */
    private IComponentSpecification _specification;

    /** Set by container */
    private ISpecificationSource _specificationSource;
    private ISpecificationResolverDelegate _delegate;
    private String _servletName;
    private ServletContext _context;

    /** Initializes in initializeService() */
    private Resource _applicationRootLocation;
    private Resource _webInfLocation;
    private Resource _webInfAppLocation;

    public void initializeService()
    {try { __CLOVER_242_0.M[1321]++;
        __CLOVER_242_0.S[6522]++;_applicationRootLocation = new ContextResource(_context, "/");

        __CLOVER_242_0.S[6523]++;_webInfLocation = _applicationRootLocation.getRelativeResource("WEB-INF/");

        __CLOVER_242_0.S[6524]++;_webInfAppLocation = _webInfLocation.getRelativeResource(_servletName + "/");
    } finally { }}

    /**
     *  Returns the {@link ISpecificationResolverDelegate} instance registered
     *  in the application specification as extension
     *  {@link Tapestry#SPECIFICATION_RESOLVER_DELEGATE_EXTENSION_NAME},
     *  or null if no such extension exists.
     * 
     **/

    public ISpecificationResolverDelegate getDelegate()
    {try { __CLOVER_242_0.M[1322]++;
        __CLOVER_242_0.S[6525]++;return _delegate;
    } finally { }}

    /**
     *  Returns the location of the servlet, within the
     *  servlet context.
     * 
     **/

    protected Resource getApplicationRootLocation()
    {try { __CLOVER_242_0.M[1323]++;
        __CLOVER_242_0.S[6526]++;return _applicationRootLocation;
    } finally { }}

    /**
     *  Invoked in subclasses to identify the resolved namespace.
     * 
     **/

    protected void setNamespace(INamespace namespace)
    {try { __CLOVER_242_0.M[1324]++;
        __CLOVER_242_0.S[6527]++;_namespace = namespace;
    } finally { }}

    /**
     *  Returns the resolve namespace.
     * 
     **/

    public INamespace getNamespace()
    {try { __CLOVER_242_0.M[1325]++;
        __CLOVER_242_0.S[6528]++;return _namespace;
    } finally { }}

    /**
     *  Returns the specification source for the running application.
     * 
     **/

    protected ISpecificationSource getSpecificationSource()
    {try { __CLOVER_242_0.M[1326]++;
        __CLOVER_242_0.S[6529]++;return _specificationSource;
    } finally { }}

    /**
     *  Returns the location of /WEB-INF/, in the servlet context.
     * 
     **/

    protected Resource getWebInfLocation()
    {try { __CLOVER_242_0.M[1327]++;
        __CLOVER_242_0.S[6530]++;return _webInfLocation;
    } finally { }}

    /**
     *  Returns the location of the application-specific subdirectory, under
     *  /WEB-INF/, in the servlet context.
     * 
     **/

    protected Resource getWebInfAppLocation()
    {try { __CLOVER_242_0.M[1328]++;
        __CLOVER_242_0.S[6531]++;return _webInfAppLocation;
    } finally { }}

    /**
     *  Returns the resolved specification.
     * 
     **/

    public IComponentSpecification getSpecification()
    {try { __CLOVER_242_0.M[1329]++;
        __CLOVER_242_0.S[6532]++;return _specification;
    } finally { }}

    /**
     *  Invoked in subclass to set the final specification the initial
     *  inputs are resolved to.
     * 
     **/

    protected void setSpecification(IComponentSpecification specification)
    {try { __CLOVER_242_0.M[1330]++;
        __CLOVER_242_0.S[6533]++;_specification = specification;
    } finally { }}

    /**
     *  Clears the namespace and specification properties.
     * 
     **/

    protected void reset()
    {try { __CLOVER_242_0.M[1331]++;
        __CLOVER_242_0.S[6534]++;_namespace = null;
        __CLOVER_242_0.S[6535]++;_specification = null;
    } finally { }}

    /** @since 3.1 */
    public void setDelegate(ISpecificationResolverDelegate delegate)
    {try { __CLOVER_242_0.M[1332]++;
        __CLOVER_242_0.S[6536]++;_delegate = delegate;
    } finally { }}

    /** @since 3.1 */
    public void setContext(ServletContext context)
    {try { __CLOVER_242_0.M[1333]++;
        __CLOVER_242_0.S[6537]++;_context = context;
    } finally { }}

    /** @since 3.1 */
    public void setServletName(String name)
    {try { __CLOVER_242_0.M[1334]++;
        __CLOVER_242_0.S[6538]++;_servletName = name;
    } finally { }}

    /** @since 3.1 */
    public void setSpecificationSource(ISpecificationSource source)
    {try { __CLOVER_242_0.M[1335]++;
        __CLOVER_242_0.S[6539]++;_specificationSource = source;
    } finally { }}

}
