/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.resolver;

import org.apache.commons.logging.Log;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.Resource;
import org.apache.tapestry.INamespace;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.spec.ComponentSpecification;
import org.apache.tapestry.spec.IComponentSpecification;

/**
 *  Performs the tricky work of resolving a page name to a page specification.
 *  The search for pages in the application namespace is the most complicated,
 *  since Tapestry searches for pages that aren't explicitly defined in the
 *  application specification.  The search, based on the <i>simple-name</i>
 *  of the page, goes as follows:
 * 
 *  <ul>
 *  <li>As declared in the application specification
 *  <li><i>simple-name</i>.page in the same folder as the application specification
 *  <li><i>simple-name</i> page in the WEB-INF/<i>servlet-name</i> directory of the context root
 *  <li><i>simple-name</i>.page in WEB-INF
 *  <li><i>simple-name</i>.page in the application root (within the context root)
 *  <li><i>simple-name</i>.html as a template in the application root, 
 *      for which an implicit specification is generated
 *  <li>By searching the framework namespace
 *  <li>By invoking {@link org.apache.tapestry.resolver.ISpecificationResolverDelegate#findPageSpecification(IRequestCycle, INamespace, String)}
 *  </ul>
 * 
 *  <p>Pages in a component library are searched for in a more abbreviated fashion:
 *  <ul>
 *  <li>As declared in the library specification
 *  <li><i>simple-name</i>.page in the same folder as the library specification
 *  <li>By searching the framework namespace
 *  <li>By invoking {@link org.apache.tapestry.resolver.ISpecificationResolverDelegate#findPageSpecification(IRequestCycle, INamespace, String)}
 *  </ul>
 *
 *  @see org.apache.tapestry.engine.IPageSource
 *  @author Howard Lewis Ship
 *  @since 3.0
 *
 */

public class PageSpecificationResolverImpl
    extends AbstractSpecificationResolver
    implements PageSpecificationResolver
{public static com.cortexeb.tools.clover.d __CLOVER_247_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /** set by container */
    private Log _log;

    /** Set by resolve() */
    private String _simpleName;

    /** @since 3.1 **/
    private INamespace _applicationNamespace;

    /** @since 3.1 **/
    private INamespace _frameworkNamespace;

    public void initializeService()
    {try { __CLOVER_247_0.M[1343]++;
        __CLOVER_247_0.S[6593]++;_applicationNamespace = getSpecificationSource().getApplicationNamespace();
        __CLOVER_247_0.S[6594]++;_frameworkNamespace = getSpecificationSource().getFrameworkNamespace();

        __CLOVER_247_0.S[6595]++;super.initializeService();
    } finally { }}

    protected void reset()
    {try { __CLOVER_247_0.M[1344]++;
        __CLOVER_247_0.S[6596]++;_simpleName = null;

        __CLOVER_247_0.S[6597]++;super.reset();
    } finally { }}

    /**
     *  Resolve the name (which may have a library id prefix) to a namespace
     *  (see {@link #getNamespace()}) and a specification (see {@link #getSpecification()}).
     * 
     *  @throws ApplicationRuntimeException if the name cannot be resolved
     * 
     */

    public void resolve(IRequestCycle cycle, String prefixedName)
    {try { __CLOVER_247_0.M[1345]++;
        __CLOVER_247_0.S[6598]++;reset();

        __CLOVER_247_0.S[6599]++;INamespace namespace = null;

        __CLOVER_247_0.S[6600]++;int colonx = prefixedName.indexOf(':');

        __CLOVER_247_0.S[6601]++;if ((((colonx > 0) && (++__CLOVER_247_0.CT[1165] != 0)) || (++__CLOVER_247_0.CF[1165] == 0))){
        {
            __CLOVER_247_0.S[6602]++;_simpleName = prefixedName.substring(colonx + 1);
            __CLOVER_247_0.S[6603]++;String namespaceId = prefixedName.substring(0, colonx);

            __CLOVER_247_0.S[6604]++;if ((((namespaceId.equals(INamespace.FRAMEWORK_NAMESPACE)) && (++__CLOVER_247_0.CT[1166] != 0)) || (++__CLOVER_247_0.CF[1166] == 0))){
                __CLOVER_247_0.S[6605]++;namespace = _frameworkNamespace;}
            else{
                __CLOVER_247_0.S[6606]++;namespace = _applicationNamespace.getChildNamespace(namespaceId);}
        }}
        else{
        {
            __CLOVER_247_0.S[6607]++;_simpleName = prefixedName;

            __CLOVER_247_0.S[6608]++;namespace = getSpecificationSource().getApplicationNamespace();
        }}

        __CLOVER_247_0.S[6609]++;setNamespace(namespace);

        __CLOVER_247_0.S[6610]++;if ((((namespace.containsPage(_simpleName)) && (++__CLOVER_247_0.CT[1167] != 0)) || (++__CLOVER_247_0.CF[1167] == 0))){
        {
            __CLOVER_247_0.S[6611]++;setSpecification(namespace.getPageSpecification(_simpleName));
            __CLOVER_247_0.S[6612]++;return;
        }}

        // Not defined in the specification, so it's time to hunt it down.

        __CLOVER_247_0.S[6613]++;searchForPage(cycle);

        __CLOVER_247_0.S[6614]++;if ((((getSpecification() == null) && (++__CLOVER_247_0.CT[1168] != 0)) || (++__CLOVER_247_0.CF[1168] == 0))){
            __CLOVER_247_0.S[6615]++;throw new ApplicationRuntimeException(
                ResolverMessages.noSuchPage(_simpleName, namespace));}
    } finally { }}

    public String getSimplePageName()
    {try { __CLOVER_247_0.M[1346]++;
        __CLOVER_247_0.S[6616]++;return _simpleName;
    } finally { }}

    private void searchForPage(IRequestCycle cycle)
    {try { __CLOVER_247_0.M[1347]++;
        __CLOVER_247_0.S[6617]++;INamespace namespace = getNamespace();

        __CLOVER_247_0.S[6618]++;if ((((_log.isDebugEnabled()) && (++__CLOVER_247_0.CT[1169] != 0)) || (++__CLOVER_247_0.CF[1169] == 0))){
            __CLOVER_247_0.S[6619]++;_log.debug("Resolving unknown page '" + _simpleName + "' in " + namespace);}

        __CLOVER_247_0.S[6620]++;String expectedName = _simpleName + ".page";

        __CLOVER_247_0.S[6621]++;Resource namespaceLocation = namespace.getSpecificationLocation();

        // See if there's a specification file in the same folder
        // as the library or application specification that's
        // supposed to contain the page.

        __CLOVER_247_0.S[6622]++;if ((((found(namespaceLocation.getRelativeResource(expectedName))) && (++__CLOVER_247_0.CT[1170] != 0)) || (++__CLOVER_247_0.CF[1170] == 0))){
            __CLOVER_247_0.S[6623]++;return;}

        __CLOVER_247_0.S[6624]++;if ((((namespace.isApplicationNamespace()) && (++__CLOVER_247_0.CT[1171] != 0)) || (++__CLOVER_247_0.CF[1171] == 0))){
        {

            // The application namespace gets some extra searching.

            __CLOVER_247_0.S[6625]++;if ((((found(getWebInfAppLocation().getRelativeResource(expectedName))) && (++__CLOVER_247_0.CT[1172] != 0)) || (++__CLOVER_247_0.CF[1172] == 0))){
                __CLOVER_247_0.S[6626]++;return;}

            __CLOVER_247_0.S[6627]++;if ((((found(getWebInfLocation().getRelativeResource(expectedName))) && (++__CLOVER_247_0.CT[1173] != 0)) || (++__CLOVER_247_0.CF[1173] == 0))){
                __CLOVER_247_0.S[6628]++;return;}

            __CLOVER_247_0.S[6629]++;if ((((found(getApplicationRootLocation().getRelativeResource(expectedName))) && (++__CLOVER_247_0.CT[1174] != 0)) || (++__CLOVER_247_0.CF[1174] == 0))){
                __CLOVER_247_0.S[6630]++;return;}

            // The wierd one ... where we see if there's a template in the application root location.

            __CLOVER_247_0.S[6631]++;String templateName = _simpleName + "." + getTemplateExtension();

            __CLOVER_247_0.S[6632]++;Resource templateResource =
                getApplicationRootLocation().getRelativeResource(templateName);

            __CLOVER_247_0.S[6633]++;if ((((templateResource.getResourceURL() != null) && (++__CLOVER_247_0.CT[1175] != 0)) || (++__CLOVER_247_0.CF[1175] == 0))){
            {
                __CLOVER_247_0.S[6634]++;setupImplicitPage(templateResource);
                __CLOVER_247_0.S[6635]++;return;
            }}

            // Not found in application namespace, so maybe its a framework page.

            __CLOVER_247_0.S[6636]++;if ((((_frameworkNamespace.containsPage(_simpleName)) && (++__CLOVER_247_0.CT[1176] != 0)) || (++__CLOVER_247_0.CF[1176] == 0))){
            {
                __CLOVER_247_0.S[6637]++;if ((((_log.isDebugEnabled()) && (++__CLOVER_247_0.CT[1177] != 0)) || (++__CLOVER_247_0.CF[1177] == 0))){
                    __CLOVER_247_0.S[6638]++;_log.debug("Found " + _simpleName + " in framework namespace.");}

                __CLOVER_247_0.S[6639]++;setNamespace(_frameworkNamespace);

                // Note:  This implies that normal lookup rules don't work
                // for the framework!  Framework pages must be
                // defined in the framework library specification.

                __CLOVER_247_0.S[6640]++;setSpecification(_frameworkNamespace.getPageSpecification(_simpleName));
                __CLOVER_247_0.S[6641]++;return;
            }}
        }}

        // Not found by any normal rule, so its time to
        // consult the delegate.

        __CLOVER_247_0.S[6642]++;IComponentSpecification specification =
            getDelegate().findPageSpecification(cycle, namespace, _simpleName);

        __CLOVER_247_0.S[6643]++;setSpecification(specification);
    } finally { }}

    private void setupImplicitPage(Resource resource)
    {try { __CLOVER_247_0.M[1348]++;
        __CLOVER_247_0.S[6644]++;if ((((_log.isDebugEnabled()) && (++__CLOVER_247_0.CT[1178] != 0)) || (++__CLOVER_247_0.CF[1178] == 0))){
            __CLOVER_247_0.S[6645]++;_log.debug("Found HTML template at " + resource);}

        // TODO The SpecFactory in Specification parser should be used in some way to 
        // create an IComponentSpecification!

        __CLOVER_247_0.S[6646]++;IComponentSpecification specification = new ComponentSpecification();
        __CLOVER_247_0.S[6647]++;specification.setPageSpecification(true);
        __CLOVER_247_0.S[6648]++;specification.setSpecificationLocation(resource);

        __CLOVER_247_0.S[6649]++;setSpecification(specification);

        __CLOVER_247_0.S[6650]++;install();
    } finally { }}

    private boolean found(Resource resource)
    {try { __CLOVER_247_0.M[1349]++;
        __CLOVER_247_0.S[6651]++;if ((((_log.isDebugEnabled()) && (++__CLOVER_247_0.CT[1179] != 0)) || (++__CLOVER_247_0.CF[1179] == 0))){
            __CLOVER_247_0.S[6652]++;_log.debug("Checking: " + resource);}

        __CLOVER_247_0.S[6653]++;if ((((resource.getResourceURL() == null) && (++__CLOVER_247_0.CT[1180] != 0)) || (++__CLOVER_247_0.CF[1180] == 0))){
            __CLOVER_247_0.S[6654]++;return false;}

        __CLOVER_247_0.S[6655]++;setSpecification(getSpecificationSource().getPageSpecification(resource));

        __CLOVER_247_0.S[6656]++;install();

        __CLOVER_247_0.S[6657]++;return true;
    } finally { }}

    private void install()
    {try { __CLOVER_247_0.M[1350]++;
        __CLOVER_247_0.S[6658]++;INamespace namespace = getNamespace();
        __CLOVER_247_0.S[6659]++;IComponentSpecification specification = getSpecification();

        __CLOVER_247_0.S[6660]++;if ((((_log.isDebugEnabled()) && (++__CLOVER_247_0.CT[1181] != 0)) || (++__CLOVER_247_0.CF[1181] == 0))){
            __CLOVER_247_0.S[6661]++;_log.debug(
                "Installing page " + _simpleName + " into " + namespace + " as " + specification);}

        __CLOVER_247_0.S[6662]++;namespace.installPageSpecification(_simpleName, specification);
    } finally { }}

    /**
     *  If the namespace defines the template extension (as property
     *  {@link Tapestry#TEMPLATE_EXTENSION_PROPERTY}, then that is used, otherwise
     *  the default is used.
     * 
     */

    private String getTemplateExtension()
    {try { __CLOVER_247_0.M[1351]++;
        __CLOVER_247_0.S[6663]++;String extension =
            getNamespace().getSpecification().getProperty(Tapestry.TEMPLATE_EXTENSION_PROPERTY);

        __CLOVER_247_0.S[6664]++;if ((((extension == null) && (++__CLOVER_247_0.CT[1182] != 0)) || (++__CLOVER_247_0.CF[1182] == 0))){
            __CLOVER_247_0.S[6665]++;extension = Tapestry.DEFAULT_TEMPLATE_EXTENSION;}

        __CLOVER_247_0.S[6666]++;return extension;
    } finally { }}

    /** @since 3.1 */

    public void setLog(Log log)
    {try { __CLOVER_247_0.M[1352]++;
        __CLOVER_247_0.S[6667]++;_log = log;
    } finally { }}

}
