/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.services.impl;

import java.util.HashMap;
import java.util.Map;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.ClassResolver;
import org.apache.hivemind.Resource;
import org.apache.hivemind.util.ClasspathResource;
import org.apache.tapestry.INamespace;
import org.apache.tapestry.engine.ISpecificationSource;
import org.apache.tapestry.engine.Namespace;
import org.apache.tapestry.event.ResetEventListener;
import org.apache.tapestry.parse.ISpecificationParser;
import org.apache.tapestry.spec.IApplicationSpecification;
import org.apache.tapestry.spec.IComponentSpecification;
import org.apache.tapestry.spec.ILibrarySpecification;
import org.apache.tapestry.spec.LibrarySpecification;
import org.apache.tapestry.util.xml.DocumentParseException;

/**
 *  Default implementation of {@link ISpecificationSource} that
 *  expects to use the normal class loader to locate component
 *  specifications from within the classpath.
 *
 * <p>Caches specifications in memory forever, or until {@link #resetDidOccur()()} is invoked.
 *
 *
 * @author Howard Lewis Ship
 * 
 */

public class SpecificationSourceImpl implements ISpecificationSource, ResetEventListener
{public static com.cortexeb.tools.clover.d __CLOVER_335_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private ClassResolver _classResolver;

    private IApplicationSpecification _specification;
    private ISpecificationParser _parser;

    private INamespace _applicationNamespace;
    private INamespace _frameworkNamespace;

    /**
     *  Contains previously parsed component specifications.
     *
     */

    private Map _componentCache = new HashMap();

    /**
     *  Contains previously parsed page specifications.
     * 
     *  @since 2.2
     * 
     */

    private Map _pageCache = new HashMap();

    /**
     *  Contains previously parsed library specifications, keyed
     *  on specification resource path.
     * 
     *  @since 2.2
     * 
     */

    private Map _libraryCache = new HashMap();

    /**
     *  Contains {@link INamespace} instances, keyed on id (which will
     *  be null for the application specification).
     * 
     */

    private Map _namespaceCache = new HashMap();

    /**
     *  Clears the specification cache.  This is used during debugging.
     *
     */

    public synchronized void resetEventDidOccur()
    {try { __CLOVER_335_0.M[1628]++;
        __CLOVER_335_0.S[7518]++;_componentCache.clear();
        __CLOVER_335_0.S[7519]++;_pageCache.clear();
        __CLOVER_335_0.S[7520]++;_libraryCache.clear();
        __CLOVER_335_0.S[7521]++;_namespaceCache.clear();

        __CLOVER_335_0.S[7522]++;_applicationNamespace = null;
        __CLOVER_335_0.S[7523]++;_frameworkNamespace = null;
    } finally { }}

    protected IComponentSpecification parseSpecification(Resource resource, boolean asPage)
    {try { __CLOVER_335_0.M[1629]++;
        __CLOVER_335_0.S[7524]++;IComponentSpecification result = null;

        __CLOVER_335_0.S[7525]++;try
        {
            __CLOVER_335_0.S[7526]++;if ((((asPage) && (++__CLOVER_335_0.CT[1298] != 0)) || (++__CLOVER_335_0.CF[1298] == 0))){
                __CLOVER_335_0.S[7527]++;result = _parser.parsePageSpecification(resource);}
            else{
                __CLOVER_335_0.S[7528]++;result = _parser.parseComponentSpecification(resource);}
        }
        catch (DocumentParseException ex)
        {
            __CLOVER_335_0.S[7529]++;throw new ApplicationRuntimeException(
                ImplMessages.unableToParseSpecification(resource),
                ex);
        }

        __CLOVER_335_0.S[7530]++;return result;
    } finally { }}

    protected ILibrarySpecification parseLibrarySpecification(Resource resource)
    {try { __CLOVER_335_0.M[1630]++;
        __CLOVER_335_0.S[7531]++;try
        {
            __CLOVER_335_0.S[7532]++;return _parser.parseLibrarySpecification(resource);
        }
        catch (DocumentParseException ex)
        {
            __CLOVER_335_0.S[7533]++;throw new ApplicationRuntimeException(
                ImplMessages.unableToParseSpecification(resource),
                ex);
        }

    } finally { }}

    /**
     *  Gets a component specification.
     * 
     *  @param resourcePath the complete resource path to the specification.
     *  @throws ApplicationRuntimeException if the specification cannot be obtained.
     * 
     */

    public synchronized IComponentSpecification getComponentSpecification(Resource resourceLocation)
    {try { __CLOVER_335_0.M[1631]++;
        __CLOVER_335_0.S[7534]++;IComponentSpecification result =
            (IComponentSpecification) _componentCache.get(resourceLocation);

        __CLOVER_335_0.S[7535]++;if ((((result == null) && (++__CLOVER_335_0.CT[1299] != 0)) || (++__CLOVER_335_0.CF[1299] == 0))){
        {
            __CLOVER_335_0.S[7536]++;result = parseSpecification(resourceLocation, false);

            __CLOVER_335_0.S[7537]++;_componentCache.put(resourceLocation, result);
        }}

        __CLOVER_335_0.S[7538]++;return result;
    } finally { }}

    public synchronized IComponentSpecification getPageSpecification(Resource resourceLocation)
    {try { __CLOVER_335_0.M[1632]++;
        __CLOVER_335_0.S[7539]++;IComponentSpecification result = (IComponentSpecification) _pageCache.get(resourceLocation);

        __CLOVER_335_0.S[7540]++;if ((((result == null) && (++__CLOVER_335_0.CT[1300] != 0)) || (++__CLOVER_335_0.CF[1300] == 0))){
        {
            __CLOVER_335_0.S[7541]++;result = parseSpecification(resourceLocation, true);

            __CLOVER_335_0.S[7542]++;_pageCache.put(resourceLocation, result);
        }}

        __CLOVER_335_0.S[7543]++;return result;
    } finally { }}

    public synchronized ILibrarySpecification getLibrarySpecification(Resource resourceLocation)
    {try { __CLOVER_335_0.M[1633]++;
        __CLOVER_335_0.S[7544]++;ILibrarySpecification result = (LibrarySpecification) _libraryCache.get(resourceLocation);

        __CLOVER_335_0.S[7545]++;if ((((result == null) && (++__CLOVER_335_0.CT[1301] != 0)) || (++__CLOVER_335_0.CF[1301] == 0))){
        {
            __CLOVER_335_0.S[7546]++;result = parseLibrarySpecification(resourceLocation);
            __CLOVER_335_0.S[7547]++;_libraryCache.put(resourceLocation, result);
        }}

        __CLOVER_335_0.S[7548]++;return result;
    } finally { }}

    public synchronized INamespace getApplicationNamespace()
    {try { __CLOVER_335_0.M[1634]++;
        __CLOVER_335_0.S[7549]++;if ((((_applicationNamespace == null) && (++__CLOVER_335_0.CT[1302] != 0)) || (++__CLOVER_335_0.CF[1302] == 0))){
            __CLOVER_335_0.S[7550]++;_applicationNamespace = new Namespace(null, null, _specification, this);}

        __CLOVER_335_0.S[7551]++;return _applicationNamespace;
    } finally { }}

    public synchronized INamespace getFrameworkNamespace()
    {try { __CLOVER_335_0.M[1635]++;
        __CLOVER_335_0.S[7552]++;if ((((_frameworkNamespace == null) && (++__CLOVER_335_0.CT[1303] != 0)) || (++__CLOVER_335_0.CF[1303] == 0))){
        {
            __CLOVER_335_0.S[7553]++;Resource frameworkLocation =
                new ClasspathResource(_classResolver, "/org/apache/tapestry/Framework.library");

            __CLOVER_335_0.S[7554]++;ILibrarySpecification ls = getLibrarySpecification(frameworkLocation);

            __CLOVER_335_0.S[7555]++;_frameworkNamespace = new Namespace(INamespace.FRAMEWORK_NAMESPACE, null, ls, this);
        }}

        __CLOVER_335_0.S[7556]++;return _frameworkNamespace;
    } finally { }}

    public void setParser(ISpecificationParser parser)
    {try { __CLOVER_335_0.M[1636]++;
        __CLOVER_335_0.S[7557]++;_parser = parser;
    } finally { }}

    public void setClassResolver(ClassResolver resolver)
    {try { __CLOVER_335_0.M[1637]++;
        __CLOVER_335_0.S[7558]++;_classResolver = resolver;
    } finally { }}

    public void setSpecification(IApplicationSpecification specification)
    {try { __CLOVER_335_0.M[1638]++;
        __CLOVER_335_0.S[7559]++;_specification = specification;
    } finally { }}
}