/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.spec;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.hivemind.Resource;
import org.apache.tapestry.Tapestry;

/**
 *  A specification for a component, as read from an XML specification file.
 *
 *  <p>A specification consists of
 *  <ul>
 *  <li>An implementing class
 *  <li>An optional template
 *  <li>An optional description
 *  <li>A set of contained components
 *  <li>Bindings for the properties of each contained component
 *  <li>A set of named assets
 *  <li>Definitions for helper beans
 *  <li>Any reserved names (used for HTML attributes)
 *  </ul>
 *
 *  <p>From this information, an actual component may be instantiated and
 *  initialized.  Instantiating a component is usually a recursive process, since
 *  to initialize a container component, it is necessary to instantiate and initialize
 *  its contained components as well.
 *
 *  @see org.apache.tapestry.IComponent
 *  @see IContainedComponent
 *  @see org.apache.tapestry.engine.IPageLoader
 *
 *  @author Howard Lewis Ship
 *
 **/

public class ComponentSpecification extends LocatablePropertyHolder implements IComponentSpecification
{public static com.cortexeb.tools.clover.d __CLOVER_346_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private String _componentClassName; 

    /** @since 1.0.9 **/

    private String _description;

    /**
     *  Keyed on component id, value is {@link IContainedComponent}.
     *
     **/

    protected Map _components;

    /**
     *  Keyed on asset name, value is {@link IAssetSpecification}.
     *
     **/

    protected Map _assets;

    /**
     *  Defines all formal parameters.  Keyed on parameter name, value is
     * {@link IParameterSpecification}.
     *
     **/

    protected Map _parameters;

    /**
     *  Defines all helper beans.  Keyed on name, value is {@link IBeanSpecification}.
     *
     *  @since 1.0.4
     **/

    protected Map _beans;

    /**
     *  The names of all reserved informal parameter names (as lower-case).  This
     *  allows the page loader to filter out any informal parameters during page load,
     *  rather than during render.
     *
     *   @since 1.0.5
     *
     **/

    protected Set _reservedParameterNames;

    /**
     *  Is the component allowed to have a body (that is, wrap other elements?).
     *
     **/

    private boolean _allowBody = true;

    /**
     *  Is the component allow to have informal parameter specified.
     *
     **/

    private boolean _allowInformalParameters = true;

    /**
     *  The XML Public Id used when the page or component specification was read
     *  (if applicable).
     * 
     *  @since 2.2
     * 
     **/

    private String _publicId;

    /**
     *  Indicates that the specification is for a page, not a component.
     * 
     *  @since 2.2
     * 
     **/

    private boolean _pageSpecification;

    /**
     *  The location from which the specification was obtained.
     * 
     *  @since 3.0
     * 
     **/

    private Resource _specificationLocation;

    /**
     *  A Map of {@link IPropertySpecification} keyed on the name
     *  of the property.
     *
     *  @since 3.0
     * 
     **/

    private Map _propertySpecifications;

    /**
     * @throws IllegalArgumentException if the name already exists.
     *
     **/

    public void addAsset(String name, IAssetSpecification asset)
    {try { __CLOVER_346_0.M[1690]++;
        __CLOVER_346_0.S[7723]++;if ((((_assets == null) && (++__CLOVER_346_0.CT[1328] != 0)) || (++__CLOVER_346_0.CF[1328] == 0))){
            __CLOVER_346_0.S[7724]++;_assets = new HashMap();}

        __CLOVER_346_0.S[7725]++;if ((((_assets.containsKey(name)) && (++__CLOVER_346_0.CT[1329] != 0)) || (++__CLOVER_346_0.CF[1329] == 0))){
            __CLOVER_346_0.S[7726]++;throw new IllegalArgumentException(
                Tapestry.format("ComponentSpecification.duplicate-asset", this, name));}

        __CLOVER_346_0.S[7727]++;_assets.put(name, asset);
    } finally { }}

    /**
     *  @throws IllegalArgumentException if the id is already defined.
     *
     **/

    public void addComponent(String id, IContainedComponent component)
    {try { __CLOVER_346_0.M[1691]++;
        __CLOVER_346_0.S[7728]++;if ((((_components == null) && (++__CLOVER_346_0.CT[1330] != 0)) || (++__CLOVER_346_0.CF[1330] == 0))){
            __CLOVER_346_0.S[7729]++;_components = new HashMap();}

        __CLOVER_346_0.S[7730]++;if ((((_components.containsKey(id)) && (++__CLOVER_346_0.CT[1331] != 0)) || (++__CLOVER_346_0.CF[1331] == 0))){
            __CLOVER_346_0.S[7731]++;throw new IllegalArgumentException(
                Tapestry.format("ComponentSpecification.duplicate-component", this, id));}

        __CLOVER_346_0.S[7732]++;_components.put(id, component);
    } finally { }}

    /**
     *  Adds the parameter.   The name is added as a reserved name.
     *
     *  @throws IllegalArgumentException if the name already exists.
     **/

    public void addParameter(String name, IParameterSpecification spec)
    {try { __CLOVER_346_0.M[1692]++;
        __CLOVER_346_0.S[7733]++;if ((((_parameters == null) && (++__CLOVER_346_0.CT[1332] != 0)) || (++__CLOVER_346_0.CF[1332] == 0))){
            __CLOVER_346_0.S[7734]++;_parameters = new HashMap();}

        __CLOVER_346_0.S[7735]++;if ((((_parameters.containsKey(name)) && (++__CLOVER_346_0.CT[1333] != 0)) || (++__CLOVER_346_0.CF[1333] == 0))){
            __CLOVER_346_0.S[7736]++;throw new IllegalArgumentException(
                Tapestry.format("ComponentSpecification.duplicate-parameter", this, name));}

        __CLOVER_346_0.S[7737]++;_parameters.put(name, spec);

        __CLOVER_346_0.S[7738]++;addReservedParameterName(name);
    } finally { }}

    /**
     *  Returns true if the component is allowed to wrap other elements (static HTML
     *  or other components).  The default is true.
     *
     *  @see #setAllowBody(boolean)
     *
     **/

    public boolean getAllowBody()
    {try { __CLOVER_346_0.M[1693]++;
        __CLOVER_346_0.S[7739]++;return _allowBody;
    } finally { }}

    /**
     *  Returns true if the component allows informal parameters (parameters
     *  not formally defined).  Informal parameters are generally used to create
     *  additional HTML attributes for an HTML tag rendered by the
     *  component.  This is often used to specify JavaScript event handlers or the class
     *  of the component (for Cascarding Style Sheets).
     *
     * <p>The default value is true.
     *
     *  @see #setAllowInformalParameters(boolean)
     **/

    public boolean getAllowInformalParameters()
    {try { __CLOVER_346_0.M[1694]++;
        __CLOVER_346_0.S[7740]++;return _allowInformalParameters;
    } finally { }}

    /**
     *  Returns the {@link IAssetSpecification} with the given name, or null
     *  if no such specification exists.
     *
     *  @see #addAsset(String,IAssetSpecification)
     **/

    public IAssetSpecification getAsset(String name)
    {try { __CLOVER_346_0.M[1695]++;

        __CLOVER_346_0.S[7741]++;return (IAssetSpecification) get(_assets, name);
    } finally { }}

    /**
     *  Returns a <code>List</code>
     *  of the String names of all assets, in alphabetical
     *  order
     *
     **/

    public List getAssetNames()
    {try { __CLOVER_346_0.M[1696]++;
        __CLOVER_346_0.S[7742]++;return sortedKeys(_assets);
    } finally { }}

    /**
     *  Returns the specification of a contained component with the given id, or
     *  null if no such contained component exists.
     *
     *  @see #addComponent(String, IContainedComponent)
     *
     **/

    public IContainedComponent getComponent(String id)
    {try { __CLOVER_346_0.M[1697]++;
        __CLOVER_346_0.S[7743]++;return (IContainedComponent) get(_components, id);
    } finally { }}

    public String getComponentClassName()
    {try { __CLOVER_346_0.M[1698]++;
        __CLOVER_346_0.S[7744]++;return _componentClassName;
    } finally { }}

    /**
     *  Returns an <code>List</code>
     *  of the String names of the {@link IContainedComponent}s
     *  for this component.
     *
     *  @see #addComponent(String, IContainedComponent)
     *
     **/

    public List getComponentIds()
    {try { __CLOVER_346_0.M[1699]++;
        __CLOVER_346_0.S[7745]++;return sortedKeys(_components);
    } finally { }}

    /**
     *  Returns the specification of a parameter with the given name, or
     *  null if no such parameter exists.
     *
     *  @see #addParameter(String, IParameterSpecification)
     *
     **/

    public IParameterSpecification getParameter(String name)
    {try { __CLOVER_346_0.M[1700]++;
        __CLOVER_346_0.S[7746]++;return (IParameterSpecification) get(_parameters, name);
    } finally { }}

    /**
     *  Returns a List of
     *  of String names of all parameters.  This list
     *  is in alphabetical order.
     *
     *  @see #addParameter(String, IParameterSpecification)
     *
     **/

    public List getParameterNames()
    {try { __CLOVER_346_0.M[1701]++;
        __CLOVER_346_0.S[7747]++;return sortedKeys(_parameters);
    } finally { }}

    public void setAllowBody(boolean value)
    {try { __CLOVER_346_0.M[1702]++;
        __CLOVER_346_0.S[7748]++;_allowBody = value;
    } finally { }}

    public void setAllowInformalParameters(boolean value)
    {try { __CLOVER_346_0.M[1703]++;
        __CLOVER_346_0.S[7749]++;_allowInformalParameters = value;
    } finally { }}

    public void setComponentClassName(String value)
    {try { __CLOVER_346_0.M[1704]++;
        __CLOVER_346_0.S[7750]++;_componentClassName = value;
    } finally { }}

    /**
     *  @since 1.0.4
     *
     *  @throws IllegalArgumentException if the bean already has a specification.
     **/

    public void addBeanSpecification(String name, IBeanSpecification specification)
    {try { __CLOVER_346_0.M[1705]++;
        __CLOVER_346_0.S[7751]++;if ((((_beans == null) && (++__CLOVER_346_0.CT[1334] != 0)) || (++__CLOVER_346_0.CF[1334] == 0))){
            __CLOVER_346_0.S[7752]++;_beans = new HashMap();}

        else{
            __CLOVER_346_0.S[7753]++;if ((((_beans.containsKey(name)) && (++__CLOVER_346_0.CT[1335] != 0)) || (++__CLOVER_346_0.CF[1335] == 0))){
                __CLOVER_346_0.S[7754]++;throw new IllegalArgumentException(
                    Tapestry.format("ComponentSpecification.duplicate-bean", this, name));}}

        __CLOVER_346_0.S[7755]++;_beans.put(name, specification);
    } finally { }}

    /**
     * Returns the {@link IBeanSpecification} for the given name, or null
     * if not such specification exists.
     *
     * @since 1.0.4
     *
     **/

    public IBeanSpecification getBeanSpecification(String name)
    {try { __CLOVER_346_0.M[1706]++;
        __CLOVER_346_0.S[7756]++;if ((((_beans == null) && (++__CLOVER_346_0.CT[1336] != 0)) || (++__CLOVER_346_0.CF[1336] == 0))){
            __CLOVER_346_0.S[7757]++;return null;}

        __CLOVER_346_0.S[7758]++;return (IBeanSpecification) _beans.get(name);
    } finally { }}

    /**
     *  Returns an unmodifiable collection of the names of all beans.
     *
     **/

    public Collection getBeanNames()
    {try { __CLOVER_346_0.M[1707]++;
        __CLOVER_346_0.S[7759]++;if ((((_beans == null) && (++__CLOVER_346_0.CT[1337] != 0)) || (++__CLOVER_346_0.CF[1337] == 0))){
            __CLOVER_346_0.S[7760]++;return Collections.EMPTY_LIST;}

        __CLOVER_346_0.S[7761]++;return Collections.unmodifiableCollection(_beans.keySet());
    } finally { }}

    /**
     *  Adds the value as a reserved name.  Reserved names are not allowed
     *  as the names of informal parameters.  Since the comparison is
     *  caseless, the value is converted to lowercase before being
     *  stored.
     *
     *  @since 1.0.5
     *
     **/

    public void addReservedParameterName(String value)
    {try { __CLOVER_346_0.M[1708]++;
        __CLOVER_346_0.S[7762]++;if ((((_reservedParameterNames == null) && (++__CLOVER_346_0.CT[1338] != 0)) || (++__CLOVER_346_0.CF[1338] == 0))){
            __CLOVER_346_0.S[7763]++;_reservedParameterNames = new HashSet();}

        __CLOVER_346_0.S[7764]++;_reservedParameterNames.add(value.toLowerCase());
    } finally { }}

    /**
     *  Returns true if the value specified is in the reserved name list.
     *  The comparison is caseless.  All formal parameters are automatically
     *  in the reserved name list, as well as any additional
     *  reserved names specified in the component specification.  The latter
     *  refer to HTML attributes generated directly by the component.
     *
     *  @since 1.0.5
     *
     **/

    public boolean isReservedParameterName(String value)
    {try { __CLOVER_346_0.M[1709]++;
        __CLOVER_346_0.S[7765]++;if ((((_reservedParameterNames == null) && (++__CLOVER_346_0.CT[1339] != 0)) || (++__CLOVER_346_0.CF[1339] == 0))){
            __CLOVER_346_0.S[7766]++;return false;}

        __CLOVER_346_0.S[7767]++;return _reservedParameterNames.contains(value.toLowerCase());
    } finally { }}

    public String toString()
    {try { __CLOVER_346_0.M[1710]++;
        __CLOVER_346_0.S[7768]++;ToStringBuilder builder = new ToStringBuilder(this);

        __CLOVER_346_0.S[7769]++;builder.append("componentClassName", _componentClassName);
        __CLOVER_346_0.S[7770]++;builder.append("pageSpecification", _pageSpecification);
        __CLOVER_346_0.S[7771]++;builder.append("specificationLocation", _specificationLocation);
        __CLOVER_346_0.S[7772]++;builder.append("allowBody", _allowBody);
        __CLOVER_346_0.S[7773]++;builder.append("allowInformalParameter", _allowInformalParameters);

        __CLOVER_346_0.S[7774]++;return builder.toString();
    } finally { }}

    /**
     *  Returns the documentation for this component.
     * 
     *  @since 1.0.9
     **/

    public String getDescription()
    {try { __CLOVER_346_0.M[1711]++;
        __CLOVER_346_0.S[7775]++;return _description;
    } finally { }}

    /**
     *  Sets the documentation for this component.
     * 
     *  @since 1.0.9
     **/

    public void setDescription(String description)
    {try { __CLOVER_346_0.M[1712]++;
        __CLOVER_346_0.S[7776]++;_description = description;
    } finally { }}

    /**
     *  Returns the XML Public Id for the specification file, or null
     *  if not applicable.
     * 
     *  <p>
     *  This method exists as a convienience for the Spindle plugin.
     *  A previous method used an arbitrary version string, the
     *  public id is more useful and less ambiguous.
     *  
     *  @since 2.2
     * 
     **/

    public String getPublicId()
    {try { __CLOVER_346_0.M[1713]++;
        __CLOVER_346_0.S[7777]++;return _publicId;
    } finally { }}

    /** @since 2.2 **/

    public void setPublicId(String publicId)
    {try { __CLOVER_346_0.M[1714]++;
        __CLOVER_346_0.S[7778]++;_publicId = publicId;
    } finally { }}

    /** 
     * 
     *  Returns true if the specification is known to be a page
     *  specification and not a component specification.  Earlier versions
     *  of the framework did not distinguish between the two, but starting
     *  in 2.2, there are seperate XML entities for pages and components.
     *  Pages omit several attributes and entities related
     *  to parameters, as parameters only make sense for components.
     *  
     *  @since 2.2 
     * 
     **/

    public boolean isPageSpecification()
    {try { __CLOVER_346_0.M[1715]++;
        __CLOVER_346_0.S[7779]++;return _pageSpecification;
    } finally { }}

    /** @since 2.2 **/

    public void setPageSpecification(boolean pageSpecification)
    {try { __CLOVER_346_0.M[1716]++;
        __CLOVER_346_0.S[7780]++;_pageSpecification = pageSpecification;
    } finally { }}

    /** @since 2.2 **/

    private List sortedKeys(Map input)
    {try { __CLOVER_346_0.M[1717]++;
        __CLOVER_346_0.S[7781]++;if ((((input == null) && (++__CLOVER_346_0.CT[1340] != 0)) || (++__CLOVER_346_0.CF[1340] == 0))){
            __CLOVER_346_0.S[7782]++;return Collections.EMPTY_LIST;}

        __CLOVER_346_0.S[7783]++;List result = new ArrayList(input.keySet());

        __CLOVER_346_0.S[7784]++;Collections.sort(result);

        __CLOVER_346_0.S[7785]++;return result;
    } finally { }}

    /** @since 2.2 **/

    private Object get(Map map, Object key)
    {try { __CLOVER_346_0.M[1718]++;
        __CLOVER_346_0.S[7786]++;if ((((map == null) && (++__CLOVER_346_0.CT[1341] != 0)) || (++__CLOVER_346_0.CF[1341] == 0))){
            __CLOVER_346_0.S[7787]++;return null;}

        __CLOVER_346_0.S[7788]++;return map.get(key);
    } finally { }}

    /** @since 3.0 **/

    public Resource getSpecificationLocation()
    {try { __CLOVER_346_0.M[1719]++;
        __CLOVER_346_0.S[7789]++;return _specificationLocation;
    } finally { }}

    /** @since 3.0 **/

    public void setSpecificationLocation(Resource specificationLocation)
    {try { __CLOVER_346_0.M[1720]++;
        __CLOVER_346_0.S[7790]++;_specificationLocation = specificationLocation;
    } finally { }}

    /**
     *  Adds a new property specification.  The name of the property must
     *  not already be defined (and must not change after being added).
     * 
     *  @since 3.0
     * 
     **/

    public void addPropertySpecification(IPropertySpecification spec)
    {try { __CLOVER_346_0.M[1721]++;
        __CLOVER_346_0.S[7791]++;if ((((_propertySpecifications == null) && (++__CLOVER_346_0.CT[1342] != 0)) || (++__CLOVER_346_0.CF[1342] == 0))){
            __CLOVER_346_0.S[7792]++;_propertySpecifications = new HashMap();}

        __CLOVER_346_0.S[7793]++;String name = spec.getName();

        __CLOVER_346_0.S[7794]++;if ((((_propertySpecifications.containsKey(name)) && (++__CLOVER_346_0.CT[1343] != 0)) || (++__CLOVER_346_0.CF[1343] == 0))){
            __CLOVER_346_0.S[7795]++;throw new IllegalArgumentException(
                Tapestry.format(
                    "ComponentSpecification.duplicate-property-specification",
                    this,
                    name));}

        __CLOVER_346_0.S[7796]++;_propertySpecifications.put(name, spec);
    } finally { }}

    /**
     *  Returns a sorted, immutable list of the names of all 
     *  {@link org.apache.tapestry.spec.IPropertySpecification}s.
     * 
     *  @since 3.0
     * 
     **/

    public List getPropertySpecificationNames()
    {try { __CLOVER_346_0.M[1722]++;
        __CLOVER_346_0.S[7797]++;return sortedKeys(_propertySpecifications);
    } finally { }}

    /**
     *  Returns the named {@link org.apache.tapestry.spec.IPropertySpecification},
     *  or null  if no such specification exist.
     * 
     *  @since 3.0
     *  @see #addPropertySpecification(IPropertySpecification)
     * 
     **/

    public IPropertySpecification getPropertySpecification(String name)
    {try { __CLOVER_346_0.M[1723]++;
        __CLOVER_346_0.S[7798]++;return (IPropertySpecification) get(_propertySpecifications, name);
    } finally { }}

}