/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.spec;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.ClassResolver;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.util.prop.OgnlUtils;

/**
 * Defines an "extension", which is much like a helper bean, but is part of a library or application
 * specification (and has the same lifecycle as the application).
 * 
 * @author Howard Lewis Ship
 * @since 2.2
 */

public class ExtensionSpecification extends LocatablePropertyHolder implements
        IExtensionSpecification
{public static com.cortexeb.tools.clover.d __CLOVER_349_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final Log LOG = LogFactory.getLog(ExtensionSpecification.class);

    private String _className;

    protected Map _configuration = new HashMap();

    private boolean _immediate;

    public String getClassName()
    {try { __CLOVER_349_0.M[1737]++;
        __CLOVER_349_0.S[7819]++;return _className;
    } finally { }}

    public void setClassName(String className)
    {try { __CLOVER_349_0.M[1738]++;
        __CLOVER_349_0.S[7820]++;_className = className;
    } finally { }}

    public void addConfiguration(String propertyName, String value)
    {try { __CLOVER_349_0.M[1739]++;
        __CLOVER_349_0.S[7821]++;if ((((_configuration.containsKey(propertyName)) && (++__CLOVER_349_0.CT[1347] != 0)) || (++__CLOVER_349_0.CF[1347] == 0))){
            __CLOVER_349_0.S[7822]++;throw new IllegalArgumentException(Tapestry.format(
                    "ExtensionSpecification.duplicate-property",
                    this,
                    propertyName));}

        __CLOVER_349_0.S[7823]++;_configuration.put(propertyName, value);
    } finally { }}

    /**
     * Returns an immutable Map of the configuration; keyed on property name, with values as
     * properties to assign.
     */

    public Map getConfiguration()
    {try { __CLOVER_349_0.M[1740]++;
        __CLOVER_349_0.S[7824]++;return Collections.unmodifiableMap(_configuration);
    } finally { }}

    /**
     * Invoked to instantiate an instance of the extension and return it. It also configures
     * properties of the extension.
     */

    public Object instantiateExtension(ClassResolver resolver)
    {try { __CLOVER_349_0.M[1741]++;
        __CLOVER_349_0.S[7825]++;if ((((LOG.isDebugEnabled()) && (++__CLOVER_349_0.CT[1348] != 0)) || (++__CLOVER_349_0.CF[1348] == 0))){
            __CLOVER_349_0.S[7826]++;LOG.debug("Instantiating extension class " + _className + ".");}
        __CLOVER_349_0.S[7827]++;Class extensionClass = null;
        __CLOVER_349_0.S[7828]++;Object result = null;

        __CLOVER_349_0.S[7829]++;try
        {
            __CLOVER_349_0.S[7830]++;extensionClass = resolver.findClass(_className);
        }
        catch (Exception ex)
        {
            __CLOVER_349_0.S[7831]++;throw new ApplicationRuntimeException(Tapestry.format(
                    "ExtensionSpecification.bad-class",
                    _className), getLocation(), ex);
        }

        __CLOVER_349_0.S[7832]++;result = instantiateInstance(extensionClass, result);

        __CLOVER_349_0.S[7833]++;initializeProperties(result);

        __CLOVER_349_0.S[7834]++;return result;
    } finally { }}

    private void initializeProperties(Object extension)
    {try { __CLOVER_349_0.M[1742]++;
        __CLOVER_349_0.S[7835]++;if ((((_configuration.isEmpty()) && (++__CLOVER_349_0.CT[1349] != 0)) || (++__CLOVER_349_0.CF[1349] == 0))){
            __CLOVER_349_0.S[7836]++;return;}

        __CLOVER_349_0.S[7837]++;Iterator i = _configuration.entrySet().iterator();
        __CLOVER_349_0.S[7838]++;while ((((i.hasNext()) && (++__CLOVER_349_0.CT[1350] != 0)) || (++__CLOVER_349_0.CF[1350] == 0))){
        {
            __CLOVER_349_0.S[7839]++;Map.Entry entry = (Map.Entry) i.next();

            __CLOVER_349_0.S[7840]++;String propertyName = (String) entry.getKey();
            __CLOVER_349_0.S[7841]++;String textValue = (String) entry.getValue();

            // Let OGNL do any conversions.

            __CLOVER_349_0.S[7842]++;OgnlUtils.set(propertyName, extension, textValue);
        }}
    } finally { }}

    private Object instantiateInstance(Class extensionClass, Object result)
    {try { __CLOVER_349_0.M[1743]++;
        __CLOVER_349_0.S[7843]++;try
        {
            __CLOVER_349_0.S[7844]++;result = extensionClass.newInstance();
        }
        catch (Exception ex)
        {
            __CLOVER_349_0.S[7845]++;throw new ApplicationRuntimeException(ex.getMessage(), getLocation(), ex);
        }

        __CLOVER_349_0.S[7846]++;return result;
    } finally { }}

    public String toString()
    {try { __CLOVER_349_0.M[1744]++;
        __CLOVER_349_0.S[7847]++;StringBuffer buffer = new StringBuffer("ExtensionSpecification@");
        __CLOVER_349_0.S[7848]++;buffer.append(Integer.toHexString(hashCode()));
        __CLOVER_349_0.S[7849]++;buffer.append('[');
        __CLOVER_349_0.S[7850]++;buffer.append(_className);

        __CLOVER_349_0.S[7851]++;if ((((_configuration != null) && (++__CLOVER_349_0.CT[1351] != 0)) || (++__CLOVER_349_0.CF[1351] == 0))){
        {
            __CLOVER_349_0.S[7852]++;buffer.append(' ');
            __CLOVER_349_0.S[7853]++;buffer.append(_configuration);
        }}

        __CLOVER_349_0.S[7854]++;buffer.append(']');

        __CLOVER_349_0.S[7855]++;return buffer.toString();
    } finally { }}

    /**
     * Returns true if the extensions should be instantiated immediately after the containing
     * {@link org.apache.tapestry.spec.LibrarySpecification}if parsed. Non-immediate extensions are
     * instantiated only as needed.
     */

    public boolean isImmediate()
    {try { __CLOVER_349_0.M[1745]++;
        __CLOVER_349_0.S[7856]++;return _immediate;
    } finally { }}

    public void setImmediate(boolean immediate)
    {try { __CLOVER_349_0.M[1746]++;
        __CLOVER_349_0.S[7857]++;_immediate = immediate;
    } finally { }}

}