/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.spec;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.builder.ToStringBuilder;
import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.ClassResolver;
import org.apache.hivemind.Location;
import org.apache.hivemind.Resource;
import org.apache.tapestry.Tapestry;

/**
 * Specification for a library. {@link org.apache.tapestry.spec.ApplicationSpecification}is a
 * specialized kind of library.
 * 
 * @author Howard Lewis Ship
 * @since 2.2
 */

public class LibrarySpecification extends LocatablePropertyHolder implements ILibrarySpecification
{public static com.cortexeb.tools.clover.d __CLOVER_361_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    /**
     * Resource resolver (used to instantiate extensions).
     */

    private ClassResolver _resolver;

    /**
     * Map of page name to page specification path.
     */

    private Map _pages;

    /**
     * Map of component alias to component specification path.
     */
    private Map _components;

    /**
     * Map of library id to library specification path.
     */

    private Map _libraries;

    private String _description;

    /**
     * Map of extension name to {@link IExtensionSpecification}.
     */

    private Map _extensions;

    /**
     * Map of extension name to Object for instantiated extensions.
     */

    private Map _instantiatedExtensions;

    /**
     * The XML Public Id used when the library specification was read (if applicable).
     * 
     * @since 2.2
     */

    private String _publicId;

    /**
     * The location of the specification.
     */

    private Resource _specificationLocation;

    public String getLibrarySpecificationPath(String id)
    {try { __CLOVER_361_0.M[1747]++;
        __CLOVER_361_0.S[7858]++;return (String) get(_libraries, id);
    } finally { }}

    /**
     * Sets the specification path for an embedded library.
     * 
     * @throws IllegalArgumentException
     *             if a library with the given id already exists
     */

    public void setLibrarySpecificationPath(String id, String path)
    {try { __CLOVER_361_0.M[1748]++;
        __CLOVER_361_0.S[7859]++;if ((((_libraries == null) && (++__CLOVER_361_0.CT[1352] != 0)) || (++__CLOVER_361_0.CF[1352] == 0))){
            __CLOVER_361_0.S[7860]++;_libraries = new HashMap();}

        __CLOVER_361_0.S[7861]++;if ((((_libraries.containsKey(id)) && (++__CLOVER_361_0.CT[1353] != 0)) || (++__CLOVER_361_0.CF[1353] == 0))){
            __CLOVER_361_0.S[7862]++;throw new IllegalArgumentException(Tapestry.format(
                    "LibrarySpecification.duplicate-child-namespace-id",
                    id));}

        __CLOVER_361_0.S[7863]++;_libraries.put(id, path);
    } finally { }}

    public List getLibraryIds()
    {try { __CLOVER_361_0.M[1749]++;
        __CLOVER_361_0.S[7864]++;return sortedKeys(_libraries);
    } finally { }}

    public String getPageSpecificationPath(String name)
    {try { __CLOVER_361_0.M[1750]++;
        __CLOVER_361_0.S[7865]++;return (String) get(_pages, name);
    } finally { }}

    public void setPageSpecificationPath(String name, String path)
    {try { __CLOVER_361_0.M[1751]++;
        __CLOVER_361_0.S[7866]++;if ((((_pages == null) && (++__CLOVER_361_0.CT[1354] != 0)) || (++__CLOVER_361_0.CF[1354] == 0))){
            __CLOVER_361_0.S[7867]++;_pages = new HashMap();}

        __CLOVER_361_0.S[7868]++;if ((((_pages.containsKey(name)) && (++__CLOVER_361_0.CT[1355] != 0)) || (++__CLOVER_361_0.CF[1355] == 0))){
            __CLOVER_361_0.S[7869]++;throw new IllegalArgumentException(Tapestry.format(
                    "LibrarySpecification.duplicate-page-name",
                    name));}

        __CLOVER_361_0.S[7870]++;_pages.put(name, path);
    } finally { }}

    public List getPageNames()
    {try { __CLOVER_361_0.M[1752]++;
        __CLOVER_361_0.S[7871]++;return sortedKeys(_pages);
    } finally { }}

    public void setComponentSpecificationPath(String alias, String path)
    {try { __CLOVER_361_0.M[1753]++;
        __CLOVER_361_0.S[7872]++;if ((((_components == null) && (++__CLOVER_361_0.CT[1356] != 0)) || (++__CLOVER_361_0.CF[1356] == 0))){
            __CLOVER_361_0.S[7873]++;_components = new HashMap();}

        __CLOVER_361_0.S[7874]++;if ((((_components.containsKey(alias)) && (++__CLOVER_361_0.CT[1357] != 0)) || (++__CLOVER_361_0.CF[1357] == 0))){
            __CLOVER_361_0.S[7875]++;throw new IllegalArgumentException(Tapestry.format(
                    "LibrarySpecification.duplicate-component-alias",
                    alias));}

        __CLOVER_361_0.S[7876]++;_components.put(alias, path);
    } finally { }}

    public String getComponentSpecificationPath(String alias)
    {try { __CLOVER_361_0.M[1754]++;
        __CLOVER_361_0.S[7877]++;return (String) get(_components, alias);
    } finally { }}

    /**
     * @since 3.0
     */

    public List getComponentTypes()
    {try { __CLOVER_361_0.M[1755]++;
        __CLOVER_361_0.S[7878]++;return sortedKeys(_components);
    } finally { }}

    public String getServiceClassName(String name)
    {try { __CLOVER_361_0.M[1756]++;
        __CLOVER_361_0.S[7879]++;throw new UnsupportedOperationException();
    } finally { }}

    public List getServiceNames()
    {try { __CLOVER_361_0.M[1757]++;
        __CLOVER_361_0.S[7880]++;return Collections.EMPTY_LIST;
    } finally { }}

    public void setServiceClassName(String name, String className)
    {try { __CLOVER_361_0.M[1758]++;
        __CLOVER_361_0.S[7881]++;throw new UnsupportedOperationException();
    } finally { }}

    private List sortedKeys(Map map)
    {try { __CLOVER_361_0.M[1759]++;
        __CLOVER_361_0.S[7882]++;if ((((map == null) && (++__CLOVER_361_0.CT[1358] != 0)) || (++__CLOVER_361_0.CF[1358] == 0))){
            __CLOVER_361_0.S[7883]++;return Collections.EMPTY_LIST;}

        __CLOVER_361_0.S[7884]++;List result = new ArrayList(map.keySet());

        __CLOVER_361_0.S[7885]++;Collections.sort(result);

        __CLOVER_361_0.S[7886]++;return result;
    } finally { }}

    private Object get(Map map, Object key)
    {try { __CLOVER_361_0.M[1760]++;
        __CLOVER_361_0.S[7887]++;if ((((map == null) && (++__CLOVER_361_0.CT[1359] != 0)) || (++__CLOVER_361_0.CF[1359] == 0))){
            __CLOVER_361_0.S[7888]++;return null;}

        __CLOVER_361_0.S[7889]++;return map.get(key);
    } finally { }}

    /**
     * Returns the documentation for this library..
     */

    public String getDescription()
    {try { __CLOVER_361_0.M[1761]++;
        __CLOVER_361_0.S[7890]++;return _description;
    } finally { }}

    /**
     * Sets the documentation for this library.
     */

    public void setDescription(String description)
    {try { __CLOVER_361_0.M[1762]++;
        __CLOVER_361_0.S[7891]++;_description = description;
    } finally { }}

    /**
     * Returns a Map of extensions; key is extension name, value is
     * {@link org.apache.tapestry.spec.IExtensionSpecification}. May return null. The returned Map
     * is immutable.
     */

    public Map getExtensionSpecifications()
    {try { __CLOVER_361_0.M[1763]++;
        __CLOVER_361_0.S[7892]++;if ((((_extensions == null) && (++__CLOVER_361_0.CT[1360] != 0)) || (++__CLOVER_361_0.CF[1360] == 0))){
            __CLOVER_361_0.S[7893]++;return null;}

        __CLOVER_361_0.S[7894]++;return Collections.unmodifiableMap(_extensions);
    } finally { }}

    /**
     * Adds another extension specification.
     * 
     * @throws IllegalArgumentException
     *             if an extension with the given name already exists.
     */

    public void addExtensionSpecification(String name, IExtensionSpecification extension)
    {try { __CLOVER_361_0.M[1764]++;
        __CLOVER_361_0.S[7895]++;if ((((_extensions == null) && (++__CLOVER_361_0.CT[1361] != 0)) || (++__CLOVER_361_0.CF[1361] == 0))){
            __CLOVER_361_0.S[7896]++;_extensions = new HashMap();}

        __CLOVER_361_0.S[7897]++;if ((((_extensions.containsKey(name)) && (++__CLOVER_361_0.CT[1362] != 0)) || (++__CLOVER_361_0.CF[1362] == 0))){
            __CLOVER_361_0.S[7898]++;throw new IllegalArgumentException(Tapestry.format(
                    "LibrarySpecification.duplicate-extension-name",
                    this,
                    name));}

        __CLOVER_361_0.S[7899]++;_extensions.put(name, extension);
    } finally { }}

    /**
     * Returns a sorted List of the names of all extensions. May return the empty list, but won't
     * return null.
     */

    public synchronized List getExtensionNames()
    {try { __CLOVER_361_0.M[1765]++;
        __CLOVER_361_0.S[7900]++;return sortedKeys(_instantiatedExtensions);
    } finally { }}

    /**
     * Returns the named IExtensionSpecification, or null if it doesn't exist.
     */

    public IExtensionSpecification getExtensionSpecification(String name)
    {try { __CLOVER_361_0.M[1766]++;
        __CLOVER_361_0.S[7901]++;if ((((_extensions == null) && (++__CLOVER_361_0.CT[1363] != 0)) || (++__CLOVER_361_0.CF[1363] == 0))){
            __CLOVER_361_0.S[7902]++;return null;}

        __CLOVER_361_0.S[7903]++;return (IExtensionSpecification) _extensions.get(name);
    } finally { }}

    /**
     * Returns true if this library specification has a specification for the named extension.
     */

    public boolean checkExtension(String name)
    {try { __CLOVER_361_0.M[1767]++;
        __CLOVER_361_0.S[7904]++;if ((((_extensions == null) && (++__CLOVER_361_0.CT[1364] != 0)) || (++__CLOVER_361_0.CF[1364] == 0))){
            __CLOVER_361_0.S[7905]++;return false;}

        __CLOVER_361_0.S[7906]++;return _extensions.containsKey(name);
    } finally { }}

    /**
     * Returns an instantiated extension. Extensions are created as needed and cached for later use.
     * 
     * @throws IllegalArgumentException
     *             if no extension specification exists for the given name.
     */

    public synchronized Object getExtension(String name)
    {try { __CLOVER_361_0.M[1768]++;
        __CLOVER_361_0.S[7907]++;return getExtension(name, null);
    } finally { }}

    /** @since 3.0 * */

    public synchronized Object getExtension(String name, Class typeConstraint)
    {try { __CLOVER_361_0.M[1769]++;
        __CLOVER_361_0.S[7908]++;if ((((_instantiatedExtensions == null) && (++__CLOVER_361_0.CT[1365] != 0)) || (++__CLOVER_361_0.CF[1365] == 0))){
            __CLOVER_361_0.S[7909]++;_instantiatedExtensions = new HashMap();}

        __CLOVER_361_0.S[7910]++;Object result = _instantiatedExtensions.get(name);
        __CLOVER_361_0.S[7911]++;IExtensionSpecification spec = getExtensionSpecification(name);

        __CLOVER_361_0.S[7912]++;if ((((spec == null) && (++__CLOVER_361_0.CT[1366] != 0)) || (++__CLOVER_361_0.CF[1366] == 0))){
            __CLOVER_361_0.S[7913]++;throw new IllegalArgumentException(Tapestry.format(
                    "LibrarySpecification.no-such-extension",
                    name));}

        __CLOVER_361_0.S[7914]++;if ((((result == null) && (++__CLOVER_361_0.CT[1367] != 0)) || (++__CLOVER_361_0.CF[1367] == 0))){
        {

            __CLOVER_361_0.S[7915]++;result = spec.instantiateExtension(_resolver);

            __CLOVER_361_0.S[7916]++;_instantiatedExtensions.put(name, result);
        }}

        __CLOVER_361_0.S[7917]++;if ((((typeConstraint != null) && (++__CLOVER_361_0.CT[1368] != 0)) || (++__CLOVER_361_0.CF[1368] == 0))){
            __CLOVER_361_0.S[7918]++;applyTypeConstraint(name, result, typeConstraint, spec.getLocation());}

        __CLOVER_361_0.S[7919]++;return result;
    } finally { }}

    /**
     * Checks that an extension conforms to the supplied type constraint.
     * 
     * @throws IllegalArgumentException
     *             if the extension fails the check.
     * @since 3.0
     */

    protected void applyTypeConstraint(String name, Object extension, Class typeConstraint,
            Location location)
    {try { __CLOVER_361_0.M[1770]++;
        __CLOVER_361_0.S[7920]++;Class extensionClass = extension.getClass();

        // Can you assign an instance of the extension to a variable
        // of type typeContraint legally?

        __CLOVER_361_0.S[7921]++;if ((((typeConstraint.isAssignableFrom(extensionClass)) && (++__CLOVER_361_0.CT[1369] != 0)) || (++__CLOVER_361_0.CF[1369] == 0))){
            __CLOVER_361_0.S[7922]++;return;}

        __CLOVER_361_0.S[7923]++;String key = (((typeConstraint.isInterface() ) && (++__CLOVER_361_0.CT[1370] != 0)) || (++__CLOVER_361_0.CF[1370] == 0))? "LibrarySpecification.extension-does-not-implement-interface"
                : "LibrarySpecification.extension-not-a-subclass";

        __CLOVER_361_0.S[7924]++;throw new ApplicationRuntimeException(Tapestry.format(
                key,
                name,
                extensionClass.getName(),
                typeConstraint.getName()), location, null);
    } finally { }}

    /**
     * Invoked after the entire specification has been constructed to instantiate any extensions
     * marked immediate.
     */

    public synchronized void instantiateImmediateExtensions()
    {try { __CLOVER_361_0.M[1771]++;
        __CLOVER_361_0.S[7925]++;if ((((_extensions == null) && (++__CLOVER_361_0.CT[1371] != 0)) || (++__CLOVER_361_0.CF[1371] == 0))){
            __CLOVER_361_0.S[7926]++;return;}

        __CLOVER_361_0.S[7927]++;Iterator i = _extensions.entrySet().iterator();

        __CLOVER_361_0.S[7928]++;while ((((i.hasNext()) && (++__CLOVER_361_0.CT[1372] != 0)) || (++__CLOVER_361_0.CF[1372] == 0))){
        {
            __CLOVER_361_0.S[7929]++;Map.Entry entry = (Map.Entry) i.next();

            __CLOVER_361_0.S[7930]++;IExtensionSpecification spec = (IExtensionSpecification) entry.getValue();

            __CLOVER_361_0.S[7931]++;if ((((!spec.isImmediate()) && (++__CLOVER_361_0.CT[1373] != 0)) || (++__CLOVER_361_0.CF[1373] == 0))){
                __CLOVER_361_0.S[7932]++;continue;}

            __CLOVER_361_0.S[7933]++;String name = (String) entry.getKey();

            __CLOVER_361_0.S[7934]++;getExtension(name);
        }}

    } finally { }}

    public ClassResolver getResourceResolver()
    {try { __CLOVER_361_0.M[1772]++;
        __CLOVER_361_0.S[7935]++;return _resolver;
    } finally { }}

    public void setResourceResolver(ClassResolver resolver)
    {try { __CLOVER_361_0.M[1773]++;
        __CLOVER_361_0.S[7936]++;_resolver = resolver;
    } finally { }}

    /**
     * Returns the extensions map.
     * 
     * @return Map of objects.
     */

    protected Map getExtensions()
    {try { __CLOVER_361_0.M[1774]++;
        __CLOVER_361_0.S[7937]++;return _extensions;
    } finally { }}

    /**
     * Updates the extension map.
     * 
     * @param extension
     *            A Map of extension specification paths keyed on extension id.
     *            <p>
     *            The map is retained, not copied.
     */

    protected void setExtensions(Map extension)
    {try { __CLOVER_361_0.M[1775]++;
        __CLOVER_361_0.S[7938]++;_extensions = extension;
    } finally { }}

    /**
     * Returns the libraries map.
     * 
     * @return Map of {@link LibrarySpecification}.
     */

    protected Map getLibraries()
    {try { __CLOVER_361_0.M[1776]++;
        __CLOVER_361_0.S[7939]++;return _libraries;
    } finally { }}

    /**
     * Updates the library map.
     * 
     * @param libraries
     *            A Map of library specification paths keyed on library id.
     *            <p>
     *            The map is retained, not copied.
     */

    protected void setLibraries(Map libraries)
    {try { __CLOVER_361_0.M[1777]++;
        __CLOVER_361_0.S[7940]++;_libraries = libraries;
    } finally { }}

    /**
     * Returns the pages map.
     * 
     * @return Map of {@link IComponentSpecification}.
     */

    protected Map getPages()
    {try { __CLOVER_361_0.M[1778]++;
        __CLOVER_361_0.S[7941]++;return _pages;
    } finally { }}

    /**
     * Updates the page map.
     * 
     * @param pages
     *            A Map of page specification paths keyed on page id.
     *            <p>
     *            The map is retained, not copied.
     */

    protected void setPages(Map pages)
    {try { __CLOVER_361_0.M[1779]++;
        __CLOVER_361_0.S[7942]++;_pages = pages;
    } finally { }}

    /**
     * Returns the services.
     * 
     * @return Map of service class names.
     * @deprecated To be removed in release 3.2.
     */

    protected Map getServices()
    {try { __CLOVER_361_0.M[1780]++;
        __CLOVER_361_0.S[7943]++;return Collections.EMPTY_MAP;
    } finally { }}

    /**
     * Updates the services map.
     * 
     * @param services
     *            A Map of the fully qualified names of classes which implement
     *            {@link org.apache.tapestry.engine.IEngineService}keyed on service id.
     *            <p>
     *            The map is retained, not copied.
     * @deprecated To be removed in release 3.2.
     */

    protected void setServices(Map services)
    {try { __CLOVER_361_0.M[1781]++;
    } finally { }}

    /**
     * Returns the components map.
     * 
     * @return Map of {@link IContainedComponent}.
     */

    protected Map getComponents()
    {try { __CLOVER_361_0.M[1782]++;
        __CLOVER_361_0.S[7944]++;return _components;
    } finally { }}

    /**
     * Updates the components map.
     * 
     * @param components
     *            A Map of {@link IContainedComponent}keyed on component id. The map is retained,
     *            not copied.
     */

    protected void setComponents(Map components)
    {try { __CLOVER_361_0.M[1783]++;
        __CLOVER_361_0.S[7945]++;_components = components;
    } finally { }}

    /**
     * Returns the XML Public Id for the library file, or null if not applicable.
     * <p>
     * This method exists as a convienience for the Spindle plugin. A previous method used an
     * arbitrary version string, the public id is more useful and less ambiguous.
     */

    public String getPublicId()
    {try { __CLOVER_361_0.M[1784]++;
        __CLOVER_361_0.S[7946]++;return _publicId;
    } finally { }}

    public void setPublicId(String publicId)
    {try { __CLOVER_361_0.M[1785]++;
        __CLOVER_361_0.S[7947]++;_publicId = publicId;
    } finally { }}

    /** @since 3.0 * */

    public Resource getSpecificationLocation()
    {try { __CLOVER_361_0.M[1786]++;
        __CLOVER_361_0.S[7948]++;return _specificationLocation;
    } finally { }}

    /** @since 3.0 * */

    public void setSpecificationLocation(Resource specificationLocation)
    {try { __CLOVER_361_0.M[1787]++;
        __CLOVER_361_0.S[7949]++;_specificationLocation = specificationLocation;
    } finally { }}

    /** @since 3.0 * */

    public synchronized String toString()
    {try { __CLOVER_361_0.M[1788]++;
        __CLOVER_361_0.S[7950]++;ToStringBuilder builder = new ToStringBuilder(this);

        __CLOVER_361_0.S[7951]++;builder.append("components", _components);
        __CLOVER_361_0.S[7952]++;builder.append("description", _description);
        __CLOVER_361_0.S[7953]++;builder.append("instantiatedExtensions", _instantiatedExtensions);
        __CLOVER_361_0.S[7954]++;builder.append("libraries", _libraries);
        __CLOVER_361_0.S[7955]++;builder.append("pages", _pages);
        __CLOVER_361_0.S[7956]++;builder.append("publicId", _publicId);
        __CLOVER_361_0.S[7957]++;builder.append("resolver", _resolver);
        __CLOVER_361_0.S[7958]++;builder.append("specificationLocation", _specificationLocation);

        __CLOVER_361_0.S[7959]++;extendDescription(builder);

        __CLOVER_361_0.S[7960]++;return builder.toString();
    } finally { }}

    /**
     * Does nothing, subclasses may override to add additional description.
     * 
     * @see #toString()
     * @since 3.0
     */

    protected void extendDescription(ToStringBuilder builder)
    {try { __CLOVER_361_0.M[1789]++;
    } finally { }}

}