/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.util;

import java.io.Serializable;

import org.apache.tapestry.IComponent;
import org.apache.tapestry.INamespace;
import org.apache.tapestry.IPage;
import org.apache.tapestry.IRequestCycle;

/**
 * The ComponentAddress class contains the path to a component, allowing it to  
 * locate an instance of that component in a different 
 * {@link org.apache.tapestry.IRequestCycle}.
 * 
 * <p>This class needs to be used mostly when working with components
 * accessed via the {@link org.apache.tapestry.IRender} interface. 
 * It allows those components to serialize and
 * pass as a service parameter information about what component they have to 
 * talk to if control returns back to them. 
 * 
 * <p>This situation often occurs when the component used via IRender contains
 * Direct or Action links.
 * 
 * @author mindbridge
 * @since 2.2
 * 
 */
public class ComponentAddress implements Serializable
{public static com.cortexeb.tools.clover.d __CLOVER_391_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private String _pageName;
    private String _idPath;

    /**
     * Creates a new ComponentAddress object that carries the identification 
     * information of the given component (the page name and the ID path).
     * @param component the component to get the address of
     */
    public ComponentAddress(IComponent component)
    {
        this(component.getPage().getPageName(), component.getIdPath());__CLOVER_391_0.S[8745]++;try { __CLOVER_391_0.M[2121]++;
    } finally { }}

    /**
     * Creates a new ComponentAddress using the given Page Name and ID Path
     * @param pageName the name of the page that contains the component
     * @param idPath the ID Path of the component
     */
    public ComponentAddress(String pageName, String idPath)
    {try { __CLOVER_391_0.M[2122]++;
        __CLOVER_391_0.S[8746]++;_pageName = pageName;
        __CLOVER_391_0.S[8747]++;_idPath = idPath;
    } finally { }}

    /**
     * Creates a new ComponentAddress using the given Page Name and ID Path
     * relative on the provided Namespace
     * @param namespace the namespace of the page that contains the component
     * @param pageName the name of the page that contains the component
     * @param idPath the ID Path of the component
     */
    public ComponentAddress(
        INamespace namespace,
        String pageName,
        String idPath)
    {
        this(namespace.constructQualifiedName(pageName), idPath);__CLOVER_391_0.S[8748]++;try { __CLOVER_391_0.M[2123]++;
    } finally { }}

    /**
     * Finds a component with the current address using the given RequestCycle.
     * @param cycle the RequestCycle to use to locate the component
     * @return IComponent a component that has been initialized for the given RequestCycle
     */
    public IComponent findComponent(IRequestCycle cycle)
    {try { __CLOVER_391_0.M[2124]++;
        __CLOVER_391_0.S[8749]++;IPage objPage = cycle.getPage(_pageName);
        __CLOVER_391_0.S[8750]++;return objPage.getNestedComponent(_idPath);
    } finally { }}

    /**
     * Returns the idPath of the component.
     * @return String the ID path of the component
     */
    public String getIdPath()
    {try { __CLOVER_391_0.M[2125]++;
        __CLOVER_391_0.S[8751]++;return _idPath;
    } finally { }}

    /**
     * Returns the Page Name of the component.
     * @return String the Page Name of the component
     */
    public String getPageName()
    {try { __CLOVER_391_0.M[2126]++;
        __CLOVER_391_0.S[8752]++;return _pageName;
    } finally { }}

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode()
    {try { __CLOVER_391_0.M[2127]++;
        __CLOVER_391_0.S[8753]++;int hash = _pageName.hashCode() * 31;
        __CLOVER_391_0.S[8754]++;if ((((_idPath != null) && (++__CLOVER_391_0.CT[1473] != 0)) || (++__CLOVER_391_0.CF[1473] == 0))){
            __CLOVER_391_0.S[8755]++;hash += _idPath.hashCode();}
        __CLOVER_391_0.S[8756]++;return hash;
    } finally { }}

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(Object obj)
    {try { __CLOVER_391_0.M[2128]++;
        __CLOVER_391_0.S[8757]++;if ((((!(obj instanceof ComponentAddress)) && (++__CLOVER_391_0.CT[1474] != 0)) || (++__CLOVER_391_0.CF[1474] == 0))){
            __CLOVER_391_0.S[8758]++;return false;}

        __CLOVER_391_0.S[8759]++;if ((((obj == this) && (++__CLOVER_391_0.CT[1475] != 0)) || (++__CLOVER_391_0.CF[1475] == 0))){
            __CLOVER_391_0.S[8760]++;return true;}

        __CLOVER_391_0.S[8761]++;ComponentAddress objAddress = (ComponentAddress) obj;
        __CLOVER_391_0.S[8762]++;if ((((!getPageName().equals(objAddress.getPageName())) && (++__CLOVER_391_0.CT[1476] != 0)) || (++__CLOVER_391_0.CF[1476] == 0))){
            __CLOVER_391_0.S[8763]++;return false;}

        __CLOVER_391_0.S[8764]++;String idPath1 = getIdPath();
        __CLOVER_391_0.S[8765]++;String idPath2 = objAddress.getIdPath();
        __CLOVER_391_0.S[8766]++;return (idPath1 == idPath2)
            || (idPath1 != null && idPath1.equals(idPath2));
    } finally { }}

}
