/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.util;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

/**
 *  Represents an HTTP content type. Allows to set various elements like
 *  the mime type, the character set, and other parameters.
 *  This is similar to a number of other implementations of the same concept in JAF, etc.
 *  We have created this simple implementation to avoid including the whole libraries. 
 * 
 *  @author mindbridge
 *  @since 3.0
 **/
public class ContentType
{public static com.cortexeb.tools.clover.d __CLOVER_392_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private String _baseType;
    private String _subType;
    private Map _parameters;

    /**
     * Creates a new empty content type
     */
    public ContentType()
    {try { __CLOVER_392_0.M[2129]++;
        __CLOVER_392_0.S[8767]++;initialize();
    } finally { }}
    
    /**
     * Creates a new content type from the argument.
     * The format of the argument has to be basetype/subtype(;key=value)* 
     * 
     * @param contentType the content type that needs to be represented
     */
    public ContentType(String contentType)
    {
        this();__CLOVER_392_0.S[8768]++;try { __CLOVER_392_0.M[2130]++;
        __CLOVER_392_0.S[8769]++;parse(contentType);
    } finally { }}
    
    private void initialize()
    {try { __CLOVER_392_0.M[2131]++;
        __CLOVER_392_0.S[8770]++;_baseType = "";
        __CLOVER_392_0.S[8771]++;_subType = "";
        __CLOVER_392_0.S[8772]++;_parameters = new HashMap();
    } finally { }}
    
    /**
     * @return the base type of the content type
     */
    public String getBaseType()
    {try { __CLOVER_392_0.M[2132]++;
        __CLOVER_392_0.S[8773]++;return _baseType;
    } finally { }}

    /**
     * @param baseType
     */
    public void setBaseType(String baseType)
    {try { __CLOVER_392_0.M[2133]++;
        __CLOVER_392_0.S[8774]++;_baseType = baseType;
    } finally { }}

    /**
     * @return the sub-type of the content type
     */
    public String getSubType()
    {try { __CLOVER_392_0.M[2134]++;
        __CLOVER_392_0.S[8775]++;return _subType;
    } finally { }}

    /**
     * @param subType
     */
    public void setSubType(String subType)
    {try { __CLOVER_392_0.M[2135]++;
        __CLOVER_392_0.S[8776]++;_subType = subType;
    } finally { }}

    /**
     * @return the MIME type of the content type
     */
    public String getMimeType()
    {try { __CLOVER_392_0.M[2136]++;
        __CLOVER_392_0.S[8777]++;return _baseType + "/" + _subType;
    } finally { }}

    /**
     * @return the list of names of parameters in this content type 
     */
    public String[] getParameterNames()
    {try { __CLOVER_392_0.M[2137]++;
        __CLOVER_392_0.S[8778]++;Set parameterNames = _parameters.keySet(); 
        __CLOVER_392_0.S[8779]++;return (String[]) parameterNames.toArray(new String[parameterNames.size()]);
    } finally { }}

    /**
     * @param key the name of the content type parameter
     * @return the value of the content type parameter
     */
    public String getParameter(String key)
    {try { __CLOVER_392_0.M[2138]++;
        __CLOVER_392_0.S[8780]++;return (String) _parameters.get(key);
    } finally { }}

    /**
     * @param key the name of the content type parameter
     * @param value the value of the content type parameter
     */
    public void setParameter(String key, String value)
    {try { __CLOVER_392_0.M[2139]++;
        __CLOVER_392_0.S[8781]++;_parameters.put(key.toLowerCase(), value);
    } finally { }}

    /**
     * Parses the argument and configures the content type accordingly.
     * The format of the argument has to be type/subtype(;key=value)* 
     * 
     * @param contentType the content type that needs to be represented
     */
    public void parse(String contentType)
    {try { __CLOVER_392_0.M[2140]++;
        __CLOVER_392_0.S[8782]++;initialize();

        __CLOVER_392_0.S[8783]++;StringTokenizer tokens = new StringTokenizer(contentType, ";");
        __CLOVER_392_0.S[8784]++;if ((((!tokens.hasMoreTokens()) && (++__CLOVER_392_0.CT[1477] != 0)) || (++__CLOVER_392_0.CF[1477] == 0))){ 
            __CLOVER_392_0.S[8785]++;return;}
        
        __CLOVER_392_0.S[8786]++;String mimeType = tokens.nextToken();
        __CLOVER_392_0.S[8787]++;StringTokenizer mimeTokens = new StringTokenizer(mimeType, "/");
        __CLOVER_392_0.S[8788]++;setBaseType((((mimeTokens.hasMoreTokens() ) && (++__CLOVER_392_0.CT[1478] != 0)) || (++__CLOVER_392_0.CF[1478] == 0))? mimeTokens.nextToken() : "");
        __CLOVER_392_0.S[8789]++;setSubType((((mimeTokens.hasMoreTokens() ) && (++__CLOVER_392_0.CT[1479] != 0)) || (++__CLOVER_392_0.CF[1479] == 0))? mimeTokens.nextToken() : "");
        
        __CLOVER_392_0.S[8790]++;while ((((tokens.hasMoreTokens()) && (++__CLOVER_392_0.CT[1480] != 0)) || (++__CLOVER_392_0.CF[1480] == 0))){ {
            __CLOVER_392_0.S[8791]++;String parameter = tokens.nextToken();

            __CLOVER_392_0.S[8792]++;StringTokenizer parameterTokens = new StringTokenizer(parameter, "=");
            __CLOVER_392_0.S[8793]++;String key = (((parameterTokens.hasMoreTokens() ) && (++__CLOVER_392_0.CT[1481] != 0)) || (++__CLOVER_392_0.CF[1481] == 0))? parameterTokens.nextToken() : "";
            __CLOVER_392_0.S[8794]++;String value = (((parameterTokens.hasMoreTokens() ) && (++__CLOVER_392_0.CT[1482] != 0)) || (++__CLOVER_392_0.CF[1482] == 0))? parameterTokens.nextToken() : "";
            __CLOVER_392_0.S[8795]++;setParameter(key, value);
        }}
    } finally { }}

    

    /**
     * @return the string representation of this content type
     */
    public String unparse()
    {try { __CLOVER_392_0.M[2141]++;
        __CLOVER_392_0.S[8796]++;StringBuffer buf = new StringBuffer(getMimeType());

        __CLOVER_392_0.S[8797]++;String[] parameterNames = getParameterNames();
        __CLOVER_392_0.S[8798]++;for (int i = 0; (((i < parameterNames.length) && (++__CLOVER_392_0.CT[1483] != 0)) || (++__CLOVER_392_0.CF[1483] == 0)); i++){
        {
            __CLOVER_392_0.S[8799]++;String key = parameterNames[i];
            __CLOVER_392_0.S[8800]++;String value = getParameter(key);
            __CLOVER_392_0.S[8801]++;buf.append(";" + key + "=" + value);
        }} 
        
        __CLOVER_392_0.S[8802]++;return buf.toString();
    } finally { }}
    
    /**
     * @return the string representation of this content type. Same as unparse().
     */
    public String toString()
    {try { __CLOVER_392_0.M[2142]++;
        __CLOVER_392_0.S[8803]++;return unparse();
    } finally { }}

}
