/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.util.exception;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.CharArrayWriter;
import java.io.IOException;
import java.io.LineNumberReader;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringReader;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

/**
 *  Analyzes an exception, creating one or more 
 *  {@link ExceptionDescription}s
 *  from it.
 *
 *  @author Howard Lewis Ship
 * 
 **/

public class ExceptionAnalyzer
{public static com.cortexeb.tools.clover.d __CLOVER_406_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private List exceptionDescriptions;
    private List propertyDescriptions;
    private CharArrayWriter writer;

    private static final int LIST_SIZE = 3;

    private boolean exhaustive = false;

    /**
     *  If true, then stack trace is extracted for each exception.  If false,
     *  the default, then stack trace is extracted for only the deepest exception.
     *
     **/

    public boolean isExhaustive()
    {try { __CLOVER_406_0.M[2189]++;
        __CLOVER_406_0.S[8991]++;return exhaustive;
    } finally { }}

    public void setExhaustive(boolean value)
    {try { __CLOVER_406_0.M[2190]++;
        __CLOVER_406_0.S[8992]++;exhaustive = value;
    } finally { }}

    /**
     *  Analyzes the exceptions.  This builds an {@link ExceptionDescription} for the
     *  exception.  It also looks for a non-null {@link Throwable}
     *  property.  If one exists, then a second {@link ExceptionDescription} 
     *  is created.  This continues until no more nested exceptions can be found.
     *
     *  <p>The description includes a set of name/value properties 
     *  (as {@link ExceptionProperty}) object.  This list contains all
     *  non-null properties that are not, themselves, {@link Throwable}.
     *
     *  <p>The name is the display name (not the logical name) of the property.  The value
     *  is the <code>toString()</code> value of the property.
     *
     *  Only properties defined in subclasses of {@link Throwable} are included.
     *
     *  <p>A future enhancement will be to alphabetically sort the properties by name.
     **/

    public ExceptionDescription[] analyze(Throwable exception)
    {try { __CLOVER_406_0.M[2191]++;
        __CLOVER_406_0.S[8993]++;ExceptionDescription[] result;

        __CLOVER_406_0.S[8994]++;if ((((writer == null) && (++__CLOVER_406_0.CT[1521] != 0)) || (++__CLOVER_406_0.CF[1521] == 0))){
            __CLOVER_406_0.S[8995]++;writer = new CharArrayWriter();}

        __CLOVER_406_0.S[8996]++;if ((((propertyDescriptions == null) && (++__CLOVER_406_0.CT[1522] != 0)) || (++__CLOVER_406_0.CF[1522] == 0))){
            __CLOVER_406_0.S[8997]++;propertyDescriptions = new ArrayList(LIST_SIZE);}

        __CLOVER_406_0.S[8998]++;if ((((exceptionDescriptions == null) && (++__CLOVER_406_0.CT[1523] != 0)) || (++__CLOVER_406_0.CF[1523] == 0))){
            __CLOVER_406_0.S[8999]++;exceptionDescriptions = new ArrayList(LIST_SIZE);}

        __CLOVER_406_0.S[9000]++;while ((((exception != null) && (++__CLOVER_406_0.CT[1524] != 0)) || (++__CLOVER_406_0.CF[1524] == 0))){
        {
            __CLOVER_406_0.S[9001]++;exception = buildDescription(exception);
        }}

        __CLOVER_406_0.S[9002]++;result = new ExceptionDescription[exceptionDescriptions.size()];
        __CLOVER_406_0.S[9003]++;result = (ExceptionDescription[]) exceptionDescriptions.toArray(result);

        __CLOVER_406_0.S[9004]++;exceptionDescriptions.clear();
        __CLOVER_406_0.S[9005]++;propertyDescriptions.clear();

        __CLOVER_406_0.S[9006]++;writer.reset();

        // We never actually close() the writer which is bad ... I'm expecting that
        // the finalize() method will close them, or that they don't need to
        // close.

        __CLOVER_406_0.S[9007]++;return result;
    } finally { }}

    protected Throwable buildDescription(Throwable exception)
    {try { __CLOVER_406_0.M[2192]++;
        __CLOVER_406_0.S[9008]++;BeanInfo info;
        __CLOVER_406_0.S[9009]++;Class exceptionClass;
        __CLOVER_406_0.S[9010]++;ExceptionProperty property;
        __CLOVER_406_0.S[9011]++;PropertyDescriptor[] descriptors;
        __CLOVER_406_0.S[9012]++;PropertyDescriptor descriptor;
        __CLOVER_406_0.S[9013]++;Throwable next = null;
        __CLOVER_406_0.S[9014]++;int i;
        __CLOVER_406_0.S[9015]++;Object value;
        __CLOVER_406_0.S[9016]++;Method method;
        __CLOVER_406_0.S[9017]++;ExceptionProperty[] properties;
        __CLOVER_406_0.S[9018]++;ExceptionDescription description;
        __CLOVER_406_0.S[9019]++;String stringValue;
        __CLOVER_406_0.S[9020]++;String message;
        __CLOVER_406_0.S[9021]++;String[] stackTrace = null;

        __CLOVER_406_0.S[9022]++;propertyDescriptions.clear();

        __CLOVER_406_0.S[9023]++;message = exception.getMessage();
        __CLOVER_406_0.S[9024]++;exceptionClass = exception.getClass();

        // Get properties, ignoring those in Throwable and higher
        // (including the 'message' property).

        __CLOVER_406_0.S[9025]++;try
        {
            __CLOVER_406_0.S[9026]++;info = Introspector.getBeanInfo(exceptionClass, Throwable.class);
        }
        catch (IntrospectionException e)
        {
            __CLOVER_406_0.S[9027]++;return null;
        }

        __CLOVER_406_0.S[9028]++;descriptors = info.getPropertyDescriptors();

        __CLOVER_406_0.S[9029]++;for (i = 0; (((i < descriptors.length) && (++__CLOVER_406_0.CT[1525] != 0)) || (++__CLOVER_406_0.CF[1525] == 0)); i++){
        {
            __CLOVER_406_0.S[9030]++;descriptor = descriptors[i];

            __CLOVER_406_0.S[9031]++;method = descriptor.getReadMethod();
            __CLOVER_406_0.S[9032]++;if ((((method == null) && (++__CLOVER_406_0.CT[1526] != 0)) || (++__CLOVER_406_0.CF[1526] == 0))){
                __CLOVER_406_0.S[9033]++;continue;}

            __CLOVER_406_0.S[9034]++;try
            {
                __CLOVER_406_0.S[9035]++;value = method.invoke(exception, null);
            }
            catch (Exception e)
            {
                __CLOVER_406_0.S[9036]++;continue;
            }

            __CLOVER_406_0.S[9037]++;if ((((value == null) && (++__CLOVER_406_0.CT[1527] != 0)) || (++__CLOVER_406_0.CF[1527] == 0))){
                __CLOVER_406_0.S[9038]++;continue;}

            // Some annoying exceptions duplicate the message property
            // (I'm talking to YOU SAXParseException), so just edit that out.

            __CLOVER_406_0.S[9039]++;if ((((message != null && message.equals(value)) && (++__CLOVER_406_0.CT[1528] != 0)) || (++__CLOVER_406_0.CF[1528] == 0))){
                __CLOVER_406_0.S[9040]++;continue;}

            // Skip Throwables ... but the first non-null
            // found is the next exception.  We kind of count
            // on there being no more than one Throwable
            // property per Exception.

            __CLOVER_406_0.S[9041]++;if ((((value instanceof Throwable) && (++__CLOVER_406_0.CT[1529] != 0)) || (++__CLOVER_406_0.CF[1529] == 0))){
            {
                __CLOVER_406_0.S[9042]++;if ((((next == null) && (++__CLOVER_406_0.CT[1530] != 0)) || (++__CLOVER_406_0.CF[1530] == 0))){
                    __CLOVER_406_0.S[9043]++;next = (Throwable) value;}

                __CLOVER_406_0.S[9044]++;continue;
            }}

            __CLOVER_406_0.S[9045]++;stringValue = value.toString().trim();

            __CLOVER_406_0.S[9046]++;if ((((stringValue.length() == 0) && (++__CLOVER_406_0.CT[1531] != 0)) || (++__CLOVER_406_0.CF[1531] == 0))){
                __CLOVER_406_0.S[9047]++;continue;}

            __CLOVER_406_0.S[9048]++;property = new ExceptionProperty(descriptor.getDisplayName(), value.toString());

            __CLOVER_406_0.S[9049]++;propertyDescriptions.add(property);
        }}

        // If exhaustive, or in the deepest exception (where there's no next)
        // the extract the stack trace.

        __CLOVER_406_0.S[9050]++;if ((((next == null || exhaustive) && (++__CLOVER_406_0.CT[1532] != 0)) || (++__CLOVER_406_0.CF[1532] == 0))){
            __CLOVER_406_0.S[9051]++;stackTrace = getStackTrace(exception);}

        // Would be nice to sort the properties here.

        __CLOVER_406_0.S[9052]++;properties = new ExceptionProperty[propertyDescriptions.size()];

        __CLOVER_406_0.S[9053]++;ExceptionProperty[] propArray =
            (ExceptionProperty[]) propertyDescriptions.toArray(properties);

        __CLOVER_406_0.S[9054]++;description =
            new ExceptionDescription(
                exceptionClass.getName(),
                message,
                propArray,
                stackTrace);

        __CLOVER_406_0.S[9055]++;exceptionDescriptions.add(description);

        __CLOVER_406_0.S[9056]++;return next;
    } finally { }}

    /**
     *  Gets the stack trace for the exception, and converts it into an array of strings.
     *
     *  <p>This involves parsing the 
     *   string generated indirectly from
     *  <code>Throwable.printStackTrace(PrintWriter)</code>.  This method can get confused
     *  if the message (presumably, the first line emitted by printStackTrace())
     *  spans multiple lines.
     *
     *  <p>Different JVMs format the exception in different ways.
     *
     *  <p>A possible expansion would be more flexibility in defining the pattern
     *  used.  Hopefully all 'mainstream' JVMs are close enough for this to continue
     *  working.
     *
     **/

    protected String[] getStackTrace(Throwable exception)
    {try { __CLOVER_406_0.M[2193]++;
        __CLOVER_406_0.S[9057]++;writer.reset();

        __CLOVER_406_0.S[9058]++;PrintWriter printWriter = new PrintWriter(writer);

        __CLOVER_406_0.S[9059]++;exception.printStackTrace(printWriter);

        __CLOVER_406_0.S[9060]++;printWriter.close();

        __CLOVER_406_0.S[9061]++;String fullTrace = writer.toString();

        __CLOVER_406_0.S[9062]++;writer.reset();

        // OK, the trick is to convert the full trace into an array of stack frames.

        __CLOVER_406_0.S[9063]++;StringReader stringReader = new StringReader(fullTrace);
        __CLOVER_406_0.S[9064]++;LineNumberReader lineReader = new LineNumberReader(stringReader);
        __CLOVER_406_0.S[9065]++;int lineNumber = 0;
        __CLOVER_406_0.S[9066]++;List frames = new ArrayList();

        __CLOVER_406_0.S[9067]++;try
        {
            __CLOVER_406_0.S[9068]++;while (true){
            {
                __CLOVER_406_0.S[9069]++;String line = lineReader.readLine();

                __CLOVER_406_0.S[9070]++;if ((((line == null) && (++__CLOVER_406_0.CT[1533] != 0)) || (++__CLOVER_406_0.CF[1533] == 0))){
                    __CLOVER_406_0.S[9071]++;break;}

                // Always ignore the first line.

                __CLOVER_406_0.S[9072]++;if ((((++lineNumber == 1) && (++__CLOVER_406_0.CT[1534] != 0)) || (++__CLOVER_406_0.CF[1534] == 0))){
                    __CLOVER_406_0.S[9073]++;continue;}

                __CLOVER_406_0.S[9074]++;frames.add(stripFrame(line));
            }}

            __CLOVER_406_0.S[9075]++;lineReader.close();
        }
        catch (IOException ex)
        {
            // Not likely to happen with this particular set
            // of readers.
        }

        __CLOVER_406_0.S[9076]++;String result[] = new String[frames.size()];

        __CLOVER_406_0.S[9077]++;return (String[]) frames.toArray(result);
    } finally { }}

    private static final int SKIP_LEADING_WHITESPACE = 0;
    private static final int SKIP_T = 1;
    private static final int SKIP_OTHER_WHITESPACE = 2;

    /**
     *  Sun's JVM prefixes each line in the stack trace
     *  with "<tab>at ", other JVMs don't.  This method
     *  looks for and strips such stuff.
     *
     **/

    private String stripFrame(String frame)
    {try { __CLOVER_406_0.M[2194]++;
        __CLOVER_406_0.S[9078]++;char array[] = frame.toCharArray();

        __CLOVER_406_0.S[9079]++;int i = 0;
        __CLOVER_406_0.S[9080]++;int state = SKIP_LEADING_WHITESPACE;
        __CLOVER_406_0.S[9081]++;boolean more = true;

        __CLOVER_406_0.S[9082]++;while ((((more) && (++__CLOVER_406_0.CT[1535] != 0)) || (++__CLOVER_406_0.CF[1535] == 0))){
        {
            // Ran out of characters to skip?  Return the empty string.

            __CLOVER_406_0.S[9083]++;if ((((i == array.length) && (++__CLOVER_406_0.CT[1536] != 0)) || (++__CLOVER_406_0.CF[1536] == 0))){
                __CLOVER_406_0.S[9084]++;return "";}

            __CLOVER_406_0.S[9085]++;char ch = array[i];

            __CLOVER_406_0.S[9086]++;switch (state)
            {
                // Ignore whitespace at the start of the line.

                case SKIP_LEADING_WHITESPACE :

                    __CLOVER_406_0.S[9087]++;if ((((Character.isWhitespace(ch)) && (++__CLOVER_406_0.CT[1537] != 0)) || (++__CLOVER_406_0.CF[1537] == 0))){
                    {
                        __CLOVER_406_0.S[9088]++;i++;
                        __CLOVER_406_0.S[9089]++;continue;
                    }}

                    __CLOVER_406_0.S[9090]++;if ((((ch == 'a') && (++__CLOVER_406_0.CT[1538] != 0)) || (++__CLOVER_406_0.CF[1538] == 0))){
                    {
                        __CLOVER_406_0.S[9091]++;state = SKIP_T;
                        __CLOVER_406_0.S[9092]++;i++;
                        __CLOVER_406_0.S[9093]++;continue;
                    }}

                    // Found non-whitespace, not 'a'
                    __CLOVER_406_0.S[9094]++;more = false;
                    __CLOVER_406_0.S[9095]++;break;

                    // Skip over the 't' after an 'a'

                case SKIP_T :

                    __CLOVER_406_0.S[9096]++;if ((((ch == 't') && (++__CLOVER_406_0.CT[1539] != 0)) || (++__CLOVER_406_0.CF[1539] == 0))){
                    {
                        __CLOVER_406_0.S[9097]++;state = SKIP_OTHER_WHITESPACE;
                        __CLOVER_406_0.S[9098]++;i++;
                        __CLOVER_406_0.S[9099]++;continue;
                    }}

                    // Back out the skipped-over 'a'

                    __CLOVER_406_0.S[9100]++;i--;
                    __CLOVER_406_0.S[9101]++;more = false;
                    __CLOVER_406_0.S[9102]++;break;

                    // Skip whitespace between 'at' and the name of the class

                case SKIP_OTHER_WHITESPACE :

                    __CLOVER_406_0.S[9103]++;if ((((Character.isWhitespace(ch)) && (++__CLOVER_406_0.CT[1540] != 0)) || (++__CLOVER_406_0.CF[1540] == 0))){
                    {
                        __CLOVER_406_0.S[9104]++;i++;
                        __CLOVER_406_0.S[9105]++;continue;
                    }}

                    // Not whitespace
                    __CLOVER_406_0.S[9106]++;more = false;
                    __CLOVER_406_0.S[9107]++;break;
            }

        }}

        // Found nothing to strip out.

        __CLOVER_406_0.S[9108]++;if ((((i == 0) && (++__CLOVER_406_0.CT[1541] != 0)) || (++__CLOVER_406_0.CF[1541] == 0))){
            __CLOVER_406_0.S[9109]++;return frame;}

        __CLOVER_406_0.S[9110]++;return frame.substring(i);
    } finally { }}

    /**
     *  Produces a text based exception report to the provided stream.
     *
     **/

    public void reportException(Throwable exception, PrintStream stream)
    {try { __CLOVER_406_0.M[2195]++;
        __CLOVER_406_0.S[9111]++;int i;
        __CLOVER_406_0.S[9112]++;int j;
        __CLOVER_406_0.S[9113]++;ExceptionDescription[] descriptions;
        __CLOVER_406_0.S[9114]++;ExceptionProperty[] properties;
        __CLOVER_406_0.S[9115]++;String[] stackTrace;
        __CLOVER_406_0.S[9116]++;String message;

        __CLOVER_406_0.S[9117]++;descriptions = analyze(exception);

        __CLOVER_406_0.S[9118]++;for (i = 0; (((i < descriptions.length) && (++__CLOVER_406_0.CT[1542] != 0)) || (++__CLOVER_406_0.CF[1542] == 0)); i++){
        {
            __CLOVER_406_0.S[9119]++;message = descriptions[i].getMessage();

            __CLOVER_406_0.S[9120]++;if ((((message == null) && (++__CLOVER_406_0.CT[1543] != 0)) || (++__CLOVER_406_0.CF[1543] == 0))){
                __CLOVER_406_0.S[9121]++;stream.println(descriptions[i].getExceptionClassName());}
            else{
                __CLOVER_406_0.S[9122]++;stream.println(
                    descriptions[i].getExceptionClassName() + ": " + descriptions[i].getMessage());}

            __CLOVER_406_0.S[9123]++;properties = descriptions[i].getProperties();

            __CLOVER_406_0.S[9124]++;for (j = 0; (((j < properties.length) && (++__CLOVER_406_0.CT[1544] != 0)) || (++__CLOVER_406_0.CF[1544] == 0)); j++){
                __CLOVER_406_0.S[9125]++;stream.println(
                    "   " + properties[j].getName() + ": " + properties[j].getValue());}

            // Just show the stack trace on the deepest exception.

            __CLOVER_406_0.S[9126]++;if ((((i + 1 == descriptions.length) && (++__CLOVER_406_0.CT[1545] != 0)) || (++__CLOVER_406_0.CF[1545] == 0))){
            {
                __CLOVER_406_0.S[9127]++;stackTrace = descriptions[i].getStackTrace();

                __CLOVER_406_0.S[9128]++;for (j = 0; (((j < stackTrace.length) && (++__CLOVER_406_0.CT[1546] != 0)) || (++__CLOVER_406_0.CF[1546] == 0)); j++){
                    __CLOVER_406_0.S[9129]++;stream.println(stackTrace[j]);}
            }}
            else{
                __CLOVER_406_0.S[9130]++;stream.println();}
        }}
    } finally { }}

}