/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.util.io;

import java.io.IOException;

import org.apache.hivemind.ClassResolver;
import org.apache.tapestry.Tapestry;
import org.apache.tapestry.services.DataSqueezer;
import org.apache.tapestry.util.AdaptorRegistry;

/**
 * A class used to convert arbitrary objects to Strings and back. This has
 * particular uses involving HTTP URLs and Cookies.
 * 
 * @author Howard Lewis Ship
 */

public class DataSqueezerImpl implements DataSqueezer
{public static com.cortexeb.tools.clover.d __CLOVER_414_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final String NULL_PREFIX = "X";

    private static final char NULL_PREFIX_CH = 'X';

    private static final int ARRAY_SIZE = 90;

    private static final int FIRST_ADAPTOR_OFFSET = 33;

    /**
     * An array of adaptors; this is used as a cheap lookup-table when
     * unsqueezing. Each adaptor is identified by a single ASCII character, in
     * the range of 33 ('!') to 122 (the letter 'z'). The offset into this table
     * is the character minus 33.
     */

    private ISqueezeAdaptor[] _adaptorByPrefix = new ISqueezeAdaptor[ARRAY_SIZE];

    /**
     * AdaptorRegistry cache of adaptors.
     */

    private AdaptorRegistry _adaptors = new AdaptorRegistry();

    /**
     * Resource resolver used to deserialize classes.
     */

    private ClassResolver _resolver;

    /**
     * Creates a new squeezer with the default set of adaptors.
     */

    public DataSqueezerImpl(ClassResolver resolver)
    {
        this(resolver, null);__CLOVER_414_0.S[9255]++;try { __CLOVER_414_0.M[2236]++;
    } finally { }}

    /**
     * Creates a new data squeezer, which will have the default set of adaptors,
     * and may add additional adaptors.
     * 
     * @param adaptors
     *            an optional list of adaptors that will be registered to the
     *            data squeezer (it may be null or empty)
     */

    public DataSqueezerImpl(ClassResolver resolver, ISqueezeAdaptor[] adaptors)
    {try { __CLOVER_414_0.M[2237]++;
        __CLOVER_414_0.S[9256]++;_resolver = resolver;

        __CLOVER_414_0.S[9257]++;registerDefaultAdaptors();

        __CLOVER_414_0.S[9258]++;if ((((adaptors != null) && (++__CLOVER_414_0.CT[1572] != 0)) || (++__CLOVER_414_0.CF[1572] == 0))){
            __CLOVER_414_0.S[9259]++;for (int i = 0; (((i < adaptors.length) && (++__CLOVER_414_0.CT[1573] != 0)) || (++__CLOVER_414_0.CF[1573] == 0)); i++){
                __CLOVER_414_0.S[9260]++;adaptors[i].register(this);}}
    } finally { }}

    private void registerDefaultAdaptors()
    {try { __CLOVER_414_0.M[2238]++;
        __CLOVER_414_0.S[9261]++;new CharacterAdaptor().register(this);
        __CLOVER_414_0.S[9262]++;new StringAdaptor().register(this);
        __CLOVER_414_0.S[9263]++;new IntegerAdaptor().register(this);
        __CLOVER_414_0.S[9264]++;new DoubleAdaptor().register(this);
        __CLOVER_414_0.S[9265]++;new ByteAdaptor().register(this);
        __CLOVER_414_0.S[9266]++;new FloatAdaptor().register(this);
        __CLOVER_414_0.S[9267]++;new LongAdaptor().register(this);
        __CLOVER_414_0.S[9268]++;new ShortAdaptor().register(this);
        __CLOVER_414_0.S[9269]++;new BooleanAdaptor().register(this);
        __CLOVER_414_0.S[9270]++;new SerializableAdaptor().register(this);
        __CLOVER_414_0.S[9271]++;new ComponentAddressAdaptor().register(this);
        __CLOVER_414_0.S[9272]++;new EnumAdaptor().register(this);
    } finally { }}

    /**
     * Registers the adaptor with one or more single-character prefixes.
     * 
     * @param prefix
     *            one or more characters, each of which will be a prefix for the
     *            adaptor.
     * @param dataClass
     *            the class (or interface) which can be encoded by the adaptor.
     * @param adaptor
     *            the adaptor which to be registered.
     */

    public synchronized void register(String prefix, Class dataClass, ISqueezeAdaptor adaptor)
    {try { __CLOVER_414_0.M[2239]++;
        __CLOVER_414_0.S[9273]++;int prefixLength = prefix.length();
        __CLOVER_414_0.S[9274]++;int offset;

        __CLOVER_414_0.S[9275]++;if ((((prefixLength < 1) && (++__CLOVER_414_0.CT[1574] != 0)) || (++__CLOVER_414_0.CF[1574] == 0))){
            __CLOVER_414_0.S[9276]++;throw new IllegalArgumentException(Tapestry.getMessage("DataSqueezer.short-prefix"));}

        __CLOVER_414_0.S[9277]++;if ((((dataClass == null) && (++__CLOVER_414_0.CT[1575] != 0)) || (++__CLOVER_414_0.CF[1575] == 0))){
            __CLOVER_414_0.S[9278]++;throw new IllegalArgumentException(Tapestry.getMessage("DataSqueezer.null-class"));}

        __CLOVER_414_0.S[9279]++;if ((((adaptor == null) && (++__CLOVER_414_0.CT[1576] != 0)) || (++__CLOVER_414_0.CF[1576] == 0))){
            __CLOVER_414_0.S[9280]++;throw new IllegalArgumentException(Tapestry.getMessage("DataSqueezer.null-adaptor"));}

        __CLOVER_414_0.S[9281]++;for (int i = 0; (((i < prefixLength) && (++__CLOVER_414_0.CT[1577] != 0)) || (++__CLOVER_414_0.CF[1577] == 0)); i++){
        {
            __CLOVER_414_0.S[9282]++;char ch = prefix.charAt(i);

            __CLOVER_414_0.S[9283]++;if ((((ch < '!' | ch > 'z') && (++__CLOVER_414_0.CT[1578] != 0)) || (++__CLOVER_414_0.CF[1578] == 0))){
                __CLOVER_414_0.S[9284]++;throw new IllegalArgumentException(Tapestry.getMessage("DataSqueezer.prefix-out-of-range"));}

            __CLOVER_414_0.S[9285]++;offset = ch - FIRST_ADAPTOR_OFFSET;

            __CLOVER_414_0.S[9286]++;if ((((_adaptorByPrefix[offset] != null) && (++__CLOVER_414_0.CT[1579] != 0)) || (++__CLOVER_414_0.CF[1579] == 0))){
                __CLOVER_414_0.S[9287]++;throw new IllegalArgumentException(Tapestry.format("DataSqueezer.adaptor-prefix-taken", prefix
                        .substring(i, i)));}

            __CLOVER_414_0.S[9288]++;_adaptorByPrefix[offset] = adaptor;

        }}

        __CLOVER_414_0.S[9289]++;_adaptors.register(dataClass, adaptor);
    } finally { }}

    /**
     * Squeezes the data object into a String by locating an appropriate adaptor
     * that can perform the conversion. data may be null.
     */

    public String squeeze(Object data) throws IOException
    {try { __CLOVER_414_0.M[2240]++;
        __CLOVER_414_0.S[9290]++;ISqueezeAdaptor adaptor;

        __CLOVER_414_0.S[9291]++;if ((((data == null) && (++__CLOVER_414_0.CT[1580] != 0)) || (++__CLOVER_414_0.CF[1580] == 0))){
            __CLOVER_414_0.S[9292]++;return NULL_PREFIX;}

        __CLOVER_414_0.S[9293]++;adaptor = (ISqueezeAdaptor) _adaptors.getAdaptor(data.getClass());

        __CLOVER_414_0.S[9294]++;return adaptor.squeeze(this, data);
    } finally { }}

    /**
     * A convience; invokes {@link #squeeze(Object)}for each element in the
     * data array. If data is null, returns null.
     */

    public String[] squeeze(Object[] data) throws IOException
    {try { __CLOVER_414_0.M[2241]++;
        __CLOVER_414_0.S[9295]++;if ((((data == null) && (++__CLOVER_414_0.CT[1581] != 0)) || (++__CLOVER_414_0.CF[1581] == 0))){
            __CLOVER_414_0.S[9296]++;return null;}

        __CLOVER_414_0.S[9297]++;int length = data.length;
        __CLOVER_414_0.S[9298]++;String[] result;

        __CLOVER_414_0.S[9299]++;result = new String[length];

        __CLOVER_414_0.S[9300]++;for (int i = 0; (((i < length) && (++__CLOVER_414_0.CT[1582] != 0)) || (++__CLOVER_414_0.CF[1582] == 0)); i++){
            __CLOVER_414_0.S[9301]++;result[i] = squeeze(data[i]);}

        __CLOVER_414_0.S[9302]++;return result;
    } finally { }}

    /**
     * Unsqueezes the string. Note that in a special case, where the first
     * character of the string is not a recognized prefix, it is assumed that
     * the string is simply a string, and return with no change.
     */

    public Object unsqueeze(String string) throws IOException
    {try { __CLOVER_414_0.M[2242]++;
        __CLOVER_414_0.S[9303]++;ISqueezeAdaptor adaptor = null;

        __CLOVER_414_0.S[9304]++;if ((((string.equals(NULL_PREFIX)) && (++__CLOVER_414_0.CT[1583] != 0)) || (++__CLOVER_414_0.CF[1583] == 0))){
            __CLOVER_414_0.S[9305]++;return null;}

        __CLOVER_414_0.S[9306]++;int offset = string.charAt(0) - FIRST_ADAPTOR_OFFSET;

        __CLOVER_414_0.S[9307]++;if ((((offset >= 0 && offset < _adaptorByPrefix.length) && (++__CLOVER_414_0.CT[1584] != 0)) || (++__CLOVER_414_0.CF[1584] == 0))){
            __CLOVER_414_0.S[9308]++;adaptor = _adaptorByPrefix[offset];}

        // If the adaptor is not otherwise recognized, the it is simply
        // an encoded String (the StringAdaptor may not have added
        // a prefix).

        __CLOVER_414_0.S[9309]++;if ((((adaptor == null) && (++__CLOVER_414_0.CT[1585] != 0)) || (++__CLOVER_414_0.CF[1585] == 0))){
            __CLOVER_414_0.S[9310]++;return string;}

        // Adaptor should never be null, because we always supply
        // an adaptor for String

        __CLOVER_414_0.S[9311]++;return adaptor.unsqueeze(this, string);
    } finally { }}

    /**
     * Convienience method for unsqueezing many strings (back into objects).
     * <p>
     * If strings is null, returns null.
     */

    public Object[] unsqueeze(String[] strings) throws IOException
    {try { __CLOVER_414_0.M[2243]++;
        __CLOVER_414_0.S[9312]++;if ((((strings == null) && (++__CLOVER_414_0.CT[1586] != 0)) || (++__CLOVER_414_0.CF[1586] == 0))){
            __CLOVER_414_0.S[9313]++;return null;}

        __CLOVER_414_0.S[9314]++;int length = strings.length;
        __CLOVER_414_0.S[9315]++;Object[] result;

        __CLOVER_414_0.S[9316]++;result = new Object[length];

        __CLOVER_414_0.S[9317]++;for (int i = 0; (((i < length) && (++__CLOVER_414_0.CT[1587] != 0)) || (++__CLOVER_414_0.CF[1587] == 0)); i++){
            __CLOVER_414_0.S[9318]++;result[i] = unsqueeze(strings[i]);}

        __CLOVER_414_0.S[9319]++;return result;
    } finally { }}

    /**
     * Checks to see if a given prefix character has a registered adaptor. This
     * is used by the String adaptor to determine whether it needs to put a
     * prefix on its String.
     */

    public boolean isPrefixRegistered(char prefix)
    {try { __CLOVER_414_0.M[2244]++;
        __CLOVER_414_0.S[9320]++;int offset = prefix - FIRST_ADAPTOR_OFFSET;

        // Special case for handling nulls.

        __CLOVER_414_0.S[9321]++;if ((((prefix == NULL_PREFIX_CH) && (++__CLOVER_414_0.CT[1588] != 0)) || (++__CLOVER_414_0.CF[1588] == 0))){
            __CLOVER_414_0.S[9322]++;return true;}

        __CLOVER_414_0.S[9323]++;if ((((offset < 0 || offset >= _adaptorByPrefix.length) && (++__CLOVER_414_0.CT[1589] != 0)) || (++__CLOVER_414_0.CF[1589] == 0))){
            __CLOVER_414_0.S[9324]++;return false;}

        __CLOVER_414_0.S[9325]++;return _adaptorByPrefix[offset] != null;
    } finally { }}

    public String toString()
    {try { __CLOVER_414_0.M[2245]++;
        __CLOVER_414_0.S[9326]++;StringBuffer buffer;

        __CLOVER_414_0.S[9327]++;buffer = new StringBuffer();
        __CLOVER_414_0.S[9328]++;buffer.append("DataSqueezer[adaptors=<");
        __CLOVER_414_0.S[9329]++;buffer.append(_adaptors.toString());
        __CLOVER_414_0.S[9330]++;buffer.append(">]");

        __CLOVER_414_0.S[9331]++;return buffer.toString();
    } finally { }}

    /**
     * Returns the resource resolver used with this squeezer.
     * 
     * @since 2.2
     */

    public ClassResolver getResolver()
    {try { __CLOVER_414_0.M[2246]++;
        __CLOVER_414_0.S[9332]++;return _resolver;
    } finally { }}

}