/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.util.io;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import org.apache.tapestry.Tapestry;
import org.apache.tapestry.services.DataSqueezer;

/**
 *  The most complicated of the adaptors, this one takes an arbitrary serializable
 *  object, serializes it to binary, and encodes it in a Base64 encoding.
 *
 *  <p>Encoding and decoding of Base64 strings uses code adapted from work in the public
 *  domain originally written by Jonathan Knudsen and published in
 *  O'reilly's "Java Cryptography". Note that we use a <em>modified</em> form of Base64 encoding,
 *  with URL-safe characters to encode the 62 and 63 values and the pad character.
 *
 *  <p>TBD:  Work out some class loader issues involved in deserializing.
 *
 *  @author Howard Lewis Ship
 * 
 **/

class SerializableAdaptor implements ISqueezeAdaptor
{public static com.cortexeb.tools.clover.d __CLOVER_422_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final String PREFIX = "O";

    /**
     *  The PAD character, appended to the end of the string to make things
     *  line up.  In normal Base64, this is the character '='.
     *
     **/

    private static final char PAD = '.';

    /**
     *  Representation for the 6-bit code 63, normally '+' in Base64.
     *
     **/

    private static final char CH_62 = '-';

    /**
     *  Representation for the 6-bit code 64, normally '/' in Base64.
     *
     **/

    private static final char CH_63 = '_';

    public String squeeze(DataSqueezer squeezer, Object data) throws IOException
    {try { __CLOVER_422_0.M[2264]++;
        __CLOVER_422_0.S[9358]++;ByteArrayOutputStream bos = null;
        __CLOVER_422_0.S[9359]++;GZIPOutputStream gos = null;
        __CLOVER_422_0.S[9360]++;ObjectOutputStream oos = null;
        __CLOVER_422_0.S[9361]++;byte[] byteData = null;

        __CLOVER_422_0.S[9362]++;try
        {
            __CLOVER_422_0.S[9363]++;bos = new ByteArrayOutputStream();
            __CLOVER_422_0.S[9364]++;gos = new GZIPOutputStream(bos);
            __CLOVER_422_0.S[9365]++;oos = new ObjectOutputStream(gos);

            __CLOVER_422_0.S[9366]++;oos.writeObject(data);
            __CLOVER_422_0.S[9367]++;oos.close();
        }
        finally
        {
            __CLOVER_422_0.S[9368]++;close(oos);
            __CLOVER_422_0.S[9369]++;close(gos);
            __CLOVER_422_0.S[9370]++;close(bos);
        }

        __CLOVER_422_0.S[9371]++;byteData = bos.toByteArray();

        __CLOVER_422_0.S[9372]++;StringBuffer encoded = new StringBuffer(2 * byteData.length);
        __CLOVER_422_0.S[9373]++;char[] base64 = new char[4];

        __CLOVER_422_0.S[9374]++;encoded.append(PREFIX);

        __CLOVER_422_0.S[9375]++;for (int i = 0; (((i < byteData.length) && (++__CLOVER_422_0.CT[1590] != 0)) || (++__CLOVER_422_0.CF[1590] == 0)); i += 3){
        {
            __CLOVER_422_0.S[9376]++;encodeBlock(byteData, i, base64);
            __CLOVER_422_0.S[9377]++;encoded.append(base64);
        }}

        __CLOVER_422_0.S[9378]++;return encoded.toString();
    } finally { }}

    private void close(OutputStream stream)
    {try { __CLOVER_422_0.M[2265]++;
        __CLOVER_422_0.S[9379]++;if ((((stream != null) && (++__CLOVER_422_0.CT[1591] != 0)) || (++__CLOVER_422_0.CF[1591] == 0))){
        {
            __CLOVER_422_0.S[9380]++;try
            {
                __CLOVER_422_0.S[9381]++;stream.close();
            }
            catch (IOException ex)
            {
                // Ignore.
            }
        }}
    } finally { }}

    private void close(InputStream stream)
    {try { __CLOVER_422_0.M[2266]++;
        __CLOVER_422_0.S[9382]++;if ((((stream != null) && (++__CLOVER_422_0.CT[1592] != 0)) || (++__CLOVER_422_0.CF[1592] == 0))){
        {
            __CLOVER_422_0.S[9383]++;try
            {
                __CLOVER_422_0.S[9384]++;stream.close();
            }
            catch (IOException ex)
            {
                // Ignore.
            }
        }}
    } finally { }}
    public Object unsqueeze(DataSqueezer squeezer, String string) throws IOException
    {try { __CLOVER_422_0.M[2267]++;
        __CLOVER_422_0.S[9385]++;ByteArrayInputStream bis = null;
        __CLOVER_422_0.S[9386]++;GZIPInputStream gis = null;
        __CLOVER_422_0.S[9387]++;ObjectInputStream ois = null;
        __CLOVER_422_0.S[9388]++;byte[] byteData;

        // Strip off the first character and decode the rest.

        __CLOVER_422_0.S[9389]++;byteData = decode(string.substring(1));

        __CLOVER_422_0.S[9390]++;try
        {
            __CLOVER_422_0.S[9391]++;bis = new ByteArrayInputStream(byteData);
            __CLOVER_422_0.S[9392]++;gis = new GZIPInputStream(bis);
            __CLOVER_422_0.S[9393]++;ois = new ResolvingObjectInputStream(squeezer.getResolver(), gis);

            __CLOVER_422_0.S[9394]++;return ois.readObject();
        }
        catch (ClassNotFoundException ex)
        {
            // The message is the name of the class.

            __CLOVER_422_0.S[9395]++;throw new IOException(
                Tapestry.format("SerializableAdaptor.class-not-found", ex.getMessage()));
        }
        finally
        {
            __CLOVER_422_0.S[9396]++;close(ois);
            __CLOVER_422_0.S[9397]++;close(gis);
            __CLOVER_422_0.S[9398]++;close(bis);
        }
    } finally { }}

    public void register(DataSqueezer squeezer)
    {try { __CLOVER_422_0.M[2268]++;
        __CLOVER_422_0.S[9399]++;squeezer.register(PREFIX, Serializable.class, this);
    } finally { }}

    private static void encodeBlock(byte[] raw, int offset, char[] base64) throws IOException
    {try { __CLOVER_422_0.M[2269]++;
        __CLOVER_422_0.S[9400]++;int block = 0;
        __CLOVER_422_0.S[9401]++;int slack = raw.length - offset - 1;
        __CLOVER_422_0.S[9402]++;int end = ((((slack >= 2) ) && (++__CLOVER_422_0.CT[1593] != 0)) || (++__CLOVER_422_0.CF[1593] == 0))? 2 : slack;

        __CLOVER_422_0.S[9403]++;for (int i = 0; (((i <= end) && (++__CLOVER_422_0.CT[1594] != 0)) || (++__CLOVER_422_0.CF[1594] == 0)); i++){
        {
            __CLOVER_422_0.S[9404]++;byte b = raw[offset + i];
            __CLOVER_422_0.S[9405]++;int neuter = ((((b < 0) ) && (++__CLOVER_422_0.CT[1595] != 0)) || (++__CLOVER_422_0.CF[1595] == 0))? b + 256 : b;
            __CLOVER_422_0.S[9406]++;block += neuter << (8 * (2 - i));
        }}

        __CLOVER_422_0.S[9407]++;for (int i = 0; (((i < 4) && (++__CLOVER_422_0.CT[1596] != 0)) || (++__CLOVER_422_0.CF[1596] == 0)); i++){
        {
            __CLOVER_422_0.S[9408]++;int sixbit = (block >>> (6 * (3 - i))) & 0x3f;
            __CLOVER_422_0.S[9409]++;base64[i] = getChar(sixbit);
        }}

        __CLOVER_422_0.S[9410]++;if ((((slack < 1) && (++__CLOVER_422_0.CT[1597] != 0)) || (++__CLOVER_422_0.CF[1597] == 0))){
            __CLOVER_422_0.S[9411]++;base64[2] = PAD;}

        __CLOVER_422_0.S[9412]++;if ((((slack < 2) && (++__CLOVER_422_0.CT[1598] != 0)) || (++__CLOVER_422_0.CF[1598] == 0))){
            __CLOVER_422_0.S[9413]++;base64[3] = PAD;}
    } finally { }}

    protected static char getChar(int sixBit) throws IOException
    {try { __CLOVER_422_0.M[2270]++;
        __CLOVER_422_0.S[9414]++;if ((((sixBit >= 0 && sixBit <= 25) && (++__CLOVER_422_0.CT[1599] != 0)) || (++__CLOVER_422_0.CF[1599] == 0))){
            __CLOVER_422_0.S[9415]++;return (char) ('A' + sixBit);}

        __CLOVER_422_0.S[9416]++;if ((((sixBit >= 26 && sixBit <= 51) && (++__CLOVER_422_0.CT[1600] != 0)) || (++__CLOVER_422_0.CF[1600] == 0))){
            __CLOVER_422_0.S[9417]++;return (char) ('a' + (sixBit - 26));}

        __CLOVER_422_0.S[9418]++;if ((((sixBit >= 52 && sixBit <= 61) && (++__CLOVER_422_0.CT[1601] != 0)) || (++__CLOVER_422_0.CF[1601] == 0))){
            __CLOVER_422_0.S[9419]++;return (char) ('0' + (sixBit - 52));}

        __CLOVER_422_0.S[9420]++;if ((((sixBit == 62) && (++__CLOVER_422_0.CT[1602] != 0)) || (++__CLOVER_422_0.CF[1602] == 0))){
            __CLOVER_422_0.S[9421]++;return CH_62;}

        __CLOVER_422_0.S[9422]++;if ((((sixBit == 63) && (++__CLOVER_422_0.CT[1603] != 0)) || (++__CLOVER_422_0.CF[1603] == 0))){
            __CLOVER_422_0.S[9423]++;return CH_63;}

        __CLOVER_422_0.S[9424]++;throw new IOException(
            Tapestry.format("SerializableAdaptor.unable-to-convert", Integer.toString(sixBit)));
    } finally { }}

    public static byte[] decode(String string) throws IOException
    {try { __CLOVER_422_0.M[2271]++;
        __CLOVER_422_0.S[9425]++;int pad = 0;
        __CLOVER_422_0.S[9426]++;char[] base64 = string.toCharArray();

        __CLOVER_422_0.S[9427]++;for (int i = base64.length - 1; (((base64[i] == PAD) && (++__CLOVER_422_0.CT[1604] != 0)) || (++__CLOVER_422_0.CF[1604] == 0)); i--){
            __CLOVER_422_0.S[9428]++;pad++;}

        __CLOVER_422_0.S[9429]++;int length = base64.length * 6 / 8 - pad;
        __CLOVER_422_0.S[9430]++;byte[] raw = new byte[length];
        __CLOVER_422_0.S[9431]++;int rawIndex = 0;

        __CLOVER_422_0.S[9432]++;for (int i = 0; (((i < base64.length) && (++__CLOVER_422_0.CT[1605] != 0)) || (++__CLOVER_422_0.CF[1605] == 0)); i += 4){
        {
            __CLOVER_422_0.S[9433]++;int block =
                (getValue(base64[i]) << 18)
                    + (getValue(base64[i + 1]) << 12)
                    + (getValue(base64[i + 2]) << 6)
                    + (getValue(base64[i + 3]));

            __CLOVER_422_0.S[9434]++;for (int j = 0; (((j < 3 && rawIndex + j < raw.length) && (++__CLOVER_422_0.CT[1606] != 0)) || (++__CLOVER_422_0.CF[1606] == 0)); j++){
                __CLOVER_422_0.S[9435]++;raw[rawIndex + j] = (byte) ((block >> (8 * (2 - j))) & 0xff);}

            __CLOVER_422_0.S[9436]++;rawIndex += 3;
        }}

        __CLOVER_422_0.S[9437]++;return raw;
    } finally { }}

    private static int getValue(char c) throws IOException
    {try { __CLOVER_422_0.M[2272]++;
        __CLOVER_422_0.S[9438]++;if ((((c >= 'A' && c <= 'Z') && (++__CLOVER_422_0.CT[1607] != 0)) || (++__CLOVER_422_0.CF[1607] == 0))){
            __CLOVER_422_0.S[9439]++;return c - 'A';}

        __CLOVER_422_0.S[9440]++;if ((((c >= 'a' && c <= 'z') && (++__CLOVER_422_0.CT[1608] != 0)) || (++__CLOVER_422_0.CF[1608] == 0))){
            __CLOVER_422_0.S[9441]++;return c - 'a' + 26;}

        __CLOVER_422_0.S[9442]++;if ((((c >= '0' && c <= '9') && (++__CLOVER_422_0.CT[1609] != 0)) || (++__CLOVER_422_0.CF[1609] == 0))){
            __CLOVER_422_0.S[9443]++;return c - '0' + 52;}

        __CLOVER_422_0.S[9444]++;if ((((c == CH_62) && (++__CLOVER_422_0.CT[1610] != 0)) || (++__CLOVER_422_0.CF[1610] == 0))){
            __CLOVER_422_0.S[9445]++;return 62;}

        __CLOVER_422_0.S[9446]++;if ((((c == CH_63) && (++__CLOVER_422_0.CT[1611] != 0)) || (++__CLOVER_422_0.CF[1611] == 0))){
            __CLOVER_422_0.S[9447]++;return 63;}

        // Pad character

        __CLOVER_422_0.S[9448]++;if ((((c == PAD) && (++__CLOVER_422_0.CT[1612] != 0)) || (++__CLOVER_422_0.CF[1612] == 0))){
            __CLOVER_422_0.S[9449]++;return 0;}

        __CLOVER_422_0.S[9450]++;throw new IOException(
            Tapestry.format(
                "SerializableAdaptor.unable-to-interpret-char",
                new String(new char[] { c })));
    } finally { }}

}