/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.util.text;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.UnsupportedEncodingException;
import java.util.Map;

/**
 * An object that loads a properties file from the provided input stream or reader.
 * This class reads the property file exactly like java.util.Properties,
 * except that it also allows the files to use an encoding other than ISO-8859-1
 * and all non-ASCII characters are read correctly using the given encoding.
 * In short, non-latin characters no longer need to be quoted using native2ascii.
 * 
 * @author mb
 * @since 3.1
 */
public class LocalizedPropertiesLoader
{public static com.cortexeb.tools.clover.d __CLOVER_437_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private static final String HEX_DIGITS = "0123456789ABCDEF";
    
    private static final ICharacterMatcher WHITESPACE = new WhitespaceMatcher(false);
    private static final ICharacterMatcher LINE_SEPARATOR = new AsciiCharacterMatcher("\n\r");
    private static final ICharacterMatcher NOT_LINE_SEPARATOR = new InverseMatcher(LINE_SEPARATOR);
    private static final ICharacterMatcher KEY_VALUE_SEPARATOR = new AsciiCharacterMatcher("=:");
    private static final ICharacterMatcher SEPARATOR = new AsciiCharacterMatcher("=:\r\n");
    private static final ICharacterMatcher COMMENT = new AsciiCharacterMatcher("#!");
    private static final ICharacterMatcher WHITESPACE_OR_SEPARATOR = 
        new CompoundMatcher(new ICharacterMatcher[] { WHITESPACE, SEPARATOR });

    private ExtendedReader _extendedReader;

    private ExtendedReader m_objExtendedReader;
    
    /**
     * Creates a new loader that will load the properties from the given input stream
     * using the default character encoding
     * 
     * @param ins the input stream to load the properties from
     */
    public LocalizedPropertiesLoader(InputStream ins)
    {
        this(new InputStreamReader(ins));__CLOVER_437_0.S[9596]++;try { __CLOVER_437_0.M[2322]++;
    } finally { }}
    
    /**
     * Creates a new loader that will load the properties from the given input stream
     * using the provided character encoding
     * 
     * @param ins the input stream to load the properties from
     * @param encoding the character encoding the be used when reading from the stream
     * @throws UnsupportedEncodingException if the name of the encoding cannot be recognized
     */
    public LocalizedPropertiesLoader(InputStream ins, String encoding) throws UnsupportedEncodingException
    {
        this(new InputStreamReader(ins, encoding));__CLOVER_437_0.S[9597]++;try { __CLOVER_437_0.M[2323]++;
    } finally { }}
    
    /**
     * Creates a new loader that will load the properties from the given reader
     * 
     * @param reader the Reader to load the properties from
     */
    public LocalizedPropertiesLoader(Reader reader)
    {try { __CLOVER_437_0.M[2324]++;
        __CLOVER_437_0.S[9598]++;_extendedReader = new ExtendedReader(new BufferedReader(reader));
    } finally { }}
    
    /**
     * Read the properties from the provided stream and store them into the given map
     * 
     * @param properties the map where the properties will be stored
     * @throws IOException if an error occurs
     */
    public void load(Map properties) throws IOException
    {try { __CLOVER_437_0.M[2325]++;
        __CLOVER_437_0.S[9599]++;while ((((!isAtEndOfStream()) && (++__CLOVER_437_0.CT[1639] != 0)) || (++__CLOVER_437_0.CF[1639] == 0))){ {
        	// we are at the beginning of a line.
        	// check whether it is a comment and if it is, skip it
            __CLOVER_437_0.S[9600]++;int nextChar = _extendedReader.peek();
            __CLOVER_437_0.S[9601]++;if ((((COMMENT.matches((char) nextChar)) && (++__CLOVER_437_0.CT[1640] != 0)) || (++__CLOVER_437_0.CF[1640] == 0))){ {
                __CLOVER_437_0.S[9602]++;_extendedReader.skipCharacters(NOT_LINE_SEPARATOR);
                __CLOVER_437_0.S[9603]++;continue;
            }}
            
            __CLOVER_437_0.S[9604]++;_extendedReader.skipCharacters(WHITESPACE);
            __CLOVER_437_0.S[9605]++;if ((((!isAtEndOfLine()) && (++__CLOVER_437_0.CT[1641] != 0)) || (++__CLOVER_437_0.CF[1641] == 0))){ {
            	// this line does not consist only of whitespace. the next word is the key
                __CLOVER_437_0.S[9606]++;String key = readQuotedLine(WHITESPACE_OR_SEPARATOR);
                __CLOVER_437_0.S[9607]++;_extendedReader.skipCharacters(WHITESPACE);
                
                // if the next char is a key-value separator, read it and skip the following spaces
                __CLOVER_437_0.S[9608]++;nextChar = _extendedReader.peek();
                __CLOVER_437_0.S[9609]++;if ((((nextChar > 0 && KEY_VALUE_SEPARATOR.matches((char) nextChar)) && (++__CLOVER_437_0.CT[1642] != 0)) || (++__CLOVER_437_0.CF[1642] == 0))){ {
                    __CLOVER_437_0.S[9610]++;_extendedReader.read();
                    __CLOVER_437_0.S[9611]++;_extendedReader.skipCharacters(WHITESPACE);
                }}

                // finally, read the value
                __CLOVER_437_0.S[9612]++;String value = readQuotedLine(LINE_SEPARATOR);
                
                __CLOVER_437_0.S[9613]++;properties.put(key, value);
            }}
            __CLOVER_437_0.S[9614]++;_extendedReader.skipCharacters(LINE_SEPARATOR);
        }}
    } finally { }}
    
    
    private boolean isAtEndOfStream() throws IOException
    {try { __CLOVER_437_0.M[2326]++;
        __CLOVER_437_0.S[9615]++;int nextChar = _extendedReader.peek();
        __CLOVER_437_0.S[9616]++;return (nextChar < 0);
    } finally { }}
    
    
    private boolean isAtEndOfLine() throws IOException
    {try { __CLOVER_437_0.M[2327]++;
        __CLOVER_437_0.S[9617]++;int nextChar = _extendedReader.peek();
        __CLOVER_437_0.S[9618]++;if ((((nextChar < 0) && (++__CLOVER_437_0.CT[1643] != 0)) || (++__CLOVER_437_0.CF[1643] == 0))){
            __CLOVER_437_0.S[9619]++;return true;}
        __CLOVER_437_0.S[9620]++;return LINE_SEPARATOR.matches((char) nextChar);
    } finally { }}
    
    
    private String readQuotedLine(ICharacterMatcher terminators) throws IOException
    {try { __CLOVER_437_0.M[2328]++;
        __CLOVER_437_0.S[9621]++;StringBuffer buf = new StringBuffer();
        
        __CLOVER_437_0.S[9622]++;while (true){ {
            // see what the next char is
            __CLOVER_437_0.S[9623]++;int nextChar = _extendedReader.peek();
            
            // if at end of stream or the char is one of the terminators, stop
            __CLOVER_437_0.S[9624]++;if ((((nextChar < 0 || terminators.matches((char) nextChar)) && (++__CLOVER_437_0.CT[1644] != 0)) || (++__CLOVER_437_0.CF[1644] == 0))){
                __CLOVER_437_0.S[9625]++;break;}

            __CLOVER_437_0.S[9626]++;try {
                // read the char (and possibly unquote it)
                __CLOVER_437_0.S[9627]++;char ch = readQuotedChar();
                __CLOVER_437_0.S[9628]++;buf.append(ch);
            } catch (IgnoreCharacterException e) {
                // simply ignore -- no character was read
            }
        }}
        
        __CLOVER_437_0.S[9629]++;return buf.toString();
    } finally { }}
    

    private char readQuotedChar() throws IOException, IgnoreCharacterException
    {try { __CLOVER_437_0.M[2329]++;
        __CLOVER_437_0.S[9630]++;int nextChar = _extendedReader.read();
        __CLOVER_437_0.S[9631]++;if ((((nextChar < 0) && (++__CLOVER_437_0.CT[1645] != 0)) || (++__CLOVER_437_0.CF[1645] == 0))){
            __CLOVER_437_0.S[9632]++;throw new IgnoreCharacterException();}
        __CLOVER_437_0.S[9633]++;char ch = (char) nextChar;
        
        // if the char is not the quotation char, simply return it
        __CLOVER_437_0.S[9634]++;if ((((ch != '\\') && (++__CLOVER_437_0.CT[1646] != 0)) || (++__CLOVER_437_0.CF[1646] == 0))){ 
            __CLOVER_437_0.S[9635]++;return ch;}

        // the character is a quotation character. unquote it
        __CLOVER_437_0.S[9636]++;nextChar = _extendedReader.read();

        // if at the end of the stream, stop
        __CLOVER_437_0.S[9637]++;if ((((nextChar < 0) && (++__CLOVER_437_0.CT[1647] != 0)) || (++__CLOVER_437_0.CF[1647] == 0))){
            __CLOVER_437_0.S[9638]++;throw new IgnoreCharacterException();}
        
        __CLOVER_437_0.S[9639]++;ch = (char) nextChar;
        __CLOVER_437_0.S[9640]++;switch (ch) {
            case 'u' :
                __CLOVER_437_0.S[9641]++;char res = 0;
                __CLOVER_437_0.S[9642]++;for (int i = 0; (((i < 4) && (++__CLOVER_437_0.CT[1648] != 0)) || (++__CLOVER_437_0.CF[1648] == 0)); i++){ {
                    __CLOVER_437_0.S[9643]++;nextChar = _extendedReader.read();
                    __CLOVER_437_0.S[9644]++;if ((((nextChar < 0) && (++__CLOVER_437_0.CT[1649] != 0)) || (++__CLOVER_437_0.CF[1649] == 0))){
                        __CLOVER_437_0.S[9645]++;throw new IllegalArgumentException("Malformed \\uxxxx encoding.");}
                    __CLOVER_437_0.S[9646]++;char digitChar = (char) nextChar;
                    __CLOVER_437_0.S[9647]++;int digit = HEX_DIGITS.indexOf(Character.toUpperCase(digitChar));
                    __CLOVER_437_0.S[9648]++;if ((((digit < 0) && (++__CLOVER_437_0.CT[1650] != 0)) || (++__CLOVER_437_0.CF[1650] == 0))){
                        __CLOVER_437_0.S[9649]++;throw new IllegalArgumentException("Malformed \\uxxxx encoding.");}
                    __CLOVER_437_0.S[9650]++;res = (char) (res * 16 + digit);
                }}
                __CLOVER_437_0.S[9651]++;return res;

            case '\r' :
                // if the next char is \n, read it and fall through
                __CLOVER_437_0.S[9652]++;nextChar = _extendedReader.peek();
                __CLOVER_437_0.S[9653]++;if ((((nextChar == '\n') && (++__CLOVER_437_0.CT[1651] != 0)) || (++__CLOVER_437_0.CF[1651] == 0))){
                    __CLOVER_437_0.S[9654]++;_extendedReader.read();}
            case '\n' :
                __CLOVER_437_0.S[9655]++;_extendedReader.skipCharacters(WHITESPACE);
                __CLOVER_437_0.S[9656]++;throw new IgnoreCharacterException();
                
            case 't' :  __CLOVER_437_0.S[9657]++;return '\t';
            case 'n' :  __CLOVER_437_0.S[9658]++;return '\n';
            case 'r' :  __CLOVER_437_0.S[9659]++;return '\r';
            default:    __CLOVER_437_0.S[9660]++;return ch;
        }
    } finally { }}
    

    private static class IgnoreCharacterException extends Exception 
    {
    }    
}
