/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry.valid;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.apache.tapestry.IMarkupWriter;
import org.apache.tapestry.IRequestCycle;
import org.apache.tapestry.form.IFormComponent;

/**
 *  Simple validation of email strings, to enforce required, and minimum length
 *  (maximum length is enforced in the client browser, by setting a maximum input
 *  length on the text field).
 *
 *
 *  @author Malcolm Edgar
 *  @since 2.3
 *
 **/

public class EmailValidator extends BaseValidator
{public static com.cortexeb.tools.clover.d __CLOVER_447_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1096998272901L);
    private int _minimumLength;
    private String _minimumLengthMessage;
    private String _invalidEmailFormatMessage;

    private String _scriptPath = "/org/apache/tapestry/valid/EmailValidator.script";

    public EmailValidator()
    {try { __CLOVER_447_0.M[2429]++;
    } finally { }}

    private EmailValidator(boolean required)
    {
        super(required);__CLOVER_447_0.S[9932]++;try { __CLOVER_447_0.M[2430]++;
    } finally { }}

    public String toString(IFormComponent field, Object value)
    {try { __CLOVER_447_0.M[2431]++;
        __CLOVER_447_0.S[9933]++;if ((((value == null) && (++__CLOVER_447_0.CT[1700] != 0)) || (++__CLOVER_447_0.CF[1700] == 0))){
            __CLOVER_447_0.S[9934]++;return null;}

        __CLOVER_447_0.S[9935]++;return value.toString();
    } finally { }}

    public Object toObject(IFormComponent field, String input) throws ValidatorException
    {try { __CLOVER_447_0.M[2432]++;
        __CLOVER_447_0.S[9936]++;if ((((checkRequired(field, input)) && (++__CLOVER_447_0.CT[1701] != 0)) || (++__CLOVER_447_0.CF[1701] == 0))){
            __CLOVER_447_0.S[9937]++;return null;}

        __CLOVER_447_0.S[9938]++;if ((((_minimumLength > 0 && input.length() < _minimumLength) && (++__CLOVER_447_0.CT[1702] != 0)) || (++__CLOVER_447_0.CF[1702] == 0))){
            __CLOVER_447_0.S[9939]++;throw new ValidatorException(
                buildMinimumLengthMessage(field),
                ValidationConstraint.MINIMUM_WIDTH);}

        __CLOVER_447_0.S[9940]++;if ((((!isValidEmail(input)) && (++__CLOVER_447_0.CT[1703] != 0)) || (++__CLOVER_447_0.CF[1703] == 0))){
            __CLOVER_447_0.S[9941]++;throw new ValidatorException(
                buildInvalidEmailFormatMessage(field),
                ValidationConstraint.EMAIL_FORMAT);}

        __CLOVER_447_0.S[9942]++;return input;
    } finally { }}

    public int getMinimumLength()
    {try { __CLOVER_447_0.M[2433]++;
        __CLOVER_447_0.S[9943]++;return _minimumLength;
    } finally { }}

    public void setMinimumLength(int minimumLength)
    {try { __CLOVER_447_0.M[2434]++;
        __CLOVER_447_0.S[9944]++;_minimumLength = minimumLength;
    } finally { }}

    public void renderValidatorContribution(
        IFormComponent field,
        IMarkupWriter writer,
        IRequestCycle cycle)
    {try { __CLOVER_447_0.M[2435]++;
        __CLOVER_447_0.S[9945]++;if ((((!isClientScriptingEnabled()) && (++__CLOVER_447_0.CT[1704] != 0)) || (++__CLOVER_447_0.CF[1704] == 0))){
            __CLOVER_447_0.S[9946]++;return;}

        __CLOVER_447_0.S[9947]++;Map symbols = new HashMap();

        __CLOVER_447_0.S[9948]++;Locale locale = field.getPage().getLocale();
        __CLOVER_447_0.S[9949]++;String displayName = field.getDisplayName();

        __CLOVER_447_0.S[9950]++;if ((((isRequired()) && (++__CLOVER_447_0.CT[1705] != 0)) || (++__CLOVER_447_0.CF[1705] == 0))){
            __CLOVER_447_0.S[9951]++;symbols.put("requiredMessage", buildRequiredMessage(field));}

        __CLOVER_447_0.S[9952]++;if ((((_minimumLength > 0) && (++__CLOVER_447_0.CT[1706] != 0)) || (++__CLOVER_447_0.CF[1706] == 0))){
            __CLOVER_447_0.S[9953]++;symbols.put("minimumLengthMessage", buildMinimumLengthMessage(field));}

        __CLOVER_447_0.S[9954]++;String pattern = getPattern(getInvalidEmailFormatMessage(), "invalid-email-format", locale);

        __CLOVER_447_0.S[9955]++;symbols.put("emailFormatMessage", formatString(pattern, displayName));

        __CLOVER_447_0.S[9956]++;processValidatorScript(_scriptPath, cycle, field, symbols);
    } finally { }}

    public String getScriptPath()
    {try { __CLOVER_447_0.M[2436]++;
        __CLOVER_447_0.S[9957]++;return _scriptPath;
    } finally { }}

    /**
     *  Allows a developer to use the existing validation logic with a different client-side
     *  script.  This is often sufficient to allow application-specific error presentation
     *  (perhaps by using DHTML to update the content of a &lt;span&gt; tag, or to use
     *  a more sophisticated pop-up window than <code>window.alert()</code>).
     * 
     **/

    public void setScriptPath(String scriptPath)
    {try { __CLOVER_447_0.M[2437]++;
        __CLOVER_447_0.S[9958]++;_scriptPath = scriptPath;
    } finally { }}

    /**
     *  Return true if the email format is valid.
     * 
     *  @param email the email string to validate
     *  @return true if the email format is valid
     */

    protected boolean isValidEmail(String email)
    {try { __CLOVER_447_0.M[2438]++;
        __CLOVER_447_0.S[9959]++;int atIndex = email.indexOf('@');

        __CLOVER_447_0.S[9960]++;if (((((atIndex == -1) || (atIndex == 0) || (atIndex == email.length() - 1)) && (++__CLOVER_447_0.CT[1707] != 0)) || (++__CLOVER_447_0.CF[1707] == 0))){
        {
            __CLOVER_447_0.S[9961]++;return false;
        }}
        else{
        {
            __CLOVER_447_0.S[9962]++;return true;
        }}
    } finally { }}

    /** @since 3.0 */

    public String getInvalidEmailFormatMessage()
    {try { __CLOVER_447_0.M[2439]++;
        __CLOVER_447_0.S[9963]++;return _invalidEmailFormatMessage;
    } finally { }}

    /** @since 3.0 */

    public String getMinimumLengthMessage()
    {try { __CLOVER_447_0.M[2440]++;
        __CLOVER_447_0.S[9964]++;return _minimumLengthMessage;
    } finally { }}

    /**
     *  Overrides the <code>invalid-email-format</code>
     *  bundle key.
     *  Parameter {0} is the display name of the field.
     * 
     *  @since 3.0
     * 
     */

    public void setInvalidEmailFormatMessage(String string)
    {try { __CLOVER_447_0.M[2441]++;
        __CLOVER_447_0.S[9965]++;_invalidEmailFormatMessage = string;
    } finally { }}

    /**
     *  Overrides the <code>field-too-short</code> bundle key.
     *  Parameter {0} is the minimum length.
     *  Parameter {1} is the display name of the field.
     * 
     *  @since 3.0
     * 
     */
    public void setMinimumLengthMessage(String string)
    {try { __CLOVER_447_0.M[2442]++;
        __CLOVER_447_0.S[9966]++;_minimumLengthMessage = string;
    } finally { }}

    /** @since 3.0 */

    protected String buildMinimumLengthMessage(IFormComponent field)
    {try { __CLOVER_447_0.M[2443]++;
        __CLOVER_447_0.S[9967]++;String pattern =
            getPattern(_minimumLengthMessage, "field-too-short", field.getPage().getLocale());

        __CLOVER_447_0.S[9968]++;return formatString(pattern, Integer.toString(_minimumLength), field.getDisplayName());
    } finally { }}

    /** @since 3.0 */

    protected String buildInvalidEmailFormatMessage(IFormComponent field)
    {try { __CLOVER_447_0.M[2444]++;
        __CLOVER_447_0.S[9969]++;String pattern =
            getPattern(
                _invalidEmailFormatMessage,
                "invalid-email-format",
                field.getPage().getLocale());

        __CLOVER_447_0.S[9970]++;return formatString(pattern, field.getDisplayName());
    } finally { }}
}