/* $$ Clover has instrumented this file $$ */// Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.tapestry;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import ognl.OgnlRuntime;

import org.apache.hivemind.ApplicationRuntimeException;
import org.apache.hivemind.ClassResolver;
import org.apache.hivemind.Messages;
import org.apache.hivemind.impl.BaseLocatable;
import org.apache.hivemind.util.PropertyAdaptor;
import org.apache.hivemind.util.PropertyUtils;
import org.apache.tapestry.bean.BeanProvider;
import org.apache.tapestry.bean.BeanProviderPropertyAccessor;
import org.apache.tapestry.engine.IPageLoader;
import org.apache.tapestry.event.ChangeObserver;
import org.apache.tapestry.event.PageDetachListener;
import org.apache.tapestry.event.PageEvent;
import org.apache.tapestry.event.PageRenderListener;
import org.apache.tapestry.event.PageValidateListener;
import org.apache.tapestry.listener.ListenerMap;
import org.apache.tapestry.param.ParameterManager;
import org.apache.tapestry.spec.IComponentSpecification;
import org.apache.tapestry.util.prop.OgnlUtils;

/**
 * Abstract base class implementing the {@link IComponent}interface.
 * 
 * @author Howard Lewis Ship
 */

public abstract class AbstractComponent extends BaseLocatable implements IComponent
{public static com.cortexeb.tools.clover.d __CLOVER_0_0 = com.cortexeb.tools.clover.aq.getRecorder(new char[] {67,58,92,119,111,114,107,115,112,97,99,101,92,106,97,107,97,114,116,97,45,116,97,112,101,115,116,114,121,92,102,114,97,109,101,119,111,114,107,92,116,97,114,103,101,116,92,99,108,111,118,101,114,45,100,98},1097019445706L);
    /**
     * Used to check that subclasses invoke this implementation of prepareForRender().
     * 
     * @see Tapestry#checkMethodInvocation(Object, String, Object)
     * @since 3.0
     */
    private final static String PREPAREFORRENDER_METHOD_ID = "AbstractComponent.prepareForRender()";

    /**
     * Used to check that subclasses invoke this implementation of cleanupAfterRender().
     * 
     * @see Tapestry#checkMethodInvocation(Object, String, Object)
     * @since 3.0
     */

    private final static String CLEANUPAFTERRENDER_METHOD_ID = "AbstractComponent.cleanupAfterRender()";

    static
    {
        // Register the BeanProviderHelper to provide access to the
        // beans of a bean provider as named properties.

        __CLOVER_0_0.S[10690]++;OgnlRuntime.setPropertyAccessor(IBeanProvider.class, new BeanProviderPropertyAccessor());
    }

    /**
     * The specification used to originally build the component.
     */

    private IComponentSpecification _specification;

    /**
     * The page that contains the component, possibly itself (if the component is in fact, a page).
     */

    private IPage _page;

    /**
     * The component which contains the component. This will only be null if the component is
     * actually a page.
     */

    private IComponent _container;

    /**
     * The simple id of this component.
     */

    private String _id;

    /**
     * The fully qualified id of this component. This is calculated the first time it is needed,
     * then cached for later.
     */

    private String _idPath;

    private static final int MAP_SIZE = 5;

    /**
     * A {@link Map}of all bindings (for which there isn't a corresponding JavaBeans property); the
     * keys are the names of formal and informal parameters.
     */

    private Map _bindings;

    private Map _components;

    private static final int BODY_INIT_SIZE = 5;

    private INamespace _namespace;

    /**
     * Used in place of JDK 1.3's Collections.EMPTY_MAP (which is not available in JDK 1.2).
     */

    private static final Map EMPTY_MAP = Collections.unmodifiableMap(new HashMap(1));

    /**
     * The number of {@link IRender}objects in the body of this component.
     */

    private int _bodyCount = 0;

    /**
     * An aray of elements in the body of this component.
     */

    private IRender[] _body;

    /**
     * The components' asset map.
     */

    private Map _assets;

    /**
     * A mapping that allows public instance methods to be dressed up as {@link IActionListener}
     * listener objects.
     * 
     * @since 1.0.2
     */

    private ListenerMap _listeners;

    /**
     * A bean provider; these are lazily created as needed.
     * 
     * @since 1.0.4
     */

    private IBeanProvider _beans;

    /**
     * Manages setting and clearing parameter properties for the component.
     * 
     * @since 2.0.3
     */

    private ParameterManager _parameterManager;

    /**
     * Provides access to localized Strings for this component.
     * 
     * @since 2.0.4
     */

    private Messages _strings;

    public void addAsset(String name, IAsset asset)
    {try { __CLOVER_0_0.M[2650]++;
        __CLOVER_0_0.S[10691]++;if ((((_assets == null) && (++__CLOVER_0_0.CT[1838] != 0)) || (++__CLOVER_0_0.CF[1838] == 0))){
            __CLOVER_0_0.S[10692]++;_assets = new HashMap(MAP_SIZE);}

        __CLOVER_0_0.S[10693]++;_assets.put(name, asset);
    } finally { }}

    public void addComponent(IComponent component)
    {try { __CLOVER_0_0.M[2651]++;
        __CLOVER_0_0.S[10694]++;if ((((_components == null) && (++__CLOVER_0_0.CT[1839] != 0)) || (++__CLOVER_0_0.CF[1839] == 0))){
            __CLOVER_0_0.S[10695]++;_components = new HashMap(MAP_SIZE);}

        __CLOVER_0_0.S[10696]++;_components.put(component.getId(), component);
    } finally { }}

    /**
     * Adds an element (which may be static text or a component) as a body element of this
     * component. Such elements are rendered by {@link #renderBody(IMarkupWriter, IRequestCycle)}.
     * 
     * @since 2.2
     */

    public void addBody(IRender element)
    {try { __CLOVER_0_0.M[2652]++;
        // Should check the specification to see if this component
        // allows body. Curently, this is checked by the component
        // in render(), which is silly.

        __CLOVER_0_0.S[10697]++;if ((((_body == null) && (++__CLOVER_0_0.CT[1840] != 0)) || (++__CLOVER_0_0.CF[1840] == 0))){
        {
            __CLOVER_0_0.S[10698]++;_body = new IRender[BODY_INIT_SIZE];
            __CLOVER_0_0.S[10699]++;_body[0] = element;

            __CLOVER_0_0.S[10700]++;_bodyCount = 1;
            __CLOVER_0_0.S[10701]++;return;
        }}

        // No more room? Make the array bigger.

        __CLOVER_0_0.S[10702]++;if ((((_bodyCount == _body.length) && (++__CLOVER_0_0.CT[1841] != 0)) || (++__CLOVER_0_0.CF[1841] == 0))){
        {
            __CLOVER_0_0.S[10703]++;IRender[] newWrapped;

            __CLOVER_0_0.S[10704]++;newWrapped = new IRender[_body.length * 2];

            __CLOVER_0_0.S[10705]++;System.arraycopy(_body, 0, newWrapped, 0, _bodyCount);

            __CLOVER_0_0.S[10706]++;_body = newWrapped;
        }}

        __CLOVER_0_0.S[10707]++;_body[_bodyCount++] = element;
    } finally { }}

    /**
     * Registers this component as a listener of the page if it implements
     * {@link org.apache.tapestry.event.PageDetachListener}or
     * {@link org.apache.tapestry.event.PageRenderListener}.
     * <p>
     * Invokes {@link #finishLoad()}. Subclasses may overide as needed, but must invoke this
     * implementation. {@link BaseComponent}loads its HTML template.
     */

    public void finishLoad(IRequestCycle cycle, IPageLoader loader,
            IComponentSpecification specification)
    {try { __CLOVER_0_0.M[2653]++;
        __CLOVER_0_0.S[10708]++;if ((((this instanceof PageDetachListener) && (++__CLOVER_0_0.CT[1842] != 0)) || (++__CLOVER_0_0.CF[1842] == 0))){
            __CLOVER_0_0.S[10709]++;_page.addPageDetachListener((PageDetachListener) this);}

        __CLOVER_0_0.S[10710]++;if ((((this instanceof PageRenderListener) && (++__CLOVER_0_0.CT[1843] != 0)) || (++__CLOVER_0_0.CF[1843] == 0))){
            __CLOVER_0_0.S[10711]++;_page.addPageRenderListener((PageRenderListener) this);}

        __CLOVER_0_0.S[10712]++;if ((((this instanceof PageValidateListener) && (++__CLOVER_0_0.CT[1844] != 0)) || (++__CLOVER_0_0.CF[1844] == 0))){
            __CLOVER_0_0.S[10713]++;_page.addPageValidateListener((PageValidateListener) this);}

        __CLOVER_0_0.S[10714]++;finishLoad();
    } finally { }}

    /**
     * Converts informal parameters into additional attributes on the curently open tag.
     * <p>
     * Invoked from subclasses to allow additional attributes to be specified within a tag (this
     * works best when there is a one-to-one corespondence between an {@link IComponent}and a HTML
     * element.
     * <p>
     * Iterates through the bindings for this component. Filters out bindings when the name matches
     * a formal parameter (as of 1.0.5, informal bindings are weeded out at page load / template
     * load time, if they match a formal parameter, or a specificied reserved name). For the most
     * part, all the bindings here are either informal parameter, or formal parameter without a
     * corresponding JavaBeans property.
     * <p>
     * For each acceptible key, the value is extracted using {@link IBinding#getObject()}. If the
     * value is null, no attribute is written.
     * <p>
     * If the value is an instance of {@link IAsset}, then {@link IAsset#buildURL(IRequestCycle)}
     * is invoked to convert the asset to a URL.
     * <p>
     * Finally, {@link IMarkupWriter#attribute(String,String)}is invoked with the value (or the
     * URL).
     * <p>
     * The most common use for informal parameters is to support the HTML class attribute (for use
     * with cascading style sheets) and to specify JavaScript event handlers.
     * <p>
     * Components are only required to generate attributes on the result phase; this can be skipped
     * during the rewind phase.
     */

    protected void renderInformalParameters(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_0_0.M[2654]++;
        __CLOVER_0_0.S[10715]++;String attribute;

        __CLOVER_0_0.S[10716]++;if ((((_bindings == null) && (++__CLOVER_0_0.CT[1845] != 0)) || (++__CLOVER_0_0.CF[1845] == 0))){
            __CLOVER_0_0.S[10717]++;return;}

        __CLOVER_0_0.S[10718]++;Iterator i = _bindings.entrySet().iterator();

        __CLOVER_0_0.S[10719]++;while ((((i.hasNext()) && (++__CLOVER_0_0.CT[1846] != 0)) || (++__CLOVER_0_0.CF[1846] == 0))){
        {
            __CLOVER_0_0.S[10720]++;Map.Entry entry = (Map.Entry) i.next();
            __CLOVER_0_0.S[10721]++;String name = (String) entry.getKey();

            __CLOVER_0_0.S[10722]++;IBinding binding = (IBinding) entry.getValue();

            __CLOVER_0_0.S[10723]++;Object value = binding.getObject();
            __CLOVER_0_0.S[10724]++;if ((((value == null) && (++__CLOVER_0_0.CT[1847] != 0)) || (++__CLOVER_0_0.CF[1847] == 0))){
                __CLOVER_0_0.S[10725]++;continue;}

            __CLOVER_0_0.S[10726]++;if ((((value instanceof IAsset) && (++__CLOVER_0_0.CT[1848] != 0)) || (++__CLOVER_0_0.CF[1848] == 0))){
            {
                __CLOVER_0_0.S[10727]++;IAsset asset = (IAsset) value;

                // Get the URL of the asset and insert that.

                __CLOVER_0_0.S[10728]++;attribute = asset.buildURL(cycle);
            }}
            else{
                __CLOVER_0_0.S[10729]++;attribute = value.toString();}

            __CLOVER_0_0.S[10730]++;writer.attribute(name, attribute);
        }}

    } finally { }}

    /**
     * Returns an object used to resolve classes.
     * 
     * @since 3.0
     */
    private ClassResolver getClassResolver()
    {try { __CLOVER_0_0.M[2655]++;
        __CLOVER_0_0.S[10731]++;return getPage().getEngine().getClassResolver();
    } finally { }}

    /**
     * Returns the named binding, or null if it doesn't exist.
     * <p>
     * This method looks for a JavaBeans property with an appropriate name, of type {@link IBinding}.
     * The property should be named <code><i>name</i>Binding</code>. If it exists and is both
     * readable and writable, then it is accessor method is invoked. Components which implement such
     * methods can access their own binding through their instance variables instead of invoking
     * this method, a performance optimization.
     * 
     * @see #setBinding(String,IBinding)
     */

    public IBinding getBinding(String name)
    {try { __CLOVER_0_0.M[2656]++;
        __CLOVER_0_0.S[10732]++;String bindingPropertyName = name + Tapestry.PARAMETER_PROPERTY_NAME_SUFFIX;

        __CLOVER_0_0.S[10733]++;if ((((PropertyUtils.isReadable(this, bindingPropertyName)
                && PropertyUtils.getPropertyType(this, bindingPropertyName).equals(IBinding.class)) && (++__CLOVER_0_0.CT[1849] != 0)) || (++__CLOVER_0_0.CF[1849] == 0))){
        {
            __CLOVER_0_0.S[10734]++;return (IBinding) PropertyUtils.read(this, bindingPropertyName);
        }}

        __CLOVER_0_0.S[10735]++;if ((((_bindings == null) && (++__CLOVER_0_0.CT[1850] != 0)) || (++__CLOVER_0_0.CF[1850] == 0))){
            __CLOVER_0_0.S[10736]++;return null;}

        __CLOVER_0_0.S[10737]++;return (IBinding) _bindings.get(name);
    } finally { }}

    /**
     * Return's the page's change observer. In practical terms, this will be an
     * {@link org.apache.tapestry.engine.IPageRecorder}.
     * 
     * @see IPage#getChangeObserver()
     * @deprecated To be removed in 3.1; use {@link IPage#getChangeObserver()}.
     */

    public ChangeObserver getChangeObserver()
    {try { __CLOVER_0_0.M[2657]++;
        __CLOVER_0_0.S[10738]++;return _page.getChangeObserver();
    } finally { }}

    public IComponent getComponent(String id)
    {try { __CLOVER_0_0.M[2658]++;
        __CLOVER_0_0.S[10739]++;IComponent result = null;

        __CLOVER_0_0.S[10740]++;if ((((_components != null) && (++__CLOVER_0_0.CT[1851] != 0)) || (++__CLOVER_0_0.CF[1851] == 0))){
            __CLOVER_0_0.S[10741]++;result = (IComponent) _components.get(id);}

        __CLOVER_0_0.S[10742]++;if ((((result == null) && (++__CLOVER_0_0.CT[1852] != 0)) || (++__CLOVER_0_0.CF[1852] == 0))){
            __CLOVER_0_0.S[10743]++;throw new ApplicationRuntimeException(Tapestry.format("no-such-component", this, id),
                    this, null, null);}

        __CLOVER_0_0.S[10744]++;return result;
    } finally { }}

    public IComponent getContainer()
    {try { __CLOVER_0_0.M[2659]++;
        __CLOVER_0_0.S[10745]++;return _container;
    } finally { }}

    public void setContainer(IComponent value)
    {try { __CLOVER_0_0.M[2660]++;
        __CLOVER_0_0.S[10746]++;if ((((_container != null) && (++__CLOVER_0_0.CT[1853] != 0)) || (++__CLOVER_0_0.CF[1853] == 0))){
            __CLOVER_0_0.S[10747]++;throw new ApplicationRuntimeException(Tapestry
                    .getMessage("AbstractComponent.attempt-to-change-container"));}

        __CLOVER_0_0.S[10748]++;_container = value;
    } finally { }}

    /**
     * Returns the name of the page, a slash, and this component's id path. Pages are different,
     * they simply return their name.
     * 
     * @see #getIdPath()
     */

    public String getExtendedId()
    {try { __CLOVER_0_0.M[2661]++;
        __CLOVER_0_0.S[10749]++;if ((((_page == null) && (++__CLOVER_0_0.CT[1854] != 0)) || (++__CLOVER_0_0.CF[1854] == 0))){
            __CLOVER_0_0.S[10750]++;return null;}

        __CLOVER_0_0.S[10751]++;return _page.getPageName() + "/" + getIdPath();
    } finally { }}

    public String getId()
    {try { __CLOVER_0_0.M[2662]++;
        __CLOVER_0_0.S[10752]++;return _id;
    } finally { }}

    public void setId(String value)
    {try { __CLOVER_0_0.M[2663]++;
        __CLOVER_0_0.S[10753]++;if ((((_id != null) && (++__CLOVER_0_0.CT[1855] != 0)) || (++__CLOVER_0_0.CF[1855] == 0))){
            __CLOVER_0_0.S[10754]++;throw new ApplicationRuntimeException(Tapestry
                    .getMessage("AbstractComponent.attempt-to-change-component-id"));}

        __CLOVER_0_0.S[10755]++;_id = value;
    } finally { }}

    public String getIdPath()
    {try { __CLOVER_0_0.M[2664]++;
        __CLOVER_0_0.S[10756]++;String containerIdPath;

        __CLOVER_0_0.S[10757]++;if ((((_container == null) && (++__CLOVER_0_0.CT[1856] != 0)) || (++__CLOVER_0_0.CF[1856] == 0))){
            __CLOVER_0_0.S[10758]++;throw new NullPointerException(Tapestry
                    .format("AbstractComponent.null-container", this));}

        __CLOVER_0_0.S[10759]++;containerIdPath = _container.getIdPath();

        __CLOVER_0_0.S[10760]++;if ((((containerIdPath == null) && (++__CLOVER_0_0.CT[1857] != 0)) || (++__CLOVER_0_0.CF[1857] == 0))){
            __CLOVER_0_0.S[10761]++;_idPath = _id;}
        else{
            __CLOVER_0_0.S[10762]++;_idPath = containerIdPath + "." + _id;}

        __CLOVER_0_0.S[10763]++;return _idPath;
    } finally { }}

    public IPage getPage()
    {try { __CLOVER_0_0.M[2665]++;
        __CLOVER_0_0.S[10764]++;return _page;
    } finally { }}

    public void setPage(IPage value)
    {try { __CLOVER_0_0.M[2666]++;
        __CLOVER_0_0.S[10765]++;if ((((_page != null) && (++__CLOVER_0_0.CT[1858] != 0)) || (++__CLOVER_0_0.CF[1858] == 0))){
            __CLOVER_0_0.S[10766]++;throw new ApplicationRuntimeException(Tapestry
                    .getMessage("AbstractComponent.attempt-to-change-page"));}

        __CLOVER_0_0.S[10767]++;_page = value;
    } finally { }}

    public IComponentSpecification getSpecification()
    {try { __CLOVER_0_0.M[2667]++;
        __CLOVER_0_0.S[10768]++;return _specification;
    } finally { }}

    public void setSpecification(IComponentSpecification value)
    {try { __CLOVER_0_0.M[2668]++;
        __CLOVER_0_0.S[10769]++;if ((((_specification != null) && (++__CLOVER_0_0.CT[1859] != 0)) || (++__CLOVER_0_0.CF[1859] == 0))){
            __CLOVER_0_0.S[10770]++;throw new ApplicationRuntimeException(Tapestry
                    .getMessage("AbstractComponent.attempt-to-change-spec"));}

        __CLOVER_0_0.S[10771]++;_specification = value;
    } finally { }}

    /**
     * Renders all elements wrapped by the receiver.
     */

    public void renderBody(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_0_0.M[2669]++;
        __CLOVER_0_0.S[10772]++;for (int i = 0; (((i < _bodyCount) && (++__CLOVER_0_0.CT[1860] != 0)) || (++__CLOVER_0_0.CF[1860] == 0)); i++){
            __CLOVER_0_0.S[10773]++;_body[i].render(writer, cycle);}
    } finally { }}

    /**
     * Adds the binding with the given name, replacing any existing binding with that name.
     * <p>
     * This method checks to see if a matching JavaBeans property (with a name of
     * <code><i>name</i>Binding</code> and a type of {@link IBinding}) exists. If so, that
     * property is updated. An optimized component can simply implement accessor and mutator methods
     * and then access its bindings via its own instance variables, rather than going through {@link
     * #getBinding(String)}.
     * <p>
     * Informal parameters should <em>not</em> be stored in instance variables if {@link
     * #renderInformalParameters(IMarkupWriter, IRequestCycle)} is to be used. It relies on using
     * the collection of bindings (to store informal parameters).
     */

    public void setBinding(String name, IBinding binding)
    {try { __CLOVER_0_0.M[2670]++;
        __CLOVER_0_0.S[10774]++;String bindingPropertyName = name + Tapestry.PARAMETER_PROPERTY_NAME_SUFFIX;

        __CLOVER_0_0.S[10775]++;if ((((PropertyUtils.isWritable(this, bindingPropertyName)
                && PropertyUtils.getPropertyType(this, bindingPropertyName).equals(IBinding.class)) && (++__CLOVER_0_0.CT[1861] != 0)) || (++__CLOVER_0_0.CF[1861] == 0))){
        {
            __CLOVER_0_0.S[10776]++;PropertyUtils.write(this, bindingPropertyName, binding);
            __CLOVER_0_0.S[10777]++;return;
        }}

        __CLOVER_0_0.S[10778]++;if ((((_bindings == null) && (++__CLOVER_0_0.CT[1862] != 0)) || (++__CLOVER_0_0.CF[1862] == 0))){
            __CLOVER_0_0.S[10779]++;_bindings = new HashMap(MAP_SIZE);}

        __CLOVER_0_0.S[10780]++;_bindings.put(name, binding);
    } finally { }}

    public String toString()
    {try { __CLOVER_0_0.M[2671]++;
        __CLOVER_0_0.S[10781]++;StringBuffer buffer;

        __CLOVER_0_0.S[10782]++;buffer = new StringBuffer(super.toString());

        __CLOVER_0_0.S[10783]++;buffer.append('[');

        __CLOVER_0_0.S[10784]++;buffer.append(getExtendedId());

        __CLOVER_0_0.S[10785]++;buffer.append(']');

        __CLOVER_0_0.S[10786]++;return buffer.toString();
    } finally { }}

    /**
     * Returns an unmodifiable {@link Map}of components, keyed on component id. Never returns null,
     * but may return an empty map. The returned map is immutable.
     */

    public Map getComponents()
    {try { __CLOVER_0_0.M[2672]++;
        __CLOVER_0_0.S[10787]++;if ((((_components == null) && (++__CLOVER_0_0.CT[1863] != 0)) || (++__CLOVER_0_0.CF[1863] == 0))){
            __CLOVER_0_0.S[10788]++;return EMPTY_MAP;}

        __CLOVER_0_0.S[10789]++;return Collections.unmodifiableMap(_components);

    } finally { }}

    public Map getAssets()
    {try { __CLOVER_0_0.M[2673]++;
        __CLOVER_0_0.S[10790]++;if ((((_assets == null) && (++__CLOVER_0_0.CT[1864] != 0)) || (++__CLOVER_0_0.CF[1864] == 0))){
            __CLOVER_0_0.S[10791]++;return EMPTY_MAP;}

        __CLOVER_0_0.S[10792]++;return Collections.unmodifiableMap(_assets);
    } finally { }}

    public IAsset getAsset(String name)
    {try { __CLOVER_0_0.M[2674]++;
        __CLOVER_0_0.S[10793]++;if ((((_assets == null) && (++__CLOVER_0_0.CT[1865] != 0)) || (++__CLOVER_0_0.CF[1865] == 0))){
            __CLOVER_0_0.S[10794]++;return null;}

        __CLOVER_0_0.S[10795]++;return (IAsset) _assets.get(name);
    } finally { }}

    public Collection getBindingNames()
    {try { __CLOVER_0_0.M[2675]++;
        // If no conainer, i.e. a page, then no bindings.

        __CLOVER_0_0.S[10796]++;if ((((_container == null) && (++__CLOVER_0_0.CT[1866] != 0)) || (++__CLOVER_0_0.CF[1866] == 0))){
            __CLOVER_0_0.S[10797]++;return null;}

        __CLOVER_0_0.S[10798]++;HashSet result = new HashSet();

        // All the informal bindings go into the bindings Map.

        __CLOVER_0_0.S[10799]++;if ((((_bindings != null) && (++__CLOVER_0_0.CT[1867] != 0)) || (++__CLOVER_0_0.CF[1867] == 0))){
            __CLOVER_0_0.S[10800]++;result.addAll(_bindings.keySet());}

        // Now, iterate over the formal parameters and add the formal parameters
        // that have a binding.

        __CLOVER_0_0.S[10801]++;List names = _specification.getParameterNames();

        __CLOVER_0_0.S[10802]++;int count = names.size();

        __CLOVER_0_0.S[10803]++;for (int i = 0; (((i < count) && (++__CLOVER_0_0.CT[1868] != 0)) || (++__CLOVER_0_0.CF[1868] == 0)); i++){
        {
            __CLOVER_0_0.S[10804]++;String name = (String) names.get(i);

            __CLOVER_0_0.S[10805]++;if ((((result.contains(name)) && (++__CLOVER_0_0.CT[1869] != 0)) || (++__CLOVER_0_0.CF[1869] == 0))){
                __CLOVER_0_0.S[10806]++;continue;}

            __CLOVER_0_0.S[10807]++;if ((((getBinding(name) != null) && (++__CLOVER_0_0.CT[1870] != 0)) || (++__CLOVER_0_0.CF[1870] == 0))){
                __CLOVER_0_0.S[10808]++;result.add(name);}
        }}

        __CLOVER_0_0.S[10809]++;return result;
    } finally { }}

    /**
     * Returns a {@link Map}of all bindings for this component. This implementation is expensive,
     * since it has to merge the disassociated bindings (informal parameters, and parameters without
     * a JavaBeans property) with the associated bindings (formal parameters with a JavaBeans
     * property).
     * 
     * @since 1.0.5
     */

    public Map getBindings()
    {try { __CLOVER_0_0.M[2676]++;
        __CLOVER_0_0.S[10810]++;Map result = new HashMap();

        // Add any informal parameters.

        __CLOVER_0_0.S[10811]++;if ((((_bindings != null) && (++__CLOVER_0_0.CT[1871] != 0)) || (++__CLOVER_0_0.CF[1871] == 0))){
            __CLOVER_0_0.S[10812]++;result.putAll(_bindings);}

        // Now work on the formal parameters

        __CLOVER_0_0.S[10813]++;Iterator i = _specification.getParameterNames().iterator();
        __CLOVER_0_0.S[10814]++;while ((((i.hasNext()) && (++__CLOVER_0_0.CT[1872] != 0)) || (++__CLOVER_0_0.CF[1872] == 0))){
        {
            __CLOVER_0_0.S[10815]++;String name = (String) i.next();

            __CLOVER_0_0.S[10816]++;if ((((result.containsKey(name)) && (++__CLOVER_0_0.CT[1873] != 0)) || (++__CLOVER_0_0.CF[1873] == 0))){
                __CLOVER_0_0.S[10817]++;continue;}

            __CLOVER_0_0.S[10818]++;IBinding binding = getBinding(name);

            __CLOVER_0_0.S[10819]++;if ((((binding != null) && (++__CLOVER_0_0.CT[1874] != 0)) || (++__CLOVER_0_0.CF[1874] == 0))){
                __CLOVER_0_0.S[10820]++;result.put(name, binding);}
        }}

        __CLOVER_0_0.S[10821]++;return result;
    } finally { }}

    /**
     * Returns a {@link ListenerMap}for the component. A {@link ListenerMap}contains a number of
     * synthetic read-only properties that implement the {@link IActionListener}interface, but in
     * fact, cause public instance methods to be invoked.
     * 
     * @since 1.0.2
     */

    public ListenerMap getListeners()
    {try { __CLOVER_0_0.M[2677]++;
        __CLOVER_0_0.S[10822]++;if ((((_listeners == null) && (++__CLOVER_0_0.CT[1875] != 0)) || (++__CLOVER_0_0.CF[1875] == 0))){
            __CLOVER_0_0.S[10823]++;_listeners = new ListenerMap(this);}

        __CLOVER_0_0.S[10824]++;return _listeners;
    } finally { }}

    /**
     * Returns the {@link IBeanProvider}for this component. This is lazily created the first time
     * it is needed.
     * 
     * @since 1.0.4
     */

    public IBeanProvider getBeans()
    {try { __CLOVER_0_0.M[2678]++;
        __CLOVER_0_0.S[10825]++;if ((((_beans == null) && (++__CLOVER_0_0.CT[1876] != 0)) || (++__CLOVER_0_0.CF[1876] == 0))){
            __CLOVER_0_0.S[10826]++;_beans = new BeanProvider(this);}

        __CLOVER_0_0.S[10827]++;return _beans;
    } finally { }}

    /**
     * Invoked, as a convienience, from
     * {@link #finishLoad(IRequestCycle, IPageLoader, IComponentSpecification)}. This implemenation
     * does nothing. Subclasses may override without invoking this implementation.
     * 
     * @since 1.0.5
     */

    protected void finishLoad()
    {try { __CLOVER_0_0.M[2679]++;
    } finally { }}

    /**
     * The main method used to render the component. Invokes
     * {@link #prepareForRender(IRequestCycle)}, then
     * {@link #renderComponent(IMarkupWriter, IRequestCycle)}.
     * {@link #cleanupAfterRender(IRequestCycle)}is invoked in a <code>finally</code> block.
     * <p>
     * Subclasses should not override this method; instead they will implement
     * {@link #renderComponent(IMarkupWriter, IRequestCycle)}.
     * 
     * @since 2.0.3
     */

    public final void render(IMarkupWriter writer, IRequestCycle cycle)
    {try { __CLOVER_0_0.M[2680]++;
        __CLOVER_0_0.S[10828]++;try
        {
            __CLOVER_0_0.S[10829]++;Tapestry.clearMethodInvocations();

            __CLOVER_0_0.S[10830]++;prepareForRender(cycle);

            __CLOVER_0_0.S[10831]++;Tapestry.checkMethodInvocation(PREPAREFORRENDER_METHOD_ID, "prepareForRender()", this);

            __CLOVER_0_0.S[10832]++;renderComponent(writer, cycle);
        }
        finally
        {
            __CLOVER_0_0.S[10833]++;Tapestry.clearMethodInvocations();

            __CLOVER_0_0.S[10834]++;cleanupAfterRender(cycle);

            __CLOVER_0_0.S[10835]++;Tapestry.checkMethodInvocation(
                    CLEANUPAFTERRENDER_METHOD_ID,
                    "cleanupAfterRender()",
                    this);
        }
    } finally { }}

    /**
     * Invoked by {@link #render(IMarkupWriter, IRequestCycle)}to prepare the component to render.
     * This implementation sets JavaBeans properties from matching bound parameters. Subclasses that
     * override this method must invoke this implementation as well.
     * 
     * @since 2.0.3
     */

    protected void prepareForRender(IRequestCycle cycle)
    {try { __CLOVER_0_0.M[2681]++;
        __CLOVER_0_0.S[10836]++;Tapestry.addMethodInvocation(PREPAREFORRENDER_METHOD_ID);

        __CLOVER_0_0.S[10837]++;if ((((_parameterManager == null) && (++__CLOVER_0_0.CT[1877] != 0)) || (++__CLOVER_0_0.CF[1877] == 0))){
        {
            // Pages inherit from this class too, but pages (by definition)
            // never have parameters.

            __CLOVER_0_0.S[10838]++;if ((((getSpecification().isPageSpecification()) && (++__CLOVER_0_0.CT[1878] != 0)) || (++__CLOVER_0_0.CF[1878] == 0))){
                __CLOVER_0_0.S[10839]++;return;}

            __CLOVER_0_0.S[10840]++;_parameterManager = new ParameterManager(this);
        }}

        __CLOVER_0_0.S[10841]++;_parameterManager.setParameters(cycle);
    } finally { }}

    /**
     * Invoked by {@link #render(IMarkupWriter, IRequestCycle)}to actually render the component
     * (with any parameter values already set). This is the method that subclasses must implement.
     * 
     * @since 2.0.3
     */

    protected abstract void renderComponent(IMarkupWriter writer, IRequestCycle cycle);

    /**
     * Invoked by {@link #render(IMarkupWriter, IRequestCycle)}after the component renders, to
     * clear any parameters back to null (or 0, or false, or whatever the correct default is).
     * Primarily, this is used to ensure that the component doesn't hold onto any objects that could
     * otherwise be garbage collected.
     * <p>
     * Subclasses may override this implementation, but must also invoke it.
     * 
     * @since 2.0.3
     */

    protected void cleanupAfterRender(IRequestCycle cycle)
    {try { __CLOVER_0_0.M[2682]++;
        __CLOVER_0_0.S[10842]++;Tapestry.addMethodInvocation(CLEANUPAFTERRENDER_METHOD_ID);

        __CLOVER_0_0.S[10843]++;if ((((_parameterManager != null) && (++__CLOVER_0_0.CT[1879] != 0)) || (++__CLOVER_0_0.CF[1879] == 0))){
            __CLOVER_0_0.S[10844]++;_parameterManager.resetParameters(cycle);}
    } finally { }}

    /** @since 3.0 * */

    public Messages getMessages()
    {try { __CLOVER_0_0.M[2683]++;
        __CLOVER_0_0.S[10845]++;if ((((_strings == null) && (++__CLOVER_0_0.CT[1880] != 0)) || (++__CLOVER_0_0.CF[1880] == 0))){
            __CLOVER_0_0.S[10846]++;_strings = getPage().getEngine().getComponentMessagesSource().getMessages(this);}

        __CLOVER_0_0.S[10847]++;return _strings;
    } finally { }}

    /**
     * Obtains the {@link IMessages}for this component (if necessary), and gets the string from it.
     */

    public String getString(String key)
    {try { __CLOVER_0_0.M[2684]++;
        __CLOVER_0_0.S[10848]++;return getMessages().getMessage(key);
    } finally { }}

    public String getMessage(String key)
    {try { __CLOVER_0_0.M[2685]++;
        // Invoke the deprecated implementation (for code coverage reasons).
        // In 3.1, remove getString() and move its implementation
        // here.

        __CLOVER_0_0.S[10849]++;return getString(key);
    } finally { }}

    /**
     * Formats a message string, using
     * {@link IMessages#format(java.lang.String, java.lang.Object[])}.
     * 
     * @param key
     *            the key used to obtain a localized pattern using {@link #getString(String)}
     * @param arguments
     *            passed to the formatter
     * @since 2.2
     * @deprecated To be removed in 3.1. Use {@link #format(String, Object[])}instead.
     */

    public String formatString(String key, Object[] arguments)
    {try { __CLOVER_0_0.M[2686]++;
        __CLOVER_0_0.S[10850]++;return getMessages().format(key, arguments);
    } finally { }}

    /**
     * Formats a localized message string, using
     * {@link IMessages#format(java.lang.String, java.lang.Object[])}.
     * 
     * @param key
     *            the key used to obtain a localized pattern using {@link #getString(String)}
     * @param arguments
     *            passed to the formatter
     * @since 3.0
     */

    public String format(String key, Object[] arguments)
    {try { __CLOVER_0_0.M[2687]++;
        // SOP: New name invokes deprecated method (consistency and
        // code coverage); in 3.1 we move the implementation here.

        __CLOVER_0_0.S[10851]++;return formatString(key, arguments);
    } finally { }}

    /**
     * Convienience method for invoking {@link IMessages#format(String, Object[])}
     * 
     * @since 2.2
     * @deprecated To be removed in 3.1. Use {@link #format(String, Object)}instead.
     */

    public String formatString(String key, Object argument)
    {try { __CLOVER_0_0.M[2688]++;
        __CLOVER_0_0.S[10852]++;return getMessages().format(key, argument);
    } finally { }}

    /**
     * Convienience method for invoking {@link IMessages#format(String, Object)}
     * 
     * @since 3.0
     */

    public String format(String key, Object argument)
    {try { __CLOVER_0_0.M[2689]++;
        __CLOVER_0_0.S[10853]++;return formatString(key, argument);
    } finally { }}

    /**
     * Convienience method for invoking {@link IMessages#format(String, Object, Object)}.
     * 
     * @since 2.2
     * @deprecated To be removed in 3.1. Use {@link #format(String, Object, Object)}instead.
     */

    public String formatString(String key, Object argument1, Object argument2)
    {try { __CLOVER_0_0.M[2690]++;
        __CLOVER_0_0.S[10854]++;return getMessages().format(key, argument1, argument2);
    } finally { }}

    /**
     * Convienience method for invoking {@link IMessages#format(String, Object, Object)}.
     * 
     * @since 3.0
     */

    public String format(String key, Object argument1, Object argument2)
    {try { __CLOVER_0_0.M[2691]++;
        __CLOVER_0_0.S[10855]++;return formatString(key, argument1, argument2);
    } finally { }}

    /**
     * Convienience method for {@link IMessages#format(String, Object, Object, Object)}.
     * 
     * @since 2.2
     * @deprecated To be removed in 3.1. Use {@link #format(String, Object, Object, Object)}
     *             instead.
     */

    public String formatString(String key, Object argument1, Object argument2, Object argument3)
    {try { __CLOVER_0_0.M[2692]++;
        __CLOVER_0_0.S[10856]++;return getMessages().format(key, argument1, argument2, argument3);
    } finally { }}

    /**
     * Convienience method for {@link IMessages#format(String, Object, Object, Object)}.
     * 
     * @since 3.0
     */

    public String format(String key, Object argument1, Object argument2, Object argument3)
    {try { __CLOVER_0_0.M[2693]++;
        __CLOVER_0_0.S[10857]++;return formatString(key, argument1, argument2, argument3);
    } finally { }}

    public INamespace getNamespace()
    {try { __CLOVER_0_0.M[2694]++;
        __CLOVER_0_0.S[10858]++;return _namespace;
    } finally { }}

    public void setNamespace(INamespace namespace)
    {try { __CLOVER_0_0.M[2695]++;
        __CLOVER_0_0.S[10859]++;_namespace = namespace;
    } finally { }}

    /**
     * Returns the body of the component, the element (which may be static HTML or components) that
     * the component immediately wraps. May return null. Do not modify the returned array. The array
     * may be padded with nulls.
     * 
     * @since 2.3
     * @see #getBodyCount()
     */

    public IRender[] getBody()
    {try { __CLOVER_0_0.M[2696]++;
        __CLOVER_0_0.S[10860]++;return _body;
    } finally { }}

    /**
     * Returns the active number of elements in the the body, which may be zero.
     * 
     * @since 2.3
     * @see #getBody()
     */

    public int getBodyCount()
    {try { __CLOVER_0_0.M[2697]++;
        __CLOVER_0_0.S[10861]++;return _bodyCount;
    } finally { }}

    /**
     * Empty implementation of
     * {@link org.apache.tapestry.event.PageRenderListener#pageEndRender(PageEvent)}. This allows
     * classes to implement {@link org.apache.tapestry.event.PageRenderListener}and only implement
     * the {@link org.apache.tapestry.event.PageRenderListener#pageBeginRender(PageEvent)}method.
     * 
     * @since 3.0
     */

    public void pageEndRender(PageEvent event)
    {try { __CLOVER_0_0.M[2698]++;
    } finally { }}

    /**
     * Sets a property of a component.
     * 
     * @see IComponent
     * @since 3.0
     */
    public void setProperty(String propertyName, Object value)
    {try { __CLOVER_0_0.M[2699]++;
        __CLOVER_0_0.S[10862]++;OgnlUtils.set(propertyName, this, value);
    } finally { }}

    /**
     * Gets a property of a component.
     * 
     * @see IComponent
     * @since 3.0
     */
    public Object getProperty(String propertyName)
    {try { __CLOVER_0_0.M[2700]++;
        __CLOVER_0_0.S[10863]++;return OgnlUtils.get(propertyName, this);
    } finally { }}
}